/************************************************************
*
*  Copyright (c) 2002-2004 Jakub Schwarzmeier
*
************************************************************/

#ifndef BH_DISTRIB_H
#define BH_DISTRIB_H

#include <stack>
#include "math/matrix.h"
#include "data/list.h"
#include "BarnesHut/bh.h"
#include "../messages.h"
#include "../server.h"

using namespace JakubSchwarzmeier::Math;
using namespace JakubSchwarzmeier::Data;

class ExtGroup;

class RemoteTree;

class LocalTree
{
private:
	
	void initCubeCorners( Vector<> & superiorCorner, RList< Vector<> > & initialCube, real & bisect);

protected:

	RList<JakubSchwarzmeier::BarnesHut::Leave> & leavesList;		//  user supplied
	RemoteTree & remoteTree;		//  user supplied

	RList< RList<JakubSchwarzmeier::BarnesHut::Node> > mostLeftAncestors;
	ExtGroup * group;	//  first used in evaluateNext
	int g;			//  actual group (node/particle) in evaluateNext

public:

	LocalTree( RList<JakubSchwarzmeier::BarnesHut::Leave> & localLeavesList,
		  RemoteTree & remoteTree
		);
	JakubSchwarzmeier::BarnesHut::Node * root;
	RList<ExtGroup> groupsList;		//  WD99

	void fillMessage( message & msg );
	void construct();
	bool evaluateNext();
	void evaluateRemote(int call);
	void integrate();

	std::stack<ExtGroup*> waitingGroups;		//   should be in protected

}; //  class LocalTree


class ExtGroup: public JakubSchwarzmeier::BarnesHut::Group
{
public:
	//  constructors are not inherited
	ExtGroup( JakubSchwarzmeier::BarnesHut::ICommonNode * n_element = NULL): JakubSchwarzmeier::BarnesHut::Group( n_element ), remoteNearList(100, false) {}

	RList<JakubSchwarzmeier::BarnesHut::ICommonNode> remoteNearList;	//  near forces
	RList<JakubSchwarzmeier::BarnesHut::ICommonNode> remoteremoteNearList;	//  near forces
};

class RequestGroup
{
public:
	
	RequestGroup(client * n_Client, double & n_mass, Vector<> & n_position, double n_cellSize, int & n_pointer): nearList(1000, false)
	{
		Client = n_Client;
		mass = n_mass;
		position = n_position;
		cellSize = n_cellSize;
		pointer = n_pointer;
		acceleration = 0;
	}

	client * Client;
	double mass;
	Vector<> position;
	int pointer;
	double cellSize;
	Vector<> acceleration;		// computed acceleration
	RList<JakubSchwarzmeier::BarnesHut::ICommonNode> nearList;	//  near forces
	
};


class RequestsList
{
public:

	RequestsList( LocalTree * n_localTree ): localTree(n_localTree)
	{ }

	void Add( RequestGroup * reqNode );
	void evaluateNext();

	LocalTree * localTree;
	RList<RequestGroup> requestNodes;
};

class RemoteNode: public JakubSchwarzmeier::BarnesHut::ICommonNode
{
public:
	
	client * Client;
	RList<JakubSchwarzmeier::BarnesHut::ICommonNode> childList;
	double bisect;

};

class RemoteTree
{
public:

	void getMessage( crackmessage & CrackMessage, client * n_Client );
	RemoteTree();
	~RemoteTree();
	bool rootsArrived;
	RemoteNode * root;

}; //  class RemoteTree

#endif //  BH_DISTRIB_H