/************************************************************
*
*  Copyright (c) 2002-2004 Jakub Schwarzmeier
*
************************************************************/

#include "general.h"
#include <reason.h>

using namespace JakubSchwarzmeier::BarnesHut;
using namespace JakubSchwarzmeier::Data;

HANDLE hSemNextComputation;
RList<Leave> * leavesList;
RemoteTree * remoteTree;
LocalTree * localTree;
RequestsList * requestsList;
long nRequests, nSatisfiedRequests;
long nOfReadyCMs;
bool bDone;
BYTE nCyclesToUISend;

DWORD WINAPI ComputingThread( LPVOID lpParameter )
{
	UNREFERENCED_PARAMETER(lpParameter);
	
	nOfReadyCMs = 0;

//	FILE * f = fopen("d:\\computing.dbg", "w");

	leavesList = new RList<Leave>;

	while (TRUE)
	{
		//  prepare for next computing step
		//
		int i;
		allDone = 0;	//  no of CMs that completed computing of interactions and waits for integration (change values)
		nRequests = nSatisfiedRequests = 0;

		remoteTree = new RemoteTree;
		localTree = new LocalTree(*leavesList, *remoteTree);
		requestsList = new RequestsList( localTree );
		SetEvent( evFillMessage );		//  permit filling remoteTree with roots from other CMs

		hSemNextComputation = CreateSemaphore(
								NULL,
								0,
								1000000,
								NULL
								);

		//  send message about readiness for next computation
		//
		
		//  but not during first run, hContinueComputing is signalized from UI's CM_LIST message
		if (dt != 0)	
		{
			//  we are ready, say that to others
			for(i = 1; i < Server->Clients.getCount(); i++)
			{
				message SendMessage(0);
				if ( Server->Clients[i].IPAddress != 0 )
				{
					SendMessage.Initialize(CM_READY_FOR_NEXT_COMPUTATION);
					SendMessage.Finish();

					Server->Clients[i].WriteFile( SendMessage.GetBuffer(), SendMessage.GetBufferSize() );
				}
			}


			EnterCriticalSection( &csContinueComputing );
			//  i'm ready for next computation
			if ( InterlockedIncrement( &nOfReadyCMs ) == Server->Clients.getCount()-1 )
				SetEvent(hContinueComputing);	//  if we are last, we must enable hContinueComputing
			LeaveCriticalSection( &csContinueComputing );
		}
		//
		//  end send readiness about continue for computing

//		fprintf(f, "Before hContinueComputing, %d\n", nCyclesToUISend); fflush(f);
		WaitForSingleObject( hContinueComputing, INFINITE );	//  wait for next computation step
//		fprintf(f, "After hContinueComputing\n"); fflush(f);
		
		if (terminateSimulationID > 1)		//  roll back
		{
			delete leavesList;
			delete remoteTree;
			delete localTree;
			delete requestsList;
			CloseHandle(hSemNextComputation);

			return 0;
		}
		
		nOfReadyCMs = 0;	//  no of CMs waiting for initiation of next computing
							//  no problems should be here, messages that influence this value are sended after
							//  CM_COMPUTATION_DONE synchronization
		bDone = false;

		localTree->construct();	//  run phase 1 & 2

		ReleaseSemaphore( hSemNextComputation, localTree->groupsList.getCount(), NULL );

		//  send my root and top level branches to other CMs
		//

		message SendMessage( 1000000  );	//  maximum size for root and two next branches (and its leaves)
		
		for(i = 1; i < Server->Clients.getCount(); i++)
		{
			if ( Server->Clients[i].IPAddress != 0 )
			{
				SendMessage.Initialize( CM_ROOT_AND_TOP_BRANCHES );
				localTree->fillMessage(SendMessage);
				SendMessage.Finish();

				Server->Clients[i].WriteFile( SendMessage.GetBuffer(), SendMessage.GetBufferSize() );
			}
		}
		//
		//  end send my root and top level branches to other CMs

		//  main computing loop
		while (TRUE)
		{
//			fprintf(f, "Pred hSemNextComputation\n"); fflush(f);
			WaitForSingleObject( hSemNextComputation, INFINITE );
//			fprintf(f, "Za hSemNextComputation\n"); fflush(f);

			EnterCriticalSection( &csRequests );
			if (requestsList->requestNodes.getCount() != 0)
			{
				requestsList->evaluateNext();
			}
			else
			{
				EnterCriticalSection( &csAcceleration );

				if ( !localTree->evaluateNext() )
				{
					if ( ( localTree->waitingGroups.empty() ) && (remoteTree->rootsArrived) 
						&& (nRequests == nSatisfiedRequests) && (!bDone) )
					{

						//  this CM made last computation over its tree:
						//  i will not have further requests, but others can
						message Message(0);
						for(int i = 1; i < Server->Clients.getCount(); i++)
						{
							if (Server->Clients[i].IPAddress != 0)
							{
								Message.Initialize(CM_COMPUTATION_DONE);
								Message.Finish();
								Server->Clients[i].WriteFile(Message.GetBuffer(), Message.GetBufferSize());
							}

						}

						bDone = true;  //  indicates that i'm done, but others can have further requests
						InterlockedIncrement( &allDone );
//						fprintf(f, "Odeslana CM_COMPUTATION_DONE, alldone = %d\n", allDone); fflush(f);
						ReleaseSemaphore(hSemNextComputation, 1, NULL);
					}

				}

				localTree->evaluateRemote(1);		//  at least try it

				LeaveCriticalSection( &csAcceleration );
			}

			LeaveCriticalSection( &csRequests );

			if ( allDone == Server->Clients.getCount()-1 )   break;	//  all CMs are done and satisfied
		
		}

		//  clean-up

		CloseHandle( hSemNextComputation );
//		fprintf(f, "After CloseHandle\n\n"); fflush(f);


		//  finish acceleration computing and integrate
		localTree->integrate();

		ResetEvent( evFillMessage );
		delete requestsList;
		delete localTree;
		delete remoteTree;

		//  send local tree to UI
		message SendProcessedData( leavesList->getCount()*8*sizeof(double)  );	//  maximum size for root and two next branches (and its leaves)
		SendProcessedData.Initialize( UI_COMPUTATION_DONE );
		for(i = 0; i < leavesList->getCount(); i++)
		{
			SendProcessedData.Add( leavesList->getAt(i).position[0] );
			SendProcessedData.Add( leavesList->getAt(i).position[1] );
			SendProcessedData.Add( leavesList->getAt(i).position[2] );
			SendProcessedData.Add( leavesList->getAt(i).velocity[0] );
			SendProcessedData.Add( leavesList->getAt(i).velocity[1] );
			SendProcessedData.Add( leavesList->getAt(i).velocity[2] );
			SendProcessedData.Add( leavesList->getAt(i).mass );
			if ( leavesList->getAt(i).load == 0 )  
				SendProcessedData.Add( 1.0 );
			else
				SendProcessedData.Add( leavesList->getAt(i).load );
		}

		SendProcessedData.Finish();

		Server->Clients[0].WriteFile(SendProcessedData.GetBuffer(), SendProcessedData.GetBufferSize());

		if (nCyclesToUISend == 0)
		{
			delete leavesList;
			leavesList = new RList<Leave>;

			theta = epsilon = dt = 0;
		}
		else
		{
			nCyclesToUISend--;
		}

	}

}

