/************************************************************
*
*  Copyright (c) 2002-2004 Jakub Schwarzmeier
*
************************************************************/

#define _WIN32_IE 0x0500
#include <winsock2.h>
#include "data/xml.h"
#include "general.h"
#include <float.h>		//  _controlfp
#include <aclapi.h>

#define WM_NOTIFY_ICON ( WM_APP + 1 )

//  Global variables

HINSTANCE g_hInstance;		//  handle to current instance
HWND g_hWnd;				//  handle to main window
TCHAR szTitle[] = TEXT("AMON - Computing Machine");	//  title bar text
server * Server;

HANDLE hContinueComputing;

CRITICAL_SECTION csRequests;
CRITICAL_SECTION csAcceleration;
CRITICAL_SECTION csContinueComputing;

HANDLE evFillMessage;

long allDone = 0;

double theta, epsilon, dt;
BYTE terminateSimulationID;

//  local main.cpp variables

TCHAR szWindowClass[] = TEXT ("JSCMClass");	// main window class name
HANDLE hComputingThread;

BOOL Wnd_OnCreate(HWND hWnd, LPCREATESTRUCT lpCreateStruct)
{
	UNREFERENCED_PARAMETER(lpCreateStruct);

	Server = new server;		//  creates worker thread

	hContinueComputing = CreateEvent(
			NULL,				//  security descriptor
			FALSE,				//  manual reset?
			FALSE,				//  is initial state signaled
			NULL				//  name
			);

	//  Create computing thread
	DWORD dwThreadId;
	hComputingThread = (void *)_beginthreadex(
		NULL,							//  security descriptor
		0,								//  stack size same as stack size of main thread
		(PTHREAD_START)ComputingThread,	//  address of function of new thread
		NULL,							//  argument list
		0,								//  thread will be running
		(unsigned *)&dwThreadId			//  thread identifier (dummy)
		);
	
//   SetThreadPriority(hComputingThread, THREAD_PRIORITY_HIGHEST);

#ifndef DEBUG
	SetPriorityClass(GetCurrentProcess(), REALTIME_PRIORITY_CLASS);
#endif

	//  associate messages from WinSock with window
	Server->BindToWindow( hWnd );

	terminateSimulationID = 0;		//  initialize

	DWORD dwAddress = MAKEIPADDRESS( 147, 228, 105, 88 );

	//  try to find .config file

	TCHAR szConfig[MAX_PATH];
	GetModuleFileName( g_hInstance, szConfig, MAX_PATH );
	lstrcat( szConfig, TEXT(".config") );

	using namespace JakubSchwarzmeier::Data;

	if (PathFileExists(szConfig))
	{
		//  read XML file
		XmlDocument xmlDoc;
		xmlDoc.Load( szConfig );

		XmlNodeList address = xmlDoc.SelectNodes( TEXT("/configuration/address/*/text()") );

		XmlNode num;
		num = address.GetItem(0);
		int first = StrToInt( num.GetText() );
		num = address.GetItem(1);
		int second = StrToInt( num.GetText() );
		num = address.GetItem(2);
		int third = StrToInt( num.GetText() );
		num = address.GetItem(3);
		int fourth = StrToInt( num.GetText() );

		dwAddress = MAKEIPADDRESS( first, second, third, fourth );
		
		ConnectLoop( dwAddress );
	}
	else
	{
		//  show connection dlg for connecting to UI
		if ( ShowIpAddressDlg( hWnd, &dwAddress ) == IDCANCEL )
		{
			return FALSE;		//  user choosed Cancel or pressed Escape, terminate app
		}

		if (!ConnectLoop( dwAddress, FALSE) )
		{
			MessageBox(TEXT("Error during connect()"));

			return FALSE;		//  terminate app
		}

		//  write XML file
		XmlDocument xmlDoc;

		XmlElement root = xmlDoc.CreateElement(TEXT("configuration"));
		xmlDoc.AppendChild( root );

		XmlElement element;
		XmlText text;

		XmlElement address = xmlDoc.CreateElement(TEXT("address"));
		root.AppendChild( address );

		element = xmlDoc.CreateElement(TEXT("first"));
		text = xmlDoc.CreateTextNode( (int)FIRST_IPADDRESS(dwAddress) );
		element.AppendChild( text );
		address.AppendChild( element );

		element = xmlDoc.CreateElement(TEXT("second"));
		text = xmlDoc.CreateTextNode( (int)SECOND_IPADDRESS(dwAddress) );
		element.AppendChild( text );
		address.AppendChild( element );

		element = xmlDoc.CreateElement(TEXT("third"));
		text = xmlDoc.CreateTextNode( (int)THIRD_IPADDRESS(dwAddress) );
		element.AppendChild( text );
		address.AppendChild( element );

		element = xmlDoc.CreateElement(TEXT("fourth"));
		text = xmlDoc.CreateTextNode( (int)FOURTH_IPADDRESS(dwAddress) );
		element.AppendChild( text );
		address.AppendChild( element );

		xmlDoc.DoHumanMarkup( root );
		xmlDoc.Save( szConfig );

	}


	//  add tray icon (icon in the taskbar status area)
	//

	NOTIFYICONDATA nid;
	ZeroMemory( &nid, sizeof(nid) );
	nid.cbSize = sizeof(nid);
	nid.hWnd = hWnd;
	nid.uID = 1;
	lstrcpy(nid.szTip, szTitle);
	nid.uFlags = NIF_ICON | NIF_MESSAGE | NIF_TIP | NIF_INFO;
	nid.uCallbackMessage = WM_NOTIFY_ICON;
	nid.hIcon = (HICON)GetClassLong(hWnd, GCL_HICONSM);
	lstrcpy( nid.szInfo, TEXT("Connected to User Interface.\n") );
	nid.uTimeout = 10000;
	lstrcpy( nid.szInfoTitle, szTitle );
	nid.dwInfoFlags = NIIF_INFO;

	Shell_NotifyIcon( NIM_ADD, &nid );

	return TRUE;
}

int nConnectedMachines = 0;

LRESULT Wnd_OnListenMessage( HWND hWnd, WPARAM wParam, LPARAM lParam )
{
	  
	SOCKADDR_IN sockname;
	SOCKET sIncoming = wParam;

	int bbc = WSAGETSELECTERROR(lParam);

	switch ( WSAGETSELECTEVENT( lParam ) ) 
	{
	int i;

	case FD_ACCEPT:
		{
		
		int iSize = sizeof(sockname);

		SOCKET sAccepted = accept(sIncoming, (sockaddr*) &sockname, &iSize );
/*
		//  disable the socket to be bound to an address that is already in use
		int optval = 1;
	//	int ss = setsockopt( sListeningSocket, SOL_SOCKET, SO_EXCLUSIVEADDRUSE, (char*)&optval, sizeof(optval) );
		int zero = 0;
		int ss = setsockopt( sAccepted, SOL_SOCKET, SO_RCVBUF, (char*)&zero, sizeof(zero) );
		ss = setsockopt( sAccepted, SOL_SOCKET, SO_SNDBUF, (char*)&zero, sizeof(zero) );
		ss = setsockopt( sAccepted, IPPROTO_TCP, TCP_NODELAY, (char*)&optval, sizeof(optval) );

*/
		//  Find client that belong to that listening socket and accept him
		for(i = 1; i < Server->Clients.getCount(); i++)
		{
			if ( sockname.sin_addr.S_un.S_addr == Server->Clients[i].IPAddress )
			{
				//  accept incoming connection and new socket associate with completion port
				Server->Clients[i].SetSocket( sAccepted );

				//  increase number of connected CMs
				nConnectedMachines++;
				//  startup reading process from freshly connected CM
				Server->Clients[i].ReadFile();
				break;
			}
		}

		}		
		
		break;

	case FD_CONNECT:

		for(i = 1; i < Server->Clients.getCount(); i++)
		{
			if ( (SOCKET)Server->Clients[i].Socket == sIncoming )
			{
				nConnectedMachines++;
				//  associate socket with completion port
				Server->Clients[i].SetSocket( (SOCKET)Server->Clients[i].Socket );

				//  startup reading process to freshly connected CM
				Server->Clients[i].ReadFile();
				break;
			}
		}
		
		break;

	default:
	
		return DefWindowProc(hWnd, WM_LISTENMESSAGE, wParam, lParam);

	} //  switch

	if ( nConnectedMachines == Server->Clients.getCount()-2 )	//  UI -1
	{
		//  free listening socket for this IP address
		closesocket( sListeningSocket );

		//  connection with every CM was made, start computing thread
		//  if there is only one CM connected to UI (me), this code will never be executed
 		ResumeThread( hComputingThread );
	}

	return 0;

}

LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	switch( uMsg )
	{
	HANDLE_MSG( hWnd, WM_CREATE, Wnd_OnCreate );
	

	case WM_LISTENMESSAGE:

		return Wnd_OnListenMessage( hWnd, wParam, lParam);

	case WM_SYSCOMMAND:
        
		switch ( wParam )
		{
		case SC_SCREENSAVE:
		case SC_MONITORPOWER:
			
			break;

		case SC_MINIMIZE: 

			ShowWindow(hWnd, SW_HIDE); 
			break;
	
		default:

			DefWindowProc(hWnd, uMsg, wParam, lParam);
			break;
		}

		return 0;

	case WM_NOTIFY_ICON:

		switch ( lParam )
		{
		case WM_LBUTTONUP:

			
			break;
		}

		return 0;


	case WM_DESTROY:

		//  remove icon from tray
		//
		NOTIFYICONDATA nid;
		ZeroMemory( &nid, sizeof(nid) );
		nid.cbSize = sizeof(nid);
		nid.hWnd = hWnd;
		nid.uID = 1;
		nid.uFlags = NIF_ICON;
		
		Shell_NotifyIcon( NIM_DELETE, &nid );

		CloseHandle( hContinueComputing );
		PostQuitMessage(0);
		return 0;


	default:

		return DefWindowProc(hWnd, uMsg, wParam, lParam);
	}

	//  window procedure should never gets here
}

BOOL RegisterWindowClass()
{
	//  Register the window class
	WNDCLASSEX wndclass;

    wndclass.cbSize = sizeof(WNDCLASSEX);
    wndclass.style = CS_OWNDC | CS_HREDRAW | CS_VREDRAW;
    wndclass.lpfnWndProc = WndProc;
    wndclass.cbClsExtra = 0;
    wndclass.cbWndExtra = 0;
    wndclass.hInstance = g_hInstance;
	wndclass.hIcon = (HICON)LoadImage(g_hInstance, MAKEINTRESOURCE(IDI_ICON), IMAGE_ICON, 0, 0, LR_DEFAULTCOLOR);  // 32x32
    wndclass.hCursor = LoadCursor(NULL, IDC_ARROW);
    wndclass.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
	wndclass.lpszMenuName = NULL;
    wndclass.lpszClassName = szWindowClass;
	wndclass.hIconSm = (HICON)LoadImage(g_hInstance, MAKEINTRESOURCE(IDI_SMALL), IMAGE_ICON, 0, 0, LR_DEFAULTCOLOR);  // 16x16

    return RegisterClassEx(&wndclass);
}

BOOL CreateNewWindow(int nCmdShow)	   
{
	//  Create main window & initialize its global variable
	g_hWnd = CreateWindowEx(
					 0, 
					 szWindowClass,
					 szTitle,
					 WS_OVERLAPPEDWINDOW | WS_CLIPSIBLINGS | WS_CLIPCHILDREN,	//  include WS_VISIBLE
					 CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
//					 HWND_MESSAGE,
					 NULL,
					 NULL,
					 g_hInstance,
					 NULL);  

	if (g_hWnd == NULL) return (FALSE);

/*
	//  Show main window according to WinMain's nCmdShow passed parameter
	//  It's not necessary to call this if style is WS_VISIBLE
	ShowWindow(g_hWnd, nCmdShow);
	UpdateWindow(g_hWnd); 
*/	return (TRUE);		

}

int APIENTRY _tWinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
					    LPTSTR lpszCmdLine, int nCmdShow )
{
	//  CM cannot connect to itself, permit only one instance
	TCHAR szMutex[] = TEXT("ComputingMachine");
	
	HANDLE hMutex = OpenMutex( MUTEX_ALL_ACCESS, FALSE, szMutex );
	if ( hMutex != NULL )
	{
		MessageBox( TEXT("Application instance already running.") );
		CloseHandle( hMutex );
		return 1;
	}
	else
	{
		hMutex = CreateMutex( NULL, TRUE, szMutex );
	}
/*
	// construct an empty DACL
	ACL acl;
	InitializeAcl(&acl, sizeof acl, ACL_REVISION);

	//  disable user mainly to terminate process and lower its priority
	SetSecurityInfo( GetCurrentProcess(), 
		SE_KERNEL_OBJECT, 
		DACL_SECURITY_INFORMATION,
		0,
		0,
		&acl,
		0);
*/
	MSG msg;

	g_hInstance = hInstance;
	UNREFERENCED_PARAMETER(hPrevInstance);	//  hPrevInstance is always NULL in Win32
	UNREFERENCED_PARAMETER(lpszCmdLine);

	//  initialization of common controls -- required under NT4
	//  if not, DialogBox with IP control will fail
	INITCOMMONCONTROLSEX iccex;
	iccex.dwSize = sizeof iccex;
	iccex.dwICC = ICC_INTERNET_CLASSES;
	InitCommonControlsEx(&iccex);

	//  Turn on FP exceptions
	int cw = _controlfp( 0,0 );   // Get the default control word.
	cw &=~(EM_OVERFLOW|EM_UNDERFLOW|EM_ZERODIVIDE|EM_DENORMAL);	// Set the exception masks OFF, turn exceptions on.
	_controlfp( cw, MCW_EM );		// Set the control word.

	//  Initialize global variables
	InitializeCriticalSection( &csRequests );
	InitializeCriticalSection( &csAcceleration );
	InitializeCriticalSection( &csContinueComputing );

	evFillMessage = CreateEvent(NULL, TRUE, FALSE, NULL);

	theta = epsilon = dt = 0;

	if (!RegisterWindowClass())
	{
		// fails if builded for UNICODE and running under 9X/ME
		MessageBox ( NULL, TEXT("This program requires Windows NT4/2000/XP+"),
					 szTitle, MB_ICONERROR );
		return 0;
	}

	do
	{
		CreateNewWindow(nCmdShow);

		while( GetMessage(&msg, NULL, 0, 0) )
		{
			TranslateMessage( &msg );
			DispatchMessage( &msg );
		}

		// Free system sources

		DestroyWindow( g_hWnd );	//  release memory needed for window
	
	} while( terminateSimulationID == 5 );

	
	WNDCLASSEX wndclass;
	GetClassInfoEx( g_hInstance, szWindowClass, &wndclass );
	
	DestroyIcon( wndclass.hIcon );	//  release memory needed for icon
	DestroyIcon( wndclass.hIconSm );	//  release memory needed for small icon

	UnregisterClass( szWindowClass, g_hInstance ); //  release memory holding info about window class

	CloseHandle( evFillMessage );

	DeleteCriticalSection( &csContinueComputing );
	DeleteCriticalSection( &csAcceleration );
	DeleteCriticalSection( &csRequests );
	if (Server != NULL)  delete Server;

	CloseHandle( hMutex );

	return msg.wParam;
}
