/************************************************************
*
*  Copyright (c) 2002-2004 Jakub Schwarzmeier
*
************************************************************/

#include "general.h"
#include "../server.h"
#include <reason.h>		//  reasons of application terminating

SOCKET sListeningSocket;

void Listen();
void Connect(int i);
void SetPrivileges();

DWORD WINAPI WorkerThread( LPVOID lpParameter )
{
	HANDLE hCompletionPort = lpParameter;

//	FILE * f= fopen("d:\\worker.dbg", "w");

	while (TRUE)
	{
		DWORD dwBytesTransffered;
		OVERLAPPED * lpOverlapped;	//  points "out"
		client * Client;			//  points "out"

		BOOL bSuccess = GetQueuedCompletionStatus( hCompletionPort, &dwBytesTransffered,
			(unsigned long *) &Client, &lpOverlapped, INFINITE );

		//  unused for now
		if ( Server->bServiceTerminating )
		{
			return 0;		//  terminates thread
		}

		if ( bSuccess )
		{
			if ( Client->ReadOperation(lpOverlapped, dwBytesTransffered) )
			{	
				//  IORead
//			fprintf(f, "BEGIN Read\n"); fflush(f);

//				if ( dwBytesTransffered == 0 )  continue;	//  why?

				while ( Client->NextMessage() )
				{
					
					crackmessage CrackMessage;
					Client->getMessage( CrackMessage );
					
					switch( CrackMessage.GetCode() )
					{
					case CM_LIST:
						{
//						fprintf(f, "CM_LIST Begin\n"); fflush(f);
						//  read list of IPs of CMs
						while ( CrackMessage.NextData() )
						{
							Server->Clients.Add( new client( CrackMessage.GetInt() ) );	//  begins at clients[1], because [0] is UI
						}

						//  Connect (listen) to other CMs
						//

						//  one side must *listen (accept connections)* and second must *connect*
						//  communication is bidirectional over one socket

						//  algorithm is following: listen to all CMs over me in list & connect 
						//  to all CMs bellow my position in list
						
						//  first find my position in list
						
						//  listen to all above
						Listen();

						message mWaitForConnect(0);
						mWaitForConnect.Initialize( UI_WAITING_FOR_CONNECT_PERMISSION );
						mWaitForConnect.Finish();
						Server->Clients[0].WriteFile( mWaitForConnect.GetBuffer(), mWaitForConnect.GetBufferSize() );
//						fprintf(f, "CM_LIST END\n"); fflush(f);

						}	//  case CM_LIST
						break;

					case CM_CONNECT_TO_OTHERS:
						{
						//  first find my position in list
//						fprintf(f, "CM_CONNECT_TO_OTHERS Begin\n"); fflush(f);
						
						int i;
						int me;

						for(i = 1; i < Server->Clients.getCount(); i++)
						{
							if ( Server->Clients[i].IPAddress == 0 )
								me = i;
						}

						//  connect to all bellow
						for(i = me+1; i < Server->Clients.getCount(); i++)
						{
							Connect(i);
						}
						}
//						fprintf(f, "CM_CONNECT_TO_OTHERS End\n"); fflush(f);

						break;	

					case CM_INITIAL_DATA:
//						fprintf(f, "CM_INITIAL_DATA Begin\n"); fflush(f);
						
						//  read computing parameters
						theta = CrackMessage.GetDouble();
						epsilon = CrackMessage.GetDouble();
						dt = CrackMessage.GetDouble();
						nCyclesToUISend = CrackMessage.GetBYTE();

						while ( CrackMessage.NextData() )
						{
							Vector<> position;
							Vector<> velocity;

							for(int i = 0; i < 3; i++)	position[i] = CrackMessage.GetDouble();
							for(i = 0; i < 3; i++)	velocity[i] = CrackMessage.GetDouble();
							double mass = CrackMessage.GetDouble();
							
							leavesList->Add( new JakubSchwarzmeier::BarnesHut::Leave(position, velocity, mass) );
						}
//						fprintf(f, "CM_INITIAL_DATA End\n"); fflush(f);

						//  we have all initial data, continue computation thread
						SetEvent( hContinueComputing );
						break;

					case CM_ROOT_AND_TOP_BRANCHES:
						
//						fprintf(f, "CM_ROOT_AND_TOP_BRANCHES Begin\n"); fflush(f);
						WaitForSingleObject( evFillMessage, INFINITE );
						remoteTree->getMessage( CrackMessage, Client );
						if ( remoteTree->root->childList.getCount() == Server->Clients.getCount()-2 )	//  -2: me + UI
						{
							remoteTree->rootsArrived = true;
							ReleaseSemaphore(hSemNextComputation, 1, NULL);
						}

//						fprintf(f, "CM_ROOT_AND_TOP_BRANCHES End\n"); fflush(f);
						break;

					case CM_COMPUTING_REQUEST:
						{

//						fprintf(f, "CM_COMPUTING_REQUEST Begin\n"); fflush(f);
						EnterCriticalSection( &csRequests );

						double mass = CrackMessage.GetDouble();
						Vector<> position;
						position[0] = CrackMessage.GetDouble();
						position[1] = CrackMessage.GetDouble();
						position[2] = CrackMessage.GetDouble();
						double cellSize = CrackMessage.GetDouble();
						int pointer = CrackMessage.GetInt();

						requestsList->Add( new RequestGroup( Client, mass, position, cellSize, pointer ) );
		
						ReleaseSemaphore(hSemNextComputation, 1, NULL);

						LeaveCriticalSection( &csRequests );
//						fprintf(f, "CM_COMPUTING_REQUEST End\n"); fflush(f);

						}

						break;

					case CM_REQUEST_SATISFIED:
						{
//						fprintf(f, "CM_REQUEST_SATISFIED Begin\n"); fflush(f);
						EnterCriticalSection( &csAcceleration );
						
						ExtGroup * group = (ExtGroup*) CrackMessage.GetInt();
						group->acceleration[0] += CrackMessage.GetDouble();
						group->acceleration[1] += CrackMessage.GetDouble();
						group->acceleration[2] += CrackMessage.GetDouble();

						while ( CrackMessage.NextData() )
						{
							double mass = CrackMessage.GetDouble();
							Vector<> position;
							position[0] = CrackMessage.GetDouble();
							position[1] = CrackMessage.GetDouble();
							position[2] = CrackMessage.GetDouble();

							group->remoteremoteNearList.Add( new JakubSchwarzmeier::BarnesHut::ICommonNode( position, mass )  );
						}

						InterlockedIncrement( &nSatisfiedRequests );
						LeaveCriticalSection( &csAcceleration );

						ReleaseSemaphore(hSemNextComputation, 1, NULL);
						}
//						fprintf(f, "CM_REQUEST_SATISFIED End\n"); fflush(f);

						break;

					
					//  CM completed computing of interactions and waits for integration (change values)
					
					case CM_COMPUTATION_DONE:	

//						fprintf(f, "CM_COMPUTATION_DONE\n"); fflush(f);
						InterlockedIncrement( &allDone );
						ReleaseSemaphore( hSemNextComputation, 1, NULL );

						break;

					//  integrations performed, readiness for next computation

					case CM_READY_FOR_NEXT_COMPUTATION:

//						fprintf(f, "CM_READY_FOR_NEXT_COMPUTATION\n"); fflush(f);
						EnterCriticalSection( &csContinueComputing );
						if ( InterlockedIncrement( &nOfReadyCMs ) == Server->Clients.getCount()-1)
							SetEvent(hContinueComputing);
						LeaveCriticalSection( &csContinueComputing );

						break;

					case CM_TERMINATE_SIMULATION:

						terminateSimulationID = CrackMessage.GetBYTE();
						SetEvent( hContinueComputing );		//  permit computing thread that will terminate itself

						delete Server;
						Server = NULL;

						switch(terminateSimulationID)
						{
		
						case 2:

							//  exit program
							PostMessage(g_hWnd, WM_CLOSE, NULL, NULL);
							_endthreadex(0);
							break;
						
						case 3:

							//  logout user
							SetPrivileges();
							ExitWindowsEx( EWX_LOGOFF , SHTDN_REASON_MAJOR_APPLICATION | SHTDN_REASON_MINOR_OTHER | SHTDN_REASON_FLAG_USER_DEFINED );	//  | EWX_FORCEIFHUNG
							PostMessage(g_hWnd, WM_CLOSE, NULL, NULL);
							_endthreadex(0);
							break;
						
						case 4:

							//  turnoff computer
							SetPrivileges();
							ExitWindowsEx( EWX_POWEROFF, SHTDN_REASON_MAJOR_APPLICATION | SHTDN_REASON_MINOR_OTHER | SHTDN_REASON_FLAG_USER_DEFINED );
							PostMessage(g_hWnd, WM_CLOSE, NULL, NULL);
							_endthreadex(0);
							break;
						
						case 5:

							//  restart program
							PostMessage(g_hWnd, WM_CLOSE, NULL, NULL);
							_endthreadex(0);
							break;
						
						}	//  end switch(terminateSimulationID)

						return 0;		//  terminate thread in every case

						break;

					default:
						
						MessageBox( (TCHAR*)Client->ReadBuffer );
						break;
					
					}	//  end switch( CrackMessage.GetCode() )

					ReleaseSemaphore(hSemNextComputation, 1, NULL);


				}	//  end while ( Client->NextMessage() )

				// pend another read request to get the next client request
				Client->ReadFile();
//			fprintf(f, "END Read\n\n"); fflush(f);

			}
			else
			{
				//  IOWrite
				//  MessageBox( TEXT("IOWrite Done") );
			}
		}
		else
		{
		//  there are many wanted occasions that can create error, so ignore it for this time
		/*
			if ( Client->ReadOper( lpOverlapped ) )
				MessageBox(TEXT("Error during read"));

			else
				MessageBox(TEXT("Error during write"));

			ShowLastError();
		*/
		}
    
		//
		// loop around to get another completed IO request
		//
	}

}


void Listen()
{
	//  set up listening to each CM in list

	//  create unbinded listening socket, overlapped attribute is default
	//  this socket will be closed after connecting
	sListeningSocket = socket( PF_INET, SOCK_STREAM, IPPROTO_TCP );
	if ( sListeningSocket == INVALID_SOCKET )
	{
		// error
	}
/*
	//  disable the socket to be bound to an address that is already in use
	int optval = 1;
//	int ss = setsockopt( sListeningSocket, SOL_SOCKET, SO_EXCLUSIVEADDRUSE, (char*)&optval, sizeof(optval) );
	int zero = 0;
	int ss = setsockopt( sListeningSocket, SOL_SOCKET, SO_RCVBUF, (char*)&zero, sizeof(zero) );
	ss = setsockopt( sListeningSocket, SOL_SOCKET, SO_SNDBUF, (char*)&zero, sizeof(zero) );
	ss = setsockopt( sListeningSocket, IPPROTO_TCP, TCP_NODELAY, (char*)&optval, sizeof(optval) );
*/

	//  Bind this socket to SOCKADDR_IN structure
	SOCKADDR_IN sockname;
	sockname.sin_family = AF_INET;
	sockname.sin_addr.S_un.S_addr = htonl(INADDR_ANY);
	sockname.sin_port = htons(1981);	//  on this port

	int bind_status = bind( sListeningSocket, (sockaddr*) &sockname, sizeof(sockname) );
	if ( bind_status == SOCKET_ERROR )
	{
		// error
		int e = WSAGetLastError();
	}

	//  use listening socket in asynchronous mode
	int async_status = WSAAsyncSelect( sListeningSocket, Server->hWnd, 
		WM_LISTENMESSAGE, FD_ACCEPT );

	if ( async_status == SOCKET_ERROR )
	{
		// error
		int e = WSAGetLastError();
	}

	//  wait for incoming connections and insert them into queue
	int listen_status = listen( sListeningSocket, SOMAXCONN );
	if ( listen_status == SOCKET_ERROR )
	{
		// error
		int e = WSAGetLastError();
	}
}

void Connect(int i)
{
	//  create unbinded socket, overlapped attribute is default
	Server->Clients[i].Socket = (HANDLE) socket( PF_INET, SOCK_STREAM, IPPROTO_TCP );
	int e = WSAGetLastError();

	/*
	//  disable the socket to be bound to an address that is already in use
	int optval = 1;
//	int ss = setsockopt( sListeningSocket, SOL_SOCKET, SO_EXCLUSIVEADDRUSE, (char*)&optval, sizeof(optval) );
	int zero = 0;
	int ss = setsockopt( sListeningSocket, SOL_SOCKET, SO_RCVBUF, (char*)&zero, sizeof(zero) );
	ss = setsockopt( sListeningSocket, SOL_SOCKET, SO_SNDBUF, (char*)&zero, sizeof(zero) );
	ss = setsockopt( sListeningSocket, IPPROTO_TCP, TCP_NODELAY, (char*)&optval, sizeof(optval) );
*/
	//  Connect this socket to SOCKADDR_IN structure
	SOCKADDR_IN sockname;
	sockname.sin_family = AF_INET;
	sockname.sin_addr.S_un.S_addr = Server->Clients[i].IPAddress;	//  connect to this machine
	sockname.sin_port = htons(1981);	//  on this port

	//  use connect() in asynchronous mode
	int async_status = WSAAsyncSelect( (SOCKET)Server->Clients[i].Socket, Server->hWnd, 
								   WM_LISTENMESSAGE, FD_CONNECT );

	e = WSAGetLastError();
	
	if ( async_status == SOCKET_ERROR )
	{
		//  error
		int e = WSAGetLastError();
	}

	//  connect to CM
	int aa = connect( (SOCKET)Server->Clients[i].Socket, (sockaddr*) &sockname, sizeof(sockname) );
	int bb =  WSAGetLastError();
}

void SetPrivileges()
{
	HANDLE hToken; 
	TOKEN_PRIVILEGES tkp; 
 
	//  get a token for this process. 
	OpenProcessToken(GetCurrentProcess(), TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY, &hToken); 
 
	//  get the LUID for the shutdown privilege. 
 	LookupPrivilegeValue(NULL, SE_SHUTDOWN_NAME, &tkp.Privileges[0].Luid); 
 
	tkp.PrivilegeCount = 1;  // one privilege to set    
	tkp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED; 
 
	//  get the shutdown privilege for this process. 
 
	AdjustTokenPrivileges(hToken, FALSE, &tkp, 0, (PTOKEN_PRIVILEGES)NULL, 0); 
}
