/************************************************************
*
*  Copyright (c) 2002-2004 Jakub Schwarzmeier
*
************************************************************/

#include "server.h"
#include "messages.h"
#include "misc.h"

client::client( int n_IPAddress, SOCKET s )
{
	WriteBuffers = new BUFFER;

	Socket = (HANDLE) s;
	//  reserve & commit memory for reading buffer
	//  should be made variable (for large no of CMs is insufficient memory)
	ReadBufferSize = 10000000;
	ReadBuffer = VirtualAlloc( NULL, ReadBufferSize, MEM_RESERVE | MEM_COMMIT, PAGE_READWRITE );
	unfinishedBuffer = VirtualAlloc( NULL, ReadBufferSize, MEM_RESERVE | MEM_COMMIT, PAGE_READWRITE );
	unfinishedUsedSize = 0;

	IPAddress = n_IPAddress;

	if ( s != 0 )
	{
		//  associate socket with completion port
		CreateIoCompletionPort( 
			Socket,
			::Server->hCompletionPort, 
			(unsigned long) this,
			0
			);
	}

	InitializeCriticalSection( &csWriteBuffers );
}

HANDLE client::SetSocket(SOCKET s)
{
	Socket = (HANDLE)s;

	return 	::CreateIoCompletionPort( 
			Socket, 
			::Server->hCompletionPort, 
			(unsigned long) this,
			0
			);
}

BOOL client::ReadOper(OVERLAPPED * ol)
{
	if ( ol == &Overlapped )		//  reading overlapped structure
	{
		return TRUE;
	}
	else
	{
		return FALSE;
	}
}

BOOL client::ReadOperation(OVERLAPPED * ol, DWORD & totalSize)
{
	if ( ol == &Overlapped )		//  reading overlapped structure
	{
		if ( unfinishedUsedSize != 0 )
		{
			//  copy new buffer behind unfinished buffer
			CopyMemory( (BYTE*)unfinishedBuffer+unfinishedUsedSize, ReadBuffer, totalSize );
			//  increment buffer size with unfinished buffer size
			totalSize += unfinishedUsedSize;
			// swap buffers
			LPVOID bak = ReadBuffer;
			ReadBuffer = unfinishedBuffer;
			unfinishedBuffer = bak;
			
			unfinishedUsedSize = 0;
		}

		BYTE * position = (BYTE *)ReadBuffer;

		while (1)
		{
			int * pactualSize = (int*) position;
			int actualSize = *pactualSize;

			if ( ( ( ((int)position)-((int)ReadBuffer)+ 4) <= totalSize )		//  ensure that size info is valid
				&& ( ((((int)position)-((int)ReadBuffer))+actualSize) <= totalSize ) )
			{
				mesBuf.push_back( position+4 );		//  4 is length of size information
				mesSize.push_back( actualSize-4 );
			}
			else
			{
				int iPos = (int)position;
				int iReadBuffer = (int)ReadBuffer;
				unfinishedUsedSize = totalSize - (iPos - iReadBuffer);
				CopyMemory( (BYTE*)unfinishedBuffer, position, unfinishedUsedSize );
				break;	//  while loop
			}
			
			if ( ((((int)position)-((int)ReadBuffer))+actualSize) == totalSize )
			{
				unfinishedUsedSize = 0;
				break;	//  while loop
			}

			position += actualSize;
		}
		
		return TRUE;
	}
	else
	{
		EnterCriticalSection( &csWriteBuffers );
	
	
		//  free buffer
		BUFFER::iterator bufIter = WriteBuffers->find( ol );
		BYTE * lpBuffer = (*bufIter).second;
		WriteBuffers->erase( bufIter );
		delete ol;	//  free writing overlapped structure
		delete [] lpBuffer;	//  free write buffer

		LeaveCriticalSection( &csWriteBuffers );

		return FALSE;
	}
}

bool client::NextMessage()
{
	return ( !mesBuf.empty() );
}

void client::getMessage( crackmessage & CrackMessage )
{
	CrackMessage.Initialize( mesBuf.front(), mesSize.front() );
	mesBuf.pop_front();
	mesSize.pop_front();
}

BOOL client::ReadFile()
{
	EnterCriticalSection( &csWriteBuffers );
	Overlapped.Internal = 0;
	Overlapped.InternalHigh = 0;
	Overlapped.Offset = 0;
	Overlapped.OffsetHigh = 0;
	Overlapped.hEvent = NULL;

	
	BOOL ret = ::ReadFile(
		Socket, 
		ReadBuffer,
		ReadBufferSize,
		NULL,
		&Overlapped
		);

	LeaveCriticalSection( &csWriteBuffers );


	return ret;
		
}

BOOL client::WriteFile(BYTE * lpBuffer, DWORD dwNumberOfBytesToWrite)
{
	EnterCriticalSection( &csWriteBuffers );

	DWORD dwBytesWritten;	
	OVERLAPPED * ol = new OVERLAPPED;
	
	ol->Internal = 0;
	ol->InternalHigh = 0;
	ol->Offset = 0;
	ol->OffsetHigh = 0;
	ol->hEvent = NULL;

	WriteBuffers->insert( BUFFER::value_type(ol, lpBuffer) );
	
	BOOL retValue = ::WriteFile( Socket,
		lpBuffer,
		dwNumberOfBytesToWrite,
		&dwBytesWritten,
		ol
		);

	LeaveCriticalSection( &csWriteBuffers );

	return retValue;
}


client::~client()
{
	DeleteCriticalSection( &csWriteBuffers );

	delete WriteBuffers;
	VirtualFree( ReadBuffer, 0, MEM_RELEASE );
	VirtualFree( unfinishedBuffer, 0, MEM_RELEASE );
	closesocket( (SOCKET) Socket );
}

server::server()
{
	//  create new completion port
	hCompletionPort = 
		CreateIoCompletionPort( INVALID_HANDLE_VALUE, NULL, 0, 0 );
	
	DWORD dwThreadId;	//  dummy for us

	//  create worker thread
	hWorkerThread = (void *)_beginthreadex(
		NULL,							//  security descriptor
		0,								//  stack size same as stack size of main thread
		(PTHREAD_START)WorkerThread,	//  address of function of new thread
		hCompletionPort,				//  argument list
		0,								//  thread will run
		(unsigned *)&dwThreadId			//  thread identifier (dummy)
		);

	SetThreadPriority(hWorkerThread, THREAD_PRIORITY_HIGHEST);

	//  initialize to WinSock 2.2
	WSADATA wsadata;
	WSAStartup( MAKEWORD( 2, 2 ), &wsadata );

	//  Get my IP address
	HOSTENT * hostent;
	IN_ADDR addr;
	hostent = gethostbyname("");
	if (hostent == 0)
	{
		MessageBox( TEXT("gethostbyname() failed") );
	}
	
	addr.S_un.S_un_b.s_b1 = hostent->h_addr_list[0][0];
	addr.S_un.S_un_b.s_b2 = hostent->h_addr_list[0][1];
	addr.S_un.S_un_b.s_b3 = hostent->h_addr_list[0][2];
	addr.S_un.S_un_b.s_b4 = hostent->h_addr_list[0][3];
	
	IPAddress = addr.S_un.S_addr;

	bServiceTerminating = FALSE;
}

server::~server()
{
	//  clean after yourself
	
	//  here should be freed remaining allocated areas from WriteBuffers

	//  thread must terminate itself by calling return or _enthreadex
	//  bServiceTerminating = TRUE;		
//	PostQueuedCompletionStatus( hCompletionPort, 0, 0, NULL );

	CloseHandle( hCompletionPort );
	
	WSACleanup();
}

void server::BindToWindow( HWND n_hWnd )
{
	hWnd = n_hWnd;
}