/************************************************************
*
*  Copyright (c) 2002-2004 Jakub Schwarzmeier
*
************************************************************/

#ifndef AUXINFO_H
#define AUXINFO_H

#include "general.h"
#include "windows/audiovideo/audiovideo.h"
#include "data/xml.h"

#define real double

using namespace JakubSchwarzmeier::Data;

class AuxInfo
{
public:

	TCHAR szDataFile [MAX_PATH];
	HANDLE hDataFile;
	TCHAR szVideoFile [MAX_PATH];

	real * lpAddress;
	double time;

	 JakubSchwarzmeier::Windows::AudioVideo avi;	// private

public:
	AuxInfo( TCHAR n_szDataFileName[], TCHAR n_szVideoFileName[] )
	{
		lstrcpy( szDataFile, n_szDataFileName );
		lstrcpy( szVideoFile, n_szVideoFileName );
		
		hDataFile = CreateFile(szDataFile, 
									GENERIC_WRITE, 
									FILE_SHARE_READ, 
									NULL, 
									OPEN_ALWAYS, 
									FILE_ATTRIBUTE_ARCHIVE,
									NULL
									);

		LARGE_INTEGER size;
		size.QuadPart = 0;
		GetFileSizeEx( hDataFile, &size );		
//		size.LowPart = GetFileSize( hDataFile, reinterpret_cast<LPDWORD>(&size.HighPart) );	//  NT4


		__int64 nOfCorrectSnapShots = 0;
		if (size.QuadPart != 0)
		{
			//  try to find .xml file

			TCHAR szXML[MAX_PATH];
			lstrcpy( szXML, szDataFile );
			PathRenameExtension( szXML, TEXT(".xml") );

			//  read XML file
			XmlDocument xmlDoc;
			xmlDoc.Load( szXML );

			XmlNode xBodies = xmlDoc.SelectSingleNode( TEXT("/simulation/count/text()") );
			int nBodies = StrToInt( xBodies.GetText() );

			LARGE_INTEGER pos;
			pos.QuadPart = 0;
			__int64 snapShotSize = 7*sizeof(double)*nBodies;

			nOfCorrectSnapShots = size.QuadPart / snapShotSize;

			if ( (size.QuadPart % snapShotSize) != 0 )
			{
				//  set pointer to last correct data snapshot, that's 2nd snapshot from the end
				pos.QuadPart += nOfCorrectSnapShots*snapShotSize;
				SetFilePointerEx( hDataFile, pos, NULL, FILE_BEGIN );	
//				SetFilePointer( hDataFile, pos.LowPart, &pos.HighPart, FILE_BEGIN );	//  NT4
			}
			else
			{
				//  file is complete or not exists before, move to the end
				SetFilePointer(hDataFile, 0, 0, FILE_END);
			}
		
		}

		time = 1.06*dt*nOfCorrectSnapShots;

		avi.CreateFile( szVideoFile, openGL->GetDibSection() );
		avi.WriteFrame( openGL->GetDibSection() );		// first write

		lpAddress = (real*) VirtualAlloc( NULL, 
										7 * sizeof(real) * unsortedBodiesList->GetCount(), 
										MEM_COMMIT | MEM_RESERVE, 
										PAGE_READWRITE 
										);

	}

	~AuxInfo()
	{
		delete unsortedBodiesList;
		unsortedBodiesList = NULL;
		VirtualFree( lpAddress, 0, MEM_DECOMMIT | MEM_RELEASE );
		CloseHandle( hDataFile );
	}

	double Time()
	{
		return time += 1.06*dt;
	}

	void WriteData()
	{

		for(int i = 0; i < unsortedBodiesList->GetCount(); i++)
		{
			lpAddress[i * 7 + 0] = unsortedBodiesList->getAt(i).mass;
			lpAddress[i * 7 + 1] = unsortedBodiesList->getAt(i).position[0];
			lpAddress[i * 7 + 2] = unsortedBodiesList->getAt(i).position[1];
			lpAddress[i * 7 + 3] = unsortedBodiesList->getAt(i).position[2];
			lpAddress[i * 7 + 4] = unsortedBodiesList->getAt(i).velocity[0];
			lpAddress[i * 7 + 5] = unsortedBodiesList->getAt(i).velocity[1];
			lpAddress[i * 7 + 6] = unsortedBodiesList->getAt(i).velocity[2];
		}

		DWORD dwWritten;

		WriteFile( hDataFile, 
			lpAddress,
			7 * sizeof(real) * unsortedBodiesList->GetCount(), 
			&dwWritten,
			NULL
			);

		//  save video and force redraw
		SendMessage( g_hWnd, WM_UPDATE_VIEW, 0, 0 );
	}
};

#undef real

#endif	//  AUXINFO_H