/************************************************************
*
*  Copyright (c) 2002-2004 Jakub Schwarzmeier
*
************************************************************/

#include "general.h"
#include "../messages.h"

SOCKET sListeningSocket;
WinString formatString(512);		//  saved in dialog resource
long nListenMachines;				//  number of listening machines waiting for connection permission

VOID ConnectionDlg_OnListenMessage( HWND hDlg, WPARAM wParam, LPARAM lParam )
{
	UNREFERENCED_PARAMETER(wParam);

	//  someone is connecting

	SOCKADDR_IN sockname;

	switch ( WSAGETSELECTEVENT( lParam ) ) 
	{
	case FD_ACCEPT:{
		//  create active connection && fill sockname structure
		int i = sizeof(sockname);
		//  accept new client
		SOCKET sConnectedSocket = accept(sListeningSocket, (sockaddr*) &sockname, &i );

		//  disable the socket to be bound to an address that is already in use
		int optval = 1;
	//	int ss = setsockopt( sListeningSocket, SOL_SOCKET, SO_EXCLUSIVEADDRUSE, (char*)&optval, sizeof(optval) );
		int zero = 0;
		int ss = setsockopt( sConnectedSocket, SOL_SOCKET, SO_RCVBUF, (char*)&zero, sizeof(zero) );
		ss = setsockopt( sConnectedSocket, SOL_SOCKET, SO_SNDBUF, (char*)&zero, sizeof(zero) );
		ss = setsockopt( sConnectedSocket, IPPROTO_TCP, TCP_NODELAY, (char*)&optval, sizeof(optval) );
	
		
		//  create new client
		Server->Clients.Add( new client(sockname.sin_addr.S_un.S_addr, sConnectedSocket) );
		//  startup reading process from CM
		Server->Clients[ Server->Clients.getCount()-1 ].ReadFile();
		
		//  add client to listbox
		WinString clientIP(512);
		DWORD dwLength = clientIP.GetMaximumLength(); 
		WSAAddressToString( (sockaddr*) &sockname, 16, NULL, clientIP.GetBuffer(), &dwLength);
		ListBox_AddString( GetDlgItem(hDlg, IDC_CMLIST), clientIP);
		
		//  update count of CMs in dialog string
		WinString showString(512);
		HWND hCountofCM = GetDlgItem( hDlg, IDC_COUNTOFCM );
		wsprintf( showString.GetBuffer(), formatString, Server->Clients.getCount() );
		SetWindowText( hCountofCM, showString );

		}
  		break;
            
	} //  switch

}

VOID ConnectionDlg_OnContinue( HWND hDlg, WPARAM wParam, LPARAM lParam )
{
	UNREFERENCED_PARAMETER(wParam);
	UNREFERENCED_PARAMETER(lParam);

	closesocket( sListeningSocket );
	
	//  send to all connected nodes list of all IP addresses
	message SendMessage( Server->Clients.getCount()*sizeof(int) );

	for(int i = 0; i < Server->Clients.getCount(); i++)
	{

		SendMessage.Initialize( CM_LIST );
	
		for(int ip = 0; ip < Server->Clients.getCount(); ip++)
		{
			if ( Server->Clients[i].IPAddress != Server->Clients[ip].IPAddress )
				SendMessage.Add( Server->Clients[ip].IPAddress );
			else

				SendMessage.Add( 0 );
		}
		
		SendMessage.Finish();

		Server->Clients[i].WriteFile( SendMessage.GetBuffer(), SendMessage.GetBufferSize() );
	}

	//  close dialog
	EndDialog( hDlg, 0 );
}

BOOL ConnectionDlg_OnInitDialog(HWND hDlg, HWND hWndFocus, LPARAM lParam)
{
	UNREFERENCED_PARAMETER(hWndFocus);
	UNREFERENCED_PARAMETER(lParam);

	nListenMachines = 0;

	//  winsock is initialized from server object

	//  display IP address of local station

	//  inet_ntoa works only with chars not wchars, so i don't use it

	TCHAR szLocalAddress[512];
	_stprintf( szLocalAddress, _T("%d.%d.%d.%d"),  
		LOBYTE( LOWORD( Server->IPAddress ) ),
		HIBYTE( LOWORD( Server->IPAddress ) ),
		LOBYTE( HIWORD( Server->IPAddress ) ),
		HIBYTE( HIWORD( Server->IPAddress ) )
		);

	SetDlgItemText(hDlg, IDC_ESTABLISH_CONNECTION_IPADDRESS, szLocalAddress );
	
	//  set up listening 

	//  create unbinded socket, overlapped attribute is default
	sListeningSocket = socket( PF_INET, SOCK_STREAM, IPPROTO_TCP );
	if ( sListeningSocket == INVALID_SOCKET )
	{
		Language.ShowLastError(IDS_LISTENING_SOCKET_ERROR);
		EndDialog ( hDlg, 1 );

	}

	//  Bind this socket to SOCKADDR_IN structure
	SOCKADDR_IN sockname;
	sockname.sin_family = AF_INET;
	sockname.sin_addr.S_un.S_addr = INADDR_ANY; //  listen to all addresses
	sockname.sin_port = htons(1981);

	int bind_status = bind( sListeningSocket, (sockaddr*) &sockname, sizeof(sockname) );
	if ( bind_status == SOCKET_ERROR )
	{
		Language.ShowLastError(IDS_BIND_ERROR);		
		EndDialog ( hDlg, 1 );
	}

	//  wait for incoming connections and insert them into queue
	int listen_status = listen( sListeningSocket, SOMAXCONN );
	if ( listen_status == SOCKET_ERROR )
	{
		Language.ShowLastError(IDS_LISTEN_ERROR);		
		EndDialog ( hDlg, 1 );
	}

	//  use listening socket in asynchronous mode, when something WinSock happen,
	//  it call WM_LISTENMESSAGE;
	//  creates new thread which is terminated as part of WSACleanup() call
	int async_status = WSAAsyncSelect( sListeningSocket, hDlg, 
								   WM_LISTENMESSAGE, FD_ACCEPT );
	if ( async_status == SOCKET_ERROR )
	{
		Language.ShowLastError(IDS_ASYNC_ERROR);		
		EndDialog ( hDlg, 1 );
	}

	//  display initial number of CMs
	WinString showString(512);

	HWND hCountofCM = GetDlgItem( hDlg, IDC_COUNTOFCM );
	GetWindowText( hCountofCM, formatString.GetBuffer(), formatString.GetMaximumLength() );
	wsprintf( showString.GetBuffer(), formatString, Server->Clients.getCount() );
	SetWindowText( hCountofCM, showString );

	return TRUE;
}

BOOL CALLBACK ConnectionDlgProc (HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch ( uMsg )
    {
    case WM_INITDIALOG: 

		return HANDLE_WM_INITDIALOG(hDlg, wParam, lParam, ConnectionDlg_OnInitDialog);

	case WM_LISTENMESSAGE:

		ConnectionDlg_OnListenMessage( hDlg, wParam, lParam);
		break;
	
	case WM_COMMAND:

		switch ( LOWORD(wParam) )
		{
		case IDC_CONTINUE:

			ConnectionDlg_OnContinue( hDlg, wParam, lParam );
			break;
		}

		break; //  case WM_COMMAND


	//  Useless for this dialog (dialog resource hasn't defined caption)
	case WM_SYSCOMMAND: 
		
		switch ( wParam ) 
		{ 
		 
		//  Called when user choosed from system menu close or clicked on closing (cross) button
		case SC_CLOSE: 

            EndDialog ( hDlg, 0 );
			return TRUE; 
		}

		break; // case WM_SYSCOMMAND

	}	// switch ( uMsg )

	return FALSE;
}


void ShowEstablishConnectionDlg( HWND hWnd )
{
	Server = new server;

	DialogBox( Language.GetResourceInstance(), MAKEINTRESOURCE(IDD_ESTABLISH_CONNECTION), hWnd, ConnectionDlgProc );
}
