/************************************************************
*
*  Copyright (c) 2002-2004 Jakub Schwarzmeier
*
************************************************************/

#include "general.h"
#include "data/xml.h"
#include <stdio.h>

#define real double

using namespace JakubSchwarzmeier::Data;

void ShowOpenDlg (HWND hWnd)
{
	TCHAR szDataFileName [MAX_PATH] = TEXT("\0");
	TCHAR szFileName [MAX_PATH] = TEXT("\0");

	OPENFILENAME ofn;
	ZeroMemory( &ofn, sizeof(ofn) );
	ofn.lStructSize		= sizeof (OPENFILENAME);
	ofn.Flags			= OFN_FILEMUSTEXIST | OFN_HIDEREADONLY;
	ofn.hwndOwner		= hWnd;
	ofn.lpstrFilter     = Language.GetFileFilterString (IDS_NBODY_FILTERSTRING);
	ofn.nFilterIndex    = 1L;
	ofn.lpstrFile       = szDataFileName;
	ofn.nMaxFile        = sizeof (szDataFileName); //  fix this?
	ofn.lpstrFileTitle  = szFileName;
	ofn.nMaxFileTitle   = sizeof (szFileName);			//  fix this?

	BOOL bOpen = GetOpenFileName (&ofn);


	if ( bOpen )
	{
		unsortedBodiesList = new JakubSchwarzmeier::Data::RList< JakubSchwarzmeier::BarnesHut::Leave >;
	
		LoadBodiesFromFile( szDataFileName, *unsortedBodiesList );

		//  set initial load to one for evry body
		for(int i = 0; i < unsortedBodiesList->getCount(); i++)  unsortedBodiesList->getAt(i).load = 1;

 		nTotalBodies = unsortedBodiesList->getCount();
		openGL->InitializeMasses();

		TCHAR szVideoFileName [MAX_PATH];
		lstrcpy( szVideoFileName, szDataFileName );
		PathRenameExtension( szVideoFileName, TEXT(".avi") );
		
		auxInfo = new AuxInfo( szDataFileName, szVideoFileName );

		auxInfo->WriteData();
	}
}

void LoadBodiesFromFile(TCHAR szFileName[], JakubSchwarzmeier::Data::RList< JakubSchwarzmeier::BarnesHut::Leave > & unsortedBodiesList )
{
	HANDLE hInFile = CreateFile(szFileName, 
		GENERIC_READ, 
		FILE_SHARE_READ, 
		NULL, 
		OPEN_EXISTING, 
		FILE_ATTRIBUTE_ARCHIVE,
		NULL
		);
	
	DWORD dwBytesRead;
	BOOL bResult;

	//  get number of bodies in file
	//

	//  try to find .xml file

	TCHAR szXML[MAX_PATH];
	lstrcpy( szXML, szFileName );
	PathRenameExtension( szXML, TEXT(".xml") );

	//  read XML file
	XmlDocument xmlDoc;
	xmlDoc.Load( szXML );

	XmlNode xBodies = xmlDoc.SelectSingleNode( TEXT("/simulation/count/text()") );
	int nBodies = StrToInt( xBodies.GetText() );
	XmlNode xTheta = xmlDoc.SelectSingleNode( TEXT("/simulation/theta/text()") );
	theta = _tstof( xTheta.GetText() );
	XmlNode xEpsilon = xmlDoc.SelectSingleNode( TEXT("/simulation/epsilon/text()") );
	epsilon = _tstof( xEpsilon.GetText() );
	XmlNode xDt = xmlDoc.SelectSingleNode( TEXT("/simulation/dt/text()") );
	dt = _tstof( xDt.GetText() );

	//
	//  end get number of bodies in file

	//  check file for completeness
	LARGE_INTEGER size;
	GetFileSizeEx( hInFile, &size );		
//	size.LowPart = GetFileSize( hInFile, reinterpret_cast<LPDWORD>(&size.HighPart) );	//  NT4

	LARGE_INTEGER pos;
	pos.QuadPart = 0;
	__int64 snapShotSize = 7*sizeof(real)*nBodies;
	__int64 nOfCorrectSnapShots = size.QuadPart / snapShotSize;
	if ( (size.QuadPart % snapShotSize) != 0 )
	{
		MessageBox( Language.GetString(IDS_FILE_INCOMPLETE_MESSAGE) );
	
		//  set pointer to last correct data snapshot, that's 2nd snapshot from the end
		pos.QuadPart += nOfCorrectSnapShots*(snapShotSize-1);
		SetFilePointerEx( hInFile, pos, NULL, FILE_BEGIN );
//		SetFilePointer( hInFile, pos.LowPart, &pos.HighPart, FILE_BEGIN );	//  NT4
	}
	else
	{
		//  get last snapshot from file
		pos.QuadPart -= snapShotSize;
		SetFilePointerEx( hInFile, pos, NULL, FILE_END );
//		SetFilePointer( hInFile, pos.LowPart, &pos.HighPart, FILE_END );	//  NT4
	}
	
	do
	{
		real adBodies[7];

		bResult = ReadFile( hInFile, &adBodies, 7*sizeof(real), &dwBytesRead, NULL );
		if ( dwBytesRead != 0 )
		{
			double mass = adBodies[0];
			Vector<> position;
			position[0] = adBodies[1];
			position[1] = adBodies[2];
			position[2] = adBodies[3];
			

			Vector<> velocity;
			velocity[0] = adBodies[4];
			velocity[1] = adBodies[5];
			velocity[2] = adBodies[6];

			unsortedBodiesList.Add( new JakubSchwarzmeier::BarnesHut::Leave(position, velocity, mass) );
		}

	} while ( bResult==0 || dwBytesRead!=0);

	CloseHandle (hInFile);
}

#undef real 
