/************************************************************
*
*  Copyright (c) 2002-2004 Jakub Schwarzmeier
*
************************************************************/

#include "distribution.h"
#include <algorithm>

using namespace JakubSchwarzmeier::Data;
using namespace JakubSchwarzmeier::Math;
using namespace JakubSchwarzmeier::BarnesHut;

void initCubeCorners( Vector<> & superiorCorner, RList< Vector<> > & initCorners, real & bisect)
{
	static int i = -1;
	Vector<> corner = superiorCorner;	//  calls "copy" constructor & performs "deep" copy

	i++;	//  actual component
	for(int j = 0; j < 2; j++)	//  bisection; every component divided into two parts
	{
		corner[i] += j*bisect;
		if (i+1 < 3)	//  3 defined as DIMENSIONS in bh.h
		{	//  still not at the end of recursion, go deeper
			initCubeCorners(corner, initCorners, bisect);
		}
		else
		{	//  one of final solutions, final coordinates of corner
			initCorners.Add( new Vector<> );
			initCorners [ initCorners.getCount()-1 ] = corner;
		}  //  end of code for one of final solutions
	
	}  // end of bisection cycle

	i--;
}

void hilbertWalkThrough(
			RList< Leave > & sourceList,
			Corner & corner,
			double & bisect,
			RList< Leave > * destinationList
			)
{

	//  taken from bh.cpp
	//

	//  create list for corners
	RList< Vector<> > initCorners(8);
	//  add corners and init their values in ordinary cube
	initCubeCorners( corner.corner, initCorners, bisect );

	//  create list for Hilbert's order
	RList< Vector<> > finalCorner(8, false);
	for(int i = 0; i < 8; i++)  finalCorner.Add( &(initCorners[i]) );	//  values has no meaning now, only for the right size of list

	//  order corners according to Hilbert's curve
	//

	//  make Hilbert's cube from ordinary cube 
	RList< Vector<> >::iterator initIter = finalCorner.begin();
	initIter += initCorners[7];	initIter += initCorners[5];
	initIter += initCorners[1];	initIter += initCorners[3];
	initIter += initCorners[2];	initIter += initCorners[0];
	initIter += initCorners[4];	initIter = initCorners[6];
	
	//  order curve according to angles
	
	//  x rotation
	for(i = 0; i < corner.angles.x; i=i+90 )
	{
		//  make tmp copy
//				std::copy( finalCorner.begin(), finalCorner.end(), initCorners.begin() );
		for(int j = 0; j < 8; j++)  initCorners.storage[j] = finalCorner.storage[j];

		RList< Vector<> >::iterator iterator = finalCorner.begin();
		iterator += initCorners[1];	iterator += initCorners[2];
		iterator += initCorners[3];	iterator += initCorners[0];
		iterator += initCorners[7];	iterator += initCorners[4];
		iterator += initCorners[5];	iterator = initCorners[6];
	}

	//  y rotation
	for(i = 0; i < corner.angles.y; i=i+90 )
	{
		//  make tmp copy	... nejde protoze on dela hlubokou kopii (nekopiruje pointery, ale hodnoty)
		// plati i pro vyse a nize
		//  nelze totiz ani
		// for(int j = 0; j < 8; j++)  initCorners[j] = finalCorner[j];
		//  on nezkopiruje pointery, ale udela hlubokou kopii
//				std::copy( finalCorner.begin(), finalCorner.end(), initCorners.begin() );
		for(int j = 0; j < 8; j++)  initCorners.storage[j] = finalCorner.storage[j];

		RList< Vector<> >::iterator iterator = finalCorner.begin();
		iterator += initCorners[7];	iterator += initCorners[6];
		iterator += initCorners[1];	iterator += initCorners[0];
		iterator += initCorners[3];	iterator += initCorners[2];
		iterator += initCorners[5];	iterator = initCorners[4];
	}

	//  z rotation
	for(i = 0; i < corner.angles.z; i=i+90 )
	{
		//  make tmp copy
//				std::copy( finalCorner.begin(), finalCorner.end(), initCorners.begin() );
		for(int j = 0; j < 8; j++)  initCorners.storage[j] = finalCorner.storage[j];

		RList< Vector<> >::iterator iterator = finalCorner.begin();
		iterator += initCorners[1];	iterator += initCorners[6];
		iterator += initCorners[5];	iterator += initCorners[2];
		iterator += initCorners[3];	iterator += initCorners[4];
		iterator += initCorners[7];	iterator = initCorners[0];
	}

	//
	//  order corners according to Hilbert's curve

	//  space partitioning according to Hilbert's curve
	//

	for(i = 0; i < 8; i++)
	{
		//  we get rightCorner by adding bisect to every component of corner vector
		Vector<> rightCorner = finalCorner[i];
		std::for_each( rightCorner.begin(), rightCorner.end(), AddValue<> (bisect) );

		int count = 0;
		int pos = 0;
		
		RList< Leave > newsourceList(100, false);
		Corner newCorner;

		//  find out how many leaves belong to this partition, so iterate over all of them
		for(int k = 0; k < sourceList.getCount(); k++)
		{
			//  look at operators >= and < ! they determinate which leave belong to this partition
			if ( (sourceList[k].position >= finalCorner[i]) && (sourceList[k].position < rightCorner) )
			{  //  leave sourceList[k] belongs to partion given by corner vector
				if ( count == 0 )
				{
					pos = k;
					count++;
				}
				else
				{
					if ( count == 1)		//  called only ones
					{
						switch (i)			//  i identifies present corner of Hilbert's cube
						{
						case 0:
							newCorner.angles.x = corner.angles.getXinc90();
							newCorner.angles.y = corner.angles.getYdec90();
							newCorner.angles.z = corner.angles.z;
							break;
						
						case 1:
						case 2:
							newCorner.angles.x = corner.angles.getXdec90();
							newCorner.angles.y = corner.angles.y;
							newCorner.angles.z = corner.angles.getZinc90();
							break;
						
						case 3:
						case 4:
							newCorner.angles.x = corner.angles.getXinc180();
							newCorner.angles.y = corner.angles.y;
							newCorner.angles.z = corner.angles.z;
							break;

						case 5:
						case 6:
							newCorner.angles.x = corner.angles.getXdec90();
							newCorner.angles.y = corner.angles.getYdec90();
							newCorner.angles.z = corner.angles.z;
							break;

						case 7:
							newCorner.angles.x = corner.angles.getXinc90();
							newCorner.angles.y = corner.angles.getYinc90();
							newCorner.angles.z = corner.angles.z;
							break;
						}

						newsourceList.Add( &( sourceList[pos] ) );		//  add previous
					}

					newsourceList.Add( &( sourceList[k] ) );
					count++;	
				}
			} 
		}  //  end of cycling through collection of positions

		if (count == 1)	//  only one child node so that's leave
		{
			destinationList->Add( &sourceList[pos] );
//			sourceList.removeAt(pos);
		}
		else
		if (count > 1)
		{	
			//  go deeper
			double newBisect = bisect / 2;
			newCorner.corner = finalCorner[i];

			hilbertWalkThrough( newsourceList, newCorner, newBisect, destinationList );
		}
	
	}	//  end of cycling through corners
	

	//
	//  taken from bh.cpp

}