/************************************************************
*
*  Copyright (c) 2002-2004 Jakub Schwarzmeier
*
************************************************************/

#include "general.h"
#include <stdio.h>	//  _stprintf
#include <float.h>	//  __controlfp

//  Global variables

 HINSTANCE g_hInstance;		//  handle to current instance
 HWND g_hWnd;				//  handle to main window
 TCHAR szTitle[SZ_LENGTH];	//  title bar text
 Lang Language;				//  international purposes
 server * Server;			
 JakubSchwarzmeier::Data::RList< JakubSchwarzmeier::BarnesHut::Leave > * unsortedBodiesList;
 OpenGLView * openGL;
 AuxInfo * auxInfo;
 BYTE terminateSimulationID;
 BYTE nCyclesToResend;
 bool isSimulationRunning;

//  Local main.cpp variables

 TCHAR szWindowClass[] = TEXT ("JSUIClass");	// main window class name
 Camera camera;

LRESULT Wnd_OnCommand( HWND hWnd, int wmId, HWND hWndControl, int wmCode )
{
	switch (wmId)	// switch( LOWORD(wParam) )
	{
	case ID_FILE_NEW_GALAXY:
		
		ShowGalaxySimulationDlg (hWnd);

		break;

	case ID_FILE_NEW_ENCOUNTER:

		ShowEncounterSimulationDlg (hWnd);

		break;

	case ID_FILE_OPEN:

		ShowOpenDlg (hWnd);

		break;

	case ID_FILE_CLOSE:

		delete auxInfo;
		auxInfo = NULL;
		delete unsortedBodiesList;
		unsortedBodiesList = NULL;

		break;

	case ID_FILE_EXIT:

		SendMessage(hWnd, WM_CLOSE, 0, 0);
		break;

	case ID_CONNECTION_ESTABLISH:
		
		ShowEstablishConnectionDlg (hWnd);
		break;

	case ID_CONNECTION_CANCEL:
		{
		
		WinString strMessage(512);
		WinString strCaption(128);
			
		strMessage.AddToString( Language.GetString(IDS_CANCEL_CONNECTION_QUESTION) );
		strCaption.AddToString( Language.GetString(IDS_CANCEL_CONNECTION_CAPTION) );

		if ( MessageBox( hWnd, strMessage, strCaption, MB_YESNO | MB_ICONQUESTION ) == IDYES )
		{
			terminateSimulationID = 5;
			DialogBox( Language.GetResourceInstance(), MAKEINTRESOURCE(IDD_WAIT_FOR_TERMINATE), hWnd, WaitForTerminateDlgProc );
		}

		}
		
		break;

	case ID_SIMULATION_RUN:

		terminateSimulationID = 0;
		currentlyRecievedBody = 0;
		nCyclesToResend = 5;
		isSimulationRunning = true;
		OrderAndSendData();

		break;

	case ID_SIMULATION_TERMINATE:

		if ( ShowTerminateSimulationDlg( hWnd ) == IDOK )	
		{
			DialogBox( Language.GetResourceInstance(), MAKEINTRESOURCE(IDD_WAIT_FOR_TERMINATE), hWnd, WaitForTerminateDlgProc );
		}

		break;


	default:

		FORWARD_WM_COMMAND( hWnd, wmId, hWndControl, wmCode, DefWindowProc);
	}

	return 0;	//  if an application processes message, it return zero
}

void Wnd_OnSize(HWND hWnd, UINT state, int cx, int cy)
{
	UNREFERENCED_PARAMETER(hWnd);
	UNREFERENCED_PARAMETER(state);

	openGL->OnSize(cx, cy);
}


void showcamerastate(HWND hWnd)
{

		 TCHAR tmpText[512], tmpText2[512];
		 lstrcpy(tmpText, TEXT(""));

		 double tmpVal[16];
		 for(int i = 0; i<16;i++)
			 tmpVal[i] = camera[i];
		 camera.Inverse();

		_stprintf(tmpText2, _T("%.*g"), 4, camera.values[12]);
		lstrcat(tmpText, tmpText2);
		lstrcat(tmpText, TEXT(", "));
		_stprintf(tmpText2, _T("%.*g"), 4, camera.values[13]);
		lstrcat(tmpText, tmpText2);
		lstrcat(tmpText, TEXT(", "));
		_stprintf(tmpText2, _T("%.*g"), 4, camera.values[14]);
		lstrcat(tmpText, tmpText2);

		lstrcat(tmpText, TEXT("; lambda: "));
		_stprintf(tmpText2, _T("%.*g"), 4, camera.GetLambda());
		lstrcat(tmpText, tmpText2);

		lstrcat(tmpText, TEXT(", fi: "));
		_stprintf(tmpText2, _T("%.*g"), 4, camera.GetFi());
		lstrcat(tmpText, tmpText2);

		lstrcat(tmpText, TEXT(", theta: "));
		 _stprintf(tmpText2, _T("%.*g"), 4, camera.GetTheta());
		lstrcat(tmpText, tmpText2);

		 for(i = 0; i<16;i++)
			 camera.values[i] = tmpVal[i];


		 SetWindowText(hWnd, tmpText);

}


void Wnd_OnKeyDown(HWND hWnd, UINT vk, BOOL fDown, int cRepeat, UINT flags)
{
	UNREFERENCED_PARAMETER(fDown);
	UNREFERENCED_PARAMETER(flags);

	switch ( vk )
	{
	case VK_LEFT:

		camera.ChangeLambdaBy(5*cRepeat);
		break;

	case VK_RIGHT:

		camera.ChangeLambdaBy(-1*5*cRepeat);
		break;

	case VK_UP:

		camera.ChangePositionBy(0, 0, -cRepeat);
		break;

	case VK_DOWN:

		camera.ChangePositionBy(0, 0, cRepeat);
		break;


	case VK_INSERT:

		camera.ChangeFiBy(5);
		break;

	case VK_DELETE:

		camera.ChangeFiBy(-5);
		break;

	case VK_HOME:

		camera.ChangeThetaBy(5);
		break;

	case VK_END:

		camera.ChangeThetaBy(-5);
		break;
	}

	showcamerastate( hWnd);
	
	openGL->Render();
	InvalidateRect(hWnd, NULL, FALSE);
}

LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	switch( uMsg )
	{
	
//	case WM_COMMAND:
		HANDLE_MSG( hWnd, WM_COMMAND, Wnd_OnCommand );

	case WM_KEYDOWN:

		HANDLE_WM_KEYDOWN( hWnd, wParam, lParam, Wnd_OnKeyDown );
		return 0;
		
	case WM_CREATE:

		openGL = new OpenGLView(hWnd);
		openGL->SetCamera( &camera );
		camera.ChangePositionBy(0, 0, 90);

		return 0;

	case WM_INITMENU:
		{
		HMENU hMenu = (HMENU) wParam;

		//  File-New and File-Open
		if ( auxInfo == NULL ) 
		{
			EnableMenuItem ( hMenu, ID_FILE_NEW_GALAXY, MF_ENABLED );
			EnableMenuItem ( hMenu, ID_FILE_NEW_ENCOUNTER, MF_ENABLED );
			EnableMenuItem ( hMenu, ID_FILE_OPEN, MF_ENABLED );
		}
		else
		{
			EnableMenuItem ( hMenu, ID_FILE_NEW_GALAXY, MF_GRAYED );
			EnableMenuItem ( hMenu, ID_FILE_NEW_ENCOUNTER, MF_GRAYED );
			EnableMenuItem ( hMenu, ID_FILE_OPEN, MF_GRAYED );
		}

		//  File-Close
		if ( ( auxInfo != NULL ) && (!isSimulationRunning) )
		{
			EnableMenuItem ( hMenu, ID_FILE_CLOSE, MF_ENABLED );
		}
		else
		{
			EnableMenuItem ( hMenu, ID_FILE_CLOSE, MF_GRAYED );
		}

		//  File-Exit

		HMENU hSystemMenu = GetSystemMenu( hWnd, FALSE );
		
		if ( Server == NULL )
		{
			EnableMenuItem ( hSystemMenu, SC_CLOSE, MF_ENABLED );
			EnableMenuItem ( hMenu, ID_FILE_EXIT, MF_ENABLED );
		}
		else
		{
			EnableMenuItem ( hSystemMenu, SC_CLOSE, MF_GRAYED );
			EnableMenuItem ( hMenu, ID_FILE_EXIT, MF_GRAYED );
		}

		//  Connection-Establish
		if ( Server == NULL )
		{
			EnableMenuItem ( hMenu, ID_CONNECTION_ESTABLISH, MF_ENABLED );
		}
		else
		{
			EnableMenuItem ( hMenu, ID_CONNECTION_ESTABLISH, MF_GRAYED );
		}

		//  Connection-Cancel
		if ( Server != NULL )
		{
			EnableMenuItem ( hMenu, ID_CONNECTION_CANCEL, MF_ENABLED );
		}
		else
		{
			EnableMenuItem ( hMenu, ID_CONNECTION_CANCEL, MF_GRAYED );
		}

		//  Simulation-Run
		if ( ( auxInfo != NULL ) && ( Server != NULL ) && (!isSimulationRunning) )
		{
			EnableMenuItem ( hMenu, ID_SIMULATION_RUN, MF_ENABLED );
		}
		else
		{
			EnableMenuItem ( hMenu, ID_SIMULATION_RUN, MF_GRAYED );
		}

		//  Simulation-Terminate
		if ( ( Server != NULL ) && (isSimulationRunning) )
		{
			EnableMenuItem ( hMenu, ID_SIMULATION_TERMINATE, MF_ENABLED );
		}
		else
		{
			EnableMenuItem ( hMenu, ID_SIMULATION_TERMINATE, MF_GRAYED );
		}

		}
		return 0;

	case WM_SIZE:

		HANDLE_WM_SIZE( hWnd, wParam, lParam, Wnd_OnSize );
		return 0;

	//  the system sends this message when there are no other messages
	//  in the application's message queue (even when there are another windows over it)
	//  it is very special -- that everything goes good, you must call BeginPaint and EndPaint
	//	else message boxes will "hidden" (will be shown after Alt pressed), function is called
	//  round about
	//  if i don't call in WM_PAINT routine Begin and EndPaint, it's send round about

	case WM_PAINT:	
		{
		HDC hDC;
		PAINTSTRUCT ps;
		hDC = BeginPaint(hWnd, &ps);

 /*
if (isSimulationRunning)
{
		for (int n = 0; n < unsortedBodiesList->getCount(); n++)
		unsortedBodiesList->getAt(n).acceleration = 0;

  for(int a = 0; a  < (unsortedBodiesList->getCount()-1); a++)		// na sebe nepusobi
    for (int b = a + 1; b < unsortedBodiesList->getCount(); b++)
	{
		JakubSchwarzmeier::Math::Vector<> rvect = unsortedBodiesList->getAt(b).position - unsortedBodiesList->getAt(a).position;

	  double d = rvect.length();

	// force that acts (b)-->(a); accel to (a) from (b)
	  unsortedBodiesList->getAt(a).acceleration += rvect*unsortedBodiesList->getAt(b).mass/pow(d,3);
	  unsortedBodiesList->getAt(b).acceleration += -1*rvect*unsortedBodiesList->getAt(a).mass/pow(d,3);
    }


  // integrate
  for(n = 0; n < unsortedBodiesList->getCount(); n++) {
  unsortedBodiesList->getAt(n).velocity+=unsortedBodiesList->getAt(n).acceleration*dt;
  unsortedBodiesList->getAt(n).position+=unsortedBodiesList->getAt(n).velocity*dt;

  }

  JakubSchwarzmeier::BarnesHut::Tree bhTree(*unsortedBodiesList);
  bhTree.step();
}
 */
	

		openGL->OnPaint( hDC );
		EndPaint(hWnd, &ps);
		}
// /*		
		if (isSimulationRunning)
			SendMessage(g_hWnd, WM_UPDATE_VIEW, 0, 0);
//*/
		return 0;

	case WM_ERASEBKGND:

		return -1;

	case WM_UPDATE_VIEW:

		openGL->Render();
		
		//  write video data
		if (auxInfo!=0)  
		{
			TCHAR szTime[512];
			_stprintf(szTime, TEXT("%.*g Myr"), 6, auxInfo->Time());
			
			RECT rect;
			GetClientRect( hWnd, &rect );

			SIZE size;
			GetTextExtentPoint32( openGL->GetDC(), szTime, lstrlen(szTime), &size);

			SetTextColor(openGL->GetDC(), RGB(255, 255, 255) );
			SetBkColor(openGL->GetDC(), RGB(0, 0, 0) );
			TextOut(openGL->GetDC(), rect.right-size.cx-20, rect.bottom-size.cy-15, szTime, lstrlen(szTime));

			auxInfo->avi.WriteFrame( openGL->GetDibSection() );			
		}

		//  force redraw
		InvalidateRect(hWnd, NULL, FALSE);
					
		return 0;

	case WM_CLOSE:

		if ( Server == NULL )  DestroyWindow( hWnd );
		return 0;

	case WM_DESTROY:

		delete openGL;

		PostQuitMessage(0);
		return 0;

	default:

		return DefWindowProc(hWnd, uMsg, wParam, lParam);
	}

	//  window procedure should never gets here
}

BOOL InitApplicationWindow(int nCmdShow)	   
{
	//  Register the window class
	WNDCLASSEX wndclass;

    wndclass.cbSize = sizeof(WNDCLASSEX);
    wndclass.style = CS_OWNDC | CS_HREDRAW | CS_VREDRAW;
    wndclass.lpfnWndProc = WndProc;
    wndclass.cbClsExtra = 0;
    wndclass.cbWndExtra = 0;
    wndclass.hInstance = g_hInstance;
	wndclass.hIcon = (HICON)LoadImage(g_hInstance, MAKEINTRESOURCE(IDI_ICON), IMAGE_ICON, 0, 0, LR_DEFAULTCOLOR);  // 32x32
    wndclass.hCursor = LoadCursor(NULL, IDC_ARROW);
    wndclass.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
	wndclass.lpszMenuName = NULL;
    wndclass.lpszClassName = szWindowClass;
	wndclass.hIconSm = (HICON)LoadImage(g_hInstance, MAKEINTRESOURCE(IDI_SMALL), IMAGE_ICON, 0, 0, LR_DEFAULTCOLOR);  // 16x16

    if ( !RegisterClassEx(&wndclass) ) return (FALSE);

	//  Create main window & initialize its global variable
	g_hWnd = CreateWindowEx(
					 0, 
					 szWindowClass,
					 szTitle,
					 WS_OVERLAPPEDWINDOW | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN,
					 CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
					 NULL,
					 NULL,
					 g_hInstance,
					 NULL);  

	if (g_hWnd == NULL) return (FALSE);

	//  Show main window according to WinMain's nCmdShow passed parameter
	//  It's not necessary to call this if style is WS_VISIBLE
	ShowWindow(g_hWnd, nCmdShow);
	UpdateWindow(g_hWnd); 
	return (TRUE);		
}


int APIENTRY _tWinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
					    LPTSTR lpszCmdLine, int nCmdShow )
{
	MSG msg;

	g_hInstance = hInstance;
	UNREFERENCED_PARAMETER(hPrevInstance);	//  hPrevInstance is always NULL
	UNREFERENCED_PARAMETER(lpszCmdLine);

	//  initialize common controls
	DWORD dwMajor, dwMinor;
	GetShell32Version( &dwMajor, &dwMinor );
	if ( (( dwMajor == 4) && (dwMinor < 72)) || (dwMajor < 3) ) 
	{
		MessageBox(TEXT("Jedes na starym exploreru"));
//		return 1;
	}


	//  Turn on FP exceptions
	int cw = _controlfp( 0,0 );   // Get the default control word.
	cw &=~(EM_OVERFLOW|EM_UNDERFLOW|EM_ZERODIVIDE|EM_DENORMAL);	// Set the exception masks OFF, turn exceptions on.
	_controlfp( cw, MCW_EM );		// Set the control word.

	//  Initialize global variables
	lstrcpy( szTitle, Language.GetString(IDS_TITLE_BAR) );  //  set title bar text
	unsortedBodiesList = NULL;
	Server = NULL;
	auxInfo = NULL;
	terminateSimulationID = 0;
	isSimulationRunning = false;

	oneDiscBodyMass = oneBulgeBodyMass = oneHaloBodyMass = oneDarkHaloBodyMass = 0;

	if (!InitApplicationWindow(nCmdShow))
    {
		// fails if builded for UNICODE and running under 9X/ME
		MessageBox ( NULL, Language.GetString(IDS_UNICODE_NEEDED),
                     szTitle, MB_ICONERROR );
        return 0;
    }

	SetMenu( g_hWnd, Language.LoadMenu(IDM_MAINMENU) );


	while( GetMessage(&msg, NULL, 0, 0) )
	{
		TranslateMessage( &msg );
		DispatchMessage( &msg );
	}

	// Free system sources

	DestroyMenu( GetMenu(g_hWnd) );	//  release memory needed for menu
	DestroyWindow( g_hWnd );	//  release memory needed for window
	
	WNDCLASSEX wndclass;
	GetClassInfoEx( g_hInstance, szWindowClass, &wndclass );
	
	DestroyIcon( wndclass.hIcon );	//  release memory needed for icon
	DestroyIcon( wndclass.hIconSm );	//  release memory needed for small icon

	UnregisterClass( szWindowClass, g_hInstance ); //  release memory holding info about window class

	if (auxInfo != NULL)	delete auxInfo;
	if (Server != NULL)  delete Server;

	return msg.wParam;
}
