#ifndef CAMERA_H
#define CAMERA_H

//  transformation matrix, specialy designed for viewing transformations

#include "math/matrix.h"

using namespace JakubSchwarzmeier::Math;

namespace JakubSchwarzmeier
{

	namespace OpenGL
	{

		class Camera: public Matrix<double>
		{
		private:

			void FixAngle( double & angle );
			void ActBy( const double rotationMatrix[] );
			void ActOn( const double rotationMatrix[] );

			void Normalize();
			
			void UpdateMatrix();
			void FinishUpdate();

			void ActByCurrentOnNewXRotationalMatrixWithAngle(double angle);
			void ActByCurrentOnNewYRotationalMatrixWithAngle(double angle);
			void ActByCurrentOnNewZRotationalMatrixWithAngle(double angle);
			void ActByCurrentOnNewTranslationMatrixWithParameters(double x, double y, double z);

		protected:

//			double lambda, fi, theta;
			Vector<> position;
			
			Vector<> xaxis, zaxis;

		public:

			Camera();

			void ChangeLambdaBy(double angle);	//  rotate around "y", lambda, heading
			void ChangeFiBy(double angle);		//  rotate around "x", fi, pitch
			void ChangeThetaBy(double angle);	//  rotate around "z", theta, roll
			void ChangePositionBy(double dx, double dy, double dz);

			double GetLambda();
			double GetFi();
			double GetTheta();
			Vector<> GetPosition();
		};
	
	}	//  end namespace OpenGL

}	//  end namespace JakubSchwarzmeier


#endif  //  CAMERA_H