#include "AudioVideo.h"

using namespace JakubSchwarzmeier::Windows;

AudioVideo::AudioVideo()
{
	pIAVIFile = NULL;
	pIAVIStream = NULL;
	pIAVICompressedStream = NULL;
	nFrame = 0;

	AVIFileInit();		//  initializes the AVIFile library
}

AudioVideo::~AudioVideo()
{
	if (pIAVIFile != NULL)  AVIFileRelease( pIAVIFile );	//  release interface returned by AVIFileOpen
	if (pIAVIStream != NULL)  AVIStreamRelease(pIAVIStream);
	if (pIAVICompressedStream != NULL)  AVIStreamRelease(pIAVICompressedStream);

	AVIFileExit();		//  decrements the reference count for the library
}

BOOL AudioVideo::CreateFile (TCHAR * strFileName, HBITMAP hDIBSection)
{
	AVIFileOpen( &pIAVIFile, strFileName, OF_CREATE | OF_SHARE_DENY_WRITE | OF_WRITE, NULL );

	DIBSECTION dibs;
	GetObject( hDIBSection, sizeof(dibs), &dibs );		//  get information about DIB section

	//  uncompressed stream creation
	//

	//  prepeare header of stream
	AVISTREAMINFO strhdr; 
	ZeroMemory( &strhdr, sizeof(strhdr) );
	
	strhdr.fccType = streamtypeVIDEO;
	strhdr.fccHandler = 0; 
	strhdr.dwScale = 1;
	strhdr.dwRate = 15;		    // 15 fps
	strhdr.dwSuggestedBufferSize	 = dibs.dsBmih.biSizeImage;
	strhdr.rcFrame.left			= 0;
	strhdr.rcFrame.top				= 0;
	strhdr.rcFrame.right			= dibs.dsBmih.biWidth;
	strhdr.rcFrame.bottom			= dibs.dsBmih.biHeight;

	//  create stream
	INT result = AVIFileCreateStream(pIAVIFile, &pIAVIStream, &strhdr);
	if ( result!=0 )  return FALSE;

	//
	//  uncompressed stream creation

	//  set up compression just before the first frame
	//

	AVICOMPRESSOPTIONS opts;
	AVICOMPRESSOPTIONS * aCompression[] = { &opts };
	ZeroMemory( &opts, sizeof(opts) );

	//  compression options dialog, AviFil32 opens dialog so main window continue in recieving messages
	BOOL bResult = AVISaveOptions( GetActiveWindow(), ICMF_CHOOSE_PREVIEW, 1, &pIAVIStream, aCompression );
	if ( bResult != TRUE )  return FALSE;
	AVISaveOptionsFree( 1, aCompression );


	result = AVIMakeCompressedStream(&pIAVICompressedStream, pIAVIStream, &opts, NULL);
	if ( result != AVIERR_OK )  return FALSE;

	result = AVIStreamSetFormat(pIAVICompressedStream, 0, &dibs.dsBmih, dibs.dsBmih.biSize+dibs.dsBmih.biClrUsed*sizeof(RGBQUAD));
	if ( result != 0 )  return FALSE;

	//
	//  compression matter

	return (result==0 ? TRUE : FALSE);
}

BOOL AudioVideo::WriteFrame (HBITMAP hDIBSection)
{
	
	DIBSECTION dibs;
	GetObject( hDIBSection, sizeof(dibs), &dibs );		//  get information about DIB section

	//  add frame
	INT result = AVIStreamWrite(pIAVICompressedStream, nFrame++, 1, dibs.dsBm.bmBits, dibs.dsBmih.biSizeImage, AVIIF_KEYFRAME, NULL, NULL);

	if ( result!=0 )  return FALSE;

	return TRUE;
}

