/************************************************************
*
*  Copyright (c) 2002-2004 Jakub Schwarzmeier
*
************************************************************/

#ifndef BH_H
#define BH_H

#include "..\math\matrix.h"
#include "..\data\list.h"

#define FI 0.7			//  in reality this is opening angle theta NOT fi

using namespace JakubSchwarzmeier::Data;
using namespace JakubSchwarzmeier::Math;

namespace JakubSchwarzmeier
{
		namespace BarnesHut
		{
			//  should serve as interface for both Leave & Node classes
			class ICommonNode
			{
			public:
				
				Vector<> position;
				real mass;
				real load;	//  costzones

				ICommonNode(Vector<> n_position = 0, real n_mass = 0)
				{
					position = n_position;
					mass = n_mass;
					load = 0;
				}

				virtual ~ICommonNode() {}	//  ensure polymorphysm
			};

			class Leave: public ICommonNode
			{
			public:
				
				Vector<> velocity;
				Vector<> acceleration;		//  neni treba, pocita se vzdy od znovu; u distributed obsahuje zrychleni udelene pouze vzdalenymi nody
				Leave(Vector<> n_position = 0, Vector<> n_velocity = 0, real n_mass = 0): ICommonNode(n_position, n_mass)
				{
					velocity = n_velocity;
					acceleration = 0;
				}
				virtual ~Leave() {}
			};


			class Node: public ICommonNode
			{
			private:
			class RCustomList: public RList<ICommonNode>
			{
			public:

				RCustomList (int n_inc_size=10, bool n_owner=true): RList<ICommonNode>(n_inc_size, n_owner)
				{}

				virtual ~RCustomList();
		
			};


			public:

				//  Hilbert
				class Angles
				{
				public:
					
					Angles(int num = 0)		//  for code: Angles ang = 0;
					{
						x = num;
						y = num;
						z = num;
					}

					Angles(int n_x, int n_y, int n_z)
					{
						x = n_x;
						y = n_y;
						z = n_z;
					}
					
					int x, y, z;

					int getXinc90()
					{
						int X = x + 90;
						properAng(X);
						return X;
					}

					int getXdec90()
					{
						int X = x - 90;
						properAng(X);
						return X;
					}

					int getXinc180()
					{
						int X = x + 180;
						properAng(X);
						return X;
					}

					int getXdec180()
					{
						int X = x - 180;
						properAng(X);
						return X;
					}

					int getYinc90()
					{
						int Y = y + 90;
						properAng(Y);
						return Y;
					}

					int getYdec90()
					{
						int Y = y - 90;
						properAng(Y);
						return Y;
					}

					int getYinc180()
					{
						int Y = y + 180;
						properAng(Y);
						return Y;
					}

					int getYdec180()
					{
						int Y = y - 180;
						properAng(Y);
						return Y;
					}

					int getZinc90()
					{
						int Z = z + 90;
						properAng(Z);
						return Z;
					}

					int getZdec90()
					{
						int Z = z - 90;
						properAng(Z);
						return Z;
					}

					int getZinc180()
					{
						int Z = z + 180;
						properAng(Z);
						return Z;
					}

					int getZdec180()
					{
						int Z = z - 180;
						properAng(Z);
						return Z;
					}


				private:
					void properAng(int & ang)
					{
						while(ang >=360)
						{
							ang -= 360;
						}
						while(ang < 0)
						{
							ang += 360;
						}
					}

				};
				Angles angles;

				Node(Vector<> & n_corner, Angles n_angles = 0);
				Vector<> & corner;	//  corner of orthogonal area
				RCustomList childList;
				RList<Leave> leaves;
				real bisect;
				bool grouped;		
				Node * parent;
				int childNo;	//  index of node in parent list [level-1]
				virtual ~Node() {	
					delete &(corner);	
				}
			};

			class Group
			{
			public:
				ICommonNode * element;	//  particle or cell
				Vector<> acceleration, velocity, position;	//  far forces
				RList<ICommonNode> nearList;	//  near forces

			public:
				Group( ICommonNode * n_element = NULL): nearList(1000, false)
				{
					element = n_element;
					//  must explicitly set up matrices to zero (proc nejde acc = vel = pos = 0 ?)
					acceleration = 0;
					velocity = 0;
					position = 0;
				}

				double getCellSize();
			};

			class Tree
			{
			private:
				
				void initCubeCorners( Vector<> & superiorCorner, RList< Vector<> > & initialCube, real & bisect);

			protected:

				RList<Leave> & leavesList;		//  user supplied
				static const real kappa;
				static const real dimensionLength;

			public:

				Tree( RList<Leave> & n_leavesList );
				void step();

			}; //  class Tree

		}	//  end namespace BarnesHut


} //  end namespace JakubSchwarzmeier

#endif //  BH_H