#pragma once

#if !defined(_WIN32_WINNT)
 #define _WIN32_WINNT 0x0400		//  CoInitializeEx need this
#endif

#include <msxml2.h>

namespace JakubSchwarzmeier
{
	namespace Data
	{
			class XmlNode;
			class XmlAttributeCollection
			{
			public:

				XmlAttributeCollection(IXMLDOMNamedNodeMap * pAttributes, IXMLDOMDocument2 * pXMLDoc);
				~XmlAttributeCollection();

				XmlNode GetNamedItem(TCHAR * szAttribute);
				XmlNode GetItem(long index);
				long GetCount();

				XmlAttributeCollection & operator=(XmlAttributeCollection original);

			private:

				IXMLDOMNamedNodeMap * m_pAttributes;
				IXMLDOMDocument2 * m_pXMLDoc;
			};

			class XmlNodeList
			{
			public:
				
				XmlNodeList(IXMLDOMNodeList * pNodes, IXMLDOMDocument2 * pXMLDoc);
				~XmlNodeList();

				int GetLength();
				XmlNode GetItem(int position);

			private:

				IXMLDOMNodeList * m_pNodes;
				IXMLDOMDocument2 * m_pXMLDoc;

			};

			class XmlNode
			{
			public:
				
				XmlNode() { m_pNode = NULL; }
				XmlNode(IXMLDOMNode * pNode, IXMLDOMDocument2 * pXMLDoc);
				virtual ~XmlNode();

				TCHAR * GetText();
				TCHAR * GetNodeValue();
				XmlAttributeCollection GetAttributes();

				XmlNodeList SelectNodes( TCHAR * szExpression );
				XmlNode SelectSingleNode( TCHAR * szExpression );

				virtual void get(IXMLDOMNode ** returnPNode) {}

				XmlNode & operator=(XmlNode original);

			public:

				IXMLDOMNode * m_pNode;
				IXMLDOMDocument2 * m_pXMLDoc;
				TCHAR m_szBuffer[1024];
			};

			class XmlElement: public XmlNode
			{
			public:

				XmlElement();
				XmlElement( IXMLDOMElement * pElement, IXMLDOMDocument2 * pXMLDoc );
				~XmlElement();

				void SetAttribute( TCHAR * name, LPCTSTR value );
				void SetAttribute( TCHAR * name, double dValue );
				void AppendChild( XmlNode & childElement );

				XmlElement & operator=(XmlElement original);

				void get(IXMLDOMNode ** returnPNode);

			private:

				IXMLDOMElement * m_pElement;
				IXMLDOMDocument2 * m_pXMLDoc;

				TCHAR m_szBuffer[1024];
			};

			class XmlProcessingInstruction: public XmlNode
			{
			public:
				
				XmlProcessingInstruction() {}
				XmlProcessingInstruction( IXMLDOMProcessingInstruction * pi );
				~XmlProcessingInstruction();

				XmlProcessingInstruction & operator=(XmlProcessingInstruction original);

				void get(IXMLDOMNode ** returnPNode);

			private:

				IXMLDOMProcessingInstruction * m_pi;
			};

			class XmlText: public XmlNode
			{
			public:

				XmlText();
				~XmlText();
				XmlText( IXMLDOMText * pText, IXMLDOMDocument2 * pXMLDoc );

				XmlText & operator=(XmlText original);

				void get(IXMLDOMNode ** returnPNode);

			private:
				
				IXMLDOMText * m_pText;
				IXMLDOMDocument2 * m_pXMLDoc;

			};

			class XmlDocument: public XmlNode
			{
			public:
				
				XmlDocument();
				XmlProcessingInstruction CreateProcessingInstruction( TCHAR * target, TCHAR * data );
				void AppendChild( XmlNode & childElement );
				XmlElement CreateElement( TCHAR * newElement );
				XmlText CreateTextNode( TCHAR * newTextNode );
				XmlText CreateTextNode( int newValue );
				XmlText CreateTextNode( double newValue );
				void Save(LPCTSTR filename);
				void Load(LPCTSTR filename);
				void DoHumanMarkup( XmlElement & pRoot );
				~XmlDocument();

			protected:

				bool addHumanMarkup(IXMLDOMNode * pElement, int depth);
			};

	} //  end namespace Data

}	//  end namespace JakubSchwarzmeier
