#include "math_ext.h"

//************************************************************
//
// non-OOP stuff
//
//

long double sin_angle(const long double angle)
{
  return sin(angle * M_PI / 180.0);
}

long double cos_angle(const long double angle)
{
  return cos(angle * M_PI / 180.0);
}


long double tan_angle(const long double angle)
{
	return tan(angle * M_PI / 180.0);
}

long double asin_angle(const double x)
{
  long double res;
  if (ceil(x) == -1.0) res = -90.0;
  else
  if (floor(x) == 1.0) res = 90;
  else
  res = asin(x) * 180.0 / M_PI;
  return res;
 
}

long double acos_angle(const double x)
{
  long double res;
  if (floor(x) == 1.0) res = 0;
  else
  if (ceil(x) == -1.0) res = 180.0;
  else
  res = acos(x) * 180.0 / M_PI;
  return res;
}

long double atan_angle(const double x)
{
  return atan(x) * 180.0 / M_PI;
}

long double atan2_angle(const double y, const double x)
{
  return atan2(y, x) * 180.0 / M_PI;
}



//  taken from Numerical recipes in C
//

double random(long idum)	// idum < 1 means initialization or reinitialization
{
	#define MBIG 1000000000
	#define MSEED 161803398
	#define MZ 0
	#define FAC (1.0/MBIG)

	static int inext,inextp;
	static long ma[56];
	static int iff=0;
	long mj,mk;
	int i,ii,k;
	if (idum < 0 || iff == 0)
	{ 
		iff=1;
		mj=labs(MSEED-labs(idum)); 
		mj %= MBIG;
		ma[55]=mj;
		mk=1;
		
		for (i=1;i<=54;i++)
		{
			ii=(21*i) % 55;
			ma[ii]=mk;
			mk=mj-mk;
			if (mk < MZ) mk += MBIG;
			mj=ma[ii];
		}

		for (k=1;k<=4;k++)
			for (i=1;i<=55;i++)
			{
				ma[i] -= ma[1+(i+30) % 55];
				if (ma[i] < MZ) ma[i] += MBIG;
			}
			inext=0; 
			inextp=31; 
	}

	if (++inext == 56) inext=1;
	if (++inextp == 56) inextp=1;
	mj=ma[inext]-ma[inextp]; 
	if (mj < MZ) mj += MBIG;
	ma[inext]=mj; 
	return mj*FAC; 

	#undef MBIG
	#undef MSEED
	#undef MZ
	#undef FAC
}
//
//	taken from from Numerical recipes in C

TCHAR * _dtot(double value, TCHAR * string)
{
	_stprintf(string, _T("%.*g"), 6, value);

	return string;
}

#ifdef UNICODE
	#if (_MSC_VER == 1200)
	double _tstof(const WCHAR szValue[])
	  {
		  WCHAR * endPtr;
		  return wcstod( szValue, &endPtr );
	  }
	#endif
#endif

double TrapezoidalIntegration(	double (*func)(double), double lower, double upper, int steps )
{
	double  integral = 0;
	double h = (upper-lower) / steps;
	double xi = lower;

	for( int k = 0; k < steps; k++)
	{
		integral = integral + h * (func(xi) + func(xi+h)) / 2;
		xi = xi + h;
	}

	return integral;
}


/**
 *
 * Purpose:      Maximize a "well-behaved" function
 *
 * Arguments:    f - Pointer to the function
 *               L - The lower bound of the search space
 *               U - The upper bound of the search space
 *               tol - The tolerance level
 *
 * Side Effects: None
 *
 * Returns:      The value of x that maximizes f(x)
 */

//  typedef double (*__func)(double);	//  takto by se slo vyhnout slozitemu zapisu a namisto
										//  prvniho parametru by pak bylo pouze __func f

double FindMaximum( double (*f)(double), double L, double U, double tol)
{

	double t_L, t_R;

  t_L = L + 0.382*(U - L);
  t_R = L + 0.618*(U - L);

  while ((U-L) > tol) {

	  if ( f(t_L) > f(t_R) ) {

      U = t_R;
      t_R = t_L;
      t_L = L + 0.382*(U - L);

    } else {

      L = t_L;
      t_L = t_R;
      t_R = L + 0.618*(U - L);
    }
  }

  return (U+L)/2.0;
}

//  hidden function
double semigausdistr( double x )
{
	// y = exp(- x^2)
	return exp( - pow(x, 2) );
}

double erf( double q )
{
	return ( (2/sqrt(M_PI)) * TrapezoidalIntegration( semigausdistr, 0, q, 5000 ) );
}

