/************************************************************
*
*  Copyright (c) 2002-2004 Jakub Schwarzmeier
*
************************************************************/

#include "halo.h"
#include "math/math_ext.h"

//  static init
double HaloModel::totalMass = 0;
double HaloModel::rhmax = 0;
double HaloModel::ah = 0;

double HaloModel::rDistribution( double r )
{	//  plummer
	return pow( ( 1 + (pow(r, 2)/pow(ah, 2)) ), (-5/2) );
}

double HaloModel::vCurve( double r )
{
	//	y=1.05*x*exp(x/55)*(2.4+(x+3)^2)^(-3/3.8)
	return  1.05 * r * exp( r/55 ) * pow( 2.4 + pow( r+3, 2 ), -3/3.8 );
}

void HaloModel::Initialize(double n_totalMass, double n_rhmax, double n_ah)
{
	totalMass = n_totalMass;
	rhmax = n_rhmax;
	ah = n_ah;
}

bool HaloModel::Create(JakubSchwarzmeier::Data::RList< JakubSchwarzmeier::BarnesHut::Leave > & list, const int & nBodies)
{
	int bodies = 0;
	random(-100);		//  init real random number generator (Numerical Recipes in C)

	double probMax = FindMaximum( rDistribution, 0.0, rhmax, 1.0 );
	probMax = rDistribution(probMax);

	while ( nBodies > bodies )
	{
		double r = 0.0 + random() * (rhmax - 0.0);	//  between 0 and rmax
		double p = 0.0 + random() * (probMax - 0.0);	//  between 0 and 1

		//  find radius from radius distribution
		if ( p <= rDistribution(r) )		//  throw out points over distribution function
		{
			bodies++;

			//  Generate positions
			//

			//  nahodne pootocim kartezskej system a pak natvrdo dam objekt pouze na x-ovou osu

			double a =  0.0 + random() * (2*M_PI - 0.0);			
			double b =  0.0 + random() * (2*M_PI - 0.0);			
			double c =  0.0 + random() * (2*M_PI - 0.0);			

			Matrix<double> transform(4, 4);
			transform.LoadIdentity();
			transform[0]  = cos(a) * cos(b) - sin(a) * sin(b) * cos(c);
			transform[1]  = cos(a) * sin(b) + sin(a) * cos(b) * cos(c);
			transform[2]  = sin(a) * sin(c);
			transform[3]  = 0;
			transform[4]  = -sin(a) * cos(b) - cos(a) * sin(b) * cos(c);
			transform[5]  = -sin(a) * sin(b) + cos(a) * cos(b) * cos(c);
			transform[6]  = cos(a) * sin(c);
			transform[7]  = 0;
			transform[8]  = sin(b) * sin(c);
			transform[9]  = -cos(b) * sin(c);
			transform[10] = cos(c);
			transform[11] = 0;

			Matrix<double> pos(4,4);
			pos.LoadIdentity();
			pos[12] = r;
			pos[13] = 0;
			pos[14] = 0;

			Matrix<double> result(4,4);
			
			result = pos * transform;

			//

			Vector<> position;
			position[0] = result[12];
			position[1] = result[13];
			position[2] = result[14];

			double lambda = atan2(position[1], position[0]);
			lambda += M_PI;

			//  Generate velocity
			//
			
			double v = vCurve(r);		//  circular (tangential) velocity

			//  convert velocity to components
			double vx = -v * sin( lambda );
			double vy = v * cos( lambda );

			Vector<> velocity;
			velocity[0] = vx;
			velocity[1] = vy;
			velocity[2] = 0;

			list.Add( new JakubSchwarzmeier::BarnesHut::Leave(position, velocity, totalMass/nBodies) );
		}
	}

	return true;
}