/************************************************************
*
*  Copyright (c) 2002-2004 Jakub Schwarzmeier
*
************************************************************/

#include <windows.h>
#include <stdexcept>

#define SPICELIBAPI __declspec(dllexport)
#include "spicelib.h"

#undef mxv_c
#undef oscelt_c
#undef conics_c

namespace __CSPICE
{
	#include "SpiceUsr.h"
	#pragma warning( push )
	#pragma warning(disable:4518)
	#include "SpiceZfc.h"
	#include "SpiceZpr.h"
	#pragma warning( pop )
}

#undef mxv_c
#undef oscelt_c
#undef conics_c

BOOL APIENTRY DllMain( HANDLE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved )
{
	__CSPICE::erract_c( "SET", 6, "REPORT" );	//  SPICE won't halt program on a error
	__CSPICE::errprt_c( "SET", 7, "NONE" );		//  no error lists to output

	return TRUE;
}

//  Auxilliary functions Unicode <--> ANSI conversions
//

//  create space for ANSI string, a user of function is responsible for disposing
void UnicodeToAnsi(LPCWSTR szUnicode/*IN*/, LPSTR & lpszAnsi/*OUT*/)
{
	int iLength = lstrlenW(szUnicode)+1;
	lpszAnsi = new char[ iLength ];
	
	//  convert
	WideCharToMultiByte(CP_ACP, 0, 
                    szUnicode,
                    -1, 
                    lpszAnsi,
                    iLength,
                    NULL, NULL);
}

void UnicodeToAnsi(LPWSTR szUnicode/*IN*/, int nLength/*IN*/, LPSTR & lpszAnsi/*OUT*/)
{
	int iLength = nLength + 1;
	lpszAnsi = new char[ iLength ];
	
	//  convert
	WideCharToMultiByte(CP_ACP, 0, 
                    szUnicode,
                    -1, 
                    lpszAnsi,
                    iLength,
                    NULL, NULL);
}

void AnsiToUnicode(LPSTR szAnsi/*IN*/, int nLength/*IN*/, LPWSTR lpszUnicode/*OUT*/)
{
	MultiByteToWideChar(CP_ACP, 0, 
                    szAnsi,
                    -1,
                    lpszUnicode,
                    nLength+1 );

}

//  Exception throwing mechanism

void CheckForErrorsA()
{
	if ( failed_c() )							//  error handling
	{
		char longms[512];
		longms[511] = '\0';
		getlms_A ( longms, sizeof(longms)/sizeof(char)-1 );
		reset_c();

#ifdef _DEBUG
		::OutputDebugString( longms );
#endif
		throw std::runtime_error( longms );
	}
}

void CheckForErrorsW()
{
	if ( failed_c() )							//  error handling
	{
		char longms[512];
		longms[511] = '\0\0';
		getlms_A ( longms, sizeof(longms)/sizeof(char)-1 );
		reset_c();

#ifdef _DEBUG
		::OutputDebugString( longms );
#endif
		throw std::runtime_error( longms );
	}
}


//  CSPICE functions
//

SPICELIBAPI void bodc2n_cA ( SpiceInt code,
                            SpiceInt namelen,
                            CHAR  * name,
                            SpiceBoolean * found )
{
	__CSPICE::bodc2n_c(code, namelen, name, found);
	CheckForErrorsA();
}

SPICELIBAPI void bodc2n_cW ( SpiceInt code,
                            SpiceInt namelen,
                            WCHAR  * name,
                            SpiceBoolean * found )
{
	SpiceChar * nameOut = NULL;

	UnicodeToAnsi(name, namelen, nameOut);
	
	__CSPICE::bodc2n_c ( code, namelen, nameOut, found );

	AnsiToUnicode(nameOut, namelen, name);	

	delete [] nameOut;

	CheckForErrorsW();
}


SPICELIBAPI void bodn2c_cA ( const CHAR * name,
                            SpiceInt * code,
                            SpiceBoolean * found )
{
	__CSPICE::bodn2c_c(name, code, found);
	CheckForErrorsA();
}

SPICELIBAPI void bodn2c_cW ( const WCHAR * name,
                            SpiceInt * code,
                            SpiceBoolean * found )
{
	SpiceChar * nameout = NULL;

	UnicodeToAnsi(name, nameout);
	
	__CSPICE::bodn2c_c(nameout, code, found);

	delete [] nameout;
	
	CheckForErrorsW();
}

SPICELIBAPI SpiceDouble lspcn_cA ( const CHAR * body,
                                  SpiceDouble et,
                                  const CHAR * abcorr )
{
	SpiceDouble ret = __CSPICE::lspcn_c(body, et, abcorr);
	CheckForErrorsA();
	
	return ret;
}

SPICELIBAPI SpiceDouble lspcn_cW ( const WCHAR * body,
                                  SpiceDouble et,
                                  const WCHAR * abcorr )
{
	SpiceChar * bodyout = NULL;
	SpiceChar * abcorrout = NULL;

	UnicodeToAnsi(body, bodyout);
	UnicodeToAnsi(abcorr, abcorrout);
	
	SpiceDouble ret = __CSPICE::lspcn_c(bodyout, et, abcorrout);

	delete [] bodyout;
	delete [] abcorrout;
	
	CheckForErrorsW();

	return ret;
}

SPICELIBAPI void recrad_cA ( ConstSpiceDouble    rectan[3],
                                SpiceDouble       * radius,
                                SpiceDouble       * ra,
                                SpiceDouble       * dec  )
{
	using namespace __CSPICE;
	recrad_c(rectan, radius, ra, dec);
	CheckForErrorsA();
}

SPICELIBAPI void recrad_cW ( ConstSpiceDouble    rectan[3],
                                SpiceDouble       * radius,
                                SpiceDouble       * ra,
                                SpiceDouble       * dec  )
{
	using namespace __CSPICE;
	recrad_c(rectan, radius, ra, dec);
	CheckForErrorsW();
}

SPICELIBAPI void radrec_cA ( SpiceDouble range,
                            SpiceDouble ra,
                            SpiceDouble dec,
                            SpiceDouble rectan[3] )
{
	using namespace __CSPICE;
	radrec_c(range, ra, dec, rectan);
	CheckForErrorsA();
}

SPICELIBAPI void radrec_cW ( SpiceDouble range,
                            SpiceDouble ra,
                            SpiceDouble dec,
                            SpiceDouble rectan[3] )
{
	using namespace __CSPICE;
	radrec_c(range, ra, dec, rectan);
	CheckForErrorsW();
}

SPICELIBAPI void pdpool_cA ( CHAR * name,
                            SpiceInt n,
                            ConstSpiceDouble * dvals )
{
	using namespace __CSPICE;
	pdpool_c(name, n, dvals);
	CheckForErrorsA();
}

SPICELIBAPI void pdpool_cW ( WCHAR * name,
                            SpiceInt n,
                            ConstSpiceDouble * dvals )
{
	using namespace __CSPICE;
	SpiceChar * nameout = NULL;

	UnicodeToAnsi(name, nameout);
	
	pdpool_c(nameout, n, dvals);
	delete [] nameout;
	
	CheckForErrorsW();
}

SPICELIBAPI void pcpool_cA ( CHAR * name,
                            SpiceInt n,
                            SpiceInt lenvals,
                            const void * cvals )
{
	using namespace __CSPICE;
	pcpool_c(name, n, lenvals, cvals);
	CheckForErrorsA();
}
                                
SPICELIBAPI void pcpool_cW ( WCHAR * name,
                            SpiceInt n,
                            SpiceInt lenvals,
                            const void * cvals )
{
	using namespace __CSPICE;
	SpiceChar * nameout = NULL;

	UnicodeToAnsi(name, nameout);
	
	pcpool_c(nameout, n, lenvals, cvals);
	delete [] nameout;
	
	CheckForErrorsW();
}

SPICELIBAPI void pipool_cA ( CHAR * name,
                            SpiceInt n,
                            ConstSpiceInt * ivals )
{
	using namespace __CSPICE;
	pipool_c(name, n, ivals);
	CheckForErrorsA();
}

SPICELIBAPI void pipool_cW ( WCHAR * name,
                            SpiceInt n,
                            ConstSpiceInt * ivals )
{
	SpiceChar * nameout = NULL;

	UnicodeToAnsi(name, nameout);

	using namespace __CSPICE;
	pipool_c(nameout, n, ivals);
	
	delete [] nameout;

	CheckForErrorsW();
}

SPICELIBAPI SpiceDouble dpr_cA ( void )
{
	SpiceDouble ret = __CSPICE::dpr_c();
	CheckForErrorsA();
	return ret;
}

SPICELIBAPI SpiceDouble dpr_cW ( void )
{
	SpiceDouble ret = __CSPICE::dpr_c();
	CheckForErrorsW();
	return ret;
}

SPICELIBAPI void reclat_cA ( ConstSpiceDouble    rectan[3],
                                SpiceDouble       * radius,
                                SpiceDouble       * longitude,
                                SpiceDouble       * latitude  )
{
	using namespace __CSPICE;
	reclat_c(rectan, radius, longitude, latitude);
	CheckForErrorsA();
}

SPICELIBAPI void reclat_cW ( ConstSpiceDouble    rectan[3],
                                SpiceDouble       * radius,
                                SpiceDouble       * longitude,
                                SpiceDouble       * latitude  )
{
	using namespace __CSPICE;
	reclat_c ( rectan, radius, longitude, latitude );
	CheckForErrorsW();
}

SPICELIBAPI SpiceInt tipbod_A (CHAR * ref, 
							   SpiceInt * body, 
							   SpiceDouble * et, 
							   SpiceDouble * tipm,
							   SpiceInt ref_len)
{
	SpiceInt ret = __CSPICE::tipbod_ ( ref, body, et, tipm, ref_len );
	CheckForErrorsA();

	return ret;
}

SPICELIBAPI SpiceInt tipbod_W (WCHAR * ref, 
							   SpiceInt * body, 
							   SpiceDouble * et, 
							   SpiceDouble * tipm,
							   SpiceInt ref_len)
{
	SpiceChar * refout = NULL;

	UnicodeToAnsi(ref, ref_len, refout);
	
	SpiceInt ret = __CSPICE::tipbod_ ( refout, body, et, tipm, ref_len );

	delete [] refout;

	CheckForErrorsW();

	return ret;
}

SPICELIBAPI SpiceInt deltet_A ( doublereal * epoch,
                             CHAR * eptype,
							doublereal * delta,
							SpiceInt ref_len)
{
	SpiceInt ret = __CSPICE::deltet_ ( epoch, eptype, delta, ref_len );
	CheckForErrorsA();

	return ret;
}

SPICELIBAPI SpiceInt deltet_W ( doublereal * epoch,
                             WCHAR * eptype,
							doublereal * delta,
							SpiceInt ref_len)
{
	SpiceChar * type = NULL;

	UnicodeToAnsi(eptype, ref_len, type);
	
	SpiceInt ret = __CSPICE::deltet_ ( epoch, type, delta, ref_len );
	
	delete [] type;

	CheckForErrorsW();

	return ret;
}


SPICELIBAPI void errdev_cA ( const CHAR * szcOperation,
                             SpiceInt lenout,
                             CHAR * szDevice    )
{
	__CSPICE::errdev_c ( szcOperation, lenout, szDevice );
	CheckForErrorsA();
}

SPICELIBAPI void errdev_cW ( const WCHAR * szcOperation,
                             SpiceInt lenout,
                             WCHAR * szDevice    )
{
	SpiceChar * operation = NULL;
	SpiceChar * device = NULL;

	UnicodeToAnsi(szcOperation, operation);
	UnicodeToAnsi(szDevice, lenout, device);
	
	__CSPICE::errdev_c ( operation, lenout, device );

	AnsiToUnicode(device, lenout, szDevice);	

	delete [] operation;
	delete [] device;

	CheckForErrorsW();
}

SPICELIBAPI void erract_cA ( const CHAR * szcOperation,
                             SpiceInt lenout,
                             CHAR * szAction )
{
	__CSPICE::erract_c ( szcOperation, lenout, szAction );
	CheckForErrorsA();
}
	
SPICELIBAPI void erract_cW ( const WCHAR * szcOperation,
                             SpiceInt lenout,
                             WCHAR * szAction )
{
	SpiceChar * operation = NULL;
	SpiceChar * action = NULL;

	UnicodeToAnsi(szcOperation, operation);
	UnicodeToAnsi(szAction, lenout, action);

	__CSPICE::erract_c ( operation, lenout, action );

	AnsiToUnicode(action, lenout, szAction);

	delete [] operation;
	delete [] action;

	CheckForErrorsW();
}

SPICELIBAPI void spkuef_c ( SpiceInt handle )
{
	__CSPICE::spkuef_c ( handle );
}

SPICELIBAPI void spklef_cA ( const CHAR * szcFilename,
                             SpiceInt * handle )
{
	__CSPICE::spklef_c ( szcFilename, handle );
	CheckForErrorsA();
}

SPICELIBAPI void spklef_cW ( const WCHAR * szcFilename,
                             SpiceInt * handle )
{
	SpiceChar * filename = NULL;

	UnicodeToAnsi(szcFilename, filename);
	
	__CSPICE::spklef_c ( filename, handle );

	delete [] filename;

	CheckForErrorsW();
}

SPICELIBAPI void spkezr_cA ( const CHAR * szcTarget,
                             SpiceDouble epoch,
                             const CHAR * szcFrame,
                             const CHAR * szcAbcorr,
                             const CHAR * szcObserver,
                             SpiceDouble state[6],
                             SpiceDouble * lt )
{
		__CSPICE::spkezr_c ( szcTarget, epoch, szcFrame, szcAbcorr, szcObserver, state, lt );
		CheckForErrorsA();
}

SPICELIBAPI void spkezr_cW ( const WCHAR * szcTarget,
                             SpiceDouble epoch,
                             const WCHAR * szcFrame,
                             const WCHAR * szcAbcorr,
                             const WCHAR * szcObserver,
                             SpiceDouble state[6],
                             SpiceDouble * lt )
{
	SpiceChar * target = NULL, * frame = NULL, * abcorr = NULL, * observer = NULL;

	UnicodeToAnsi(szcTarget, target);
	UnicodeToAnsi(szcFrame, frame);
	UnicodeToAnsi(szcAbcorr, abcorr);
	UnicodeToAnsi(szcObserver, observer);

	__CSPICE::spkezr_c ( target, epoch, frame, abcorr, observer, state, lt );

	delete [] target;
	delete [] frame;
	delete [] abcorr;
	delete [] observer;

	CheckForErrorsW();
}

SPICELIBAPI void pxform_cA ( const CHAR * szcFrom,
                             const CHAR * szcTo,
                             SpiceDouble et,
                             SpiceDouble rotate[3][3] )
{
	__CSPICE::pxform_c ( szcFrom, szcTo, et, rotate );
	CheckForErrorsA();
}

SPICELIBAPI void pxform_cW ( const WCHAR * szcFrom,
                             const WCHAR * szcTo,
                             SpiceDouble et,
                             SpiceDouble rotate[3][3] )
{
	SpiceChar * from = NULL, * to = NULL;

	UnicodeToAnsi(szcFrom, from);
	UnicodeToAnsi(szcTo, to);

	__CSPICE::pxform_c ( from, to, et, rotate );

	delete [] from;
	delete [] to;

	CheckForErrorsW();
}

SPICELIBAPI void mxv_c( ConstSpiceDouble m1[3][3], ConstSpiceDouble vin[3], SpiceDouble vout[3] )
{
	__CSPICE:: mxv_c ( CONST_MAT(m1), CONST_VEC(vin), (vout) );
}

SPICELIBAPI void errprt_cA ( const CHAR * szcOperation,
                             SpiceInt lenout,
                             CHAR * szList )
{
	__CSPICE::errprt_c(szcOperation, lenout, szList);
	CheckForErrorsA();
}

SPICELIBAPI void errprt_cW ( const WCHAR * szcOperation,
                             SpiceInt lenout,
                             WCHAR * szList )
{
	SpiceChar * operation = NULL;
	SpiceChar * list = NULL;

	UnicodeToAnsi(szcOperation, operation);
	UnicodeToAnsi(szList, lenout, list);

	__CSPICE::errprt_c(operation, lenout, list);

	AnsiToUnicode(list, lenout, szList);

	delete [] operation;
	delete [] list;

	CheckForErrorsW();
}

SPICELIBAPI void spkez_cA ( SpiceInt target,
                            SpiceDouble epoch,
                            const CHAR * szcFrame,
                            const CHAR * szcAbcorr,
                            SpiceInt observer,
                            SpiceDouble state[6],
                            SpiceDouble * lt )
{
	__CSPICE::spkez_c ( target, epoch, szcFrame, szcAbcorr, observer, state, lt );
	CheckForErrorsA();
}

SPICELIBAPI void spkez_cW ( SpiceInt target,
                            SpiceDouble epoch,
                            const WCHAR * szcFrame,
                            const WCHAR * szcAbcorr,
                            SpiceInt observer,
                            SpiceDouble state[6],
                            SpiceDouble * lt )
{
	SpiceChar * frame = NULL, * abcorr = NULL;

	UnicodeToAnsi(szcFrame, frame);
	UnicodeToAnsi(szcAbcorr, abcorr);

	__CSPICE::spkez_c ( target, epoch, frame, abcorr, observer, state, lt );

	delete [] frame;
	delete [] abcorr;

	CheckForErrorsW();
}

SPICELIBAPI SpiceBoolean failed_c ( )
{
	return __CSPICE::failed_c();
}

SPICELIBAPI int getlms_A (CHAR * szMsg, long msg_len)
{
	return __CSPICE::getlms_(szMsg, msg_len);
}

SPICELIBAPI int getlms_W (WCHAR * szMsg, long msg_len)
{
	char * msg = NULL;

	UnicodeToAnsi(szMsg, msg_len, msg);

	int i = __CSPICE::getlms_(msg, msg_len);

	AnsiToUnicode(msg, msg_len, szMsg);	

	delete [] msg;

	return i;
}

SPICELIBAPI void reset_c ()
{
	return __CSPICE::reset_c ();
}

SPICELIBAPI void ldpool_cA ( const CHAR * szcFileName )
{
	__CSPICE::ldpool_c(szcFileName);
	CheckForErrorsA();
}

SPICELIBAPI void ldpool_cW ( const WCHAR * szcFileName )
{
	SpiceChar * filename = NULL;

	UnicodeToAnsi(szcFileName, filename);

	__CSPICE::ldpool_c(filename);

	delete [] filename;

	CheckForErrorsW();
}

SPICELIBAPI void bodvar_cA ( SpiceInt body,
                             const CHAR * szcItem,
                             SpiceInt * dim,
                             SpiceDouble * values )
{
	__CSPICE::bodvar_c(body, szcItem, dim, values);
	CheckForErrorsA();
}

SPICELIBAPI void bodvar_cW ( SpiceInt body,
                             const WCHAR * szcItem,
                             SpiceInt * dim,
                             SpiceDouble * values )
{
	SpiceChar * item = NULL;

	UnicodeToAnsi(szcItem, item);

	__CSPICE::bodvar_c(body, item, dim, values);

	delete [] item;

 	CheckForErrorsW();
}


SPICELIBAPI void spkobj_cA ( const CHAR * spk,
                              SpiceCell * ids )
{
	__CSPICE::spkobj_c(spk, ids);
	CheckForErrorsA();
}

SPICELIBAPI void spkobj_cW ( const WCHAR * szcSpk,
                              SpiceCell * ids )
{
	SpiceChar * filename = NULL;

	UnicodeToAnsi(szcSpk, filename);

	__CSPICE::spkobj_c(filename, ids);

	delete [] filename;

	CheckForErrorsW();
}

SPICELIBAPI SpiceInt card_c ( SpiceCell * cell )
{
	return __CSPICE::card_c( cell );
}

SPICELIBAPI void spkcov_cA	( const CHAR * spk,
                              SpiceInt idcode,
                              SpiceCell * cover )
{
	__CSPICE::spkcov_c(spk, idcode, cover);
	CheckForErrorsA();
}

SPICELIBAPI void spkcov_cW	( const WCHAR * szcSpk,
                              SpiceInt idcode,
                              SpiceCell * cover )
{
	SpiceChar * filename = NULL;

	UnicodeToAnsi(szcSpk, filename);

	__CSPICE::spkcov_c(filename, idcode, cover);

	delete [] filename;

	CheckForErrorsW();
}

SPICELIBAPI void wnfetd_c	( SpiceCell * window,
                              SpiceInt n,
                              SpiceDouble * left,
                              SpiceDouble * right )
{
	__CSPICE::wnfetd_c( window, n, left, right );
	CheckForErrorsA();
}

SPICELIBAPI void getfat_cA   ( const CHAR * file,
                              SpiceInt arclen,
                              SpiceInt typlen,
                              CHAR * arch,
                              CHAR * type )
{
	__CSPICE::getfat_c( file, arclen, typlen, arch, type );
	CheckForErrorsA();
}

SPICELIBAPI void getfat_cW   ( const WCHAR * szcFile,
                              SpiceInt arclen,
                              SpiceInt typlen,
                              WCHAR * szArch,
                              WCHAR * szType )
{
	SpiceChar * filename = NULL;
	SpiceChar * arch = NULL;
	SpiceChar * type = NULL;

	UnicodeToAnsi(szcFile, filename);
	UnicodeToAnsi(szArch, arclen, arch);
	UnicodeToAnsi(szType, typlen, type);

	__CSPICE::getfat_c(filename, arclen, typlen, arch, type );

	AnsiToUnicode(arch, arclen, szArch);	
	AnsiToUnicode(type, typlen, szType);	

	delete [] filename;
	delete [] arch;
	delete [] type;

	CheckForErrorsW();
}

SPICELIBAPI void spkw08_cA	( SpiceInt			   handle,
							  SpiceInt			   body,
							  SpiceInt             center,
							  const CHAR	     * frame,
							  SpiceDouble          first,
							  SpiceDouble          last,
                              const CHAR	     * segid,
                              SpiceInt             degree,
                              SpiceInt             n,
                              ConstSpiceDouble     states[][6],
                              SpiceDouble          epoch1,
                              SpiceDouble          step )
{
	using namespace __CSPICE;
	spkw08_c(  handle,  body, center, frame, first, last, segid,
        degree, n, states,  epoch1, step );
	CheckForErrorsA();
}

SPICELIBAPI void spkw08_cW	( SpiceInt			   handle,
							  SpiceInt			   body,
							  SpiceInt             center,
							  const WCHAR	     * szcFrame,
							  SpiceDouble          first,
							  SpiceDouble          last,
                              const WCHAR     *	   szcSegid,
                              SpiceInt             degree,
                              SpiceInt             n,
                              ConstSpiceDouble     states[][6],
                              SpiceDouble          epoch1,
                              SpiceDouble          step )
{
	SpiceChar * frame = NULL;
	SpiceChar * segid = NULL;

	UnicodeToAnsi(szcFrame, frame);
	UnicodeToAnsi(szcSegid, segid);

	using namespace __CSPICE;
	spkw08_c(  handle,  body, center, frame, first, last, segid,
        degree, n, states,  epoch1, step );

	delete [] frame;
	delete [] segid;

	CheckForErrorsW();
}

//  extern int spkw08_(integer *handle, integer *body, integer *center, char *frame, doublereal *first, doublereal *last, char *segid, integer *degree, integer *n, doublereal *states, doublereal *epoch1, doublereal *step, ftnlen frame_len, ftnlen segid_len);

SPICELIBAPI int spkw08_A	(SpiceInt handle, 
								SpiceInt body, 
								SpiceInt center, 
								const CHAR *frame, 
								SpiceDouble first, 
								SpiceDouble last, 
								const CHAR *segid, 
								SpiceInt degree, 
								SpiceInt n, 
								SpiceDouble * states, 
								SpiceDouble epoch1, 
								SpiceDouble step)
{
	using namespace __CSPICE;
	int ret = spkw08_(  &handle,  &body, &center, ( char * )frame, &first, &last,( char * ) segid,
        &degree, &n, states,  &epoch1, &step, (__CSPICE::ftnlen)strlen(frame), (__CSPICE::ftnlen)strlen(segid) );
	CheckForErrorsA();

	return ret;
}

SPICELIBAPI int spkw08_W	(SpiceInt handle, 
								SpiceInt body, 
								SpiceInt center, 
								const WCHAR * szcFrame, 
								SpiceDouble first, 
								SpiceDouble last, 
								const WCHAR * szcSegid, 
								SpiceInt degree, 
								SpiceInt n, 
								SpiceDouble * states, 
								SpiceDouble epoch1, 
								SpiceDouble step)
{
	SpiceChar * frame = NULL;
	SpiceChar * segid = NULL;

	UnicodeToAnsi(szcFrame, frame);
	UnicodeToAnsi(szcSegid, segid);

	using namespace __CSPICE;
	int ret = spkw08_(  &handle,  &body, &center, ( char * )frame, &first, &last,( char * ) segid,
        &degree, &n, states,  &epoch1, &step, (__CSPICE::ftnlen)strlen(frame), (__CSPICE::ftnlen)strlen(segid) );
	delete [] frame;
	delete [] segid;

	CheckForErrorsW();

	return ret;
}

SPICELIBAPI void spkcls_c	( SpiceInt handle )
{
	__CSPICE::spkcls_c( handle );
}

SPICELIBAPI void spkopn_cA	( const CHAR    * name,
                              const CHAR    * ifname,
                              SpiceInt            ncomch,
                              SpiceInt          * handle  )
{
	__CSPICE::spkopn_c(  name,  ifname, ncomch, handle );
	CheckForErrorsA();
}

SPICELIBAPI void spkopn_cW	( const WCHAR    * szcName,
                              const WCHAR    * szcIfname,
                              SpiceInt            ncomch,
                              SpiceInt          * handle  )
{
	SpiceChar * name = NULL;
	SpiceChar * ifname = NULL;

	UnicodeToAnsi(szcName, name);
	UnicodeToAnsi(szcIfname, ifname);

	__CSPICE::spkopn_c(  name,  ifname, ncomch, handle );

	delete [] name;
	delete [] ifname;

	CheckForErrorsW();
}

SPICELIBAPI void spkopa_cA	( const CHAR    * file,
                              SpiceInt          * handle )
{
	__CSPICE::spkopa_c(  file,  handle );
	CheckForErrorsA();
}

SPICELIBAPI void spkopa_cW	( const WCHAR    * szFile,
                              SpiceInt          * handle )
{
	SpiceChar * file = NULL;

	UnicodeToAnsi(szFile, file);

	__CSPICE::spkopa_c(  file,  handle );
	delete [] file;

	CheckForErrorsW();
}

SPICELIBAPI int bodeul_		(SpiceInt *body, 
							 SpiceDouble *et, 
							 SpiceDouble *ra, 
							 SpiceDouble *dec, 
							 SpiceDouble *w, 
							 SpiceDouble *lambda)
{
	int r = __CSPICE::bodeul_( body, et, ra, dec, w, lambda );
	CheckForErrorsA();
    return r;
}

SPICELIBAPI void oscelt_c	( ConstSpiceDouble    state[6],
                            SpiceDouble         et      ,
                            SpiceDouble         mu      ,
                            SpiceDouble         elts[8]  )
{
	__CSPICE::oscelt_c( state, et, mu, elts );
	CheckForErrorsA();
}

SPICELIBAPI void conics_c	( ConstSpiceDouble    elts[8],
                                SpiceDouble         et,
                                SpiceDouble         state[6] )
{
	__CSPICE::conics_c( elts, et, state );
	CheckForErrorsA();
}

SPICELIBAPI void bodvcd_cA	( SpiceInt             body,
                                const CHAR     * item,
                                SpiceInt             maxn,
                                SpiceInt           * dim ,
                                SpiceDouble        * values )
{
	__CSPICE::bodvcd_c(  body,  item, maxn, dim, values );
	CheckForErrorsA();
}

SPICELIBAPI void bodvcd_cW	( SpiceInt             body,
                                const WCHAR     * szItem,
                                SpiceInt             maxn,
                                SpiceInt           * dim ,
                                SpiceDouble        * values )
{
	SpiceChar * item = NULL;

	UnicodeToAnsi(szItem, item);

	__CSPICE::bodvcd_c(  body,  item, maxn, dim, values );
	delete [] item;

	CheckForErrorsW();
}


SPICELIBAPI int spcac_A	( SpiceInt    *         handle,
                                SpiceInt		*   unit,
                                const CHAR		* szcBmark,
                                const CHAR		* szcEmark
                                )
{
	using namespace __CSPICE;
	int ret = spcac_( handle, unit, (char *)szcBmark, (char *)szcEmark, (__CSPICE::ftnlen)strlen(szcBmark), (__CSPICE::ftnlen)strlen(szcEmark));
	CheckForErrorsA();

	return ret;
}

SPICELIBAPI int spcac_W	( SpiceInt    *         handle,
                                SpiceInt	*	   unit,
                                const WCHAR		* szcBmark,
                                const WCHAR		* szcEmark
                                )
{
	SpiceChar * bmark = NULL;
	SpiceChar * emark = NULL;

	UnicodeToAnsi(szcBmark, bmark);
	UnicodeToAnsi(szcEmark, emark);

	using namespace __CSPICE;
	int ret = spcac_( handle, unit, (char *)bmark, (char *)emark, (__CSPICE::ftnlen)strlen(bmark), (__CSPICE::ftnlen)strlen(emark));

	delete [] bmark;
	delete [] emark;

	CheckForErrorsW();

	return ret;
}

SPICELIBAPI void ftncls_cA ( SpiceInt            unit )
{
	__CSPICE::ftncls_c( unit );
	CheckForErrorsA();
}

SPICELIBAPI void ftncls_cW ( SpiceInt            unit )
{
	__CSPICE::ftncls_c( unit );
	CheckForErrorsW();
}


SPICELIBAPI int txtopr_A (const CHAR * szcFname, 
						SpiceInt * unit)
{
	using namespace __CSPICE;
	int ret = txtopr_( (char*)szcFname, unit, (__CSPICE::ftnlen)strlen(szcFname) );

	CheckForErrorsA();

	return ret;
}

SPICELIBAPI int txtopr_W (const WCHAR * szcFname, 
						SpiceInt * unit)
{
	SpiceChar * fname = NULL;

	UnicodeToAnsi(szcFname, fname);

	using namespace __CSPICE;
	int ret = txtopr_( (char*)fname, unit, (__CSPICE::ftnlen)strlen(fname) );

	delete [] fname;

	CheckForErrorsW();

	return ret;
}