/************************************************************
*
*  Copyright (c) 2002-2004 Jakub Schwarzmeier
*
************************************************************/

//  Single-threaded (ST) dynamic-link library with an interface for CSPICE.
//  CSPICE could not be used directly in multi-threaded (MT) programs
//  in form of static library lib, because it uses some global 
//  variables from ST C-RTL that aren't in MT C-RTL.
//  Library is Unicode/ANSI.

//  Called functions are (c) by the California Institute of Technology (CALTECH).

//  note1: Parameter lenout is usually used for IN/OUT parameter
//  of type string terminated with \0. I can pass string of some 
//  length but lenout specifies maximal size of string that can 
//  function return into this buffer. External buffer must be 
//  one character bigger than lenout for inserting ending sing \0.

//  note2: Functions can throw exception CSpiceException declared here.
//  This isn't a standard behaviour of CSPICE.

//  note3: Operators new & delete must be overloaded. This workaround
//  helps to solve a problem with RTL that tracks object allocated
//  on heap. There is one heap for DLL and one for calling module.
//  Overloading ensures that exception object is allocated and freed from
//  the same (this DLL) heap. Cannot throw MFC exceptions because
//  MFC is multi-threaded.

#pragma once

#undef min
#undef max

//  definitions of spice datatypes (SpiceInt, SpiceDouble, ...)
#include <D:\Dokumenty\_space\spice\cspice\Include\spicezdf.h>
//  definitions of f2c datatypes (doublereal, ...)
#include <D:\Dokumenty\_space\spice\cspice\Include\f2c.h>
//  definitions for the SpiceCell data type
#include <D:\Dokumenty\_space\spice\cspice\Include\SpiceCel.h>

#define CHAR char
#define WCHAR wchar_t

#ifndef SPICELIBAPI
	#define SPICELIBAPI __declspec (dllimport)

	//  UNICODE and ANSI function defines
	//
	#ifndef UNICODE
	#define bodc2n_c bodc2n_cA
	#define bodn2c_c bodn2c_cA
	#define lspcn_c lspcn_cA
	#define recrad_c recrad_cA
	#define radrec_c radrec_cA
	#define pdpool_c pdpool_cA
	#define pcpool_c pcpool_cA
	#define pipool_c pipool_cA
	#define dpr_c dpr_cA
	#define reclat_c reclat_cA
	#define tipbod_ tipbod_A
	#define deltet_ deltet_A
	#define errdev_c errdev_cA
	#define erract_c erract_cA
	#define spklef_c spklef_cA
	#define spkezr_c spkezr_cA
	#define pxform_c pxform_cA
	#define errprt_c errprt_cA
	#define spkez_c  spkez_cA
	#define getlms_  getlms_A
	#define ldpool_c ldpool_cA
	#define bodvar_c bodvar_cA
	#define spkobj_c spkobj_cA
	#define spkcov_c spkcov_cA
	#define getfat_c getfat_cA
	#define spkw08_c spkw08_cA
	#define spkopn_c spkopn_cA
	#define spkopa_c spkopa_cA
	#define spkw08_ spkw08_A
	#define bodvcd_c bodvcd_cA
	#define spcac_ spcac_A
	#define ftncls_c ftncls_cA
	#define txtopr_ txtopr_A
	#define CSpiceException CSpiceExceptionA
	#else
	#define bodc2n_c bodc2n_cW
	#define bodn2c_c bodn2c_cW
	#define lspcn_c lspcn_cW
	#define recrad_c recrad_cW
	#define radrec_c radrec_cW
	#define pdpool_c pdpool_cW
	#define pcpool_c pcpool_cW
	#define pipool_c pipool_cW
	#define dpr_c dpr_cW
	#define reclat_c reclat_cW
	#define tipbod_ tipbod_W
	#define deltet_ deltet_W
	#define errdev_c errdev_cW
	#define erract_c erract_cW
	#define spklef_c spklef_cW
	#define spkezr_c spkezr_cW
	#define pxform_c pxform_cW
	#define errprt_c errprt_cW
	#define spkez_c  spkez_cW
	#define getlms_  getlms_W
	#define ldpool_c ldpool_cW
	#define bodvar_c bodvar_cW
	#define spkobj_c spkobj_cW
	#define spkcov_c spkcov_cW
	#define getfat_c getfat_cW
	#define spkw08_c spkw08_cW
	#define spkopn_c spkopn_cW
	#define spkopa_c spkopa_cW
	#define spkw08_ spkw08_W
	#define bodvcd_c bodvcd_cW
	#define spcac_ spcac_W
	#define ftncls_c ftncls_cW
	#define txtopr_ txtopr_W
	#define CSpiceException CSpiceExceptionW
	#endif
#endif

SPICELIBAPI void bodc2n_cA ( SpiceInt code,
                            SpiceInt namelen,
                            CHAR  * name,
                            SpiceBoolean * found );

SPICELIBAPI void bodc2n_cW ( SpiceInt code,
                            SpiceInt namelen,
                            WCHAR  * name,
                            SpiceBoolean * found );

SPICELIBAPI void bodn2c_cA ( const CHAR * name,
                            SpiceInt * code,
                            SpiceBoolean * found );

SPICELIBAPI void bodn2c_cW ( const WCHAR * name,
                            SpiceInt * code,
                            SpiceBoolean * found );

SPICELIBAPI SpiceDouble lspcn_cA ( const CHAR * body,
                                  SpiceDouble et,
                                  const CHAR * abcorr );

SPICELIBAPI SpiceDouble lspcn_cW ( const WCHAR * body,
                                  SpiceDouble et,
                                  const WCHAR * abcorr );

SPICELIBAPI void recrad_cA ( ConstSpiceDouble    rectan[3],
                                SpiceDouble       * radius,
                                SpiceDouble       * ra,
                                SpiceDouble       * dec  );

SPICELIBAPI void recrad_cW ( ConstSpiceDouble    rectan[3],
                                SpiceDouble       * radius,
                                SpiceDouble       * ra,
                                SpiceDouble       * dec  );

SPICELIBAPI void radrec_cA ( SpiceDouble range,
                            SpiceDouble ra,
                            SpiceDouble dec,
                            SpiceDouble rectan[3] );

SPICELIBAPI void radrec_cW ( SpiceDouble range,
                            SpiceDouble ra,
                            SpiceDouble dec,
                            SpiceDouble rectan[3] );

SPICELIBAPI void pdpool_cA ( CHAR * name,
                            SpiceInt n,
                            ConstSpiceDouble * dvals );

SPICELIBAPI void pdpool_cW ( WCHAR * name,
                            SpiceInt n,
                            ConstSpiceDouble * dvals );

SPICELIBAPI void pcpool_cA ( CHAR * name,
                            SpiceInt n,
                            SpiceInt lenvals,
                            const void * cvals );
                                
SPICELIBAPI void pcpool_cW ( WCHAR * name,
                            SpiceInt n,
                            SpiceInt lenvals,
                            const void * cvals );

SPICELIBAPI void pipool_cA ( CHAR * name,
                            SpiceInt n,
                            ConstSpiceInt * ivals );

SPICELIBAPI void pipool_cW ( WCHAR * name,
                            SpiceInt n,
                            ConstSpiceInt * ivals );

SPICELIBAPI SpiceDouble dpr_cA ( );
SPICELIBAPI SpiceDouble dpr_cW ( );

SPICELIBAPI  void reclat_cA ( ConstSpiceDouble    rectan[3],
                                SpiceDouble       * radius,
                                SpiceDouble       * longitude,
                                SpiceDouble       * latitude  );

SPICELIBAPI  void reclat_cW ( ConstSpiceDouble    rectan[3],
                                SpiceDouble       * radius,
                                SpiceDouble       * longitude,
                                SpiceDouble       * latitude  );


SPICELIBAPI SpiceInt tipbod_A (CHAR * ref, 
							   SpiceInt * body, 
							   SpiceDouble * et, 
							   SpiceDouble * tipm,
							   SpiceInt ref_len);

SPICELIBAPI SpiceInt tipbod_W (WCHAR * ref, 
							   SpiceInt * body, 
							   SpiceDouble * et, 
							   SpiceDouble * tipm,
							   SpiceInt ref_len);

SPICELIBAPI SpiceInt deltet_A (  doublereal * epoch,
									CHAR * eptype,
									doublereal * delta,
									SpiceInt ref_len);

SPICELIBAPI SpiceInt deltet_W ( doublereal * epoch,
                             WCHAR * eptype,
                             doublereal * delta,
						     SpiceInt ref_len);

SPICELIBAPI void errdev_cA ( const CHAR * szcOperation,
                             SpiceInt lenout,
                             CHAR * szDevice );

SPICELIBAPI void errdev_cW ( const WCHAR * szcOperation,
                             SpiceInt lenout,
                             WCHAR * szDevice );

SPICELIBAPI void erract_cA ( const CHAR * szcOperation,
                             SpiceInt lenout,
                             CHAR * szAction );

SPICELIBAPI void erract_cW ( const WCHAR * szcOperation,
                             SpiceInt lenout,
                             WCHAR * szAction );

SPICELIBAPI void spkuef_c ( SpiceInt handle );

SPICELIBAPI void spklef_cA ( const CHAR * filename,
                             SpiceInt * handle );

SPICELIBAPI void spklef_cW ( const WCHAR * filename,
                             SpiceInt * handle );

SPICELIBAPI void spkezr_cA ( const CHAR * target,
                             SpiceDouble epoch,
                             const CHAR * szcFrame,
                             const CHAR * szcAbcorr,
                             const CHAR * szcObserver,
                             SpiceDouble state[6],
                             SpiceDouble * lt );

SPICELIBAPI void spkezr_cW ( const WCHAR * target,
                             SpiceDouble epoch,
                             const WCHAR * szcFrame,
                             const WCHAR * szcAbcorr,
                             const WCHAR * szcObserver,
                             SpiceDouble state[6],
                             SpiceDouble * lt );

SPICELIBAPI void pxform_cA ( const CHAR * from,
                             const CHAR * szTo,
                             SpiceDouble et,
                             SpiceDouble rotate[3][3] );

SPICELIBAPI void pxform_cW ( const WCHAR * from,
                             const WCHAR * szTo,
                             SpiceDouble et,
                             SpiceDouble rotate[3][3] );

SPICELIBAPI void mxv_c ( ConstSpiceDouble m1[3][3],
						 ConstSpiceDouble vin[3],
						 SpiceDouble vout[3] );

SPICELIBAPI void errprt_cA ( const CHAR * szOperation,
                             SpiceInt lenout,
                             CHAR * szList );

SPICELIBAPI void errprt_cW ( const WCHAR * szOperation,
                             SpiceInt lenout,
                             WCHAR * szList );

SPICELIBAPI void spkez_cA ( SpiceInt target,
                            SpiceDouble epoch,
                            const CHAR * szcFrame,
                            const CHAR * szcAbcorr,
                            SpiceInt observer,
                            SpiceDouble state[6],
                            SpiceDouble * lt );
 
SPICELIBAPI void spkez_cW ( SpiceInt target,
                            SpiceDouble epoch,
                            const WCHAR * szcFrame,
                            const WCHAR * szcAbcorr,
                            SpiceInt observer,
                            SpiceDouble state[6],
                            SpiceDouble * lt );
 
SPICELIBAPI SpiceBoolean failed_c ();

SPICELIBAPI int getlms_A (CHAR * szMsg, long msg_len);
SPICELIBAPI int getlms_W (WCHAR * szMsg, long msg_len);

SPICELIBAPI void reset_c ( void );

SPICELIBAPI void ldpool_cA ( const CHAR * szcFileName );
SPICELIBAPI void ldpool_cW ( const WCHAR * szcFileName );

SPICELIBAPI void bodvar_cA ( SpiceInt body,
                             const CHAR * szcItem,
                             SpiceInt * dim,
                             SpiceDouble * values );
SPICELIBAPI void bodvar_cW ( SpiceInt body,
                             const WCHAR * szcItem,
                             SpiceInt * dim,
                             SpiceDouble * values );

SPICELIBAPI void spkobj_cA	( const CHAR * spk,
                              SpiceCell * ids );

SPICELIBAPI void spkobj_cW	( const WCHAR * spk,
                              SpiceCell * ids );

SPICELIBAPI SpiceInt card_c ( SpiceCell * cell );

SPICELIBAPI void spkcov_cA	( const CHAR * spk,
                              SpiceInt idcode,
                              SpiceCell * cover );

SPICELIBAPI void spkcov_cW	( const WCHAR * spk,
                              SpiceInt idcode,
                              SpiceCell * cover );

SPICELIBAPI void wnfetd_c	( SpiceCell * window,
                              SpiceInt n,
                              SpiceDouble * left,
                              SpiceDouble * right ); 

SPICELIBAPI void getfat_cA   ( const CHAR * file,
                              SpiceInt arclen,
                              SpiceInt typlen,
                              CHAR * arch,
                              CHAR * type ); 

SPICELIBAPI void getfat_cW   ( const WCHAR * file,
                              SpiceInt arclen,
                              SpiceInt typlen,
                              WCHAR * arch,
                              WCHAR * type ); 

SPICELIBAPI void spkw08_cA	( SpiceInt			   handle,
							  SpiceInt			   body,
							  SpiceInt             center,
							  const CHAR	     * frame,
							  SpiceDouble          first,
							  SpiceDouble          last,
                              const CHAR     * segid,
                              SpiceInt             degree,
                              SpiceInt             n,
                              ConstSpiceDouble     states[][6],
                              SpiceDouble          epoch1,
                              SpiceDouble          step );

SPICELIBAPI void spkw08_cW	( SpiceInt			   handle,
							  SpiceInt			   body,
							  SpiceInt             center,
							  const WCHAR	     * frame,
							  SpiceDouble          first,
							  SpiceDouble          last,
                              const WCHAR     * segid,
                              SpiceInt             degree,
                              SpiceInt             n,
                              ConstSpiceDouble     states[][6],
                              SpiceDouble          epoch1,
                              SpiceDouble          step );


SPICELIBAPI int spkw08_A	(SpiceInt handle, 
								SpiceInt body, 
								SpiceInt center, 
								const CHAR *frame, 
								SpiceDouble first, 
								SpiceDouble last, 
								const CHAR *segid, 
								SpiceInt degree, 
								SpiceInt n, 
								SpiceDouble * states, 
								SpiceDouble epoch1, 
								SpiceDouble step);

SPICELIBAPI int spkw08_W	(SpiceInt handle, 
								SpiceInt body, 
								SpiceInt center, 
								const WCHAR *frame, 
								SpiceDouble first, 
								SpiceDouble last, 
								const WCHAR *segid, 
								SpiceInt degree, 
								SpiceInt n, 
								SpiceDouble * states, 
								SpiceDouble epoch1, 
								SpiceDouble step);

SPICELIBAPI void spkcls_c	( SpiceInt handle );

SPICELIBAPI void spkopn_cA	( const CHAR    * name,
                              const CHAR    * ifname,
                              SpiceInt            ncomch,
                              SpiceInt          * handle  );

SPICELIBAPI void spkopn_cW	( const WCHAR    * name,
                              const WCHAR    * ifname,
                              SpiceInt            ncomch,
                              SpiceInt          * handle  );

SPICELIBAPI void spkopa_cA	( const CHAR    * file,
                              SpiceInt          * handle );

SPICELIBAPI void spkopa_cW	( const WCHAR    * file,
                              SpiceInt          * handle );

SPICELIBAPI int bodeul_		(SpiceInt *body, 
							 SpiceDouble *et, 
							 SpiceDouble *ra, 
							 SpiceDouble *dec, 
							 SpiceDouble *w, 
							 SpiceDouble *lambda);

SPICELIBAPI void oscelt_c	( ConstSpiceDouble    state[6],
                            SpiceDouble         et      ,
                            SpiceDouble         mu      ,
                            SpiceDouble         elts[8]  );

SPICELIBAPI void conics_c	( ConstSpiceDouble    elts[8],
                                SpiceDouble         et,
                                SpiceDouble         state[6] );

SPICELIBAPI void bodvcd_cA	( SpiceInt             body,
                                const CHAR     * item,
                                SpiceInt             maxn,
                                SpiceInt           * dim ,
                                SpiceDouble        * values );

SPICELIBAPI void bodvcd_cW	( SpiceInt             body,
                                const WCHAR     * item,
                                SpiceInt             maxn,
                                SpiceInt           * dim ,
                                SpiceDouble        * values );

SPICELIBAPI int spcac_A	( SpiceInt       *      handle,
                                SpiceInt	*	   unit,
                                const CHAR		* szcBmark,
                                const CHAR		* szcEmark
                                );

SPICELIBAPI int spcac_W	( SpiceInt        *     handle,
                                SpiceInt	*	   unit,
                                const WCHAR		* szcBmark,
                                const WCHAR		* szcEmark
                                );

SPICELIBAPI void ftncls_cA ( SpiceInt            unit );
SPICELIBAPI void ftncls_cW ( SpiceInt            unit );

SPICELIBAPI int txtopr_A (const CHAR * szcFname, 
						SpiceInt * unit);

SPICELIBAPI int txtopr_W (const WCHAR * szcFname, 
						SpiceInt * unit);
