#include "StdAfx.h"

#include "datetime.h"

#include <ctype.h>
#include <locale>
#include <time.h>
#include <wx/intl.h>
#include <schwarzmeier/wx.h>
#include <schwarzmeier/xmllib.h>

using namespace JakubSchwarzmeier::Astronomy;

CDateTime::CDateTime()
{
	Set(2451545.0);
}

CDateTime::~CDateTime(void)
{
}

CDateTime::CDateTime(GregorianCalendar * gc)
{
	UErrorCode success = U_ZERO_ERROR;
	int32_t yy = gc->get( UCAL_YEAR, success );
	int32_t mm = gc->get( UCAL_MONTH, success );
	int32_t dd = gc->get( UCAL_DATE, success );
	
	int32_t hh = gc->get( UCAL_HOUR_OF_DAY , success );
	int32_t min = gc->get( UCAL_MINUTE, success );
	int32_t ss = gc->get( UCAL_SECOND, success );
	int32_t ms = gc->get( UCAL_MILLISECOND, success );

	Set( dd, static_cast<wxDateTime::Month>( mm ), yy, hh, min, ss, ms );
}

CDateTime::CDateTime( const CDateTime & original )
{
	operator=( original );
}

CDateTime::CDateTime( double jdn ): wxDateTime( jdn )
{
}

CDateTime::CDateTime(const wxDateTime & wxDT): wxDateTime( wxDT )
{
}

bool CDateTime::SetMonthInt(int month)
{
	bool ok = true;
	
	wxDateTime::Month wxmonth;

	switch (month)
	{
	case 1:

		wxmonth = Jan;
		break;

	case 2:

		wxmonth = Feb;
		break;

	case 3:

		wxmonth = Mar;
		break;

	case 4:

		wxmonth = Apr;
		break;

	case 5:

		wxmonth = May;
		break;

	case 6:

		wxmonth = Jun;
		break;

	case 7:

		wxmonth = Jul;
		break;

	case 8:

		wxmonth = Aug;
		break;

	case 9:

		wxmonth = Sep;
		break;

	case 10:

		wxmonth = Oct;
		break;

	case 11:

		wxmonth = Nov;
		break;

	case 12:

		wxmonth = Dec;
		break;

	default:

		ok = false;
		break;
	}

	SetMonth( wxmonth );
	
	return ok;
}

int CDateTime::GetMonthInt()
{
	int month;

	switch ( GetMonth() )
	{
	case Jan:

		month = 1;
		break;

	case Feb:

		month = 2;
		break;

	case Mar:

		month = 3;
		break;

	case Apr:

		month = 4;
		break;

	case May:

		month = 5;
		break;

	case Jun:

		month = 6;
		break;

	case Jul:

		month = 7;
		break;

	case Aug:

		month = 8;
		break;

	case Sep:

		month = 9;
		break;

	case Oct:

		month = 10;
		break;

	case Nov:

		month = 11;
		break;

	case Dec:

		month = 12;
		break;

	default:

		month = 0;
		break;
	}

	return month;
}

/*static*/
wxChar CDateTime::GetDateSeparator()
{
	wxDateTime dt;
	dt.SetToCurrent();
	wxString strDate = dt.FormatDate();

	for(size_t i = 0; i < strlen( strDate.c_str() ); i++)
	{
		if ( isalnum( static_cast<unsigned char>( strDate.GetChar(i) ) ) == 0 )
		{
			return strDate.GetChar(i);
		}
	}

	return wxT('.');
}

wxString CDateTime::FormatDateLong()
{
	wxString strDate;
	wxChar dateSeparator = CDateTime::GetDateSeparator();

	wxString strPrintf( wxT("%d") );
	strPrintf.Append( dateSeparator );
	strPrintf.Append( wxT("%d") );
	strPrintf.Append( dateSeparator );
	strPrintf.Append( wxT("%d") );

	std::locale loc( wxGetLocale()->GetSysName() );
	std::time_get <char>::dateorder order = std::use_facet <std::time_get <char> >( loc ).date_order ( );

	switch (order)
	{
		case std::time_base::ydm: 

			strDate.Printf( strPrintf, GetYear(), GetDay(), GetMonth() );
			break;

		case std::time_base::ymd: 

			strDate.Printf( strPrintf, GetYear(), GetMonth(), GetDay() );
			break;
		
		case std::time_base::mdy:

			strDate.Printf( strPrintf, GetMonth(), GetDay(), GetYear() );
			break;

		case std::time_base::dmy:
		case std::time_base::no_order:
		default:

			strDate.Printf( strPrintf, GetDay(), GetMonthInt(), GetYear() );
			break;
	}

	return strDate;
}

/*static*/
wxChar CDateTime::GetTimeSeparator()
{
	wxDateTime dt;
	dt.SetToCurrent();
	wxString strTime = dt.FormatTime();

	for(size_t i = 0; i < strlen( strTime.c_str() ); i++)
	{
		if ( isalnum( static_cast<unsigned char>( strTime.GetChar(i) ) ) == 0 )
		{
			return strTime.GetChar(i);
		}
	}

	return wxT(':');
}

wxString CDateTime::FormatTimeLong()
{
	wxString strTime;
	wxChar timeSeparator = CDateTime::GetTimeSeparator();

	wxString strPrintf( wxT("%02d") );
	strPrintf.Append( timeSeparator );
	strPrintf.Append( wxT("%02d") );
	strPrintf.Append( timeSeparator );
	strPrintf.Append( wxT("%02d") );

	strTime.Printf( strPrintf, GetHour(), GetMinute(), GetSecond() );

	return strTime;
}

CDateTime & CDateTime::operator=( const CDateTime & right_side )
{
	wxDateTime::operator=( right_side );
/*	Set( right_side.GetDay(), right_side.GetMonth(), right_side.GetYear(),
		right_side.GetHour(), right_side.GetMinute(), right_side.GetSecond(),
		right_side.GetMillisecond() );
*/	
	return *this;
}

void CDateTime::Serialize(xmlDoc * pXmlDoc, xmlNode * branch)
{
	xmlNode * datetimeNode = xmlNewNode( NULL, (const xmlChar *)s_strDateTime.c_str() );
	xmlAddChild( branch, datetimeNode );

	SetTextNode( pXmlDoc, datetimeNode, GetJulianDayNumber() );
}

void CDateTime::Deserialize(xmlDoc * pXmlDoc, xmlNode * pXmlNode)
{
	std::string xpathDate( s_strDateTime );
	xpathDate += _T("/text()");

	std::string strTime = GetXPathResponse( pXmlDoc, pXmlNode, xpathDate );
	double time = _tstof( strTime.c_str() );
	Set( time );
}

std::string CDateTime::s_strDateTime( _T("datetime") );