/************************************************************
*
*  Copyright (c) 2002-2004 Jakub Schwarzmeier
*
************************************************************/

#pragma once

#include <string>
#include <map>
#include <set>
#include <schwarzmeier/std.h>

namespace JakubSchwarzmeier
{
	namespace Astronomy
	{
		class CSpice
		{
		public:

			CSpice();
			~CSpice();
		
			void LoadSpkFile( std::string strFileName );
			void UnloadSpkFile( std::string strFileName );
			void UnloadSpkFiles();
			void LoadKernelFile( std::string strFileName );
			//  Get ephemeris in rectangular ecliptical coordinates
			void GetEphemerisEclipticRectangular( int observerId/*Sun*/, int targetId, double utcJD, 
				JakubSchwarzmeier::Std::Math::Vector<> & position, 
				JakubSchwarzmeier::Std::Math::Vector<> & velocity );
			// Compute L_s, the planetocentric longitude of the sun, as seen from center
			double GetLSun( int observerId, double utcJD );
			//  Get ephemeris in planetocentric coordinates
			void GetEphemerisPlanetocentricLatLon( int observerId, int targetId, double utcJD, 
				double & latitude, double & longitude );
			//  Get ephemeris in horizon coordinates -- elevation (altitude) and azimuth
			void GetEphemerisHorizon( int observerId, int targetId, double utcJD,
				double targetLat, double targetLon, 
				double & elevation, double & azimuth );

			static std::set<int> GetSpkObjectsIDs( std::set<std::string> & files );
			static std::map<std::string, int> GetSpkObjects( std::set<std::string> & files );

			void GetPoleEcliptic( int bodyID, double utcJD, 
				double & longitude, double & latitude );
			double GetPM( int bodyID, double utcJD );
			void GetRadii( int bodyID, double & radius );
			void GetMass( int bodyID, double & mass );
			void GetName( int bodyID, std::string & name );

			static std::string GetFileType(std::string file);

		private:

			typedef std::map<std::string, int> MAP;
			MAP m_spkFiles;
		};

	}	//  namespace Astronomy

} // namespace JakubSchwarzmeier