/************************************************************
*
*  Copyright (c) 2002-2004 Jakub Schwarzmeier
*
************************************************************/

#include "stdafx.h"

#include <SFC.Graphics.h>
#include "ObserverPageDlg.h"

using namespace JakubSchwarzmeier::SFC::Graphics;
using namespace JakubSchwarzmeier::Std::Math;

CObserver::CObserver()		//  base class constructor (Camera's) is called automatically
{
	//  holder of property page, created only when needed
	m_dlgSettings = NULL;

	//  initialize field of pressed keys
	for(int i = 0; i < 0xFF; i++)	m_keys[i] = false;

	m_bMouseRightDrag = false;

	m_stepSize.position = 1;
	m_stepSize.angle = 5;
}

CObserver::~CObserver()
{
	if (m_dlgSettings != NULL)
	{
		ASSERT_VALID( m_dlgSettings );
		delete m_dlgSettings;
	}
}

void CObserver::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	m_keys[nChar] = true;
	UpdateInputs();

	m_view->ReleaseRenderingThread();
}

void CObserver::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	UpdateInputs();
	m_keys[nChar] = false;

	m_view->ReleaseRenderingThread();
}

void CObserver::UpdateInputs()
{
	bool shift;
//	if ( GetKeyState(VK_SHIFT) & 0x10000 ) 
	if (m_keys[16])
		shift = true;
	else
		shift = false;

	if ( m_keys[VK_UP] & !shift )		ChangePosition(Vector<>(0, 0, -m_stepSize.position));
	if ( m_keys[VK_DOWN] & !shift )		ChangePosition(Vector<>(0, 0, m_stepSize.position));
	if ( m_keys[VK_LEFT] & !shift )		ChangeHeading( m_stepSize.angle );
	if ( m_keys[VK_RIGHT] & !shift )	ChangeHeading( -m_stepSize.angle );
	if ( m_keys[VK_HOME] & !shift )		ChangeRoll(m_stepSize.angle);
	if ( m_keys[VK_END] & !shift )		ChangeRoll(-m_stepSize.angle);
		

	if ( m_keys[VK_UP] && shift )		ChangePitch( m_stepSize.angle );
	if ( m_keys[VK_DOWN]&& shift )		ChangePitch( -m_stepSize.angle );
	if ( m_keys[VK_LEFT] && shift )		ChangePosition(Vector<>(-m_stepSize.position, 0, 0));
	if ( m_keys[VK_RIGHT] && shift )	ChangePosition(Vector<>(m_stepSize.position, 0, 0));
//	if ( m_keys[VK_HOME] && shift )		ChangePosition(Vector<>(0, m_stepSize.position, 0));
//	if ( m_keys[VK_END] && shift )		ChangePosition(Vector<>(0, -m_stepSize.position, 0));
	if ( m_keys[VK_HOME] && shift )		ChangeLambda(m_stepSize.angle);
	if ( m_keys[VK_END] && shift )		ChangeLambda(-m_stepSize.angle);
}

void CObserver::SetPositionStepSize(double stepSize)
{
	m_stepSize.position = stepSize;
}

void CObserver::SetAngleStepSize(double stepSize)
{
	m_stepSize.angle = stepSize;
}

CObserver::STEP_SIZE CObserver::GetStepSize()
{
	return m_stepSize;
}

void CObserver::OnRButtonDown(UINT nFlags, int x, int y)
{
	m_bMouseRightDrag = true;
	m_posRMouse.x = x;
	m_posRMouse.y = y;

	UpdateInputs();
	m_view->ReleaseRenderingThread();
}

void CObserver::OnRButtonUp(UINT nFlags, int x, int y)
{
	m_bMouseRightDrag = false;
	UpdateInputs();

	m_view->ReleaseRenderingThread();
}

void CObserver::OnMouseMove(UINT nFlags, int x, int y)
{
	if (m_bMouseRightDrag)
	{
 		POINT m_RMouseRotate;
		m_RMouseRotate.x = (x - m_posRMouse.x);
 		m_RMouseRotate.y = (y - m_posRMouse.y);
		
//		m_base.ChangeFiBy( m_RMouseRotate.y * m_stepSize.angle );
//		m_base.ChangeLambdaBy( m_RMouseRotate.x * m_stepSize.angle );

		m_posRMouse.x = x;
		m_posRMouse.y = y;
	}
	UpdateInputs();

	m_view->ReleaseRenderingThread();
}

void CObserver::SetView( COpenGLView * view )
{
	ASSERT_VALID( view );
	m_view = view;
}

CPropertyPage * CObserver::GetSettingsPage()
{
	if (m_dlgSettings == NULL)
	{
		m_dlgSettings = new CObserverPageDlg(this);
		ASSERT_VALID( m_dlgSettings );
	}
	
	return m_dlgSettings;
}
