/*
*
*  Copyright (c) 2002-2004 Jakub Schwarzmeier
*
************************************************************/

#include "stdafx.h"

#include <SFC.Graphics.h>

#include <windowsx.h>	//  useful macros like SelectBitmap that makes correct casts
#include <gdiplus.h>

#pragma comment(lib, "opengl32.lib")
#pragma comment(lib, "glu32.lib")
#pragma comment(lib, "gdiplus.lib")

#ifdef _DEBUG
	#pragma comment(lib, "glprocs_debug.lib")
#else
	#pragma comment(lib, "glprocs_release.lib")
#endif

#include "OpenGLStateDlg.h"

#define MAX_PFORMATS 256

using namespace JakubSchwarzmeier::SFC::Graphics;
using namespace Gdiplus;
using namespace JakubSchwarzmeier::Std::Math;

IMPLEMENT_DYNCREATE (COpenGLView, CView)

BEGIN_MESSAGE_MAP(COpenGLView, CView)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_ERASEBKGND()
	ON_WM_KEYDOWN()
	ON_WM_KEYUP()
	ON_WM_MOUSEMOVE()
	ON_WM_RBUTTONUP()
	ON_WM_RBUTTONDOWN()
	ON_WM_SIZE()
END_MESSAGE_MAP()


COpenGLView::COpenGLView(): m_eventRender(TRUE)
{
	m_bFullscreen = false;
	m_bConstBitmap = false;

	m_bTerminateThread = true;
	m_bOnSize = true;
	m_bSetPixelFormat = true;

	//  screen
	m_hDcView = NULL;
	m_hRC = NULL;

	//  painting, DIB section
	m_hDcPaintMem = NULL;
	m_hOriginalPaintDIB = NULL;
	m_hPaintDibSection = NULL;

	m_hDcPBuffer = NULL;

	m_widthOfDIB = 0;
	m_heightOfDIB = 0;

	m_bitsPerPixel = 0;

	AddObserver( new CObserver() );
	m_currentObserver = 0;

	m_quadric = NULL;
	m_lastList = 0;
	m_iFontDisplayListBegin = 0;

	//  available number of lights
	for(int i = 0; i < GL_MAX_LIGHTS; i++)
		m_freeLights[i] = true;
}

COpenGLView::~COpenGLView()
{
}

int COpenGLView::OnCreate( LPCREATESTRUCT lpCreateStruct )
{
	if (__super::OnCreate(lpCreateStruct) == -1)
		return -1;

	//  Create a new DC, it's a memory DC, that is compatible (means device depended) 
	//  with screen. Its display surface is exactly one monochrome pixel wide and 
	//  one monochrome pixel high. Before an application can use a memory DC for drawing 
	//  operations, it must select a bitmap of the correct width and height into the DC. 
	m_hDcPaintMem = CreateCompatibleDC(NULL);
	if (m_hDcPaintMem == FALSE)
	{
		THROW_WINDOWS_EXCEPTION()
	}

	m_hDcView = ::GetDC( this->m_hWnd );
	if (m_hDcView == FALSE)
	{
		THROW_WINDOWS_EXCEPTION()
	}

	//  StartUp GDI+
	Status state = GdiplusStartup( &m_GdiPlusId, &GdiplusStartupInput(), NULL);
	if (state != Ok)
	{
		ATLTRACE2( traceAppMsg, 0, "GDI+ startup error" );
		THROW_WINDOWS_EXCEPTION()
	}

	//  set number of color bits for each pixel
	m_bitsPerPixel = ::GetDeviceCaps( ::GetDC(m_hWnd), BITSPIXEL );

	//  Startup rendering thread
	m_bTerminateThread = false;
	//  create rendering thread
	m_renderingThread = AfxBeginThread(
		RenderingThread,		//  address of function of new thread
		this,					//  argument list
		THREAD_PRIORITY_NORMAL,	//  priority
		0,						//  default stack size
		CREATE_SUSPENDED		//  thread is stopped
		);

	m_renderingThread->m_bAutoDelete = false;
	m_renderingThread->ResumeThread();	

	return 0;
}

 void COpenGLView::OnDestroy( )
{
	__super::OnDestroy();

	//  Terminate rendering thread
	//
	m_bTerminateThread = true;
	ReleaseRenderingThread();
	DWORD ret = ::WaitForSingleObject(m_renderingThread->m_hThread, INFINITE);
	if (ret == WAIT_FAILED)
	{
		THROW_WINDOWS_EXCEPTION()
	}

	delete m_renderingThread;

	//
	//  Terminate rendering thread

	//  if exists quadric object release it
	if (m_quadric != NULL)  gluDeleteQuadric( m_quadric );

	if (m_iFontDisplayListBegin != 0)  glDeleteLists(m_iFontDisplayListBegin, 256);
	
	if (m_bFullscreen)
	{
		ChangeDisplaySettings(NULL, 0);
		ShowCursor(TRUE);
	}

	if ( m_bConstBitmap )
	{
		HGLRC hRcTmp = 	wglCreateContext( m_hDcView );
		BOOL success = wglMakeCurrent(m_hDcView, hRcTmp);

		if (m_hDcPBuffer != NULL )
		{	wglReleasePbufferDCARB( m_hbuffer, m_hDcPBuffer );
			wglDestroyPbufferARB( m_hbuffer );
		}

		if ( m_hDcPaintMem != NULL )  DeleteDC( m_hDcPaintMem );
		if ( m_hDcPBuffer != NULL )  DeleteDC( m_hDcPBuffer );

		if ( m_hOriginalPaintDIB != NULL )
		{
			HBITMAP hbmPrev = SelectBitmap(m_hDcPaintMem, m_hOriginalPaintDIB);
			DeleteObject( hbmPrev );
		}

		wglMakeCurrent(NULL, NULL);
		wglDeleteContext(hRcTmp);
	}

	wglMakeCurrent(NULL, NULL);
	wglDeleteContext(m_hRC);

	if ( m_hDcView != NULL )	DeleteDC( m_hDcView );

	GdiplusShutdown( m_GdiPlusId );
}

void COpenGLView::OnSize( UINT nType, int cx, int cy )
{
	//  This trigger an UI actualization routines, eg. toolbar & statusbar area
	//  repainting. It must be forced this way, because the rendering thread
	//  consumes so much of the processor time, that OnIdle is called too little much.
	UpdateDialogControls( GetParent(), TRUE );
	
	m_bOnSize = true;
	ReleaseRenderingThread();
}

void COpenGLView::SetPixelFormat(int bitsPerPixel)
{
	m_bitsPerPixel = bitsPerPixel;

	m_bSetPixelFormat = true;
	ReleaseRenderingThread();
}

/*static*/ 
UINT COpenGLView::RenderingThread( LPVOID lpParameter )
{
	COpenGLView * openGLView = reinterpret_cast<COpenGLView *>( lpParameter );

	while (true)
	{
		::WaitForSingleObject( openGLView->m_eventRender, INFINITE );
		if (openGLView->m_bTerminateThread)  break;

        openGLView->m_criticalSection.Lock();
		if ( openGLView->m_bOnSize || openGLView->m_bSetPixelFormat )
		{
			CRect rect;
			openGLView->GetClientRect( &rect );

			if (openGLView->m_bConstBitmap)
			{
				//  Create DIB section
				//

				BITMAPINFOHEADER BIH;
								
				//  create a BITMAPINFOHEADER structure to describe the DIB
				ZeroMemory( &BIH, sizeof ( BIH ) );

				//  fill in the header info
				BIH.biSize			= sizeof ( BIH );
				BIH.biWidth			= openGLView->m_widthOfDIB;
				BIH.biHeight		= openGLView->m_heightOfDIB;
				BIH.biPlanes		= 1;
				BIH.biBitCount		= openGLView->m_bitsPerPixel;
				BIH.biCompression	= BI_RGB;

				//  Create the DIB section for painting
				if (openGLView->m_hPaintDibSection != NULL)   
					DeleteObject(openGLView->m_hPaintDibSection);
				
				openGLView->m_hPaintDibSection = 
					::CreateDIBSection( openGLView->m_hDcPaintMem,
										(BITMAPINFO*)&BIH,
										DIB_PAL_COLORS,
										&openGLView->m_pBits,
										NULL,
										0);

				if (openGLView->m_hPaintDibSection == NULL)
				{
					THROW_WINDOWS_EXCEPTION()
				}

				//  Select the new bitmap -- DIB section -- into the painting memory DC.
				if ( openGLView->m_hOriginalPaintDIB == NULL )
				{
					openGLView->m_hOriginalPaintDIB = 
						SelectBitmap(openGLView->m_hDcPaintMem, openGLView->m_hPaintDibSection);
				}
				else
				{
					HBITMAP hPrevDibSection = 
						SelectBitmap(openGLView->m_hDcPaintMem, openGLView->m_hPaintDibSection);
					DeleteObject( hPrevDibSection );
				}


				//
				//  Create DIB section
			}
			else
			{
				openGLView->m_widthOfDIB = rect.Width();
				openGLView->m_heightOfDIB = rect.Height();
			}

			openGLView->m_bOnSize = false;

			if (openGLView->m_bSetPixelFormat)
			{
				//  Set the pixel format for the screen DC
				//

				PIXELFORMATDESCRIPTOR pfd;
				ZeroMemory( &pfd, sizeof ( pfd ) );

				pfd.nSize		= sizeof ( pfd );
				pfd.nVersion	= 1;
				pfd.dwFlags		= PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL;
				pfd.iPixelType	= PFD_TYPE_RGBA;
				pfd.cColorBits	= openGLView->m_bitsPerPixel;
				pfd.cDepthBits	= 24;			//  count of bits per pixel for depth (z) buffer
				pfd.iLayerType	= PFD_MAIN_PLANE;

				if (!openGLView->m_bConstBitmap)	pfd.dwFlags |= PFD_DOUBLEBUFFER;

				int iFormat = ChoosePixelFormat( openGLView->m_hDcView, &pfd );
				if (iFormat == 0)
				{
					THROW_WINDOWS_EXCEPTION()
				}
				//  painting
				BOOL success = ::SetPixelFormat( openGLView->m_hDcView, iFormat, &pfd );
				if (!success)
				{
					THROW_WINDOWS_EXCEPTION()
				}


				//  We will create new rendering context (RC), so delete the old one.
				if (openGLView->m_hRC != NULL)
				{
					BOOL success = wglDeleteContext( openGLView->m_hRC );
					if (success != TRUE)
					{
						THROW_WINDOWS_EXCEPTION()
					}
				}

				//  Create the rendering context that has the same pixel format as 
				//  the device context.
				openGLView->m_hRC = wglCreateContext( openGLView->m_hDcView );
				if (openGLView->m_hRC == NULL)
				{
					THROW_WINDOWS_EXCEPTION()
				}

				if (openGLView->m_bConstBitmap)
				{
					pfd.dwFlags		= PFD_DRAW_TO_BITMAP | PFD_SUPPORT_OPENGL;
					iFormat = ChoosePixelFormat( openGLView->m_hDcPaintMem, &pfd );
					if (iFormat == 0)
					{
						THROW_WINDOWS_EXCEPTION()
					}

					success = ::SetPixelFormat( openGLView->m_hDcPaintMem, iFormat, &pfd );
					if (!success)
					{
						THROW_WINDOWS_EXCEPTION()
					}

				/*
					PIXELFORMATDESCRIPTOR pfd_new;
					DescribePixelFormat (openGLView->m_hDcView, iFormat, sizeof(PIXELFORMATDESCRIPTOR), &pfd_new);

					int generic_format = pfd_new.dwFlags & PFD_GENERIC_FORMAT;
					int generic_accelerated = pfd_new.dwFlags & PFD_GENERIC_ACCELERATED;

					if (generic_format && ! generic_accelerated)
					{
					// software
						int x = 0;
					}
					else if (generic_format && generic_accelerated)
					{
					// hardware - MCD
						int x = 0;
					}
					else if (! generic_format && ! generic_accelerated)
					{
					// hardware - ICD
						int x = 0;
					}
				*/

					success = wglMakeCurrent( openGLView->m_hDcView, openGLView->m_hRC );
					if (success != TRUE)
					{
						THROW_WINDOWS_EXCEPTION()
					}

					//  Attribute arrays must be "0" terminated
					int ifmtList[] = {
						WGL_DRAW_TO_PBUFFER_ARB,	GL_TRUE,  //  enable render to pbuffer
						WGL_SUPPORT_OPENGL_ARB,		GL_TRUE,  //  automatic for NVIDIA for compatibility issues
						WGL_ACCELERATION_ARB,		WGL_FULL_ACCELERATION_ARB,
						WGL_DEPTH_BITS_ARB,         24,	//  the depth of the depth (z-axis) buffer; not working for 32
						WGL_COLOR_BITS_ARB,         openGLView->m_bitsPerPixel,	
						WGL_ALPHA_BITS_ARB,         8,
						WGL_STENCIL_BITS_ARB,       0,
						WGL_DOUBLE_BUFFER_ARB,		GL_FALSE,	//  we dont require double buffering
						0										//  zero terminates the list
					};


					int pformats[MAX_PFORMATS];
					UINT nformats;

					//  Ask the hardware to find pixel format defined in ifmtList.
					//  Note: Before this step must be set valid current 
					//  rendering context.
					BOOL ret = wglChoosePixelFormatARB( openGLView->m_hDcView, 
						ifmtList, NULL, MAX_PFORMATS, pformats, &nformats );

					if ( (ret == 0) || (nformats < 1) )
					{
						THROW_WINDOWS_EXCEPTION()
					}

					//  Create pixel buffer (pbuffer) based on format.
					int iattributes[1] = {0};

					openGLView->m_hbuffer =	
						wglCreatePbufferARB( openGLView->m_hDcView,	pformats[0], 
						openGLView->m_widthOfDIB, openGLView->m_heightOfDIB, iattributes );

					if (openGLView->m_hbuffer == NULL)
					{
						THROW_WINDOWS_EXCEPTION_CODE(::GetLastError() & 0xFFFF)
					}

					//  Create a device context for the newly created pbuffer.
					openGLView->m_hDcPBuffer = wglGetPbufferDCARB( openGLView->m_hbuffer );
					if (openGLView->m_hDcPBuffer == NULL)
					{
						THROW_WINDOWS_EXCEPTION()
					}

					//  Create an OpenGL rendering context and associate it with 
					//  the handle for the pbuffer.
					HGLRC pbufglctx = wglCreateContext( openGLView->m_hDcPBuffer );
					if (pbufglctx == NULL)
					{
						THROW_WINDOWS_EXCEPTION()
					}

					//  Make pbuffer's OpenGL rendering context the current context.
					wglMakeCurrent( openGLView->m_hDcPBuffer, pbufglctx );

					//  Delete screen rendering context.
					wglDeleteContext( openGLView->m_hRC );

					//  save new context
					openGLView->m_hRC = pbufglctx;

					wglMakeCurrent( NULL, NULL );
				}

				openGLView->m_bSetPixelFormat = false;
			
			}	//  end if (m_bSetPixelFormat)

		} //  end if (openGLView->m_bOnSize || m_bSetPixelFormat)

		HDC hDC;
		if (openGLView->m_bConstBitmap)	hDC = openGLView->m_hDcPBuffer;
		else
			hDC = openGLView->m_hDcView;

		BOOL success = wglMakeCurrent( hDC, openGLView->m_hRC );
		if (success != TRUE)
		{
			THROW_WINDOWS_EXCEPTION()
		}

		//  User's rendering
		openGLView->PreRender(openGLView);
		openGLView->OnRender(openGLView);

		if (openGLView->m_bConstBitmap)
		{
			glReadPixels(0, 0, openGLView->m_widthOfDIB, openGLView->m_heightOfDIB,
				GL_RGBA, GL_UNSIGNED_BYTE, openGLView->m_pBits);

			if (success != TRUE)
			{
				THROW_WINDOWS_EXCEPTION()
			}
	

		}

		wglMakeCurrent( NULL, NULL );

		openGLView->m_criticalSection.Unlock();

		//  Force redrawing
		openGLView->Invalidate();
	}

	return 0;
}

BOOL COpenGLView::OnEraseBkgnd( CDC* pDC )
{
	return FALSE;
}
/*
BOOL COpenGLView::Create(LPCTSTR lpszClassName, LPCTSTR lpszWindowName,
	DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext)
{
		// register window class
		CString m_strWindowClass = AfxRegisterWndClass(CS_OWNDC);

		return __super::Create(m_strWindowClass, lpszWindowName, dwStyle, rect, pParentWnd, nID, pContext);
}
*/
BOOL COpenGLView::PreCreateWindow(CREATESTRUCT& cs) 
{
	// An OpenGL window must be created with the following flags and must not
    // include CS_PARENTDC for the class style. Refer to SetPixelFormat
    // documentation in the "Remarks" section for further information.
    cs.style |= WS_CLIPSIBLINGS | WS_CLIPCHILDREN;

	return CView::PreCreateWindow(cs);
}

/*
void COpenGLView::SetObserver( Observer * observer )
{
	m_observer = observer;
}

Observer * COpenGLView::GetObserver()
{
	return m_observer;
}
*/
/*virtual*/
void COpenGLView::PreRender(COpenGLView * openGLView)
{
	//  called if window is resized

	//  global properties

	glClearColor ( 0.0, 0.0, 0.0, 0.0 );

	//  set depth (z-buffer) attributes
	glEnable(GL_DEPTH_TEST);			//  glClear must then have set GL_DEPTH_BUFFER_BIT flag, otherwise it cause displaying errors!!!
	glDepthFunc(GL_LEQUAL);
	glClearDepth(1.0f);	//  filled with a value that's as far from the viewpoint as possible, so any object is nearer than that

	//  performance and quality attributes
	glShadeModel(GL_SMOOTH);		// make the shading smooth
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	glEnable(GL_POINT_SMOOTH);
	glEnable(GL_CULL_FACE);

	//  projection properties

	glViewport(0, 0, m_widthOfDIB, m_heightOfDIB);		//  set transformation from device to window coordinates

	glMatrixMode(GL_PROJECTION); 
	glLoadIdentity();

	m_glAspect = static_cast<GLfloat>(m_widthOfDIB)/static_cast<GLfloat>(m_heightOfDIB);

	//  point of this is that eye is every time in 0,0,0 
	//  glFrustum (-glAspect, glAspect, -1, 1, 1.0, 5000.0);
	//  multiply projection matrix 
	gluPerspective(80, m_glAspect, 1.0, 5000.0);

	glMatrixMode   ( GL_MODELVIEW );
}

void COpenGLView::OnRender(COpenGLView * openGLView)
{
	//  clear the color and depth buffers 
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); 

	//  modeling transformation in ModelView matrix

	glLoadIdentity();

	//  simulate viewing transformation in ModelView matrix
	glMultMatrixd( m_observers[m_currentObserver]->GetMatrix() );

	//  objects drawing

	glBegin( GL_QUADS );
		glColor3f( 1.0f, 0.0f, 0.0f ); glVertex2f( 1, 1 );
		glColor3f( 0.0f, 1.0f, 0.0f ); glVertex2f( -1, 1 );
		glColor3f( 0.0f, 0.0f, 1.0f ); glVertex2f( -1, -1 );
		glColor3f( 0.0f, 1.0f, 0.0f ); glVertex2f( 1, -1 );

		glColor3f( 1.0f, 0.0f, 0.0f ); glVertex2f( 1, 1 );
		glColor3f( 0.0f, 1.0f, 0.0f ); glVertex2f( 1, -1 );
		glColor3f( 0.0f, 0.0f, 1.0f ); glVertex2f( -1, -1 );
		glColor3f( 0.0f, 1.0f, 0.0f ); glVertex2f( -1, 1 );
	glEnd();

	glBegin( GL_LINES );
		glColor3f( 1.0f, 1.0f, 1.0f );
	for(int i = -40; i <= 40; i+=2)
	{
		//  perpendicular
		glVertex3f( 100, 0, i );	glVertex3f( -100, 0, i );
		glVertex3f( i, 0, 100 );	glVertex3f( i, 0, -100 );

		//  z
		glVertex3f( -15, -100, i );	glVertex3f( -15, 100, i );
		glVertex3f( -15, i, -100 );	glVertex3f( -15, i, 100 );
	}
		glEnd();

	//  y-axis
	glBegin( GL_LINES );
		glVertex3f( 0, 100, 0 );	glVertex3f( 0, -100, 0 );
	glEnd();


	//  flush rendering pipe
	glFinish();
}

void COpenGLView::OnDraw( CDC* pDC )
{
	//  bitblt from painting memory DC to screen DC
    m_criticalSection.Lock();
	ASSERT( pDC->m_hDC );
	ASSERT( m_hDcPaintMem );

	if (m_bConstBitmap)
	{
		RECT rect;
		GetClientRect( &rect );

		pDC->SetStretchBltMode( STRETCH_HALFTONE );
		BOOL success = StretchBlt( /*target DC*/pDC->m_hDC, 0, 0, rect.right, rect.bottom,
			/*source DC*/m_hDcPaintMem, 0, 0, m_widthOfDIB, m_heightOfDIB, SRCCOPY);
/*
		success = BitBlt( pDC->m_hDC, 0, 0, m_widthOfDIB, m_heightOfDIB, 
			m_hDcPaintMem, 0, 0, SRCCOPY);
*/
		if (!success)
		{
			THROW_WINDOWS_EXCEPTION()
		}
	}
	else
	{
		::wglMakeCurrent( m_hDcView, m_hRC );
		::SwapBuffers( m_hDcView );
		::wglMakeCurrent( NULL, NULL );
	}

    m_criticalSection.Unlock();

}

void COpenGLView::ReleaseRenderingThread()
{
	m_eventRender.SetEvent();
}

void COpenGLView::AddObserver( CObserver * observer )
{
	ASSERT_VALID( observer );
	CAutoPtr<CObserver> aObserver( observer );
	m_observers.Add( aObserver );
	observer->SetView( this );
}

void COpenGLView::SwitchObserver( size_t no )
{
	if ( no < m_observers.GetCount() )
	{
		m_currentObserver = no;
	}
}

size_t COpenGLView::GetCurrentObserver()
{
	return m_currentObserver;
}

const CAutoPtrArray< CObserver > * COpenGLView::GetObservers()
{
	return &m_observers;
}

void COpenGLView::SetConstBitmap( bool constBitmap, int width, int height )
{
	m_bConstBitmap = constBitmap;
	m_widthOfDIB = width;
	m_heightOfDIB = height;
}

GLUquadricObj * COpenGLView::GetQuadric()
{
	if ( m_quadric == NULL )  m_quadric = gluNewQuadric();

	return m_quadric;
}

CString COpenGLView::GetLastError()
{
	return CString( reinterpret_cast<const char *>( gluErrorString(glGetError()) ) );
}

int COpenGLView::ReserveLight()
{
	for(int i = GL_LIGHT0; i < GL_LIGHT0 + GL_MAX_LIGHTS; i++)
	{
		if (m_freeLights[i-GL_LIGHT0])
		{
			m_freeLights[i-GL_LIGHT0] = false;
			
			return i;
		}
	}

	return -1;
}

void COpenGLView::DisposeLight(int light)
{
	m_freeLights[light-GL_LIGHT0] = true;
}

GLuint COpenGLView::GetList()
{
	return ++m_lastList;
}

GLuint COpenGLView::LoadTexture(const TCHAR * fileName)
{
	GLuint textureID;

	//  OpenGL 
	glGenTextures(1, &textureID);
	glBindTexture(GL_TEXTURE_2D, textureID);

	//  GDI+ intermezzo
	Bitmap image( fileName, FALSE );

	int width = image.GetWidth();
	int height = image.GetHeight();
	
	//  GDI+ returns color bytes in BGR order
	BitmapData bitmap;
	Rect rect(0, 0, width, height);
	image.LockBits( &rect, ImageLockModeRead, PixelFormat24bppRGB, &bitmap );

	//  OpenGL cont'd

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );	//  if GL_DECAL, it don't respect lighting

	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

	//  fastest (in meaning of speed) is to use glTextImage2D
	//  then is usable to use textures with power of two and gluBuild2DMipmaps
	//  and slowest is to use textures with no power of two and gluBuild2DMipmaps

	//  glTexImage2D requires textures in power of two; gluBuild2DMipmaps rescale size if needed
	//  glTexImage2D(GL_TEXTURE_2D, 0, 3, width, height, 0, GL_BGR_EXT, GL_UNSIGNED_BYTE,  bitmap.Scan0);

	gluBuild2DMipmaps(GL_TEXTURE_2D, 3, width, height, GL_BGR_EXT, GL_UNSIGNED_BYTE, bitmap.Scan0);

	image.UnlockBits( &bitmap );

	return textureID;
}

void COpenGLView::DrawText(const TCHAR * szText, const TCHAR * szFont, int size)
{
	if (m_iFontDisplayListBegin == 0)
	{
		//  Begining in the OpenGL's display list is at 1000.
		m_iFontDisplayListBegin = 1000;

		HFONT hOldFont;

		if (szFont != NULL)
		{
			LOGFONT     lf;

			//  Let's create a TrueType font to display.
			ZeroMemory( &lf, sizeof(LOGFONT) );
			lf.lfHeight               =   -14;
			lf.lfWeight               =   FW_SEMIBOLD;
			lf.lfCharSet              =   DEFAULT_CHARSET;
			lf.lfOutPrecision         =   OUT_DEFAULT_PRECIS;
			lf.lfClipPrecision        =   CLIP_DEFAULT_PRECIS;
			lf.lfQuality              =   CLEARTYPE_QUALITY;
			lf.lfPitchAndFamily       =   FF_DONTCARE|DEFAULT_PITCH;
			lstrcpy(lf.lfFaceName, szFont);

			HFONT hFont = CreateFontIndirect(&lf);
			hOldFont = SelectFont(m_hDcPaintMem, hFont);
		}
		else
		{
			HGDIOBJ x = SelectObject (m_hDcPaintMem, GetStockObject (SYSTEM_FONT)); 
		}
		//  Create a set of display lists based on the TT font we selected.
		//  The other function that we may use was wglUseFontBitmaps.
//		BOOL x = wglUseFontOutlines(m_hRenderMemDC, 0, 256, m_iFontDisplayListBegin , 0.0f, 0.0f,
//			WGL_FONT_POLYGONS, NULL);

		BOOL x = wglUseFontBitmaps(m_hDcPaintMem, 0, 1024, m_iFontDisplayListBegin);
		if (szFont != NULL)	DeleteObject( SelectObject(m_hDcPaintMem, hOldFont) );
	}
		
	glPushAttrib(GL_LIST_BIT);

	  glListBase(m_iFontDisplayListBegin); // indicate start of display lists
#ifdef _UNICODE
	  glCallLists( lstrlen( szText ), GL_UNSIGNED_SHORT, szText );
#else
	  glCallLists( lstrlen( szText ), GL_UNSIGNED_BYTE, szText );
#endif

	glPopAttrib();
}

GLdouble COpenGLView::GetAspect()
{

	return m_glAspect;
}

/*virtual*/ void COpenGLView::ShowStateVariables()
{
	m_criticalSection.Lock();

	COpenGLStateDlg dlg(this);
	dlg.DoModal();

	m_criticalSection.Unlock();
}

void COpenGLView::OnKeyDown( UINT nChar, UINT nRepCnt, UINT nFlags )
{
	switch( nChar )
	{
	case VK_F1:

		ShowStateVariables();
		break;
	}

	m_observers[m_currentObserver]->OnKeyDown(nChar, nRepCnt, nFlags);
}

void COpenGLView::OnKeyUp( UINT nChar, UINT nRepCnt, UINT nFlags )
{
	m_observers[m_currentObserver]->OnKeyUp(nChar, nRepCnt, nFlags);
}

void COpenGLView::OnRButtonDown( UINT nFlags, CPoint point )
{
	m_observers[m_currentObserver]->OnRButtonDown(nFlags, point.x, point.y);
}

void COpenGLView::OnRButtonUp( UINT nFlags, CPoint point )
{
	m_observers[m_currentObserver]->OnRButtonUp(nFlags, point.x, point.y);
}

void COpenGLView::OnMouseMove( UINT nFlags, CPoint point )
{
	m_observers[m_currentObserver]->OnMouseMove(nFlags, point.x, point.y);
}