/************************************************************
*
*  Copyright (c) 2002-2004 Jakub Schwarzmeier
*
************************************************************/

#pragma once

#include <gl/gl.h>		//  OpenGL header
#include <gl/glu.h>		//  GL utility header
#include <gl/glprocs.h>		//  Intel's static library containing wrapper functions for extensions

#include "OpenGLStateDlg.h"

namespace JakubSchwarzmeier
{
	namespace SFC
	{
		namespace Graphics
		{
			class AFX_EXT_CLASS COpenGLView: public ::CView
			{
			public:
				COpenGLView();
				~COpenGLView();

			//  implementations
				virtual BOOL PreCreateWindow(CREATESTRUCT& cs);

				HBITMAP GetDibSection();
				void SetPixelFormat(int bitsPerPixel);
				void SetConstBitmap( bool constBitmap, int width = 0, int height = 0 );

				void AddObserver( CObserver * observer );
				void SwitchObserver( size_t no );
				size_t GetCurrentObserver();
				const CAutoPtrArray< CObserver > * GetObservers();

				int ReserveLight();
				void DisposeLight(int light);

				GLuint GetList();

				GLUquadricObj * GetQuadric();
				CString GetLastError();

				GLuint LoadTexture(const TCHAR * fileName);
				void DrawText(const TCHAR * szText, const TCHAR * szFont = NULL, 
					int size = -20);

				GLdouble GetAspect();

				void ReleaseRenderingThread();

			protected:
			//  overrides
				virtual void OnDraw (CDC* pDC);

			//  implementations
				virtual void OnRender(COpenGLView * openGLView);
				virtual void PreRender(COpenGLView * openGLView);
				
				virtual void ShowStateVariables();

				static UINT RenderingThread( LPVOID lpParameter );

			//  message handlers
				afx_msg int OnCreate( LPCREATESTRUCT lpCreateStruct );
				afx_msg void OnDestroy();
				afx_msg void OnSize( UINT nType, int cx, int cy );
				afx_msg BOOL OnEraseBkgnd( CDC* pDC );

				afx_msg void OnKeyDown( UINT nChar, UINT nRepCnt, UINT nFlags );
				afx_msg void OnKeyUp( UINT nChar, UINT nRepCnt, UINT nFlags );
				afx_msg void OnRButtonDown( UINT nFlags, CPoint point );
				afx_msg void OnRButtonUp( UINT nFlags, CPoint point );
				afx_msg void OnMouseMove( UINT nFlags, CPoint point );

			//  data fields
				bool m_bFullscreen;
				bool m_bConstBitmap;

				bool m_bTerminateThread;
				CWinThread * m_renderingThread;
				::CEvent m_eventRender;
				bool m_bOnSize, m_bSetPixelFormat;
				::CCriticalSection m_criticalSection;

				HDC m_hDcView;	//  View's DC
				HGLRC m_hRC;		//  view or pbuffer

				//  painting fields
				HDC		m_hDcPaintMem;	
				HBITMAP m_hOriginalPaintDIB;
				HBITMAP m_hPaintDibSection;

				//  pbuffer
				HDC		m_hDcPBuffer;	
				HPBUFFERARB m_hbuffer;

				int		m_widthOfDIB, m_heightOfDIB;

				ULONG	m_GdiPlusId;
				int		m_bitsPerPixel;

				void* m_pBits;

				CAutoPtrArray< CObserver > m_observers;
				size_t m_currentObserver;
				GLUquadricObj * m_quadric;
				bool m_freeLights[GL_MAX_LIGHTS];
				GLuint m_lastList;
				int m_iFontDisplayListBegin;

				GLdouble m_glAspect;

				friend CObserver;
				friend COpenGLStateDlg;

				DECLARE_MESSAGE_MAP()
				DECLARE_DYNCREATE (COpenGLView)
			
			};	//  end class COpenGLView
		
		}	//  end namespace Graphics
	
	}	//  end namespace SFC

}	//  end namespace JakubSchwarzmeier

