#include "stdafx.h"

#include "MFC.Net.h"

#include <atlcoll.h>

using namespace JakubSchwarzmeier::MFC::Net::Sockets;

CHttpProcessor::CHttpProcessor(): m_unfinishedBuffer(true)
{
}

bool CHttpProcessor::GetLine(const char buffer[], 
							 const DWORD dwSize, 
							 DWORD & dwNumberOfChars)
{
	for(DWORD i = 0; i < dwSize-1; i++)
	{
		if ( ChrCmpIA( buffer[i], '\r' ) == 0 && ChrCmpIA( buffer[i+1], '\n' ) == 0 )
		{
			dwNumberOfChars = i;
			return true;
		}
	}

	return false;
}

bool CHttpProcessor::GetSpace(const char buffer[], 
		const DWORD dwSize, 
		DWORD & dwNumberOfChars)
{
	for(DWORD i = 0; i < dwSize; i++)
	{
		if ( isspace( static_cast<unsigned char>(buffer[i]) ) != 0 )
		{
			dwNumberOfChars = i;
			return true;
		}
	}

	return false;
}

bool CHttpProcessor::HandleRequest(const char n_buffer[], const DWORD n_dwSize, 
						DWORD & n_dwPosition, char ** outData, DWORD & dwOutDataSize)
{
	char * buffer;				//  input stream
	DWORD dwSize;				//	
	DWORD dwLastOccurence = 0;	//	last used position in input buffer

	//  Are there some data from the last call?
	if ( (dwSize = m_unfinishedBuffer.GetLength()) == 0 )
	{
		dwSize = n_dwSize;	
		buffer = const_cast<char*>(n_buffer);

		//  no data from last call
		int isasci = _istascii( buffer[n_dwPosition] );
		//  Is it ascii? If it's so, we believe that's HTTP!
		if (isasci <= 0)  return false;
	}
	else
	{
		//  yes!, unfinished buffer
		m_unfinishedBuffer.Append( &n_buffer[n_dwPosition], n_dwSize );	//  append new data
		dwSize += n_dwSize;
		buffer = m_unfinishedBuffer.GetBuffer();
	}

	//  Here are stored headers other then Content-Length.
	CAtlMap<CStringA, CStringA, CStringAHashTraits, CStringAHashTraits> headerMap;
	CStringA strHeader, strContent;			//  This is tmp buffer for each
											//  header/content string.

	//  Now parse STATUS LINE. It should be in these forms (eg.):
	//
	//  (1) request: POST /Example1/Service1.asmx HTTP/1.0
	//  (2) response: HTTP/1.1 100 Continue
	//  Both of them are formed from three parts. Moreover, If the status line 
	//  is valid, it must contain \r\n at the end.

	DWORD dwNoOfCharsOnLine = 0;
	bool ret = GetLine( buffer, dwSize, dwNoOfCharsOnLine );

	if (!ret)
	{
		//  copy buffer	
		if (m_unfinishedBuffer.GetLength() == 0)
			m_unfinishedBuffer.Append( &n_buffer[n_dwPosition], n_dwSize );

		return true;
	}

	DWORD dwNoOfCharsToSpace = 0;
	CStringA strStatus1, strStatus2, strStatus3;		//  holds those three parts

	//  first token
	ret = GetSpace( &buffer[dwLastOccurence], dwSize-dwLastOccurence, dwNoOfCharsToSpace );
	if (ret)
	{
		StrCpyNA( strStatus1.GetBuffer(dwNoOfCharsToSpace+1), &buffer[dwLastOccurence], dwNoOfCharsToSpace+1 );	 // +1 for trailing \0
		strStatus1.ReleaseBuffer();
		dwLastOccurence += dwNoOfCharsToSpace+1;		
		if (dwLastOccurence < dwSize)
		{
			//  second token
			ret = GetSpace( &buffer[dwLastOccurence], dwSize-dwLastOccurence, dwNoOfCharsToSpace );
			if (ret)
			{
				StrCpyNA( strStatus2.GetBuffer(dwNoOfCharsToSpace+1), &buffer[dwLastOccurence], dwNoOfCharsToSpace+1 );
				strStatus2.ReleaseBuffer();
				dwLastOccurence += dwNoOfCharsToSpace+1;
				if (dwLastOccurence < dwNoOfCharsOnLine)
				{
					//  third token till the end of line
					StrCpyNA( strStatus3.GetBuffer(dwNoOfCharsOnLine-dwLastOccurence+1), &buffer[dwLastOccurence], dwNoOfCharsOnLine-dwLastOccurence+1 ); // +1 is \0
					strStatus3.ReleaseBuffer();
					dwLastOccurence += dwNoOfCharsOnLine - dwLastOccurence + 2;

					//  First or third part must contain HTTP string like HTTP/1.1
					if ((strStatus1.Find( m_szcHTTP ) == -1)
						&& (strStatus3.Find( m_szcHTTP ) == -1) )
					{
						ret = false;
					}
				}
				else
					ret = false;
			}
		}
		else
			ret = false;
	}

	if (!ret)
	{
		//  not found
		//  This is not an HTTP message. Bad Content-Length?
		CHttpResponse response;
		response.Initialize( m_strHTTP11, 
			CStringA("400"), 
			m_strBadRequest,
			headerMap, 
			0, 
			&buffer[dwLastOccurence], 
			&dwOutDataSize, 
			outData );
		response.Create();

		goto finish;
	}

	//  Now parse HEADER BLOCK. It's form is in form: eg. 
	//
	//  Content-Type: text/xml; charset=utf-8\r\n
	//  And look for the end of the header (the \r\n\r\n).

	unsigned contentLength = 0;				//  This can be zero even when 
											//  the item is found.
	//  Sometimes (eg. HTTP/1.1 100 Continue) isn't Content-Length header present.
	//  So it's critical to look after the \r\n\r\n. 
	bool bEndOfHeaders = false;
	
	while (dwLastOccurence < dwSize)
	{
		ret = GetLine( &buffer[dwLastOccurence], dwSize-dwLastOccurence, dwNoOfCharsOnLine );
		if (ret)
		{
			if (dwNoOfCharsOnLine == 0)
			{
				//  end of headers, \r\n\r\n occured
				dwLastOccurence += 2;
				bEndOfHeaders = true;
				break;
			}
			
			//  first token, we don't have trailing ':'
			GetSpace( &buffer[dwLastOccurence], dwSize-dwLastOccurence, dwNoOfCharsToSpace );
			StrCpyNA( strHeader.GetBuffer(dwNoOfCharsToSpace), &buffer[dwLastOccurence], dwNoOfCharsToSpace );
			strHeader.ReleaseBuffer();
			dwLastOccurence += dwNoOfCharsToSpace + 1;

			//  second token, till the end of line
			StrCpyNA( strContent.GetBuffer(dwNoOfCharsOnLine-dwNoOfCharsToSpace), &buffer[dwLastOccurence], dwNoOfCharsOnLine-dwNoOfCharsToSpace ); 
			strContent.ReleaseBuffer();
			dwLastOccurence += dwNoOfCharsOnLine - dwNoOfCharsToSpace + 1;

			//  Insert into map
			//  Check whether it is a Content-Length header
			if ( StrCmpIA( strHeader, m_strContentLength ) == 0 )
			{
				//  yes, it is
				contentLength = StrToIntA( strContent );
			}
			else
			{
				//  no it isn't, so insert header/content pair into map
				headerMap.SetAt( strHeader, strContent );
			}
		}
		else
		{
			//  copy buffer	
			if (m_unfinishedBuffer.GetLength() == 0)
				m_unfinishedBuffer.Append( &n_buffer[n_dwPosition], n_dwSize );

			return true;
		}
	}

	//  Note Content-Length gives minimal no. of bytes that HTTP message carries.

	//  Contains buffer content of HTTP message?
	if (( (dwSize-dwLastOccurence) >= contentLength ) && (bEndOfHeaders) )
	{
		//  Whole message is here. Should I construct CResponse or CRequest? Status lines:
		//  CResponse: HTTP/1.1 200 OK
		//  CRequest:  POST /Example1/Service1.asmx HTTP/1.0

		//  So if second part is a number, then i will suppose, that's HTTP RESPONSE.
		if ( isdigit( strStatus2[0] ) != 0 )
		{
			//  construct CHttpResponse
			CHttpResponse response;
			response.Initialize( strStatus1, 
				strStatus2, 
				strStatus3,
				headerMap, 
				contentLength, 
				&buffer[dwLastOccurence], 
				&dwOutDataSize, 
				outData );
			response.Process();
		}
		else
		{
			//  construct CHttpRequest
			CHttpRequest request;
			request.Initialize( strStatus1, 
				strStatus2, 
				strStatus3,
				headerMap, 
				contentLength, 
				&buffer[dwLastOccurence], 
				&dwOutDataSize, 
				outData );
			request.Process();
		}

		dwLastOccurence += contentLength;	//  no +1 because THIS IS length
	}
	else
	{
		//  copy buffer	
		if (m_unfinishedBuffer.GetLength() == 0)
			m_unfinishedBuffer.Append( &n_buffer[n_dwPosition], n_dwSize );

		return true;
	}

finish:
	//  Return correct n_dwPosition:
	if (m_unfinishedBuffer.GetLength() == 0)
		n_dwPosition = dwLastOccurence;
	else
	{
		n_dwPosition = dwLastOccurence - (m_unfinishedBuffer.GetLength() - n_dwSize);
		m_unfinishedBuffer.Free();
	}

	return false;
}

const char CHttpProcessor::m_szcHTTP[] = "HTTP";
