#include "stdafx.h"

#include <MFC.Net.h>

using namespace JakubSchwarzmeier::MFC::Net::Sockets;

JakubSchwarzmeier::MFC::Net::Sockets::CHttpRequest::CHttpRequest()
{
	CoInitialize(NULL);
}

JakubSchwarzmeier::MFC::Net::Sockets::CHttpRequest::~CHttpRequest()
{
	CoUninitialize();
}

bool JakubSchwarzmeier::MFC::Net::Sockets::CHttpRequest::GetWebServiceFromUrl(
	CStringA & strURL,
    CStringA & strServiceName)
{
	//  Extract from URL in status line name of webservice class.
	//  e.g. http://www.edie.com:81/...?Param1=Value1&Handler=Name... and returns Name
	//  http ... scheme
	//  www.edie.com ... host name
	//  81 ... port
	//  Param1=Value1&Handler=Name ... query string

	int p = strURL.Find( m_strHandler );
	if (p != -1)
	{
		int charsToCopy = strURL.GetLength()-p-m_strHandler.GetLength()+1;
		StrCpyNA( strServiceName.GetBuffer(charsToCopy), static_cast<LPCSTR>(strURL)+p+m_strHandler.GetLength(), charsToCopy );
		strServiceName.ReleaseBuffer();

		return true;
	}

	return false;
}

void JakubSchwarzmeier::MFC::Net::Sockets::CHttpRequest::GetWebMethodFromSOAPAction(
	CStringA & strSoapActionA, CStringW & strSoapActionW)
{
	int sp = strSoapActionA.ReverseFind('\\');
	if (sp == -1)	sp = strSoapActionA.ReverseFind('#');
	if (sp == -1)	sp = 0;

	int ep;
	for(ep = sp+1; ep < strSoapActionA.GetLength(); ep++)
	{
		if ( __iscsym( strSoapActionA[ep] ) == 0 )	break;
		strSoapActionW += strSoapActionA[ep];
	}
	
	return;
}
//  Returns true if query part of URL contains four characters "WSDL".
//
bool JakubSchwarzmeier::MFC::Net::Sockets::CHttpRequest::IsWSDLRequest(CStringA & strURL)
{
	char szWSDL[5];

	DWORD dwSize = sizeof(szWSDL);
	int i = UrlGetPartA(strURL, szWSDL, &dwSize, URL_PART_QUERY, 0);
	if (i == 0)
	{
		if ( m_strWSDL.CompareNoCase( szWSDL ) == 0 )
			return true;
	}
	
	return false;
}

bool JakubSchwarzmeier::MFC::Net::Sockets::CHttpRequest::ConvertURLToLocal(CStringA & strURL, CStringA & strReqFileName)
{
	TCHAR szPath[MAX_PATH];
	DWORD ret = GetModuleFileName( AfxGetApp()->m_hInstance, szPath, sizeof(szPath)/sizeof(TCHAR) );
	if (ret == 0)
	{
		//  failure
	}

	BOOL removed = PathRemoveFileSpec( szPath );
	if (removed != FALSE)
	{
		for(int i = 0; i < strURL.GetLength(); i++)
		{
			TCHAR chActual = strURL[i];
			if ( ChrCmpI( chActual, '\?' ) != 0 )
			{
				if ( ChrCmpI( chActual, '/' ) == 0 )
				{
					TCHAR szBackSlash[] = _T("\\");
					StrNCat( szPath, szBackSlash, 2 );
				}
				else
				{
					StrNCat( szPath, &chActual, 2 );
				}
			}
			else
			{
				break;
			}
		}

		BOOL fileExists = PathFileExists( szPath );
		if (fileExists == TRUE)
		{
            strReqFileName = szPath;
			return true;
		}
	}

	return false;
}

void JakubSchwarzmeier::MFC::Net::Sockets::CHttpRequest::Process()
{
	//  TODO: nejakej HTTP code, podle kteryho se nakonec vygeneruje HTTP hlavka

	CStringA strSoapActionA;
	bool isSOAPAction = m_headerMap.Lookup(m_strSoapAction, strSoapActionA);

	bool isWSDLRequest = IsWSDLRequest(m_s2);

	//  Destroys inner buffer after going out of scope.
	JakubSchwarzmeier::MFC::System::IO::CBuffer outputHTTPContentBuffer(true);

	//  Convert /something/something.dll?bo into C:\MySuperApp\something\something.dll
	CStringA strRequestedFileName;
	bool bFileExist = ConvertURLToLocal(m_s2, strRequestedFileName);

	if (bFileExist)
	{
		if (isSOAPAction || isWSDLRequest)
		{
			//  SOAP request

			//  Try to load specified file (DLL) with executable code.
			bool isDLLLoaded = true;
			bool bUnloadDLL = true;

			//  Check whether is DLL loaded.
			if ( GetModuleHandle(strRequestedFileName) != NULL )
			{
				//  DLL already loaded, disable unload
				bUnloadDLL = false;
			}

			HMODULE hModule = LoadLibraryA(strRequestedFileName);

			GETWEBSERVICENAME GetWebServiceName;
			EXECUTEWEBMETHOD ExecuteWebMethod;
			WSDL wsdl;
			if (hModule == NULL)	
			{
				isDLLLoaded = false;
			}
			else
			{
				GetWebServiceName = (GETWEBSERVICENAME)GetProcAddress(hModule, "GetWebServiceName");
				ExecuteWebMethod = (EXECUTEWEBMETHOD)GetProcAddress(hModule, "ExecuteWebMethod");
				wsdl = (WSDL)GetProcAddress(hModule, "WSDL");
				if ( (GetWebServiceName == NULL) || (ExecuteWebMethod == NULL)
					|| (wsdl == NULL) )
				{
					isDLLLoaded = false;
				}
			}

			if (isDLLLoaded && isWSDLRequest)
			{
				wsdl(outputHTTPContentBuffer);
			}

			//  If this is a ATL SOAP Request then Handler query string must be equal
			//  to Web Service name returned from DLL.
			if (isDLLLoaded && !isWSDLRequest)
			{
				CStringA strServiceName;
				bool isATLSOAPQueryString = GetWebServiceFromUrl(m_s2, strServiceName);
				if (isATLSOAPQueryString)
				{
					if ( strServiceName.CompareNoCase( GetWebServiceName() ) != 0 )
					{    	
						isSOAPAction = false;
					}
				}
			}

			if (isDLLLoaded && isSOAPAction && !isWSDLRequest)
			{
				//  Extract web method name from SOAPAction
				CStringW strSoapActionW;
				GetWebMethodFromSOAPAction(strSoapActionA, strSoapActionW);

				JakubSchwarzmeier::MFC::System::IO::CBuffer inputSoapBuffer(
					const_cast<char*>(m_contentBuffer), 
					m_contentLength);
	
				ExecuteWebMethod( 
					inputSoapBuffer, 
					strSoapActionW, 
					outputHTTPContentBuffer );
			}

			if (isDLLLoaded && bUnloadDLL)
			{
				if (FreeLibrary(hModule) == 0)
				{
					//  failure
				}
			}

		}
		else
		{
			//  "ordinary" web request, probably to web server

			//  TODO: find requested file and return it to outputHTTPContentBuffer
		}

	}
	else
	{
		// bFileExist is false, return HTTP error.

	}

	//  Create HTTP response
	//
	JakubSchwarzmeier::MFC::System::IO::CBuffer outputHTPPCompleteResponseBuffer;

	CAtlMap<CStringA, CStringA, 
				CStringAHashTraits, CStringAHashTraits> headerMap;

	DWORD dwOutHttpHeaderLength = 0;
	char * outHttpHeaderBuffer = NULL;

	CHttpResponse response;
	response.Initialize( m_strHTTP11, CStringA("200"),
		m_strOK,
		headerMap,
		outputHTTPContentBuffer.GetLength(),
		outputHTTPContentBuffer.GetBuffer(),
		&dwOutHttpHeaderLength,
		&outHttpHeaderBuffer
		);
	
	response.Create();
	
	outputHTPPCompleteResponseBuffer.Append(
		outHttpHeaderBuffer,
		dwOutHttpHeaderLength
		);

	delete [] outHttpHeaderBuffer;

	outputHTPPCompleteResponseBuffer.Append(
		outputHTTPContentBuffer.GetBuffer(),
		outputHTTPContentBuffer.GetLength()
		);

	*m_outBuffer = outputHTPPCompleteResponseBuffer.GetBuffer();
	*m_dwOutBufferLength = outputHTPPCompleteResponseBuffer.GetLength();

}

