#pragma once

namespace JakubSchwarzmeier
{
	namespace MFC
	{
		namespace Net
		{
			namespace WebServices
			{
				_ATLSOAP_DECLARE_WSDL_SRF();

				template <class THandler>
				class CSDLGenerator:
					public _CSDLGenerator,
					public IHttpServerContext,
					public CComObjectRootEx<CComSingleThreadModel>
				{
				public:

					HRESULT STDMETHODCALLTYPE QueryInterface( 
						/* [in] */ REFIID riid,
						/* [iid_is][out] */ void __RPC_FAR *__RPC_FAR *ppvObject)
					{
						return E_NOTIMPL;
					}
				    
					ULONG STDMETHODCALLTYPE AddRef( void)
					{
						return 1;
					}
				    
					ULONG STDMETHODCALLTYPE Release( void)
					{
						return 1;
					}


					HTTP_CODE RenderWSDL(JakubSchwarzmeier::MFC::System::IO::CBuffer & outputBuffer)
					{
						CStreamBuffer writeStream;
						writeStream.SetBuffer( &outputBuffer );

						CComObjectStack<THandler> handler;
						if (FAILED(InitializeSDL(&handler)))
						{
							return HTTP_FAIL;
						}

						CStencil s;
						HTTP_CODE hcErr = s.LoadFromString(s_szAtlsWSDLSrf, (DWORD) strlen(s_szAtlsWSDLSrf));
						if (hcErr == HTTP_SUCCESS)
						{
							hcErr = HTTP_FAIL;
							if (s.ParseReplacements(this) != false)
							{

								s.FinishParseReplacements();

								SetStream(&writeStream);
								SetWriteStream(&writeStream);
								SetHttpServerContext(this);

								ATLASSERT( s.ParseSuccessful() != false );

								hcErr = s.Render(this, &writeStream);
							}
						}

						return hcErr;
					}

					const char * GetHandlerName()
					{
						return THandler::ServiceName();
					}

					// IHttpServerContext pseudo-implementation
					LPCSTR GetRequestMethod(){return NULL;}
					LPCSTR GetQueryString(){return NULL;}
					LPCSTR GetPathInfo(){return NULL;}
					LPCSTR GetPathTranslated(){return NULL;}
					LPCSTR GetScriptPathTranslated(){return NULL;}
					DWORD GetTotalBytes(){return NULL;}
					DWORD GetAvailableBytes(){return NULL;}
					BYTE *GetAvailableData(){return NULL;}
					LPCSTR GetContentType(){return NULL;}

					BOOL GetServerVariable(
						LPCSTR pszVariableName,
						LPSTR pvBuffer,
						DWORD *pdwSize)
					{
						LPCSTR		szVal = NULL;
						char		hostName[MAX_PATH];
						CStringA	szReturn;
						ATLASSERT( pdwSize );

						if( strcmp(pszVariableName, "URL") == 0)
						{
							//  return something in form eg. something.dll
							strcpy( szReturn.GetBuffer(1024), GetHandlerName() );
							strcat( szReturn.GetBuffer(), ".dll" );
							szReturn.ReleaseBuffer();

							szVal = szReturn;
						}
						else if( strcmp(pszVariableName, "SERVER_NAME") == 0)
						{
							TRACE0( "If an RPC exception is reported, it's created it is throwed by gethostname(): " );
							::gethostname( hostName, sizeof(hostName) );

							szReturn.Format("%s", hostName);
							szVal = szReturn;
						}
						else if( strcmp(pszVariableName, "SERVER_PORT")== 0)
						{
							unsigned int nListeningPort = 80;
							szReturn.Format("%d/", nListeningPort);
							szVal = szReturn;
						}
						else if( strcmp(pszVariableName, "PROTOCOL")== 0)
							szVal =	"http";

						if( szVal )
						{
							if( strlen(szVal) < *pdwSize )
							{
								strcpy( pvBuffer, szVal );
								return TRUE;
							}
							*pdwSize	=	(DWORD)strlen(szVal) + 1;
						}

						return FALSE;
					}
					BOOL GetImpersonationToken(HANDLE * ){return FALSE;}
					BOOL WriteClient(void *, DWORD *){return FALSE;}
					BOOL AsyncWriteClient(void *, DWORD *){	return FALSE;}
					BOOL ReadClient(void *, DWORD *){return FALSE;}
					BOOL AsyncReadClient(void *, DWORD *){return FALSE;}
					BOOL SendRedirectResponse(LPCSTR ){return FALSE;}
					BOOL SendResponseHeader(LPCSTR , LPCSTR , BOOL ){return FALSE;}
					BOOL DoneWithSession(DWORD ){return FALSE;}
					BOOL RequestIOCompletion(PFN_HSE_IO_COMPLETION , DWORD *){return FALSE;}
					BOOL TransmitFile(HANDLE , PFN_HSE_IO_COMPLETION , void *, 
						LPCSTR , DWORD , DWORD ,void *, DWORD , void *,DWORD , DWORD ){return FALSE;}
					BOOL AppendToLog(LPCSTR , DWORD* ){return FALSE;}
					BOOL MapUrlToPathEx(LPCSTR , DWORD , HSE_URL_MAPEX_INFO *){return FALSE;}

				}; // class CSDLGenerator
			
			}	//  end namespace WebServices
		
		}	//  end namespace Net

	} 	//  end namespace MFC

}	//  end namespace JakubSchwarzmeier

