#pragma once

#include "streambuffer.h"

namespace JakubSchwarzmeier
{
	namespace MFC
	{
		namespace Net
		{
			namespace WebServices
			{
				template <class _T>
				class CSoapInvoker: public CSoapHandler<_T>
				{
				public:

					bool GenerateSoapFault(
						SOAP_ERROR_CODE errCode, 
						IWriteStream * pWriteStream,
						HRESULT & hr
						)
					{
						ATLASSERT( pWriteStream != NULL );

						CSoapFault fault;	//  ATL class for error SOAP message creation
						LPSTR szErrorA;			//  Gets error message from COM
						wchar_t szErrorW[512];		

						_ATLTRY
						{
							FormatMessageA(FORMAT_MESSAGE_ALLOCATE_BUFFER|FORMAT_MESSAGE_FROM_SYSTEM,
										NULL,
										hr,
										MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
										reinterpret_cast<LPSTR>(&szErrorA),
										0,
										NULL );

							//  ANSI --> UNICODE
							MultiByteToWideChar(CP_UTF8, 
									0, 
									szErrorA,
									strlen(szErrorA),
									szErrorW,
									sizeof(szErrorW)
									);

							fault.m_strDetail += szErrorW;

							fault.m_soapErrCode = errCode;
							fault.GenerateFault(pWriteStream);

							LocalFree( szErrorA );
						}
						_ATLCATCHALL()
						{
							ATLTRACE2(traceAppMsg, 0, "GenerateSoapFault: Out of memory.\r\n");
						}

						//  Reset state of SoapHandler
						ResetClientState(true);

						return false;
					}

					bool InvokeSoapMethod(
						JakubSchwarzmeier::MFC::System::IO::CBuffer & inputBuffer,
						CStringW strSOAPAction,
						JakubSchwarzmeier::MFC::System::IO::CBuffer & outputBuffer
						) throw()
					{
						CStreamBuffer writeStream;
						writeStream.SetBuffer( &outputBuffer );

						HRESULT hr = InitializeSOAP(NULL);

						if (FAILED(hr))	return GenerateSoapFault(SOAP_E_SERVER, &writeStream, hr);

						_ATLTRY
						{
							SetSoapMapFromName( strSOAPAction, -1, GetNamespaceUri(), -1, true);
						}
						_ATLCATCHALL()
						{
							UninitializeSOAP();
							return GenerateSoapFault(SOAP_E_SERVER, &writeStream, hr);
						}

						CStreamBuffer readStream;
						readStream.SetBuffer( &inputBuffer );

						hr = BeginParse(&readStream);

						if (FAILED(hr))
						{
							UninitializeSOAP();
							return GenerateSoapFault(SOAP_E_CLIENT, &writeStream, hr);
						}

						hr = CallFunctionInternal();
						if (FAILED(hr))
						{
							Cleanup();
							GenerateSoapFault(SOAP_E_SERVER, &writeStream, hr);
							UninitializeSOAP();
							return GenerateSoapFault(SOAP_E_SERVER, &writeStream, hr);
						}

						hr = GenerateResponse(&writeStream);
						Cleanup();
						if (FAILED(hr))
						{
							UninitializeSOAP();
							return GenerateSoapFault(SOAP_E_SERVER, &writeStream, hr);
						}

						UninitializeSOAP();

						//  Reset state of SoapHandler
						ResetClientState(true);

						return true;
					}
				
				};	//  end class CSoapInvoker

			}	//  end namespace WebServices
		
		}	//  end namespace Net

	} 	//  end namespace MFC

}	//  end namespace JakubSchwarzmeier

