#pragma once


namespace JakubSchwarzmeier
{
	namespace MFC
	{
		namespace Net
		{
			namespace Sockets
			{
				using namespace JakubSchwarzmeier::MFC::System::IO;
				class AFX_EXT_CLASS CSocket: 
					public JakubSchwarzmeier::MFC::System::IO::IIOCPStream					
				{
				public:
					// Construction/destruction
					
					//  Read buffer size in Bs
					CSocket(SOCKET n_socket = 0, CSocketPool * n_pSocketPool = NULL, size_t n_readBufferSize = 256);

					virtual ~CSocket();

					// Implementation
					void AddProtocol( IProtocol * protocol );
					void SetReadBufferSize(size_t n_newSize);

					CString GetLocalAddress();
					unsigned int GetLocalPort();
					CString GetRemoteAddress();
					unsigned int GetRemotePort();

					// Overridables
					virtual void ReadFile();
					virtual void WriteFile(char * lpBuffer, DWORD dwNumberOfBytesToWrite);

					virtual void OnRead(OVERLAPPEDPLUS * olp, DWORD dwSize);
					virtual void OnWrite(OVERLAPPEDPLUS * olp, DWORD dwSize);

					virtual void OnErrorRead(OVERLAPPEDPLUS * olp, DWORD dwSize);
					virtual void OnErrorWrite(OVERLAPPEDPLUS * olp, DWORD dwSize);

					virtual operator HANDLE() const;

					// Attributes
					operator SOCKET() const;

				protected:

					SOCKET m_hSocket;
					CSocketPool * m_pSocketPool;

					//  read buffer
					size_t m_readBufferSize;
					char * m_readBuffer;
					OVERLAPPEDPLUS m_readingOverlapped;  //  not suitable for multiple 
														 //  worker threads

					//  write buffer
					CRBMap<OVERLAPPEDPLUS*, char*> m_writeBuffersMap;

					//  protocols
					CAtlList<IProtocol*> m_protocolList;
					IProtocol * m_directRouteProtocol;


					//  thread safety
					::CCriticalSection m_criticalSection;

					DECLARE_DYNAMIC( CSocket )

				};	//  end class CSocket

			}	//  end namespace Sockets

		}	//  end namespace Net

	} 	//  end namespace MFC

}	//  end namespace JakubSchwarzmeier

