#include "stdafx.h"

#include "MFC.Net.h"

#define WM_NETWORK_EVENT	WM_USER + 1

using namespace JakubSchwarzmeier::MFC::Net::Sockets;

BEGIN_MESSAGE_MAP(CSocketPool::CMessageWnd, CWnd)
	ON_MESSAGE( WM_NETWORK_EVENT, OnNetworkEvent )
END_MESSAGE_MAP()

IMPLEMENT_DYNAMIC( CSocketPool, CObject )

void CSocketPool::CMessageWnd::SetSocketPool(CSocketPool * n_socketPool)
{
	ASSERT_VALID(n_socketPool);
	m_socketPool = n_socketPool;

	return;
}

LRESULT CSocketPool::CMessageWnd::OnNetworkEvent( WPARAM wParam, LPARAM lParam )
{
	try
	{
		SOCKET sIncoming = wParam;

		int iErrorCode = WSAGETSELECTERROR(lParam);
		if (iErrorCode != 0)
		{
			throw new CWinsockException(TRUE, iErrorCode);
		}

		switch ( WSAGETSELECTEVENT( lParam ) ) 
		{
		case FD_ACCEPT:
		{
			//  sIncoming socket is a listening socket
			SOCKADDR_IN sockname;
			int iSize = sizeof(sockname);

			//  Use WSAAccept for conditional accept/reject.
//			SOCKET sAccepted = accept(sIncoming, (sockaddr*) &sockname, &iSize );
			SOCKET sAccepted = WSAAccept( sIncoming, (sockaddr*) &sockname, &iSize,
				Condition, reinterpret_cast<DWORD_PTR>( &m_socketPool->m_acceptedRemoteHosts ) );

			if ( sAccepted == INVALID_SOCKET )
			{
				throw new CWindowsException(TRUE);
			}

			m_socketPool->OnAccept( new CSocket(sAccepted, m_socketPool) );
		}		
			
			break;

		case FD_CONNECT:
		{

			//  sIncoming is a connected socket
			
			//  remove socket from CONNECTING sockets and add it to CONNECTED sockets
			SOCKET connectingSocket;
			POSITION old;
			POSITION pos = m_socketPool->m_hConnectingSockets.GetHeadPosition();
			while (pos != NULL)
			{
				old = pos;
				connectingSocket = m_socketPool->m_hConnectingSockets.GetNext(pos);

				if (connectingSocket == sIncoming)
				{
					m_socketPool->m_hConnectingSockets.RemoveAt(old);
					break;	//  exit while loop
				}
				//  go to next socket in list
			}

 			m_socketPool->OnConnect( new CSocket(sIncoming, m_socketPool) );
		}
			
			break;

		} //  switch
	}

	catch(JakubSchwarzmeier::MFC::System::CBaseException * excp)
	{
		excp->ReportError();
		excp->Delete();
	}

	return 0;
}

CSocketPool::CSocketPool()
{
}

CSocketPool::~CSocketPool()
{
	Close();

	POSITION pos = m_connectedSocketsPool.GetHeadPosition();
	while (pos != NULL)
	{
		CSocket * socket = m_connectedSocketsPool.GetNext(pos);

		ASSERT_VALID(socket);
		delete socket;		
	}
}

unsigned short CSocketPool::Listen(unsigned short n_port, 
	bool bListenOther /*= false*/)
{
	//  Create new unbinded listening socket, overlapped attribute is default.
	//  This socket is only for listening.
	SOCKET listeningSocket = socket( AF_INET, SOCK_STREAM, IPPROTO_TCP );
	if ( listeningSocket == INVALID_SOCKET )
	{
		throw new CWindowsException(TRUE);
	}

	//  Socket will be bound for exclusive use.
	BOOL optval = TRUE;
	int sso = setsockopt( listeningSocket, SOL_SOCKET, SO_EXCLUSIVEADDRUSE, (char*)&optval, sizeof(optval) );
	if (sso != 0)
	{
		throw new CWindowsException(TRUE);
	}

	unsigned short port = n_port;
	//  Bind this socket to SOCKADDR_IN structure
	SOCKADDR_IN sockname;
	sockname.sin_family = AF_INET;
	sockname.sin_addr.S_un.S_addr = htonl(INADDR_ANY);	//  local address, can change 
														//  if the host is multihomed
	sockname.sin_port = htons(port);	//  on this port
	
	int bind_status = bind( listeningSocket, (sockaddr*) &sockname, sizeof(sockname) );
	if ( bind_status == SOCKET_ERROR )
	{
		if ( (bListenOther) && ( GetLastError() == WSAEADDRINUSE ) )
		{
			port = 1000;
			do
			{
				//  try to find free port
				sockname.sin_port = htons(++port);
				bind_status = bind( listeningSocket, (sockaddr*) &sockname, 
					sizeof(sockname) );

			} while (bind_status == SOCKET_ERROR && GetLastError()== WSAEADDRINUSE);

			if (bind_status == SOCKET_ERROR)  throw new CWindowsException(TRUE);
		}
		else
		{
				throw new CWindowsException(TRUE);
		}

	}

	//  Create a window that will recieve the network events.
	if (m_messageWnd.GetSafeHwnd() == 0)	CreateMessageWindow();

	//  Use a listening socket in an asynchronous mode.
	int async_status = 	WSAAsyncSelect( listeningSocket, m_messageWnd, 
		WM_NETWORK_EVENT, FD_ACCEPT);
	if ( async_status == SOCKET_ERROR )
	{
		throw new CWindowsException(TRUE);
	}

	//  Wait for incoming connections and insert them into queue
	int listen_status = listen( listeningSocket, SOMAXCONN );
	if ( listen_status == SOCKET_ERROR )
	{
		throw new CWindowsException(TRUE);
	}

	//  add socket to listening sockets
	m_hListeningSockets.AddTail( listeningSocket );

	return port;
}

void CSocketPool::Connect(int n_ipAddress, unsigned short n_port, 
						  bool synchronous /*= false*/)
{
	//  create unbinded socket, overlapped attribute is default
	SOCKET connectingSocket = socket( PF_INET, SOCK_STREAM, IPPROTO_TCP );
	if (connectingSocket == INVALID_SOCKET)
	{
		throw new CWindowsException(TRUE);
	}

	//  Disable the socket to be bound to an address that is already in use.
	int optval = 1;
	int sso = setsockopt( connectingSocket, SOL_SOCKET, SO_EXCLUSIVEADDRUSE, (char*)&optval, sizeof(optval) );
	if (sso != 0)
	{
		throw new CWindowsException(TRUE);
	}

	//  Connect this socket to SOCKADDR_IN structure
	SOCKADDR_IN sockname;
	sockname.sin_family = AF_INET;
	sockname.sin_addr.S_un.S_addr = htonl(n_ipAddress);	//  connect to this remote machine
	sockname.sin_port = htons(n_port);	//  on this port

	//  Create a window that will recieve the network events.
	if (m_messageWnd.GetSafeHwnd() == 0)	CreateMessageWindow();

	if (!synchronous)
	{
		//  Use connect() in an asynchronous mode.
		int async_status = 	WSAAsyncSelect( connectingSocket, m_messageWnd, 
			WM_NETWORK_EVENT, FD_CONNECT);
		if ( async_status == SOCKET_ERROR )
		{
			throw new CWindowsException(TRUE);
		}
	}

	//  connect
	int con = connect( connectingSocket, (sockaddr*) &sockname, sizeof(sockname) );
	if ( con != 0 )
	{
		throw new CWindowsException(TRUE);
	}
	else if (synchronous)
	{
		unsigned long ul = TRUE;
		int nRet = ioctlsocket(connectingSocket, FIONBIO, static_cast<unsigned long *>(&ul) );
		if (nRet == SOCKET_ERROR)
		{
			// Failed to put the socket into nonblocking mode
			THROW_WINDOWS_EXCEPTION()
		}

		sockaddr_in sadr;
		int iSize = sizeof(sadr);
		int i = getpeername( connectingSocket, reinterpret_cast<sockaddr *>(&sadr), &iSize );

		OnConnect( new CSocket(connectingSocket, this) );
	}
	else
	{
		//  add socket to connecting sockets
		m_hConnectingSockets.AddTail( connectingSocket );
	}

	return;
}

void CSocketPool::CreateMessageWindow()
{
	m_strWindowClass = AfxRegisterWndClass(0);
	m_messageWnd.CreateEx(0, m_strWindowClass, NULL, 0, 0, 0, 0, 0, HWND_MESSAGE, NULL);
	m_messageWnd.SetSocketPool( this );
}

void CSocketPool::Close()
{
	//  clean remaining listening sockets
	//
	SOCKET listeningSocket;
	POSITION pos = m_hListeningSockets.GetHeadPosition();
	while (pos != NULL)
	{
		listeningSocket = m_hListeningSockets.GetNext(pos);

		//  cancel recieving of asynchrous network messages
		WSAAsyncSelect( listeningSocket, m_messageWnd, 0, 0 );

		//  close listening socket
		int err = closesocket( listeningSocket );
		if ( err == SOCKET_ERROR )	throw new CWindowsException(TRUE);
		
	}

	m_hListeningSockets.RemoveAll();

	//  clean remaining connecting sockets
	//
	SOCKET connectingSocket;
	pos = m_hConnectingSockets.GetHeadPosition();
	while (pos != NULL)
	{
		connectingSocket = m_hConnectingSockets.GetNext(pos);

		//  cancel recieving of asynchrous network messages
		WSAAsyncSelect( connectingSocket, m_messageWnd, 0, 0 );

		//  close listening socket
		int err = closesocket( connectingSocket );
		if ( err == SOCKET_ERROR )	throw new CWindowsException(TRUE);
	}

	m_hConnectingSockets.RemoveAll();

}

void CSocketPool::OnAccept(CSocket * n_socket)
{
	//  ready for override
	ASSERT_VALID(n_socket);

	m_connectedSocketsPool.AddTail( n_socket );
}

void CSocketPool::OnConnect(CSocket * n_socket)
{
	//  ready for override	
	ASSERT_VALID(n_socket);

	m_connectedSocketsPool.AddTail( n_socket );
}

void CSocketPool::RemoveSocket( CSocket * socketToRemove )
{
	ASSERT_VALID(socketToRemove);

	m_criticalSection.Lock();

	POSITION pos = m_connectedSocketsPool.GetHeadPosition();
	while (pos != NULL)
	{
		CSocket * socket = m_connectedSocketsPool.GetAt(pos);
		if (socketToRemove == socket)
		{
			m_connectedSocketsPool.RemoveAt(pos);
			break;
		}
		
		m_connectedSocketsPool.GetNext(pos);
	}

	m_criticalSection.Unlock();
}

int CSocketPool::Condition(
	/*[IN]*/ LPWSABUF lpCallerId,
	/*[IN]*/ LPWSABUF lpCallerData,
	/*[IN/OUT]*/ LPQOS lpSQOS,
	/*[IN/OUT]*/ LPQOS lpGQOS,
	/*[IN]*/ LPWSABUF lpCalleeId,
	/*[OUT]*/ LPWSABUF lpCalleeData,
	/*[OUT]*/ GROUP FAR *g,
	/*[IN]*/ DWORD_PTR dwCallbackData
	)
{
	//  This is a static function, class data member is send through dwCallbackData.
	CAtlList<DWORD> * acceptedRemoteHosts =
		reinterpret_cast< CAtlList<DWORD> * >(dwCallbackData);

	// Try to find address of connecting remote host in our records.
	SOCKADDR_IN * sock = reinterpret_cast<SOCKADDR_IN*>(lpCallerId->buf);
	POSITION pos = acceptedRemoteHosts->GetHeadPosition();
	while (pos)
	{
		//  Is connecting entity in trusted list?
		if ( sock->sin_addr.S_un.S_addr == acceptedRemoteHosts->GetNext( pos ) )
		{
			return CF_ACCEPT;
		}
	}

	//  Not found, reject connection.
	return CF_REJECT;
}

void CSocketPool::SetTrustedListening(DWORD dwIPAddress)
{
	m_acceptedRemoteHosts.AddTail( htonl(dwIPAddress) );
}

BOOL CSocketPool::SetWildTrustedListening(CString & strWildAcceptedRemoteHosts)
{
	//  BLIND STUB

	//  Check strWildAcceptedRemoteHosts for correctness.
	CAtlRegExp<> re;
	re.Parse( _T("{[0-255]?[0-255]?[0-255]}.") \
			  _T("{[0-255]?[0-255]?[0-255]}.") \
			  _T("{[0-255]?[0-255]?[0-255]}.") \
			  _T("{[0-255]?[0-255]?[0-255]}") );

	//  Match the regular expression against a string
	CAtlREMatchContext<> mcIPAddress;
	if ( re.Match( strWildAcceptedRemoteHosts, &mcIPAddress ) == TRUE )
	{
		for (UINT nGroupIndex = 0; nGroupIndex < mcIPAddress.m_uNumGroups;
				++nGroupIndex)
		{
			const CAtlREMatchContext<>::RECHAR* szStart = 0;
			const CAtlREMatchContext<>::RECHAR* szEnd = 0;
			mcIPAddress.GetMatch(nGroupIndex, &szStart, &szEnd);

			ptrdiff_t nLength = szEnd - szStart;
			printf("%d: \"%.*s\"\n", nGroupIndex, nLength, szStart);
		}
	
		m_strWildAcceptedRemoteHosts = strWildAcceptedRemoteHosts;
		return TRUE;
	}
	else
		return FALSE;
}

