#pragma once

class CStreamBuffer: public IWriteStream, public IStreamImpl
{
protected:

	JakubSchwarzmeier::MFC::System::IO::CBuffer * m_pBuffer;
	long m_nCurr;
	long m_nBodyLen;
	long l_refCount;

public:

	CStreamBuffer(): m_nCurr(0), m_nBodyLen(0), l_refCount(0), m_pBuffer(NULL) {};

	HRESULT WINAPI QueryInterface(REFIID riid, void **ppv)
	{
		if (ppv == NULL)
		{
			return E_POINTER;
		}

		*ppv = NULL;

		if (InlineIsEqualGUID(riid, IID_IUnknown) ||
			InlineIsEqualGUID(riid, IID_IStream) ||
			InlineIsEqualGUID(riid, IID_ISequentialStream))
		{
			*ppv = static_cast<IStream*>(this);
			return S_OK;
		}

		return E_NOINTERFACE;
	}

	ULONG WINAPI AddRef()
	{
		::InterlockedIncrement( &l_refCount );
		return 1;
	}

	ULONG WINAPI Release()
	{
		if( l_refCount > 0 )
		{
			::InterlockedDecrement( &l_refCount );
			if( l_refCount == 0 )
			{
				m_nCurr = 0;
				m_nBodyLen = 0;
			}
		}
		return 1;
	}

	void SetBuffer(JakubSchwarzmeier::MFC::System::IO::CBuffer * pBuff)
	{
		m_pBuffer = pBuff;
		m_nBodyLen = pBuff->GetLength();
	}

	HRESULT WINAPI Read(void *pDest, ULONG nMaxLen, ULONG *pnRead)
	{
		ATLASSERT( pDest != NULL );

		if (pnRead != NULL)
		{
			*pnRead = 0;
		}

		long nRead = m_nCurr;
		if (nRead < m_nBodyLen)
		{
			long nLength = min((int)(m_nBodyLen - nRead), (LONG) nMaxLen);
			memcpy(pDest, m_pBuffer->GetBuffer() + nRead, nLength);
			m_nCurr+= nLength;

			if (pnRead != NULL)
			{
				*pnRead = (ULONG) nLength;
			}
		}

		return S_OK;
	}

	HRESULT WriteStream(LPCSTR szOut, int nLen, LPDWORD pdwWritten)
	{
		ATLASSERT( szOut != NULL );

		if (nLen < 0)
		{
			nLen = (int) strlen(szOut);
		}

		_ATLTRY
		{
			m_pBuffer->Append(szOut, nLen);
			m_nBodyLen	=	m_pBuffer->GetLength();
		}
		_ATLCATCHALL()
		{
			return E_OUTOFMEMORY;
		}

		if (pdwWritten != NULL)
		{
			*pdwWritten = (DWORD) nLen;
		}

		return S_OK;
	}


	HRESULT FlushStream()
	{
		return S_OK;
	}

}; // class CStreamBuffer
