// SFC.System.cpp : Defines the initialization routines for the DLL.
//

#include "stdafx.h"
#include <SFC.System.h>

#include <afxdllx.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

static AFX_EXTENSION_MODULE SFCSystemDLL = { NULL, NULL };

extern "C" int APIENTRY
DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
{
	// Remove this if you use lpReserved
	UNREFERENCED_PARAMETER(lpReserved);

	if (dwReason == DLL_PROCESS_ATTACH)
	{
		TRACE0("SFCSystem.DLL Initializing!\n");
		
		// Extension DLL one-time initialization
		if (!AfxInitExtensionModule(SFCSystemDLL, hInstance))
			return 0;

		// Insert this DLL into the resource chain
		// NOTE: If this Extension DLL is being implicitly linked to by
		//  an MFC Regular DLL (such as an ActiveX Control)
		//  instead of an MFC application, then you will want to
		//  remove this line from DllMain and put it in a separate
		//  function exported from this Extension DLL.  The Regular DLL
		//  that uses this Extension DLL should then explicitly call that
		//  function to initialize this Extension DLL.  Otherwise,
		//  the CDynLinkLibrary object will not be attached to the
		//  Regular DLL's resource chain, and serious problems will
		//  result.

		JakubSchwarzmeier::SFC::System::LoadResourceDll( SFCSystemDLL );
		new CDynLinkLibrary(SFCSystemDLL);
	}
	else if (dwReason == DLL_PROCESS_DETACH)
	{
		TRACE0("SFCSystem.DLL Terminating!\n");

		JakubSchwarzmeier::SFC::System::FreeResourceDll( SFCSystemDLL );

		// Terminate the library before destructors are called
		AfxTermExtensionModule(SFCSystemDLL);
	}
	return 1;   // ok
}

void JakubSchwarzmeier::SFC::System::LoadResourceDll( AFX_EXTENSION_MODULE & extModule )
{
	TCHAR szModuleFileName[MAX_PATH];
	//  eg. szModuleFileName = C:\APPDIR\SFC.SYSTEM.DLL
	DWORD dwSize = GetModuleFileName( static_cast<HMODULE>( extModule.hModule ), 
		szModuleFileName, 
		MAX_PATH );

	if (dwSize == 0)
	{
		ErrorMessageBox( _T(__FILE__), __LINE__ );
		throw;
	}

	TCHAR szAppDir[MAX_PATH];
	StrCpy( /* [out] */ szAppDir, /* [in] */ szModuleFileName );

	//  eg. szAppDir = C:\APPDIR
	PathRemoveFileSpec( szAppDir );

	//  eg. szModuleFileName = SFC.SYSTEM.DLL
	PathStripPath( szModuleFileName );

	//  Get language and country code, eg. cs and CZ
	LANGID langid = GetUserDefaultUILanguage();
	WORD wPrimaryLang = PRIMARYLANGID(langid);
	WORD wSubLang = SUBLANGID(langid);

	DWORD lcid = MAKELCID(MAKELANGID(wPrimaryLang, wSubLang), SORT_DEFAULT);
	LCID alcid = ConvertDefaultLocale(lcid);

	TCHAR szLangCode[3], szCntryCode[3];
	GetLocaleInfo(alcid, LOCALE_SISO639LANGNAME, szLangCode, sizeof(szLangCode)/sizeof(TCHAR));
	GetLocaleInfo(alcid, LOCALE_SISO3166CTRYNAME, szCntryCode, sizeof(szCntryCode)/sizeof(TCHAR));
//	GetLocaleInfo(alcid, LOCALE_SNATIVELANGNAME, szLangCode, 4);

	//  Build language-country string for directory, append backslash and filename
	//  eg. C:\APPDIR\cs-CZ\SFC.SYSTEM.DLL
	PathAddBackslash( szAppDir );
	StrCat( szAppDir, szLangCode );
	StrCat( szAppDir, _T("-") );
	StrCat( szAppDir, szCntryCode );
	PathAddBackslash( szAppDir );
	StrCat( szAppDir, szModuleFileName );

	//  Load localized satellite resource DLL
	SFCSystemDLL.hResource = LoadLibraryEx( szAppDir, NULL, DONT_RESOLVE_DLL_REFERENCES );
	if (SFCSystemDLL.hResource == 0)
	{
		ErrorMessageBox( _T(__FILE__), __LINE__ );
		throw;
	}
}

void JakubSchwarzmeier::SFC::System::FreeResourceDll( AFX_EXTENSION_MODULE & extModule )
{
	if ( extModule.hResource != extModule.hModule )
	{
		FreeLibrary( extModule.hResource );
		extModule.hResource = extModule.hModule;
	}
}

void JakubSchwarzmeier::SFC::System::ErrorMessageBox( TCHAR szFile[], long lLine )
{
	LPTSTR szError;
	//  lpMessage contains error message from Windows
	FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM, NULL, 
		GetLastError(), 0, reinterpret_cast<LPTSTR>(&szError), 0, NULL);

	TCHAR szMessage[4096];
	_stprintf(szMessage, TEXT("%s\nSource: %s. Line: %d.\n"), szError, szFile, lLine);

	MessageBox( AfxGetMainWnd()->GetSafeHwnd(), szMessage, NULL, MB_OK | MB_ICONERROR );
	LocalFree( szError );
}
