#pragma once

namespace JakubSchwarzmeier
{
	namespace SFC
	{
		namespace System
		{
			namespace IO
			{
				class CBuffer
				{
				protected:
					char * m_pBuffer;
					DWORD m_dwAlloc, m_dwInflate, m_dwLen;
					bool m_bOwner;

				public:
					CBuffer(bool n_bOwner = false, DWORD n_dwInflate = 1024)
					{
						m_dwLen = 0;
						m_dwInflate = n_dwInflate;
						m_bOwner = n_bOwner;

						m_pBuffer = new char[m_dwInflate];
						m_dwAlloc = m_dwInflate;
					}

					CBuffer(char * n_pBuffer, DWORD n_dwLen,
						bool n_bOwner = false, DWORD n_dwInflate = 1024)
					{
						m_dwLen = n_dwLen;
						m_dwInflate = n_dwInflate;
						m_bOwner = n_bOwner;

						m_pBuffer = n_pBuffer;
						m_dwAlloc = n_dwLen;
					}

					~CBuffer() throw()
					{
						if (m_bOwner)	delete [] m_pBuffer;
					}

					void Free()
					{
						m_dwLen = 0;
					}

					BOOL Append(LPCSTR sz, int nLen = -1)
					{
						if (nLen == -1)
							nLen = (int) strlen(sz);

						DWORD newSize = nLen + m_dwLen;

						bool bNewBuffer = false;
                        while (newSize > m_dwAlloc)
						{
							//  just find the right size
							m_dwAlloc += m_dwInflate;
							bNewBuffer = true;
						}

						if (bNewBuffer)
						{
							char * newBuffer = new char[m_dwAlloc];
							memcpy(newBuffer, m_pBuffer, m_dwLen);
                            delete [] m_pBuffer;
							m_pBuffer = newBuffer;
						}

						memcpy(m_pBuffer + m_dwLen, sz, nLen);
						m_dwLen += nLen;
						return TRUE;
					}

					char * GetBuffer()
					{
						return m_pBuffer;
					}
					
					DWORD GetLength()
					{
						return m_dwLen;
					}

					CBuffer& operator+=(LPCSTR sz)
					{
						Append(sz);
						return *this;
					}
				};	//  end class CBuffer
				
				#undef BUFFER_SIZE 

			}	//  end namespace IO
		
		}	//  end namespace System

	} 	//  end namespace SFC

}	//  end namespace JakubSchwarzmeier

