#pragma once

namespace JakubSchwarzmeier
{
	namespace SFC
	{
		namespace System
		{
			//  abstract class
			class AFX_EXT_CLASS CBaseException: public ::CException
			{
			public:
				//  set bAutoDelete TRUE if allocated on the heap
				CBaseException(BOOL bAutoDelete) = 0;
				void SetCause(CString & strCauseBody);
				void SetCause(CString & strCauseBody, 
					CString & strCauseDetails, int & iHelpID);

				virtual void ReportError();

			protected:

				CString m_strCauseBody;
				CString m_strCauseDetails;
				int m_iHelpID;
			};

			class AFX_EXT_CLASS CComException: public CBaseException
			{
			public:

				CComException(BOOL bAutoDelete, HRESULT hr);
			};

			class AFX_EXT_CLASS CWindowsException: public CBaseException
			{
			public:

				CWindowsException(BOOL bAutoDelete, DWORD dwCode = 0);
			};
		
		}	//  end namespace System

	}	//  end namespace AfxWin

}	//  end namespace JakubSchwarzmeier

#ifdef _DEBUG
	#define TRACE_ERROR() \
		LPTSTR __szError; \
		::FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM, NULL, \
			::GetLastError(), 0, reinterpret_cast<LPTSTR>(&__szError), 0, NULL); \
		ATLTRACE2(traceAppMsg, 0, TEXT("TRACE_ERROR: %s, %s\r\nGetLastError(): %s\r\n"), __FUNCTION__, __FILE__, __szError);	\
		LocalFree( __szError ); 

	#define TRACE_ERROR_CODE(code) \
		LPTSTR __szError; \
		::FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM, NULL, \
			code, 0, reinterpret_cast<LPTSTR>(&__szError), 0, NULL); \
		ATLTRACE2(traceAppMsg, 0, TEXT("TRACE_ERROR: %s, %s\r\nGetLastError(): %s\r\n"), __FUNCTION__, __FILE__, __szError);	\
		LocalFree( __szError ); 
		
	#define REPORT_CATCH_EXCEPTION() \
		ATLTRACE2(traceAppMsg, 0, "CATCH_EXCEPTION: %s, %s\r\n", __FUNCTION__, __FILE__);
#else
	#define TRACE_ERROR()
	#define TRACE_ERROR_CODE(code)
	#define REPORT_CATCH_EXCEPTION()
#endif // #ifdef _DEBUG

#define THROW_WINDOWS_EXCEPTION() \
	TRACE_ERROR() \
	throw new CWindowsException(TRUE);

#define THROW_WINDOWS_EXCEPTION_CODE(code) \
	TRACE_ERROR_CODE(code) \
	throw new CWindowsException(TRUE, code);
