/************************************************************
*
*  Copyright (c) 2002-2004 Jakub Schwarzmeier
*
************************************************************/

#ifndef LIST_H
#define LIST_H

#include <iostream>

namespace JakubSchwarzmeier
{
	namespace Std
	{
		namespace Data
		{

			template<typename T>
			class RList
			{
			public:
				//  ma byt v protected, kvuli tomu, ze mi delal hlubokou kopii a ja chtel zrovna jenom kopirovat pointery
				T ** storage;	// ukazatel na ukazatel typu

			protected:
				int inc_size;		// velikost, o kterou pole ukazatelu naroste
				int real_size;		// skutecna aktualni velikost pole ukazatelu, ne pocet alokovanych ukazatelu na ukazatel objektu
				int count;	   		// index v poli ukazatelu, ktere se BUDE alokovat,
   									// celkovy pocet alokovanych ukazatelu na objekt v poli
				bool owner;			// statut vlastnictvi; true = kontejner, false = uzivatel
				void inflate();

			public:
			RList (int inc_size = 10, bool n_owner = true);
			virtual ~RList();			// uvolni uzivatelske objekty a zrusi pole ukazatelu na ukazatele na T

			int Add(T * ptr_T);	// po vlozeni ukazatele se PList stava vlastnikem objektu, na ktery odkazuje

			void own(bool n_owner);

			T& getAt(int index);   // vracime referenci na objekt, ne ukazatel

			T * GetAddress(int index);   // vracime ukazatel na objekt

			T& operator[] (int index);

			void removeAt(int index);

			int getCount();
			int GetCount()
	{
   				return (count);
	}

			class iterator;
			friend class iterator;
			class iterator
			{
				RList & list;
				int index;
			public:

				iterator(RList & n_list, int n_index = 0): list(n_list), index(n_index) {}
				T& operator+=(T& rv)	
				{
	//				 *list.storage[index] = rv;		... pokud chci vyvolat pretizeny operator T a provest hlubokou kopii, v hlavicce pak const
					// nasledujici jenom prekopiruje pointer
					list.storage[index] = &rv;
					return list[index++];
				}
				T& operator=(T& rv)	
				{
	//				 *list.storage[index] = rv;		... pokud chci vyvolat pretizeny operator T a provest hlubokou kopii, v hlavicce pak const
					// nasledujici jenom prekopiruje pointer
					list.storage[index] = &rv;
					return list[index];
				}
	//  tyhle melky kopie by mi vygeneroval sam kompiler; prepni na level4 a uvidis

				T& operator*() const
				{
					return *list.storage[index];
				}


 				bool operator!=(const iterator& rv) const
				{
					return index != rv.index;
				}

				T& operator++() //  Prefix
				{
			 		return list[++index];	 //  increment BEFORE evaluation of expression
				}

			};	//  class iterator

			iterator begin() { 
				iterator it = iterator(*this);
				return it; }
			iterator end() { 
				iterator it = iterator(*this, count-1);
				return it; }

			};


			template <typename T>
			RList<T>::RList (int inc_size, bool n_owner)
			{
				RList::inc_size = inc_size;
				real_size = inc_size;
				count = 0;
				owner = n_owner;

				storage = new T* [inc_size];
			}

			template <typename T>
			RList<T>::~RList()			// uvolni uzivatelske objekty a zrusi pole ukazatelu na ukazatele na T
			{
				if (owner)
				{

					if (count != 0)
					{
						for (int i = 0; i < count; i++)
						{
								// smaze objekt, na ktery nam uzivatel dal ukazatel
         					if (storage[i] != NULL)
							{
            					delete  storage[i];
							}
						}
					}
				}

 				delete [] storage;		// a nakonec pole ukazatelu na ukazatele, to kazdopadne existuje (z konstruktoru)
			}

			// po vlozeni ukazatele se RList stava vlastnikem objektu, na ktery odkazuje
			template <typename T>
			int RList<T>::Add(T * ptr_T)
			{
   				if (real_size <= count) inflate();
				storage[count] =  ptr_T;

				return (count++) ;		// vrati aktualni count a az pote ho navysi pro dalsi prvek
			}

			// vracime referenci na objekt, ne ukazatel
			template <typename T>
			T& RList<T>::getAt(int index)
			{
				#ifdef _DEBUG
				if (index >= count)
				{
					throw std::runtime_error("Pokus o pristup k prvku mimo rozsah pole!");

				}
				#endif

				return *storage[index];
			}

			template <typename T>
			T * RList<T>::GetAddress (int index)
			{
				#ifdef _DEBUG
				if (index >= count)
				{
					throw std::runtime_error("Pokus o pristup k prvku mimo rozsah pole!");

				}
				#endif

				return storage[index];
			}

			template <typename T>
			T& RList<T>::operator[] (int index)
			{
				return getAt(index);
			}

			template <typename T>
			void RList<T>::own(bool n_owner)
			{
				owner = n_owner;
			}

			template <typename T>
			void RList<T>::removeAt(int index)
			{
				if ( !(index >= count) )
				{
					// smazani objektu
	     			if (owner) delete storage[index];

					// "sesypani" ukazatelu
					for (int i = index; i < count-1; i++)
					{
							storage[i]=storage[i+1];
					}

					// posledni v poli ukazatelu je tam vlivem presypani dvakrat, tj. smazeme odkaz
					storage[--count] = NULL;  // jeste pred provedenim se count snizi

				}
 				#ifdef DEBUG
				else
				{
      				cerr << "Pokus o vyjmuti prvku mimo rozsah pole." << endl;
				}
				#endif
			}

			template <typename T>
			int RList<T>::getCount()
			{
   				return (count);
			}


			template <typename T>
			void RList<T>::inflate()
			{
				// vytvorime nove pole ukazatelu
				T ** new_storage = new T* [real_size+inc_size];
				// prekopirujeme ukazately ze stavajiciho pole do noveho
				for (int i = 0; i < real_size; i++) new_storage[i] = storage[i];
				// nastavime novou velikost pole ukazatelu
				real_size += inc_size;
				// zrusime puvodni pole ukazatalu, ale jen pokud jiz existuje
				if (count != 0) delete [] storage;
					// a pole ukazatelu bude z noveho pole ukazatelu
     				storage = new_storage;
			}

		}  // end namespace Data

	}	//  end namespace Std

}  //  end namespace JakubSchwarzmeier


#endif // LIST_H

