/************************************************************
*
*  Copyright (c) 2002-2004 Jakub Schwarzmeier
*
************************************************************/

#ifndef CAMERA_H
#define CAMERA_H

//  OpenGL works with modelview matrix -- this means, that modelling
//  and viewing matrix are combinated into this one.
//  This class is a transformation matrix, specialy designed for 
//  viewing transformations.

#include <schwarzmeier/std.h>

namespace JakubSchwarzmeier
{
	namespace Std
	{
		using namespace Math;

		namespace Graphics
		{

			class RotationMatrix: public Matrix<double>
			{
			protected:

                Quaternion<>	m_q;

				void Transform( Quaternion<> & rot );

			public:

				RotationMatrix();
				RotationMatrix( const RotationMatrix & original );
				RotationMatrix( const Matrix<> & original );
				RotationMatrix( const Quaternion<> & q );
				RotationMatrix( const Vector<> & v, const Quaternion<> & q );
				virtual ~RotationMatrix() {}
				RotationMatrix & operator=( const RotationMatrix & right_side );

				//  Like in an aeroplane
				//
				virtual void ChangeHeading( double angle );	//  rotate around local "y", knodwn as "yaw" too
				virtual void ChangePitch( double angle );	//  around "x"
				virtual void ChangeRoll( double angle );	//  around "z"
                virtual void ChangeRotation( Matrix<> & rotation );
                virtual void ChangePosition( Vector<> & position );

				//  With respect to cartesian coordinate system
				//

				//  gets
				Vector<> GetZAxis();				//  get z axis
                Vector<> GetYAxis();				//  y
                Vector<> GetXAxis();				//  x

				Quaternion<> GetQuaternion();

//				double GetLambda();					//  get angle of rotation around y-axis
  //            double GetPhi();					//  x
//              double GetTheta();					//  z

				virtual Vector<> GetPosition() const;

				//  sets
				virtual void ChangeLambda( double angle );	//  rotate around y
				virtual void ChangePhi( double angle );		//  rotate around x
				virtual void ChangeTheta( double angle );	//  rotate around z

				void ResetAngles(double lambda, double phi, double theta);

//				virtual void SetLambda( double angle );
//				virtual void SetPhi( double angle );
//				virtual void SetTheta( double angle );

				virtual void SetPosition( Vector<> & position );

				RotationMatrix & operator=( const Matrix<> & right_side );
			};
		
		}	//  end namespace Graphics

	}	//  end namespace Std

}	//  end namespace JakubSchwarzmeier


#endif  //  CAMERA_H
