/************************************************************
*
*  Copyright (c) 2002-2004 Jakub Schwarzmeier
*
************************************************************/

#ifndef MODELMATRIX_H
#define MODELMATRIX_H

//  OpenGL works with modelview matrix -- this means, that modelling
//  and viewing matrix are combinated into this one.
//  This class is a transformation matrix, specialy designed for 
//  viewing transformations.

#include <schwarzmeier/std.h>

namespace JakubSchwarzmeier
{
	namespace Std
	{
		using namespace Math;

		namespace Graphics
		{

			class ModelMatrix: public Matrix<double>
			{
			protected:

                Quaternion<>	m_q;

				void Transform( Quaternion<> & rot );

			public:

				ModelMatrix();
				ModelMatrix( const ModelMatrix & original );
				ModelMatrix( const Quaternion<> & q );
				virtual ~ModelMatrix() {}
				ModelMatrix & operator=( const ModelMatrix & right_side );

				//  Like in an aeroplane
				//
				void ChangeHeading( double angle );	//  rotate around local "y", knodwn as "yaw" too
				void ChangePitch( double angle );	//  around "x"
				void ChangeRoll( double angle );	//  around "z"
                void ChangeRotation( Matrix<> & rotation );
                void ChangePosition( Vector<> & position );

				//  With respect to cartesian coordinate system
				//

				//  gets
				Vector<> GetZAxis();				//  get z axis
                Vector<> GetYAxis();				//  y
                Vector<> GetXAxis();				//  x

				Quaternion<> GetQuaternion();

				double GetLambda();					//  get angle of rotation around y-axis
                double GetPhi();					//  x
                double GetTheta();					//  z

				Vector<> GetPosition();

				//  sets
				void ChangeLambda( double angle );	//  rotate around y

				void SetLambda( double angle );
				void SetPhi( double angle );
				void SetTheta( double angle );

				void SetPosition( Vector<> & position );

				ModelMatrix & operator=( const Matrix<> & right_side );
			};
		
		}	//  end namespace Graphics

	}	//  end namespace Std

}	//  end namespace JakubSchwarzmeier


#endif  //  MODELMATRIX_H
