#ifndef InterpolationApproximation_h
#define InterpolationApproximation_h

#include <vector>

namespace JakubSchwarzmeier
{
	namespace Std
	{
		namespace Math
		{
			class ValuePair
			{
			public:

				size_t GetCount();
				void Add( double x, double y );
				std::vector<double> & GetX();
				std::vector<double> & GetY();

			protected:

				std::vector<double> m_x;
				std::vector<double> m_y;
			};

			#ifdef __cplusplus_cli 
			  public // abstract */
			#endif
			class InterpolationApproximation
			{
			public:

				virtual ~InterpolationApproximation() {}

				virtual double GetValue( double x ) = 0;
				
				// the Points property
				ValuePair GetPoints();
				void SetPoints( ValuePair & values );
				
			protected:

				ValuePair m_values;
			};


			#ifdef __cplusplus_cli 
			  public 
			#endif
			class Spline: public InterpolationApproximation
			{
			public:

				Spline();
				~Spline();

				/* override */ virtual double GetValue( double point );
			protected:

				bool m_initialized;
				double * m_y2;   //  contains the second derivatives of the interpolating function at the tabulated points xi

			};

			class Spline2: public InterpolationApproximation
			{
			public: 

				Spline2();
				~Spline2();

				/*override*/ virtual double GetValue( double point );
				double SplDif( double point );

			protected:

				int m_lasti;

				bool m_initialized;
				double * m_coef;

				void spline( double coef[], std::vector<double> & x, std::vector<double> & y, int n );
				void splsub( double b[],
					std::vector<double> & x, std::vector<double> & y,
					int n );

				double seval( double x0, std::vector<double> & x, std::vector<double> & y, double coef[], int n );
				double spldif( double x0, std::vector<double> & x, std::vector<double> & y, double coef[], int n );
				double spldif2( double x0, std::vector<double> & x, std::vector<double> & y, double coef[], int n );

				int interval( double u, std::vector<double> & x, int n );

			};

		}	//  namespace Math

	}	//  namespace Std

}	//  namespace JakubSchwarzmeier

#endif //  InterpolationApproximation_h
