/************************************************************
*
*  Copyright (c) 2002-2004 Jakub Schwarzmeier
*
************************************************************/

#ifndef MATH_EXT_H
#define MATH_EXT_H

#define _USE_MATH_DEFINES
#include <math.h>

//************************************************************
//
// Some non-OOP math hairy stuff
//
// 
/*
// VC++ 6.0 doesn't have PI & other values defined
#ifndef DBL_EPSILON
	#define M_PI				3.14159265358979323846
	#define DBL_EPSILON         2.2204460492503131E-16
	#define FLT_EPSILON         1.19209290E-07F
	#define LDBL_EPSILON        1.084202172485504434e-019L

	// smallest positive IEEE normal numbers
	#define DBL_MIN             2.2250738585072014E-308
	#define FLT_MIN             1.17549435E-38F 	
#endif
*/
namespace JakubSchwarzmeier
{
	namespace Std
	{
		namespace Math
		{
			class Functions
			{
			public:

				typedef double (*Integral)(double);

				static long double sin_angle(const long double angle);
				static long double cos_angle(const long double angle);
				static long double tan_angle(const long double angle);
				static long double asin_angle(const double x);
				static long double acos_angle(const double x);
				static long double atan_angle(const double x);
				static long double atan2_angle(const double y, const double x);

				static double Random(long idum = 1);

				static double TrapezoidalIntegration( Integral func, double lower, double upper, int steps );
	            static double SimpsonIntegration( Integral f, double lower,
                    double upper, int steps);

				typedef double (*Function)(double);
				static double FindMinimum( double (*f)(double), double L, double U, double tol);
				static double FindMaximum( double (*f)(double), double L, double U, double tol);

				//  Error and Zeta Functions
				/*
				The error and Riemann zeta functions frequently occur in probability and statistics problems.
				ERF(z) is the integral of the Gaussian distribution from 0 to z.  
				*/
				static double erf( double q );

				//  n = number of points, x = values, f = function values, val = requested value
				static double LagrangeInterpolation(int n, double x[], double f[], double val);
				static void PolynomialInterpolation(double xa[], double ya[], int n, double x, double & y/*out*/, double & dy/*out*/);
				static void RationalInterpolation(double xa[], double ya[], int n, double x, double & y, double & dy);
			};

   			class AdditiveIntegral
			{
			public:

				AdditiveIntegral();
				double Integrate(Functions::Integral func, double upper, int steps);

			protected:

				double m_integral;
				double m_xi;
			};

		}	//  end namespace Math

	}	//  end namespace Std

}	//  end namespace JakubSchwarzmeier

#endif // MATH_EXT_H
