#include "../../include/schwarzmeier/xmllib.h"

#include <strstream>
#include <string>
#include <iomanip>

#define START_LOC	\

#define FINISH_LOC \

std::string GetXPathResponse( xmlDoc * pXmlDoc, xmlNode * pNode, std::string strXPath )
{
	xmlXPathContext * context = xmlXPathNewContext(pXmlDoc);
	context->node = pNode;
	xmlXPathObject * xpath = xmlXPathEvalExpression( (const xmlChar *)strXPath.c_str(), context);
	xmlXPathFreeContext(context);

	std::string result;
	if(!xmlXPathNodeSetIsEmpty(xpath->nodesetval))
	{
		xmlChar * textValue = xmlNodeListGetString(pXmlDoc, xpath->nodesetval->nodeTab[0], 1);
		result = ((const char *)textValue);
		xmlFree(textValue);
	}
	else
	{
		result = _T("");
	}


	return result;
}

xmlNode * SelectSingleNode( xmlDoc * pXmlDoc, xmlNode * pNode, std::string strXPath )
{
	xmlXPathContext * context = xmlXPathNewContext(pXmlDoc);
	context->node = pNode;
	xmlXPathObject * xpath = xmlXPathEvalExpression( (const xmlChar *)strXPath.c_str(), context);
	xmlXPathFreeContext(context);

	if(xmlXPathNodeSetIsEmpty(xpath->nodesetval))
	{
		return NULL;
	}

	return xpath->nodesetval->nodeTab[0];
}


xmlNodeSet * SelectNodes( xmlDoc * pXmlDoc, xmlNode * pNode, std::string strXPath )
{
	xmlXPathContext * context = xmlXPathNewContext(pXmlDoc);
	context->node = pNode;
	xmlXPathObject * xpath = xmlXPathEvalExpression( (const xmlChar *)strXPath.c_str(), context);
	xmlXPathFreeContext(context);

	if(xmlXPathNodeSetIsEmpty(xpath->nodesetval))
	{
		return NULL;
	}

	return xpath->nodesetval;
}

void SetAttribute( xmlNode * pNode, std::string attName, double attValue )
{
	START_LOC

	char szTmp[32];
	sprintf(szTmp, "%.18e", attValue );
	xmlNewProp( pNode, (const xmlChar *)attName.c_str(), (const xmlChar *)szTmp );

	FINISH_LOC
}

void SetAttribute( xmlNode * pNode, std::string attName, int attValue )
{
	char szTmp[32];
	sprintf(szTmp, "%d", attValue );
	xmlNewProp( pNode, (const xmlChar *)attName.c_str(), (const xmlChar *)szTmp );
}

void SetAttribute( xmlNode * pNode, std::string attName, bool attValue )
{
	char szTmp[32];
	if (attValue)
	{
		strcpy( szTmp, "true" );
	}
	else
	{
		strcpy( szTmp, "false" );
	}

	xmlNewProp( pNode, (const xmlChar *)attName.c_str(), (const xmlChar *)szTmp );
}

void SetTextNode( xmlDoc * pXmlDoc, xmlNode * pNode, std::string textValue )
{
	xmlNode * textNode = xmlNewDocText( pXmlDoc, (const xmlChar *)textValue.c_str() );
	xmlAddChild( pNode, textNode );
}

void SetTextNode( xmlDoc * pXmlDoc, xmlNode * pNode, int textValue )
{
	char szTmp[32];
	sprintf(szTmp, "%d", textValue );
	xmlNode * textNode = xmlNewDocText( pXmlDoc, (const xmlChar *)szTmp );
	xmlAddChild( pNode, textNode );
}


void SetTextNode( xmlDoc * pXmlDoc, xmlNode * pNode, double textValue )
{
	START_LOC

	char szTmp[32];
	sprintf(szTmp, "%.18e", textValue );
	xmlNode * textNode = xmlNewDocText( pXmlDoc, (const xmlChar *)szTmp );
	xmlAddChild( pNode, textNode );

	FINISH_LOC
}

void SetTextNode( xmlDoc * pXmlDoc, xmlNode * pNode, JakubSchwarzmeier::Std::Math::Matrix<> & matrix )
{
	START_LOC

	std::strstream strStream;
	strStream << std::setiosflags( std::ios_base::scientific | std::ios_base::showpoint );
	strStream << std::setprecision( 18 );
	for(size_t row = 0; row < matrix.GetRows(); row++)
	{
		for(size_t column = 0; column < matrix.GetColumns(); column++)
		{
			strStream << matrix.GetValue(column, row) << " ";
		}
	}

	strStream << '\0';

	xmlNode * textNode = xmlNewDocText( pXmlDoc, (const xmlChar *)strStream.str() );
	xmlAddChild( pNode, textNode );
	SetAttribute( pNode, "r", (int)matrix.GetRows() );
	SetAttribute( pNode, "c", (int)matrix.GetColumns() );

	FINISH_LOC
}

void SetTextNode( xmlDoc * pXmlDoc, xmlNode * pNode, JakubSchwarzmeier::Std::Math::Quaternion<> & q )
{
	START_LOC

	xmlNode * pScalarNode = xmlNewNode( NULL, (const xmlChar *)"scalar" );
	xmlAddChild( pNode, pScalarNode );
	SetTextNode( pXmlDoc, pScalarNode, q.GetScalar() );

	xmlNode * pVectorNode = xmlNewNode( NULL, (const xmlChar *)"vector" );
	xmlAddChild( pNode, pVectorNode );
	SetTextNode( pXmlDoc, pVectorNode, q.GetVector() );

	FINISH_LOC
}

void SetTextNode( xmlDoc * pXmlDoc, xmlNode * pNode, JakubSchwarzmeier::Std::Math::Vector<> & v )
{
	START_LOC

	std::strstream strStream;
	strStream << std::setiosflags( std::ios_base::scientific | std::ios_base::showpoint );
	strStream << std::setprecision( 18 );
	for(size_t row = 0; row < v.GetRows(); row++)
	{
		for(size_t column = 0; column < v.GetColumns(); column++)
		{
			strStream << v.GetValue(column, row) << " ";
		}
	}

	strStream << '\0';

	xmlNode * textNode = xmlNewDocText( pXmlDoc, (const xmlChar *)strStream.str() );
	xmlAddChild( pNode, textNode );
	SetAttribute( pNode, "r", (int)v.GetRows() );
	SetAttribute( pNode, "c", (int)v.GetColumns() );

	FINISH_LOC
}


void GetTextNode( xmlDoc * pXmlDoc, xmlNode * pNode, JakubSchwarzmeier::Std::Math::Matrix<> * matrix )
{
	START_LOC

	xmlChar * attr = xmlGetProp(pNode, (const xmlChar *) "r" );
	int rows = atoi( (const char*) attr );
	xmlFree(attr);

	attr = xmlGetProp(pNode, (const xmlChar *) "c" );
	int columns = atoi( (const char*) attr );
	xmlFree(attr);

	std::string strMatrix = GetXPathResponse( pXmlDoc, pNode, "./text()" );
	std::strstream strStream;
	strStream << strMatrix;
	
	for(int row = 0; row < rows; row++)
	{
		for(int column = 0; column < columns; column++)
		{
			double d;
			strStream >> d;
			matrix->SetValue(column, row, d);
		}
	}	
	
	FINISH_LOC
}

void GetTextNode( xmlDoc * pXmlDoc, xmlNode * pNode, JakubSchwarzmeier::Std::Math::Quaternion<> * q )
{
	START_LOC

	std::string xpathScalar( "scalar/text()" );
	std::string strScalar = GetXPathResponse( pXmlDoc, pNode, xpathScalar );
	double scalar = atof( strScalar.c_str() );
	q->SetScalar( scalar );

	xmlNode * pVectorNode = SelectSingleNode( pXmlDoc, pNode, "vector" );
	JakubSchwarzmeier::Std::Math::Vector<> vec;
	GetTextNode( pXmlDoc, pVectorNode, &vec );
	q->SetVector( vec );

	FINISH_LOC
}

void GetTextNode( xmlDoc * pXmlDoc, xmlNode * pNode, JakubSchwarzmeier::Std::Math::Vector<> * v )
{
	START_LOC

	xmlChar * attr = xmlGetProp(pNode, (const xmlChar *) "r" );
	int rows = atoi( (const char*) attr );
	xmlFree(attr);

	attr = xmlGetProp(pNode, (const xmlChar *) "c" );
	int columns = atoi( (const char*) attr );
	xmlFree(attr);

	std::string strMatrix = GetXPathResponse( pXmlDoc, pNode, "./text()" );
	std::strstream strStream;
	strStream << strMatrix;
	
	for(int row = 0; row < rows; row++)
	{
		for(int column = 0; column < columns; column++)
		{
			double d;
			strStream >> d;
			v->SetValue(column, row, d);
		}
	}	
	
	FINISH_LOC
}

void GetAttribute( xmlNode * pNode, std::string attName, double * retValue )
{
	START_LOC

	xmlChar * attr = xmlGetProp(pNode, (const xmlChar *) attName.c_str() );
	*retValue = _tstof( (const char*) attr );
	xmlFree(attr);

	FINISH_LOC
}

void GetAttribute( xmlNode * pNode, std::string attName, int * retValue )
{
	xmlChar * attr = xmlGetProp(pNode, (const xmlChar *) attName.c_str() );
	*retValue = atoi( (const char*) attr );
	xmlFree(attr);
}

void GetAttribute( xmlNode * pNode, std::string attName, bool * retValue )
{
	xmlChar * attr = xmlGetProp(pNode, (const xmlChar *) attName.c_str() );
	if ( ( strcmp( reinterpret_cast<const char *>(attr), "true" ) == 0 ) || ( strcmp( reinterpret_cast<const char *>(attr), "1" ) == 0 ) )
	{
		*retValue = true;
	}
	else
	{
		*retValue = false;
	}

	xmlFree(attr);
}

