/////////////////////////////////////////////////////////////////////////////
// Name:        Observer.cpp
// Purpose:     
// Author:      
// Modified by: 
// Created:     08/21/06 10:24:11
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

#include "stdwx.h"

#if defined(__GNUG__) && !defined(__APPLE__)
#pragma implementation "Observer.h"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
////@end includes

#include "Observer.h"
#include "valdouble.h"

////@begin XPM images
////@end XPM images

/*!
 * CObserver type definition
 */

using namespace JakubSchwarzmeier::Std::Graphics;
using namespace JakubSchwarzmeier::Std::Math;
using namespace JakubSchwarzmeier::wx;

IMPLEMENT_DYNAMIC_CLASS( CObserver, wxDialog )

/*!
 * CObserver event table definition
 */

BEGIN_EVENT_TABLE( CObserver, wxDialog )

////@begin CObserver event table entries
    EVT_BUTTON( XRCID("IDC_ARROW_RIGHT_UP"), CObserver::OnCArrowRightUpClick )

    EVT_BUTTON( XRCID("IDC_ARROW_UP"), CObserver::OnCArrowUpClick )

    EVT_BUTTON( XRCID("IDC_ARROW_LEFT"), CObserver::OnCArrowLeftClick )

    EVT_BUTTON( XRCID("IDC_ARROW_RIGHT"), CObserver::OnCArrowRightClick )

    EVT_BUTTON( XRCID("IDC_ARROW_DOWN"), CObserver::OnCArrowDownClick )

    EVT_BUTTON( XRCID("IDC_ARROW_LEFT_DOWN"), CObserver::OnCArrowLeftDownClick )

    EVT_BUTTON( XRCID("ID_HEADING_LEFT"), CObserver::OnHeadingLeftClick )

    EVT_BUTTON( XRCID("ID_HEADING_RIGHT"), CObserver::OnHeadingRightClick )

    EVT_BUTTON( XRCID("ID_PITCH_UP"), CObserver::OnPitchUpClick )

    EVT_BUTTON( XRCID("ID_PITCH_DOWN"), CObserver::OnPitchDownClick )

    EVT_BUTTON( XRCID("ID_ROLL_CCW"), CObserver::OnRollCcwClick )

    EVT_BUTTON( XRCID("ID_ROLL_CW"), CObserver::OnRollCwClick )

    EVT_BUTTON( XRCID("ID_APPLY2"), CObserver::OnApply2Click )

    EVT_BUTTON( wxID_CANCEL, CObserver::OnCancelClick )

    EVT_BUTTON( XRCID("ID_RESETANGLES"), CObserver::OnResetAnglesClick )

    EVT_BUTTON( XRCID("ID_INITIAL"), CObserver::OnInitialClick )

    EVT_BUTTON( wxID_APPLY, CObserver::OnApplyClick )

////@end CObserver event table entries

END_EVENT_TABLE()

/*!
 * CObserver constructors
 */

 CObserver::CObserver( )
{
	//  initialize field of pressed keys
	for(int i = 0; i < 0x400; i++)	m_keys[i] = false;

	m_stepSize.position = 0.09;
	m_stepSize.angle = 0.8;

	m_framesPerSecond = 0.0;

	m_pCamera = NULL;
}

void CObserver::OnCreateCamera()
{
	if (m_pCamera == NULL)
		m_pCamera = new RotationMatrix;
}


CObserver::CObserver( wxWindow* parent, JakubSchwarzmeier::wx::COpenGLCanvas * pglCanvas, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Create(parent, pglCanvas, id, caption, pos, size, style);
}

CObserver::~CObserver()
{
	if (m_pCamera != NULL)
		delete m_pCamera;
}

/*!
 * CObserver creator
 */

bool CObserver::Create( wxWindow* parent, JakubSchwarzmeier::wx::COpenGLCanvas * pglCanvas, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CObserver member initialisation
////@end CObserver member initialisation

    m_pglCanvas = pglCanvas;

	SetExtraStyle(GetExtraStyle()|wxWS_EX_VALIDATE_RECURSIVELY);

////@begin CObserver creation
    SetExtraStyle(GetExtraStyle()|wxWS_EX_BLOCK_EVENTS);
    SetParent(parent);
    CreateControls();
    GetSizer()->Fit(this);
    GetSizer()->SetSizeHints(this);
    Centre();
////@end CObserver creation
    return true;
}

/*!
 * Control creation for CObserver
 */

void CObserver::CreateControls()
{    
	wxXmlResource::Get()->Load(wxT("res/observer.xrc"));

////@begin CObserver content construction
    wxXmlResource::Get()->LoadDialog(this, GetParent(), _T("IDD_OBSERVER"));
    // Set validators
    if (FindWindow(XRCID("ID_TEXTCTRL")))
        FindWindow(XRCID("ID_TEXTCTRL"))->SetValidator( wxDoubleValidator(& m_movementPositionStep) );
    if (FindWindow(XRCID("ID_TEXTCTRL1")))
        FindWindow(XRCID("ID_TEXTCTRL1"))->SetValidator( wxDoubleValidator(& m_movementAngleStep) );
    if (FindWindow(XRCID("ID_ABS_PX")))
        FindWindow(XRCID("ID_ABS_PX"))->SetValidator( wxDoubleValidator(& m_abs_px) );
    if (FindWindow(XRCID("ID_ABS_PY")))
        FindWindow(XRCID("ID_ABS_PY"))->SetValidator( wxDoubleValidator(& m_abs_py) );
    if (FindWindow(XRCID("ID_ABS_PZ")))
        FindWindow(XRCID("ID_ABS_PZ"))->SetValidator( wxDoubleValidator(& m_abs_pz) );
    if (FindWindow(XRCID("ID_HEADING")))
        FindWindow(XRCID("ID_HEADING"))->SetValidator( wxDoubleValidator(& m_heading) );
    if (FindWindow(XRCID("ID_PITCH")))
        FindWindow(XRCID("ID_PITCH"))->SetValidator( wxDoubleValidator(& m_pitch) );
    if (FindWindow(XRCID("ID_ROLL")))
        FindWindow(XRCID("ID_ROLL"))->SetValidator( wxDoubleValidator(& m_roll) );
    if (FindWindow(XRCID("ID_REL_PX")))
        FindWindow(XRCID("ID_REL_PX"))->SetValidator( wxDoubleValidator(& m_rel_px) );
    if (FindWindow(XRCID("ID_REL_PY")))
        FindWindow(XRCID("ID_REL_PY"))->SetValidator( wxDoubleValidator(& m_rel_py) );
    if (FindWindow(XRCID("ID_REL_PZ")))
        FindWindow(XRCID("ID_REL_PZ"))->SetValidator( wxDoubleValidator(& m_rel_pz) );
////@end CObserver content construction

    // Create custom windows not generated automatically here.

////@begin CObserver content initialisation

////@end CObserver content initialisation

	m_initial = *m_pCamera;
    OnUpdateInputs();
}

/*!
 * Should we show tooltips?
 */

bool CObserver::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CObserver::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CObserver bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CObserver bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CObserver::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CObserver icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CObserver icon retrieval
}
/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for IDC_ARROW_RIGHT_UP
 */

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_INITIAL
 */

void CObserver::OnInitialClick( wxCommandEvent& event )
{
	*m_pCamera = m_initial;
    OnUpdateInputs();
	m_pglCanvas->ReRender();
}


void CObserver::OnUpdateInputs()
{
	//  absolute values

	SetAbsPx( m_pCamera->GetPosition().GetValue(0,0) );
	SetAbsPy( m_pCamera->GetPosition().GetValue(0,1) );
	SetAbsPz( m_pCamera->GetPosition().GetValue(0,2) );

	SetMovementPositionStep( m_stepSize.position );
	SetMovementAngleStep( m_stepSize.angle );

	//  end absolute values

	//  relative values

	SetHeading( 0.0 );
	SetPitch( 0.0 );
	SetRoll( 0.0 );

	SetRelPx( 0.0 );
	SetRelPy( 0.0 );
	SetRelPz( 0.0 );

	//  end relative values

    TransferDataToWindow();
}

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_APPLY
 */

void CObserver::OnApplyClick( wxCommandEvent& event )
{
    if ( Validate() && TransferDataFromWindow() )
	{
		//  set absolute values
		if ( Vector<>( GetAbsPx(), GetAbsPy(), GetAbsPz() ) != m_pCamera->GetPosition() )
		{
			m_pCamera->SetPosition( Vector<>( GetAbsPx(), GetAbsPy(), GetAbsPz() ) );
		}
		//  end set absolute values

		//  set relative values
		if (GetHeading() != 0) 
		{
			m_pCamera->ChangeHeading( GetHeading() );
		}
		if (GetPitch() != 0)
		{
			m_pCamera->ChangePitch( GetPitch() );
		}
		if (GetRoll() != 0)
		{
			m_pCamera->ChangeRoll( GetRoll() );
		}
		if (GetRelPx() != 0)
		{
			m_pCamera->ChangePosition( Vector<>( GetRelPx(), 0, 0 ) );
		}
		if (GetRelPy() != 0)
		{
			m_pCamera->ChangePosition( Vector<>( 0, GetRelPy(), 0 ) );
		}
		if (GetRelPz() != 0)
		{
			m_pCamera->ChangePosition( Vector<>( 0, 0, GetRelPz() ) );
		}
		//  end set relative values

		OnUpdateInputs();

		TransferDataToWindow();

		m_pglCanvas->ReRender();
	}
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_CANCEL
 */

void CObserver::OnCancelClick( wxCommandEvent& event )
{
	wxDialog::OnCancel( event );
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for IDC_ARROW_RIGHT_UP
 */

void CObserver::OnCArrowRightUpClick( wxCommandEvent& event )
{
	m_pCamera->ChangePosition( Vector<>(0, 0, -m_stepSize.position) );
	OnUpdateInputs();
	m_pglCanvas->ReRender();

////@begin wxEVT_COMMAND_BUTTON_CLICKED event handler for IDC_ARROW_RIGHT_UP in CObserver.
    // Before editing this code, remove the block markers.
    event.Skip();
////@end wxEVT_COMMAND_BUTTON_CLICKED event handler for IDC_ARROW_RIGHT_UP in CObserver. 
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for IDC_ARROW_UP
 */

void CObserver::OnCArrowUpClick( wxCommandEvent& event )
{
	m_pCamera->ChangePosition( Vector<>(0, m_stepSize.position, 0) );
	OnUpdateInputs();
	m_pglCanvas->ReRender();

////@begin wxEVT_COMMAND_BUTTON_CLICKED event handler for IDC_ARROW_UP in CObserver.
    // Before editing this code, remove the block markers.
    event.Skip();
////@end wxEVT_COMMAND_BUTTON_CLICKED event handler for IDC_ARROW_UP in CObserver. 
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for IDC_ARROW_LEFT
 */

void CObserver::OnCArrowLeftClick( wxCommandEvent& event )
{
	m_pCamera->ChangePosition( Vector<>(-m_stepSize.position, 0, 0) );
	OnUpdateInputs();
	m_pglCanvas->ReRender();

////@begin wxEVT_COMMAND_BUTTON_CLICKED event handler for IDC_ARROW_LEFT in CObserver.
    // Before editing this code, remove the block markers.
    event.Skip();
////@end wxEVT_COMMAND_BUTTON_CLICKED event handler for IDC_ARROW_LEFT in CObserver. 
}

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for IDC_ARROW_RIGHT
 */

void CObserver::OnCArrowRightClick( wxCommandEvent& event )
{
	m_pCamera->ChangePosition( Vector<>(m_stepSize.position, 0, 0) );
	OnUpdateInputs();
	m_pglCanvas->ReRender();

////@begin wxEVT_COMMAND_BUTTON_CLICKED event handler for IDC_ARROW_RIGHT in CObserver.
    // Before editing this code, remove the block markers.
    event.Skip();
////@end wxEVT_COMMAND_BUTTON_CLICKED event handler for IDC_ARROW_RIGHT in CObserver. 
}

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for IDC_ARROW_DOWN
 */

void CObserver::OnCArrowDownClick( wxCommandEvent& event )
{
	m_pCamera->ChangePosition( Vector<>(0, -m_stepSize.position, 0) );
	OnUpdateInputs();
	m_pglCanvas->ReRender();

////@begin wxEVT_COMMAND_BUTTON_CLICKED event handler for IDC_ARROW_DOWN in CObserver.
    // Before editing this code, remove the block markers.
    event.Skip();
////@end wxEVT_COMMAND_BUTTON_CLICKED event handler for IDC_ARROW_DOWN in CObserver. 
}

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for IDC_ARROW_LEFT_DOWN
 */

void CObserver::OnCArrowLeftDownClick( wxCommandEvent& event )
{
	m_pCamera->ChangePosition( Vector<>(0, 0, m_stepSize.position) );
	OnUpdateInputs();
	m_pglCanvas->ReRender();

////@begin wxEVT_COMMAND_BUTTON_CLICKED event handler for IDC_ARROW_LEFT_DOWN in CObserver.
    // Before editing this code, remove the block markers.
    event.Skip();
////@end wxEVT_COMMAND_BUTTON_CLICKED event handler for IDC_ARROW_LEFT_DOWN in CObserver. 
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_HEADING_LEFT
 */

void CObserver::OnHeadingLeftClick( wxCommandEvent& event )
{
	m_pCamera->ChangeHeading( m_stepSize.angle );
	OnUpdateInputs();
	m_pglCanvas->ReRender();

////@begin wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_HEADING_LEFT in CObserver.
    // Before editing this code, remove the block markers.
    event.Skip();
////@end wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_HEADING_LEFT in CObserver. 
}

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_HEADING_RIGHT
 */

void CObserver::OnHeadingRightClick( wxCommandEvent& event )
{
	m_pCamera->ChangeHeading( -m_stepSize.angle );
	OnUpdateInputs();
	m_pglCanvas->ReRender();

////@begin wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_HEADING_RIGHT in CObserver.
    // Before editing this code, remove the block markers.
    event.Skip();
////@end wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_HEADING_RIGHT in CObserver. 
}

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_PITCH_UP
 */

void CObserver::OnPitchUpClick( wxCommandEvent& event )
{
	m_pCamera->ChangePitch( -m_stepSize.angle );
	OnUpdateInputs();
	m_pglCanvas->ReRender();

////@begin wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_PITCH_UP in CObserver.
    // Before editing this code, remove the block markers.
    event.Skip();
////@end wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_PITCH_UP in CObserver. 
}

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_PITCH_DOWN
 */

void CObserver::OnPitchDownClick( wxCommandEvent& event )
{
	m_pCamera->ChangePitch( m_stepSize.angle );
	OnUpdateInputs();
	m_pglCanvas->ReRender(); 

////@begin wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_PITCH_DOWN in CObserver.
    // Before editing this code, remove the block markers.
    event.Skip();
////@end wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_PITCH_DOWN in CObserver. 
}

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_ROLL_CCW
 */

void CObserver::OnRollCcwClick( wxCommandEvent& event )
{
	m_pCamera->ChangeRoll( -m_stepSize.angle );
	OnUpdateInputs();
	m_pglCanvas->ReRender();
////@begin wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_ROLL_CCW in CObserver.
    // Before editing this code, remove the block markers.
    event.Skip();
////@end wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_ROLL_CCW in CObserver. 
}

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_ROLL_CW
 */

void CObserver::OnRollCwClick( wxCommandEvent& event )
{
	m_pCamera->ChangeRoll( m_stepSize.angle );
	OnUpdateInputs();
	m_pglCanvas->ReRender();

////@begin wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_ROLL_CW in CObserver.
    // Before editing this code, remove the block markers.
    event.Skip();
////@end wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_ROLL_CW in CObserver. 
}

void CObserver::SetCamera( JakubSchwarzmeier::Std::Graphics::RotationMatrix & camera )
{
	*m_pCamera = camera;
}

JakubSchwarzmeier::Std::Graphics::RotationMatrix * CObserver::GetCamera()
{
	return m_pCamera;
}

void CObserver::CalculateFrameRate()
{
	static double framesPerSecond    = 0.0f;		// This will store our fps
    static double lastTime			= 0.0f;	// This will hold the time from the last frame
	static double frameInterval = 0.0f;			

	// Here we get the current tick count and multiply it by 0.001 to convert it from milliseconds to seconds.
	// GetTickCount() returns milliseconds (1000 ms = 1 second) so we want something more intuitive to work with.
    double currentTime = clock() / CLOCKS_PER_SEC;

	// Here we store the elapsed time between the current and last frame,
	// then keep the current frame in our static variable for the next frame.
 	frameInterval = currentTime - lastTime;
	lastTime = currentTime;

	// Increase the frame counter
    ++framesPerSecond;

	// Now we want to subtract the current time by the last time that was stored
	// to see if the time elapsed has been over a second, which means we found our FPS.
    if( frameInterval > 1.0f )
    {
		// Here we set the lastTime to the currentTime
	    lastTime = currentTime;
		
		//  framesPerSecond is valid value
		m_framesPerSecond = framesPerSecond;

		// Reset the frames per second
        framesPerSecond = 0;
    }

}
 
void CObserver::SetPositionSpeed(double stepSize)
{
	m_stepSize.position = stepSize;
}

void CObserver::SetAngleSpeed(double stepSize)
{
	m_stepSize.angle = stepSize;
}

double CObserver::GetPositionSpeed()
{
	return m_stepSize.position;
}

double CObserver::GetAngleSpeed()
{
	return m_stepSize.angle;
}


void CObserver::OnMouse(wxMouseEvent& event)
{
}

void CObserver::OnKeyDown(wxKeyEvent & event)
{
	m_keys[ event.GetKeyCode() ] = true;
	if ( UpdateInputs() )
	{
		m_pglCanvas->ReRender();
	}
	else
	{
		event.Skip();
	}
}

void CObserver::OnKeyUp(wxKeyEvent & event)
{
	m_keys[ event.GetKeyCode() ] = false;
	if ( UpdateInputs() )
	{
		m_pglCanvas->ReRender();
	}
	else
	{
		event.Skip();
	}
}

bool CObserver::UpdateInputs()
{
	bool shift;
	if (m_keys[WXK_SHIFT])
		shift = true;
	else
		shift = false;

	CalculateFrameRate();

	// Once we have the frame interval, we just need to find the current speed.
	// This is done by multiplying our speed by the elapsed time between frames.
	// We can then pass that speed into our camera movement functions.
	float positionSpeed = m_stepSize.position / m_framesPerSecond;
	float angleSpeed = m_stepSize.angle / m_framesPerSecond;

	bool used = false;

	if ( m_keys[WXK_UP] && !shift )		
	{
		m_pCamera->ChangePosition(Vector<>(0, 0, -positionSpeed));
		OnUpdateInputs();
		used = true;
	}

	if ( m_keys[WXK_DOWN] && !shift )
	{
		m_pCamera->ChangePosition(Vector<>(0, 0, positionSpeed));
		OnUpdateInputs();
		used = true;
	}

	if ( m_keys[WXK_LEFT] && !shift )
	{
		m_pCamera->ChangeHeading( angleSpeed );
		OnUpdateInputs();
		used = true;
	}

	if ( m_keys[WXK_RIGHT] && !shift )	
	{
		m_pCamera->ChangeHeading( -angleSpeed );
		OnUpdateInputs();
		used = true;
	}

	if ( m_keys[WXK_HOME] && !shift )
	{
		m_pCamera->ChangeRoll(angleSpeed);
		OnUpdateInputs();
		used = true;
	}

	if ( m_keys[WXK_END] && !shift )		
	{
		m_pCamera->ChangeRoll(-angleSpeed);
		OnUpdateInputs();
		used = true;
	}

	if ( m_keys[WXK_PRIOR] && !shift )
	{
		m_pCamera->ChangePosition(Vector<>(0, positionSpeed, 0));
		OnUpdateInputs();
		used = true;
	}

	if ( m_keys[WXK_NEXT] && !shift )	
	{
		m_pCamera->ChangePosition(Vector<>(0, -positionSpeed, 0));
		OnUpdateInputs();
		used = true;
	}
		
	if ( m_keys[WXK_INSERT] && !shift )		
	{
		m_pCamera->ChangePitch( angleSpeed );
		OnUpdateInputs();
		used = true;
	}

	if ( m_keys[WXK_DELETE]&& !shift )
	{
		m_pCamera->ChangePitch( -angleSpeed );
		OnUpdateInputs();
		used = true;
	}

	if ( m_keys[WXK_LEFT] && shift )		
	{
		m_pCamera->ChangePosition(Vector<>(-positionSpeed, 0, 0));
		OnUpdateInputs();
		used = true;
	}

	if ( m_keys[WXK_RIGHT] && shift )	
	{
		m_pCamera->ChangePosition(Vector<>(positionSpeed, 0, 0));
		OnUpdateInputs();
		used = true;
	}

	return used;
}

void CObserver::Show( wxWindow * parent, JakubSchwarzmeier::wx::COpenGLCanvas * pglCanvas )
{
	//  try to find me in apparent parent's list
    for ( wxWindowList::compatibility_iterator node = parent->GetChildren().GetFirst();
            node;
            node = node->GetNext() )
    {
        wxWindow * win = (wxWindow *)node->GetData();
		if (win == this)
		{
			win->Show( true );
			return;
		}
	}

	Create( parent, pglCanvas );
	wxDialog::Show(true);

	return;
}

void CObserver::SetOpenGLCanvas( JakubSchwarzmeier::wx::COpenGLCanvas * pglCanvas )
{
	m_pglCanvas = pglCanvas;
}

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_APPLY2
 */

void CObserver::OnApply2Click( wxCommandEvent& event )
{
    if ( Validate() && TransferDataFromWindow() )
	{
		if ( GetMovementPositionStep() != m_stepSize.position )
		{
			SetPositionSpeed( GetMovementPositionStep() );
		}
		if ( GetMovementAngleStep() != m_stepSize.angle )
		{
			SetAngleSpeed( GetMovementAngleStep() );
		}

		TransferDataToWindow();

		m_pglCanvas->ReRender();

	}
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_RESETANGLES
 */

void CObserver::OnResetAnglesClick( wxCommandEvent& event )
{
	m_pCamera->ResetAngles( 0, 0, 0 );
	OnUpdateInputs();
	m_pglCanvas->ReRender();

////@begin wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_RESETANGLES in CObserver.
    // Before editing this code, remove the block markers.
    event.Skip();
////@end wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_RESETANGLES in CObserver. 
}


