/////////////////////////////////////////////////////////////////////////////
// Name:        ObserverDlg.cpp
// Purpose:     
// Author:      
// Modified by: 
// Created:     11/01/05 15:43:56
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

#include "stdwx.h"

#if defined(__GNUG__) && !defined(__APPLE__)
#pragma implementation "OpenGLCanvasDlg.h"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
////@end includes

#include "OpenGLCanvasDlg.h"
#include <schwarzmeier/std.h>

using namespace JakubSchwarzmeier::Std::Math;

////@begin XPM images
////@end XPM images

/*!
 * COpenGLCanvasDlg type definition
 */

using namespace JakubSchwarzmeier::wx;

IMPLEMENT_DYNAMIC_CLASS( COpenGLCanvasDlg, wxDialog )

/*!
 * COpenGLCanvasDlg event table definition
 */

BEGIN_EVENT_TABLE( COpenGLCanvasDlg, wxDialog )

////@begin COpenGLCanvasDlg event table entries
    EVT_CHECKBOX( XRCID("ID_CHECKBOX"), COpenGLCanvasDlg::OnTexturingSelected )

    EVT_CHECKBOX( XRCID("ID_LIGHTING"), COpenGLCanvasDlg::OnLightingClick )

    EVT_CHECKBOX( XRCID("ID_COLORMATERIAL"), COpenGLCanvasDlg::OnColormaterialClick )

    EVT_CHECKBOX( XRCID("ID_FACECULLING"), COpenGLCanvasDlg::OnFacecullingClick )

    EVT_CHOICE( XRCID("ID_POLYGON_MODE"), COpenGLCanvasDlg::OnPolygonModeSelected )

////@end COpenGLCanvasDlg event table entries

END_EVENT_TABLE()

/*!
 * COpenGLCanvasDlg constructors
 */

COpenGLCanvasDlg::COpenGLCanvasDlg( )
{
}

COpenGLCanvasDlg::COpenGLCanvasDlg( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Create(parent, id, caption, pos, size, style);
}

COpenGLCanvasDlg::COpenGLCanvasDlg( wxWindow* parent, JakubSchwarzmeier::wx::COpenGLCanvas * pglCanvas, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
	m_pglCanvas = pglCanvas;

	Create(parent, id, caption, pos, size, style);
}

COpenGLCanvasDlg::~COpenGLCanvasDlg( )
{
}

/*!
 * COpenGLCanvasDlg creator
 */

bool COpenGLCanvasDlg::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin COpenGLCanvasDlg member initialisation
    m_notebook = NULL;
    m_texturingCtrl = NULL;
    m_lightingCtrl = NULL;
    m_colorMaterialCtrl = NULL;
    m_faceCullingCtrl = NULL;
    m_polygonModeCtrl = NULL;
    m_infosizer = NULL;
    m_vendor = NULL;
    m_renderer = NULL;
    m_version = NULL;
    m_extensions = NULL;
////@end COpenGLCanvasDlg member initialisation

////@begin COpenGLCanvasDlg creation
    SetExtraStyle(GetExtraStyle()|wxWS_EX_BLOCK_EVENTS);
    SetParent(parent);
    CreateControls();
    GetSizer()->Fit(this);
    GetSizer()->SetSizeHints(this);
    Centre();
////@end COpenGLCanvasDlg creation
    return true;
}

/*!
 * Control creation for COpenGLCanvasDlg
 */

void COpenGLCanvasDlg::CreateControls()
{    
	wxXmlResource::Get()->Load(wxT("res/openglcanvasdlg.xrc"));
////@begin COpenGLCanvasDlg content construction
    wxXmlResource::Get()->LoadDialog(this, GetParent(), _T("ID_OPENGLCANVASDLG"));
    m_notebook = XRCCTRL(*this, "ID_NOTEBOOK", wxNotebook);
    m_texturingCtrl = XRCCTRL(*this, "ID_CHECKBOX", wxCheckBox);
    m_lightingCtrl = XRCCTRL(*this, "ID_LIGHTING", wxCheckBox);
    m_colorMaterialCtrl = XRCCTRL(*this, "ID_COLORMATERIAL", wxCheckBox);
    m_faceCullingCtrl = XRCCTRL(*this, "ID_FACECULLING", wxCheckBox);
    m_polygonModeCtrl = XRCCTRL(*this, "ID_POLYGON_MODE", wxChoice);
    m_vendor = XRCCTRL(*this, "ID_VENDOR", wxStaticText);
    m_renderer = XRCCTRL(*this, "ID_RENDERER", wxStaticText);
    m_version = XRCCTRL(*this, "ID_VERSION", wxStaticText);
    m_extensions = XRCCTRL(*this, "ID_LISTCTRL", wxListCtrl);
////@end COpenGLCanvasDlg content construction

//	m_winPosture->SetExtraStyle(GetExtraStyle()|wxWS_EX_VALIDATE_RECURSIVELY);

	m_pglCanvas->GetMyContext()->SetCurrent();
	m_vendor->SetLabel( glGetString(GL_VENDOR) );
	m_renderer->SetLabel( glGetString(GL_RENDERER) );
	m_version->SetLabel( glGetString(GL_VERSION) );

	//  Extensions	
	//
	wxListItem itemCol;
    itemCol.m_format = wxLIST_FORMAT_LEFT;
	itemCol.m_mask = wxLIST_MASK_FORMAT;
    m_extensions->InsertColumn(0, itemCol);

    // to speed up inserting we hide the control temporarily
    m_extensions->Hide();

	long l = 0;
    char * p = (char *) glGetString(GL_EXTENSIONS);
    char * end = p + strlen(p);
    while (p < end)
	{
		size_t n = strcspn(p, " ");
		wxString extName(p, n);
		m_extensions->InsertItem( l++, extName );

		p += (n + 1);
    }
	m_pglCanvas->GetMyContext()->UnsetCurrent();

	m_extensions->SetColumnWidth(0, wxLIST_AUTOSIZE);
	m_extensions->Show();

	//  Recalc size
	m_infosizer = (wxStaticBoxSizer*)FindWindow( XRCID("ID_VENDOR") )->GetContainingSizer();
	m_infosizer->GetStaticBox()->GetParent()->Layout();
	//
	//  Extensions	

	//  Settings
	//

	//  texturing
	if ( m_pglCanvas->m_bTexturing )
		m_texturingCtrl->SetValue(true);
	else
		m_texturingCtrl->SetValue(false);

	//  polygon mode
	switch( m_pglCanvas->m_polygonMode )
	{
	case GL_POINT: 
		
		m_polygonModeCtrl->SetSelection(0);
		break;

	case GL_LINE: 
		
		m_polygonModeCtrl->SetSelection(1);
		break;

	case GL_FILL: 
		
		m_polygonModeCtrl->SetSelection(2);
		break;
	}

	//  lighting
	if ( m_pglCanvas->m_bLighting )
		m_lightingCtrl->SetValue(true);
	else
		m_lightingCtrl->SetValue(false);

	//  material
	if ( m_pglCanvas->m_bColorMaterial )
		m_colorMaterialCtrl->SetValue(true);
	else
		m_colorMaterialCtrl->SetValue(false);

	//  face culling
	if ( m_pglCanvas->m_bCullFace )
		m_faceCullingCtrl->SetValue(true);
	else
		m_faceCullingCtrl->SetValue(false);


    // Create custom windows not generated automatically here.

////@begin COpenGLCanvasDlg content initialisation

////@end COpenGLCanvasDlg content initialisation
}

/*!
 * Should we show tooltips?
 */

bool COpenGLCanvasDlg::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap COpenGLCanvasDlg::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin COpenGLCanvasDlg bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end COpenGLCanvasDlg bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon COpenGLCanvasDlg::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin COpenGLCanvasDlg icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end COpenGLCanvasDlg icon retrieval
}
/*!
 * wxEVT_COMMAND_RADIOBUTTON_SELECTED event handler for ID_TEXTURING_ON
 */

void COpenGLCanvasDlg::OnTexturingSelected( wxCommandEvent& event )
{
	m_pglCanvas->m_bTexturing = m_texturingCtrl->GetValue();
	m_pglCanvas->ReRender();
}


/*!
 * wxEVT_COMMAND_CHOICE_SELECTED event handler for ID_POLYGON_MODE
 */

void COpenGLCanvasDlg::OnPolygonModeSelected( wxCommandEvent& event )
{
	switch( m_polygonModeCtrl->GetSelection() )
	{
	case 0: 
		
		m_pglCanvas->m_polygonMode = GL_POINT;
		break;

	case 1: 
		
		m_pglCanvas->m_polygonMode = GL_LINE;
		break;

	case 2: 
		
		m_pglCanvas->m_polygonMode = GL_FILL;
		break;
	}

	m_pglCanvas->ReRender();
}


/*!
 * wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_LIGHTING
 */

void COpenGLCanvasDlg::OnLightingClick( wxCommandEvent& event )
{
	m_pglCanvas->m_bLighting = m_lightingCtrl->GetValue();
	m_pglCanvas->ReRender();
}


/*!
 * wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_COLORMATERIAL
 */

void COpenGLCanvasDlg::OnColormaterialClick( wxCommandEvent& event )
{
	m_pglCanvas->m_bColorMaterial = m_colorMaterialCtrl->GetValue();
	m_pglCanvas->ReRender();
}


/*!
 * wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_FACECULLING
 */

void COpenGLCanvasDlg::OnFacecullingClick( wxCommandEvent& event )
{
	m_pglCanvas->m_bCullFace = m_faceCullingCtrl->GetValue();
	m_pglCanvas->ReRender();
}


