#include "stdwx.h"

#include "SerializeReaderWriter.h"
#include "DocChildFrame.h"
#include "OpenGLView.h"

using namespace JakubSchwarzmeier::wx;

IMPLEMENT_CLASS(CSerializeReaderWriter, wxObject)
IMPLEMENT_CLASS(CSerializeWriter, CSerializeReaderWriter)
IMPLEMENT_CLASS(CSerializeReader, CSerializeReaderWriter)

bool CSerializeReaderWriter::m_bIsResurrection = false;

bool CSerializeReaderWriter::IsResurrection()
{
	return m_bIsResurrection;
}

CSerializeWriter::CSerializeWriter( const wxString & fileName )
{
	m_fileName = fileName;
	m_id = 0;
	m_pXmlDoc = xmlNewDoc( (const xmlChar *)"1.0" );

	//  create new node
	xmlNode * pNode = xmlNewNode( NULL, (const xmlChar *)"archive" );
	xmlNewProp( pNode, (const xmlChar *)"class", (const xmlChar *)"CSerializeWriter" );

	//  connect root node to document
	xmlDocSetRootElement( m_pXmlDoc, pNode );

	m_nodeStack.push( pNode );
}

CSerializeWriter::~CSerializeWriter()
{
	xmlThrDefIndentTreeOutput( 1 );
	xmlKeepBlanksDefault(0);
	if ( xmlSaveFormatFile( m_fileName.GetData(), m_pXmlDoc, 1 ) == -1 )
	{
		//  failure
		return;
	}
}

bool CSerializeWriter::Value( double * val, wxString & name )
{
    

	return true;
}

bool CSerializeWriter::Value( wxString * val, wxString & name )
{
	CreateValueNode( wxString("wxString"), name, *val );
	return true;
}

bool CSerializeWriter::Value( void * val, wxString & name )
{
	wxString strVal;
	strVal << (int)val;
	CreateValueNode( wxString("pvoid"), name, strVal );
	return true;
}

bool CSerializeWriter::Append( wxObject ** obj, wxString & name )
{
	//  vytvor novy node v XML stromu
	//  <node class="obj->GetClassInfo()->GetClassName()" id=xxx>
	//  uloz pointer do hashe

	int num;
	bool isId = AddToMap( &num, *obj );
	CreateNode( wxString( (*obj)->GetClassInfo()->GetClassName() ), name, num, isId);
	if (isId == false)  
	{
		m_nodeStack.pop();
		return true;
	}

	wxObject * pObjTmp;

	//  Is it some known class?
	if ( (*obj)->IsKindOf( CLASSINFO(wxDocument) ) )
	{
		wxDocument * doc = wxDynamicCast(*obj, wxDocument);
        this->Append( &doc->GetViews(), wxString("m_documentViews") );
	}
	else
	if ( (*obj)->IsKindOf( CLASSINFO(wxView) ) )
	{
		wxView * view = wxDynamicCast(*obj, wxView);
        this->Value( &view->GetViewName(), wxString("m_viewTypeName") );
		pObjTmp = view->GetDocument();
		this->Append( reinterpret_cast<wxObject**>( &pObjTmp ), wxString("m_viewDocument") );
		pObjTmp = view->GetFrame();
		this->Append( reinterpret_cast<wxObject**>( &pObjTmp ), wxString("m_viewFrame") );
	}
	else
	if ( (*obj)->IsKindOf( CLASSINFO(wxDocMDIChildFrame) ) )
	{
		wxDocMDIChildFrame * frame = wxDynamicCast(*obj, wxDocMDIChildFrame);
		pObjTmp = frame->GetDocument();
		this->Append( reinterpret_cast<wxObject**>( &pObjTmp ), wxString("m_childDocument") );
		pObjTmp = frame->GetView();
		this->Append( reinterpret_cast<wxObject**>( &pObjTmp ), wxString("m_childView") );
	}	

	//  serialize rest of member values in descendant
	(*obj)->OnSerialize( this );

	m_nodeStack.pop();

	return true;
}

bool CSerializeWriter::Append( wxList * list, wxString & name )
{
	int num;
	bool isId = AddToMap( &num, list );
	CreateNode(wxString("wxList"), name, num, isId);

	wxString strCount;
	strCount << list->GetCount();
	xmlNewProp( m_nodeStack.top(), (const xmlChar *)"count", (const xmlChar *)strCount.GetData() );

	if (isId == false)  
	{
		m_nodeStack.pop();
		return true;
	}

	long i = 1;
	wxNode * node = list->GetFirst();
	while (node)
	{
		wxString strListElement("list_element");
		wxString strIndex;
		strIndex << i++;
		strListElement.Append(strIndex);

		wxObject * obj = wxDynamicCast(node->GetData(), wxObject);
		this->Append( &obj, strListElement );

		node = node->GetNext();
	}
	
	m_nodeStack.pop();

	return true;
}

void CSerializeWriter::CreateNode( wxString & className, wxString & name, int id, bool isId )
{
	//  create new node
	xmlNode * pNewNode = xmlNewNode( NULL, (const xmlChar *)name.GetData() );
	xmlNewProp( pNewNode, (const xmlChar *)"class", (const xmlChar *)className.GetData() );
	wxString strNum;
	strNum << id;
	wxString strIdRef;
	if (isId)
	{
		strIdRef << "id";
	}
	else
	{
		strIdRef << "ref";
	}

	xmlNewProp( pNewNode, (const xmlChar *)strIdRef.GetData(), (const xmlChar *)strNum.GetData() );

	//  plug new node into tree
	if ( xmlAddChild( m_nodeStack.top(), pNewNode ) == NULL )
	{
		//  failure
		return;
	}

	m_nodeStack.push( pNewNode );
}


void CSerializeWriter::CreateValueNode( wxString & className, wxString & name, wxString & value )
{
	//  create new node
	xmlNode * pNewNode = xmlNewNode( NULL, (const xmlChar *)name.GetData() );
	xmlNewProp( pNewNode, (const xmlChar *)"class", (const xmlChar *)className.GetData() );
	xmlNode * text = xmlNewDocText( m_pXmlDoc, (const xmlChar *)value.GetData() );
	xmlAddChild( pNewNode, text );

	//  plug new node into tree
	if ( xmlAddChild( m_nodeStack.top(), pNewNode ) == NULL )
	{
		//  failure
		return;
	}
}

bool CSerializeWriter::AddToMap( int * id, void * obj )
{
	bool isId;
	std::pair< MAP::iterator, bool> ret;
	ret = m_pointerMap.insert( MAP::value_type(obj, m_id) );
	if (ret.second)
	{
		isId = true;
		*id = m_id++;
	}
	else
	{
		isId = false;
		*id = (ret.first)->second;
	}

	return isId;
}

CSerializeReader::CSerializeReader( const wxString & fileName )
{
	m_pXmlDoc = xmlParseFile( (const char *)fileName.GetData() );
	if (m_pXmlDoc == NULL )
	{
//		fprintf(stderr,"Document not parsed successfully. \n");
		return;
	}

	m_id = 0;

	xmlNode * pNode = xmlDocGetRootElement(m_pXmlDoc);
	if (pNode == NULL)
	{
//		fprintf(stderr,"empty document\n");
		xmlFreeDoc(m_pXmlDoc);
		return;
	}

	m_nodeStack.push( pNode );

	CSerializeReaderWriter::m_bIsResurrection = true;
}

bool CSerializeReader::Append( wxObject ** obj, wxString & name )
{
	bool isId = ReadNode(name, obj);
	if (isId == false)  
	{
		m_nodeStack.pop();
		return true;
	}

	//  deserialize rest of member values in descendant
	(*obj)->OnSerialize( this );

	wxString strTmp;
	wxObject * pObjTmp;

	//  deserialize know classes
	if ( (*obj)->IsKindOf( CLASSINFO(wxDocument) ) )
	{
		wxDocument * doc = wxDynamicCast(*obj, wxDocument);
		this->Append( &doc->GetViews(), wxString("m_documentViews") );
	}
	else
	if ( (*obj)->IsKindOf( CLASSINFO(wxView) ) )
	{
		wxView * view = wxDynamicCast(*obj, wxView);
        
		this->Value( &strTmp, wxString("m_viewTypeName") );
		view->SetViewName(strTmp);

		this->Append( reinterpret_cast<wxObject**>( &pObjTmp ), wxString("m_viewDocument") );
		view->SetDocument( reinterpret_cast<wxDocument *>(pObjTmp) );
		this->Append( reinterpret_cast<wxObject**>( &pObjTmp  ), wxString("m_viewFrame") );
		view->SetFrame( reinterpret_cast<wxFrame *>(pObjTmp) );
	}
	else
	if ( (*obj)->IsKindOf( CLASSINFO(wxDocMDIChildFrame) ) )
	{
		wxDocMDIChildFrame * frame = wxDynamicCast(*obj, wxDocMDIChildFrame);
		this->Append( reinterpret_cast<wxObject**>( &pObjTmp ), wxString("m_childDocument") );
		frame->SetDocument( reinterpret_cast<wxDocument *>(pObjTmp) );
		this->Append( reinterpret_cast<wxObject**>( &pObjTmp ), wxString("m_childView") );
		frame->SetView( reinterpret_cast<wxView *>(pObjTmp) );

	}

	if ( (*obj)->IsKindOf( CLASSINFO(JakubSchwarzmeier::wx::CDocChildFrame) ) )
	{
		JakubSchwarzmeier::wx::CDocChildFrame * frame = wxDynamicCast(*obj, JakubSchwarzmeier::wx::CDocChildFrame);

		wxView * view = frame->GetView();
		frame->Create( frame->GetDocument(), NULL, reinterpret_cast<wxMDIParentFrame*>( wxTheApp->GetTopWindow() ), true );
		frame->SetView( view );
	}

	m_nodeStack.pop();

	return true;
}


bool CSerializeReader::Append( wxList * list, wxString & name )
{
	bool isId = ReadNode( name, reinterpret_cast<wxObject**>( &list ) );
	if (isId == false)  
	{
		m_nodeStack.pop();
		return true;
	}

	xmlChar * xmlCount = xmlGetProp(m_nodeStack.top(), (const xmlChar *)"count");
	wxString strCount(xmlCount);
	xmlFree(xmlCount);

	long l;
	strCount.ToLong( &l );
	
	for(long i = 1; i <= l; i++)
	{
		wxString strListElement("list_element");
		wxString strIndex;
		strIndex << i;
		strListElement.Append(strIndex);

		wxObject * obj = NULL;
		this->Append( &obj, strListElement );
		
		//  There is no need for manually adding view into document's view list.
		//  It is done automatically by the framework when calling view->SetDocument( ).
	}
	
	m_nodeStack.pop();

	return true;
}

bool CSerializeReader::ReadNode( wxString & name, wxObject ** pObj )
{
	wxString strXPathExp( (const xmlChar *)"./" );
	strXPathExp.Append(name.GetData());

	xmlXPathContext * context = xmlXPathNewContext(m_pXmlDoc);
	context->node = m_nodeStack.top();
	xmlXPathObject * xpath = xmlXPathEvalExpression( (const xmlChar *)strXPathExp.GetData(), context);
	xmlXPathFreeContext(context);

	xmlNode * pNode = xpath->nodesetval->nodeTab[0];
	xmlXPathFreeObject (xpath);

	//  Get class name
	xmlChar * classFromXml = xmlGetProp(pNode, (const xmlChar *)"class");
	if (classFromXml == NULL)
	{
		//  failure
	}

	wxString strClassName( classFromXml );
	xmlFree(classFromXml);

	//  Check if is it a new object or just reference to existing instance, read id
	xmlChar * refOrId = xmlGetProp(pNode, (const xmlChar *)"id");
	bool isId;
	if (refOrId == NULL)
	{
        refOrId = xmlGetProp(pNode, (const xmlChar *)"ref");	
		if (refOrId == NULL)
		{
			//  failure
		}
		//  ref
        isId = false;		
	}
	else
	{
		//  id
		isId = true;
	}

	wxString id(refOrId);
	xmlFree(refOrId);
	long l;
	if ( !id.ToLong( &l ) )
	{
		//  failure
	}

	if (isId)
	{
		wxClassInfo * serClassInfo = wxClassInfo::FindClass(strClassName);
		if ( serClassInfo->IsKindOf( CLASSINFO(wxList) ) )
		{
		}
		else
		if ( serClassInfo->IsKindOf( CLASSINFO(wxDocument) ) )
		{
		}
		else
		if ( serClassInfo->IsKindOf( CLASSINFO(wxDocChildFrame) ) )
		{
		}
		else
		{
            *pObj = ::wxCreateDynamicObject( strClassName );
		}
	}

	//  now compare values from xml with values from map
	std::pair< MAP::iterator, bool> ret;
	ret = m_pointerMap.insert( MAP::value_type(l, *pObj) );
	if (ret.second)
	{
		if (isId != true)  { /*  failure */ }
	}
	else
	{
		if (isId != false)  { /*  failure  */ }
		*pObj = (ret.first)->second;
	}

	m_nodeStack.push( pNode );

	return isId;
}

void CSerializeReader::ReadValueNode( wxString & className, wxString & name, wxString & value )
{
	wxString strXPathExp( (const xmlChar *)"./" );
	strXPathExp.Append(name.GetData());
	strXPathExp.Append( wxString("/text()") );

	xmlXPathContext * context = xmlXPathNewContext(m_pXmlDoc);
	context->node = m_nodeStack.top();
	xmlXPathObject * xpath = xmlXPathEvalExpression( (const xmlChar *)strXPathExp.GetData(), context);
	if(xmlXPathNodeSetIsEmpty(xpath->nodesetval))
	{
    //    printf("No result\n");
		return;
	}
	xmlXPathFreeContext(context);

	xmlChar * textValue = xmlNodeListGetString(m_pXmlDoc, xpath->nodesetval->nodeTab[0], 1);
	wxString strTextVal(textValue);
	xmlFree(textValue);

	value = strTextVal;

	//  Get class name
	xmlChar * classFromXml = xmlGetProp(xpath->nodesetval->nodeTab[0], (const xmlChar *)"class");
	if (classFromXml == NULL)
	{
		//  failure
	}
	xmlXPathFreeObject (xpath);

	wxString strClassName( classFromXml );
	xmlFree(classFromXml);

	//  compare class names
	if ( strClassName.CompareTo( strClassName ) != 0 )
	{
		//  failure
		return;
	}
}

bool CSerializeReader::Value( double * val, wxString & name )
{
	return true;
}

bool CSerializeReader::Value( wxString * val, wxString & name )
{
	ReadValueNode( wxString("wxString"), name, *val );
	return true;
}

bool CSerializeReader::Value( void * val, wxString & name )
{
	wxString strVal;
	ReadValueNode( wxString("pvoid"), name, strVal );
	strVal.ToLong( static_cast<long*>( val ) );
	return true;
}

CSerializeReader::~CSerializeReader()
{
	CSerializeReaderWriter::m_bIsResurrection = false;
}