/************************************************************
*
*  Copyright (c) 2005 Jakub Schwarzmeier
*
************************************************************/

#ifndef _WX_SERIALIZEREADERWRITER_H_
#define _WX_SERIALIZEREADERWRITER_H_

#include <stack>
#include <map>

#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <libxml/xpath.h>

namespace JakubSchwarzmeier
{
	namespace wx
	{
		class CSerializeReaderWriter: public wxObject
		{
		public:
			
			virtual ~CSerializeReaderWriter() {}

			virtual bool Value( double * val, wxString & name ) = 0;
			virtual bool Value( wxString * val, wxString & name ) = 0;
			virtual bool Value( void * val, wxString & name ) = 0;
/*			virtual bool Value( int * val, wxString & name ) = 0;
			virtual bool Value( wxString * val, wxString & name ) = 0;  */
			virtual bool Append( wxObject ** obj, wxString & name ) = 0;
			virtual bool Append( wxList * list, wxString & name ) = 0;

			static bool IsResurrection();

		protected:

			static bool m_bIsResurrection;

            DECLARE_CLASS(CSerializeReaderWriter)
		};	//  end class CSerializeReaderWriter

	
		class CSerializeWriter: public CSerializeReaderWriter
		{
		public:

			CSerializeWriter( const wxString & fileName );
			
			virtual bool Value( double * val, wxString & name );
			virtual bool Value( wxString * val, wxString & name );
			virtual bool Value( void * val, wxString & name );
/*			virtual bool Value( int * val, wxString & name );
			virtual bool Value( wxString * val, wxString & name ); */
			virtual bool Append( wxObject ** obj, wxString & name );
			virtual bool Append( wxList * list, wxString & name );

			virtual ~CSerializeWriter();

		protected:

			void CreateNode( wxString & className, wxString & name, int id, bool isId );
			void CreateValueNode( wxString & className, wxString & name, wxString & value );
			bool AddToMap( int * id, void * obj );

			int m_id;
			wxString m_fileName;
			xmlDoc * m_pXmlDoc;
			std::stack<xmlNode*> m_nodeStack;
			
			typedef std::map<void*, int> MAP;
			MAP m_pointerMap;

            DECLARE_CLASS(CSerializeWriter)

		};	//  end class CSerializeWriter

		class CSerializeReader: public CSerializeReaderWriter
		{
		public:
			
			CSerializeReader( const wxString & fileName );
			
			virtual bool Value( double * val, wxString & name );
			virtual bool Value( wxString * val, wxString & name );
			virtual bool Value( void * val, wxString & name );
/*			virtual bool Value( int * val, wxString & name );
			virtual bool Value( wxString * val, wxString & name ); */
			virtual bool Append( wxObject ** obj, wxString & name );
			virtual bool Append( wxList * list, wxString & name );

			virtual ~CSerializeReader();

		protected:

			bool ReadNode( wxString & name, wxObject ** pObj );
			void ReadValueNode( wxString & className, wxString & name, wxString & value );

			int m_id;
			xmlDoc * m_pXmlDoc;
			std::stack<xmlNode*> m_nodeStack;
			
			typedef std::map<int, wxObject*> MAP;
			MAP m_pointerMap;

            DECLARE_CLASS(CSerializeReader)

		};	//  end class CSerializeReader

	}	//  end namespace wx

}	//  end namespace JakubSchwarzmeier

#endif // _WX_SERIALIZEREADERWRITER_H_