#include "stdwx.h"

#include "ThreedModel.h"

#include "ms3dmodel.h"
#include "objmodel.h"
#include "3dsmodel.h"

using namespace JakubSchwarzmeier::wx;

CThreedModel::CThreedModel(COpenGLCanvas * openGLCanvas)
{
	m_pOpenGLCanvas = openGLCanvas;
	
	m_displayList = 0;
}

CThreedModel::~CThreedModel()
{
	if (m_displayList != 0)
		glDeleteLists(m_displayList, 1);
}

void CThreedModel::Render()
{
	glColor3ub(93, 93, 93);
	glPushMatrix();
	glCallList(m_displayList);
	glPopMatrix();
}

CThreedModel * CThreedModel::Create( std::string filename, COpenGLCanvas * openGLCanvas )
{
	CThreedModel * model = NULL;

    wxFileName fn( filename.c_str() );
	
	if ( fn.GetExt().CmpNoCase( "ms3d" ) == 0 )
	{
		model = new CMS3DModel( openGLCanvas );
	}
	else
	if ( fn.GetExt().CmpNoCase( "obj" ) == 0 )
	{
		model = new CObjModel( openGLCanvas );
	}
	else
	if ( fn.GetExt().CmpNoCase( "3ds" ) == 0 )
	{
		model = new C3DSModel( openGLCanvas );
	}

	if (model != NULL)	model->Load( filename );

	return model;
}
