#include "stdwx.h"

#include "freetype.h"

// FreeType headers
#include <ft2build.h>
#include <freetype/freetype.h>
#include <freetype/ftglyph.h>
#include <freetype/ftoutln.h>
#include <freetype/fttrigon.h>

// STL headers
#include <vector>
#include <string>
#include <stdexcept>
#include <algorithm>

#pragma comment(lib, "freetype.lib")

using namespace JakubSchwarzmeier::wx;
using namespace std;

#define CHAR_NUM 383

inline int next_p2(int a)// Vrt nsledujc mocninu sla 2
{
	int rval = 1;// Nastav bit vpravo do jedniky
	while(rval < a)// Dokud je nalezen mocnina men ne minimum
	{
		rval <<= 1;// Zskn dal mocniny (rotace bit doleva, rychlej zpsob, jak napsat rval *= 2)
	}
	return rval;// Vrcen vsledku
}

// Vytvo display list pro dan znak
void make_dlist(FT_Face face, wchar_t ch, GLuint list_base, GLuint* tex_base, int * fontWidth)
{
	//The first thing we do is get FreeType to render our character
	//into a bitmap.  This actually requires a couple of FreeType commands:

	//Load the Glyph for our character.
	if(FT_Load_Glyph( face, FT_Get_Char_Index( face, ch ), FT_LOAD_DEFAULT ))
		throw std::runtime_error("FT_Load_Glyph failed");

	//Move the face's glyph into a Glyph object.
    FT_Glyph glyph;
    if(FT_Get_Glyph( face->glyph, &glyph ))
		throw std::runtime_error("FT_Get_Glyph failed");

	//Convert the glyph to a bitmap.
	FT_Glyph_To_Bitmap( &glyph, ft_render_mode_normal, 0, 1 );
    FT_BitmapGlyph bitmap_glyph = (FT_BitmapGlyph)glyph;

	//This reference will make accessing the bitmap easier
	FT_Bitmap& bitmap=bitmap_glyph->bitmap;

	//Use our helper function to get the widths of
	//the bitmap data that we will need in order to create
	//our texture.
	int width = next_p2( bitmap.width );
	int height = next_p2( bitmap.rows );

	//Allocate memory for the texture data.
	GLubyte* expanded_data = new GLubyte[ 2 * width * height];

	//Here we fill in the data for the expanded bitmap.
	//Notice that we are using two channel bitmap (one for
	//luminocity and one for alpha), but we assign
	//both luminocity and alpha to the value that we
	//find in the FreeType bitmap. 
	//We use the ?: operator so that value which we use
	//will be 0 if we are in the padding zone, and whatever
	//is the the Freetype bitmap otherwise.
	for(int j=0; j <height;j++) {
		for(int i=0; i < width; i++){
			expanded_data[2*(i+j*width)]= expanded_data[2*(i+j*width)+1] = 
				(i>=bitmap.width || j>=bitmap.rows) ?
				0 : bitmap.buffer[i + bitmap.width*j];
		}
	}


	//Now we just setup some texture paramaters.
    glBindTexture( GL_TEXTURE_2D, tex_base[ch]);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);

	//Here we actually create the texture itself, notice
	//that we are using GL_LUMINANCE_ALPHA to indicate that
	//we are using 2 channel data.
    glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, width, height,
		  0, GL_LUMINANCE_ALPHA, GL_UNSIGNED_BYTE, expanded_data );

	//With the texture created, we don't need to expanded data anymore
    delete [] expanded_data;

	//So now we can create the display list
	glNewList(list_base+ch,GL_COMPILE);

	glBindTexture(GL_TEXTURE_2D,tex_base[ch]);

	glPushMatrix();

	//first we need to move over a little so that
	//the character has the right amount of space
	//between it and the one before it.
	glTranslatef(bitmap_glyph->left,0,0);

	//Now we move down a little in the case that the
	//bitmap extends past the bottom of the line 
	//(this is only true for characters like 'g' or 'y'.
	glTranslatef(0,bitmap_glyph->top-bitmap.rows,0);

	//Now we need to account for the fact that many of
	//our textures are filled with empty padding space.
	//We figure what portion of the texture is used by 
	//the actual character and store that information in 
	//the x and y variables, then when we draw the
	//quad, we will only reference the parts of the texture
	//that we contain the character itself.
	double	x=(double)bitmap.width / (double)width,
			y=(double)bitmap.rows / (double)height;

	//Here we draw the texturemaped quads.
	//The bitmap that we got from FreeType was not 
	//oriented quite like we would like it to be,
	//so we need to link the texture to the quad
	//so that the result will be properly aligned.
	glBegin(GL_QUADS);
	glTexCoord2d(0,0); glVertex2f(0,bitmap.rows);
	glTexCoord2d(0,y); glVertex2f(0,0);
	glTexCoord2d(x,y); glVertex2f(bitmap.width,0);
	glTexCoord2d(x,0); glVertex2f(bitmap.width,bitmap.rows);
	glEnd();
	glPopMatrix();

	fontWidth[ch] = face->glyph->advance.x >> 6;
	glTranslatef(fontWidth[ch] ,0,0);

	//increment the raster position as if we were a bitmap font.
	//(only needed if you want to calculate text length)
	//glBitmap(0,0,0,0,face->glyph->advance.x >> 6,0,NULL);

	//Finnish the display list
	glEndList();
}

CFreetype::CFreetype()
{
	m_textures = NULL;
	m_fontWidths = NULL;
	m_listBase = 0;
}

void CFreetype::Create(std::string fileName, unsigned int h)
{
	m_fileName = fileName;

	m_textures = new GLuint[CHAR_NUM];// Pam pro ID textur
	m_fontWidths = new int[CHAR_NUM];// Pam pro ku znak
	this->m_height = h;

	//Create and initilize a freetype font library.
	FT_Library library;
	if (FT_Init_FreeType( &library )) 
		throw std::runtime_error("FT_Init_FreeType failed");

	//The object in which Freetype holds information on a given
	//font is called a "face".
	FT_Face face;

	//This is where we load in the font information from the file.
	//Of all the places where the code might die, this is the most likely,
	//as FT_New_Face will die if the font file does not exist or is somehow broken.
	if (FT_New_Face( library, fileName.c_str(), 0, &face )) 
		throw std::runtime_error("FT_New_Face failed (there is probably a problem with your font file)");

	//For some twisted reason, Freetype measures font size
	//in terms of 1/64ths of pixels.  Thus, to make a font
	//h pixels high, we need to request a size of h*64.
	//(h << 6 is just a prettier way of writting h*64)
	FT_Set_Char_Size( face, h << 6, h << 6, 96, 96);

	//Here we ask opengl to allocate resources for
	//all the textures and displays lists which we
	//are about to create.  
	m_listBase=glGenLists(CHAR_NUM);
	glGenTextures( CHAR_NUM, m_textures );

	//This is where we actually create each of the fonts display lists.
	for(wchar_t i=0;i<CHAR_NUM;i++)
		make_dlist(face,i,m_listBase,m_textures,m_fontWidths);

	//We don't need the face information now that the display
	//lists have been created, so we free the assosiated resources.
	FT_Done_Face(face);

	//Ditto for the library.
	FT_Done_FreeType(library);
}


CFreetype::~CFreetype()
{
	if (m_listBase != 0)	glDeleteLists(m_listBase, CHAR_NUM);
	if (m_textures != NULL)
	{
		glDeleteTextures(CHAR_NUM, m_textures);
		delete [] m_textures;
	}
	if (m_fontWidths != NULL)
	{
		delete [] m_fontWidths;
	}
}

inline void pushScreenCoordinateMatrix()// Pepne do pravohl projekce
{
	glPushAttrib(GL_TRANSFORM_BIT);
	GLint viewport[4];
	glGetIntegerv(GL_VIEWPORT, viewport);
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	gluOrtho2D(viewport[0], viewport[2], viewport[1], viewport[3]);
	glPopAttrib();
}


inline void pop_projection_matrix()// Obnov perspektivu
{
	glPushAttrib(GL_TRANSFORM_BIT);
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glPopAttrib();
}

// Rendering textu
void CFreetype::PrintWindow(double x, double y, std::string strText, POS pos)
{
	// We want a coordinate system where things coresponding to window pixels.
//	pushScreenCoordinateMatrix();					
	glPushAttrib(GL_TRANSFORM_BIT);
	GLint	viewport[4];
	glGetIntegerv(GL_VIEWPORT, viewport);
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	gluOrtho2D(viewport[0],viewport[2],viewport[1],viewport[3]);
	glPopAttrib();
	
	GLuint font= m_listBase;
	double h=m_height/.63f;						//We make the height about 1.5* that of
	


	////////////////////////////////////
	//NEWCESTINA
	//
	//Nasledujici kod konvertuje znaky s hacky do spravneho kodovani.
	//Znaky s carky maji stejne kodovani
	////////////////////////////////////

	wstring line;
	vector<wstring> lines;//NEWCESTINA
	wchar_t znak;
	for(int loop1=0; loop1<strText.size(); loop1++) {
		switch(strText[loop1]) {
			case '':
				znak=268;
				break;

			case '':
				znak=269;
				break;

			case '':
				znak=270;
				break;

			case '':
				znak=271;
				break;

			case '':
				znak=282;
				break;

			case '':
				znak=283;
				break;

			case '':
				znak=327;
				break;

			case '':
				znak=328;
				break;

			case '':
				znak=344;
				break;

			case '':
				znak=345;
				break;

			case '':
				znak=352;
				break;

			case '':
				znak=353;
				break;

			case '':
				znak=356;
				break;

			case '':
				znak=357;
				break;

			case '':
				znak=366;
				break;

			case '':
				znak=367;
				break;

			case '':
				znak=381;
				break;

			case '':
				znak=382;
				break;

			case '\n':

				lines.push_back( line );
				line.clear();
				break;

			default :
				znak=(unsigned char) strText[loop1];
				break; //znaky s carky maji v char zaporne hodnoty
				       //pomoci pretypovani z nich udelame kladne hodnoty
				       //znaky bez hacek a carek se nemeni
		}

		if (strText[loop1] != '\n')	line.append(1, znak );
		if ( (loop1 == strText.size()-1) && (strText[loop1] != '\n') )
		{
				lines.push_back( line );
		}
	}

	glPushAttrib(GL_LIST_BIT | GL_CURRENT_BIT  | GL_ENABLE_BIT | GL_TRANSFORM_BIT);	
	glMatrixMode(GL_MODELVIEW);
	glDisable(GL_LIGHTING);
	glEnable(GL_TEXTURE_2D);
	glDisable(GL_DEPTH_TEST);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);	

	glListBase(font);

	double modelview_matrix[16];	
	glGetDoublev(GL_MODELVIEW_MATRIX, modelview_matrix);

	//This is where the text display actually happens.
	//For each line of text we reset the modelview matrix
	//so that the line's text will start in the correct position.
	//Notice that we need to reset the matrix, rather than just translating
	//down by h. This is because when each character is
	//draw it modifies the current matrix so that the next character
	//will be drawn immediatly after it.  
	for(size_t i=0;i<lines.size();i++) {
		

		glPushMatrix();
		glLoadIdentity();
		
		int strWidth = 0;
		switch (pos)
		{
		case TOP_LEFT:

			glTranslatef(x, viewport[3] - y - h*i - m_height, 0);// Pesun na odpovdajc pozici
			break;

		case TOP_RIGHT:

			for(int j = 0; j < lines[i].size(); j++)
				strWidth += m_fontWidths[ lines[i][j] ];
			glTranslatef(viewport[2] - x - strWidth, viewport[3] - y - h*i - m_height, 0);// Pesun na odpovdajc pozici
			break;

		case BOTTOM_LEFT:

			glTranslatef(x, y - h*i, 0);// Pesun na odpovdajc pozici
			break;

		case BOTTOM_RIGHT:

			for(int j = 0; j < lines[i].size(); j++)
				strWidth += m_fontWidths[ lines[i][j] ];
			glTranslatef(viewport[2] - x - strWidth, y - h*i, 0);// Pesun na odpovdajc pozici
			break;

		}
		glMultMatrixd(modelview_matrix);

	//  The commented out raster position stuff can be useful if you need to
	//  know the length of the text that you are creating.
	//  If you decide to use it make sure to also uncomment the glBitmap command
	//  in make_dlist().
	//	glRasterPos2f(0,0);
		glCallLists(lines[i].length(), GL_UNSIGNED_SHORT, lines[i].c_str());
	//	double rpos[4];
	//	glGetFloatv(GL_CURRENT_RASTER_POSITION ,rpos);
	//	double len=x-rpos[0];

		glPopMatrix();
	}


	glPopAttrib();		

	pop_projection_matrix();
}

void CFreetype::Print(std::string text)
{
	glMatrixMode(GL_MODELVIEW);
	glDisable(GL_LIGHTING);
	glEnable(GL_TEXTURE_2D);
	glDisable(GL_DEPTH_TEST);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	glPushAttrib(GL_LIST_BIT);

	  glListBase(this->m_listBase); // indicate start of display lists
	  glCallLists( text.length(), GL_UNSIGNED_BYTE, text.c_str() );

	glPopAttrib();
}

std::string CFreetype::GetFileName()
{
	return m_fileName;
}

double CFreetype::GetHeight()
{
	return m_height;
}
