#include "stdwx.h"
#include "objmodel.h"

#include <fstream>
#include <schwarzmeier/std.h>
#include <vector>

using namespace JakubSchwarzmeier::wx;
using namespace JakubSchwarzmeier::Std::Math;

CObjModel::CObjModel(COpenGLCanvas * openGLCanvas): CThreedModel(openGLCanvas)
{
}

CObjModel::~CObjModel()
{
}

void CObjModel::Load(std::string filename)
{
	std::ifstream m_is;
	m_is.open( filename.c_str(), std::ios::in | std::ios::binary  );
	if (!m_is)	throw;

	bool m_bObjectHasUV = false;
	char code;		//  first char from line
	std::vector< Vector<> > vertices, vertTextures;	//  list of vertices
	std::vector< Vector<int> > faces, facesTextures;		//  list of faces
	std::vector< Vector<> > normals;			//  normal to each vertex

	vertices.reserve( 300000 );
	vertTextures.reserve( 300000 );
	faces.reserve( 300000 );
	facesTextures.reserve( 300000 );

	while (m_is.eof()==0)
	{
		m_is >> code;

		switch (code)
		{
		case 'v':			

			switch(m_is.get())
			{
			case ' ':		//  read vertex

				double x, y, z;
				m_is >> x >> y >> z;	//  extract data
				
				vertices.push_back( Vector<>(x, y, z) );
				break;

			case 'n':		//  read normal

				m_is >> x >> y >> z;	//  extract data
				
				normals.push_back( Vector<>(x, y, z) );
				break;

			case 't':		//  read texture vertex

				double u, v;
				m_is >> u >> v;
				vertTextures.push_back( Vector<>(u, v, 0) );

				// Set the flag that tells us this object has texture coordinates.
				// Now we know that the face information will list the vertice AND UV index.
				// For example, ("f 1 3 2" verses "f 1/1 2/2 3/3")
				m_bObjectHasUV = true;

				break;
			}
			break;

		case 'f':			//  read face (=polygon, not necessary triangle)

			// Check if this object has texture coordinates before reading in the values
			int v1, v2, v3;
			if( m_bObjectHasUV )
			{
				int t1, t2, t3;
				char ch;
				m_is >> v1 >> ch >> t1 >> v2 >> ch >> t2 >> v3 >> ch >> t3;
			    facesTextures.push_back( Vector<int>(t1-1, t2-1, t3-1) );
			}
			else										// The object does NOT have texture coordinates
			{
				m_is >> v1 >> v2 >> v3;
			}

			faces.push_back( Vector<int>(v1-1, v2-1, v3-1) );	//  for regular -1
			break;

		}	//  end switch

		//  discard to end of line
		char line[1024];
		m_is.getline(line, sizeof(line));
		m_is >> std::ws;

	}	//  end while

	//  create display list
	m_displayList = glGenLists(1);

	glNewList(m_displayList, GL_COMPILE);

	if ( facesTextures.empty() )
	{
		glPushAttrib( GL_ENABLE_BIT );
			glDisable( GL_TEXTURE_2D );
	}

	//  render
	glBegin(GL_TRIANGLES);
		for(int i = 0; i < faces.size(); i++)
			for(int j = 0; j < 3; j++)
			{
				size_t ind = faces[i][j];


				if ( normals.size() == vertices.size() )
                    glNormal3d( normals[ind][0], normals[ind][1], normals[ind][2]);
				glVertex3d( vertices[ind][0], vertices[ ind ][1], vertices[ ind ][2]);
			}
	glEnd();

	if ( facesTextures.empty() )
	{
		glPopAttrib();
	}

	glEndList();
	//  end create display list

	return;
}
