/************************************************************
*
*  Copyright (c) 2005 Jakub Schwarzmeier
*
************************************************************/

#ifndef _WX_OPENGLCANVAS_H_
#define _WX_OPENGLCANVAS_H_

#include <gl/glew.h>
#include <wx/glcanvas.h>
#include "observer.h"
#include "glCanvas.h"
#include <memory>
#include <deque>
#include <gl/glu.h>
#include "OpenGLCanvasDlg.h"
#include "Observer.h"

namespace JakubSchwarzmeier
{
	namespace wx
	{

		//  Forward references
		class CObserver;
		class COpenGLCanvas;
		class wxMyGLContext;
		class COpenGLCanvasDlg;
		class CObserverDlg;

		class CRenderingThread: public wxThread
		{
		public:

			CRenderingThread();
			void Create(COpenGLCanvas * glCanvas, bool signaled = true);

			// thread execution starts here
			virtual void * Entry();

		protected:

			COpenGLCanvas * m_glCanvas;
			bool m_bTerminateThread;
			wxCondition m_condition;
			wxMutex m_mutex;
			bool m_bSignaled;
		
			friend class COpenGLCanvas;

		};	//  end class CRenderingThread

		class COpenGLCanvas: public wxGLCanvas
		{
		public:

			COpenGLCanvas() {}	//  for two step construction

			COpenGLCanvas(wxWindow *parent, wxWindowID id = wxID_ANY,
				const wxPoint& pos = wxDefaultPosition,
				const wxSize& size = wxDefaultSize, long style = 0,
				const wxString& name = wxGLCanvasName, int *attribList = 0,
				const wxPalette& palette = wxNullPalette);

			virtual void Create(wxWindow *parent, bool bDeserialize = false, wxWindowID id = wxID_ANY,
				const wxPoint& pos = wxDefaultPosition,
				const wxSize& size = wxDefaultSize, long style = 0,
				const wxString& name = wxGLCanvasName, int *attribList = 0,
				const wxPalette& palette = wxNullPalette);

			virtual ~COpenGLCanvas();
			virtual bool OnSerialize( JakubSchwarzmeier::wx::CSerializeReaderWriter * dir );

		//  implementations

			void SetObserver( CObserver * observer );
			CObserver * GetObserver( );

			int ReserveLight();
			void DisposeLight(int light);
			GLuint GetList();
			GLUquadricObj * GetQuadric();

			wxString GetLastError();

			virtual void ReRender();
			virtual void ForceReRender();
			void SwapBuffers();

			void SetSignaled( bool signaled );

			double GetFov();
			double GetAspect();

			GLuint LoadTexture(wxString fileName);

			wxMyGLContext * GetMyContext();

			void GetCopyImage(wxImage & newImage);

			void SetAutoRender(bool autoRender);
			bool GetAutoRender();

			virtual bool Destroy();

		protected:
		//  overrides
			virtual void OnDraw(wxDC * pDC);

		//  implementations
			virtual void ThreadedReRender();
			virtual void OnRender();
			virtual void PreRender();
			virtual void OnContextTerminate();
			virtual void RenderFinished();
			virtual void SetRenderingParameters();

			virtual void ShowOpenGLInfo();
			virtual void ShowObserver();

		//  message handlers
			void OnPaint(wxPaintEvent & event);
			void OnSize(wxSizeEvent & event);
			void OnEraseBkgnd(wxEraseEvent & event);
			void OnMouse(wxMouseEvent & event);
			void OnKeyDown(wxKeyEvent & event);
			void OnKeyUp(wxKeyEvent & event);
			void OnMinimize(wxIconizeEvent & event);

			CObserver * m_observer;
			CObserver * m_internalObserver;

			GLdouble m_nearClippingPlane;
			GLdouble m_farClippingPlane;
            GLdouble m_fov;
			GLdouble m_aspect;

			bool m_bTexturing;
			GLenum m_polygonMode;
			bool m_bLighting;
			bool m_bColorMaterial;
			bool m_bCullFace;

			GLUquadricObj * m_quadric;
			bool m_freeLights[GL_MAX_LIGHTS];
			GLuint m_lastList;

			bool m_bSize;
			bool m_bAutoRender;
			bool m_bWasAutoRender;

			friend class COpenGLCanvasDlg;
			friend class wxMyGLContext;

		protected:

			//  threading
			CRenderingThread * m_pRenderingThread;
			friend class CRenderingThread;
			wxCriticalSection m_csRenderingContext;
			wxCriticalSection m_csRenderingBitmap;

			DECLARE_EVENT_TABLE()
			DECLARE_DYNAMIC_CLASS(COpenGLCanvas)
		
		};	//  end class COpenGLCanvas

		
	}	//  end namespace wx

}	//  end namespace JakubSchwarzmeier

#endif // _WX_OPENGLCANVAS_H_