/************************************************************
*
*  Copyright (c) 2005 Jakub Schwarzmeier
*
************************************************************/

#include "stdwx.h"

#include "valdouble.h"

IMPLEMENT_DYNAMIC_CLASS(wxDoubleValidator, wxValidator)

wxDoubleValidator::wxDoubleValidator(double * val)
{
    m_pVal = val;
}

wxDoubleValidator::wxDoubleValidator(const wxDoubleValidator& val)
    : wxValidator()
{
    Copy(val);
}

bool wxDoubleValidator::Copy(const wxDoubleValidator& val)
{
    wxValidator::Copy(val);

    m_pVal = val.m_pVal;

    return true;
}


// Called when the value in the window must be validated.
// This function can pop up an error message.
bool wxDoubleValidator::Validate(wxWindow *parent)
{
    wxControl *control = (wxTextCtrl *) m_validatorWindow;

    // If window is disabled, simply return
    if ( !control->IsEnabled() )
        return true;

    wxString val(control->GetLabel());

    bool ok = true;

    // NB: this format string should contian exactly one '%s'
    wxString errormsg;

	double dummy;
	if ( val.ToDouble( &dummy ) == false )
    {
        ok = false;
        errormsg = _("'%s' should only contain real numbers.");
    }

    if ( !ok )
    {
        wxASSERT_MSG( !errormsg.empty(), _T("you forgot to set errormsg") );

        m_validatorWindow->SetFocus();

        wxString buf;
        buf.Printf(errormsg, val.c_str());

        wxMessageBox(buf, _("Validation conflict"),
                     wxOK | wxICON_EXCLAMATION, parent);
    }

    return ok;
}

// Called to transfer data to the window
bool wxDoubleValidator::TransferToWindow(void)
{
    if ( m_pVal )
    {
        wxControl *control = (wxTextCtrl *) m_validatorWindow;

		char sz[100];
		sprintf(sz, "%.18e", *m_pVal);
		control->SetLabel( sz );
    }

    return true;
}

// Called to transfer data to the window
bool wxDoubleValidator::TransferFromWindow(void)
{
    if ( m_pVal )
    {
        wxControl *control = (wxTextCtrl *) m_validatorWindow;

		*m_pVal = atof( control->GetLabel() );
	}

    return true;
}
