/************************************************************
*
*  Copyright (c) 2005 Jakub Schwarzmeier
*
************************************************************/

#include "stdwx.h"

#include "valint.h"

IMPLEMENT_DYNAMIC_CLASS(wxIntegerValidator, wxValidator)

wxIntegerValidator::wxIntegerValidator(int * val)
{
    m_pVal = val;
}

wxIntegerValidator::wxIntegerValidator(const wxIntegerValidator& val)
    : wxValidator()
{
    Copy(val);
}

bool wxIntegerValidator::Copy(const wxIntegerValidator& val)
{
    wxValidator::Copy(val);

    m_pVal = val.m_pVal;

    return true;
}


// Called when the value in the window must be validated.
// This function can pop up an error message.
bool wxIntegerValidator::Validate(wxWindow *parent)
{
    wxTextCtrl *control = (wxTextCtrl *) m_validatorWindow;

    // If window is disabled, simply return
    if ( !control->IsEnabled() )
        return true;

    wxString val(control->GetValue());

    bool ok = true;

    // NB: this format string should contian exactly one '%s'
    wxString errormsg;

	for(size_t i = 0; i < val.Length(); i++)
	{
		if ( ( val[i] == '-' ) || (val[i] == '+') )  continue;
		if ( isdigit( val[i] ) == 0 )
		{
			ok = false;
			break;
		}
	}

	if (val.Length() == 0)  ok = false;
	errormsg = _("'%s' should contain integer number.");

    if ( !ok )
    {
        wxASSERT_MSG( !errormsg.empty(), _T("you forgot to set errormsg") );

        m_validatorWindow->SetFocus();

        wxString buf;
        buf.Printf(errormsg, val.c_str());

        wxMessageBox(buf, _("Validation conflict"),
                     wxOK | wxICON_EXCLAMATION, parent);
    }

    return ok;
}

// Called to transfer data to the window
bool wxIntegerValidator::TransferToWindow(void)
{
    if ( m_pVal )
    {
        wxTextCtrl *control = (wxTextCtrl *) m_validatorWindow;
        wxString val;
		val << *m_pVal;
		control->SetValue( val );
    }

    return true;
}

// Called to transfer data to the window
bool wxIntegerValidator::TransferFromWindow(void)
{
    if ( m_pVal )
    {
        wxTextCtrl *control = (wxTextCtrl *) m_validatorWindow;
		*m_pVal = atoi( control->GetValue().c_str() );
	}

    return true;
}
