#include "StdWx.h"

#include "valnotnull.h"

IMPLEMENT_DYNAMIC_CLASS(wxNotNullValidator, wxValidator)

wxNotNullValidator::wxNotNullValidator()
{
	m_pVal = NULL;
}

wxNotNullValidator::wxNotNullValidator(wxString * val, wxString errString)
{
    m_pVal = val;
	m_errString = errString;
}

wxNotNullValidator::wxNotNullValidator(const wxNotNullValidator& val)
    : wxValidator()
{
    Copy(val);
}

bool wxNotNullValidator::Copy(const wxNotNullValidator& val)
{
    wxValidator::Copy(val);

    m_pVal = val.m_pVal;
	m_errString = val.m_errString;

    return true;
}


// Called when the value in the window must be validated.
// This function can pop up an error message.
bool wxNotNullValidator::Validate(wxWindow *parent)
{
    wxControl *control = (wxControl *) m_validatorWindow;

    // If window is disabled, simply return
    if ( !control->IsEnabled() )
        return true;

	//  Value entered by user.
    wxString val(control->GetLabel());

    bool ok = true;

    // NB: this format string should contian exactly one '%s'
    wxString errormsg;
	
	if ( val.Length() < 1 )  
		ok = false;

    if ( !ok )
    {
        wxASSERT_MSG( !m_errString.empty(), _T("you forgot to set errormsg") );

        m_validatorWindow->SetFocus();

        wxString buf;
        buf.Printf(m_errString, val.c_str());

        wxMessageBox(buf, _("Validation conflict"),
                     wxOK | wxICON_EXCLAMATION, parent);
    }

    return ok;
}

// Called to transfer data to the window
bool wxNotNullValidator::TransferToWindow(void)
{
    if ( m_pVal )
    {
        wxControl * control = (wxControl *) m_validatorWindow;
		control->SetLabel( *m_pVal );
    }

    return true;
}

// Called to transfer data to the window
bool wxNotNullValidator::TransferFromWindow(void)
{
    if ( m_pVal )
    {
        wxControl *control = (wxControl *) m_validatorWindow;
        *m_pVal = control->GetLabel();
    }

    return true;
}
