/////////////////////////////////////////////////////////////////////////////
// Name:        BodiesDlg.cpp
// Purpose:     
// Author:      Jakub Schwarzmeier
// Modified by: 
// Created:     08/11/06 11:33:24
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#if defined(__GNUG__) && !defined(__APPLE__)
#pragma implementation "BodiesDlg.h"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif


////@begin includes
#include "SpiceFilesDlg.h"
////@end includes

#include "BodiesDlg.h"
#include "simulation.h"
#include "PhaethonDoc.h"
#include "PhaethonApp.h"
#include <schwarzmeier/astronomy.h>
#include "body.h"
#include "comet.h"
#include "bodypropertiesdlg.h"
#include "bodynewdlg.h"

////@begin XPM images
////@end XPM images

/*!
 * CBodiesDlg type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CBodiesDlg, wxDialog )


/*!
 * CBodiesDlg event table definition
 */

BEGIN_EVENT_TABLE( CBodiesDlg, wxDialog )

////@begin CBodiesDlg event table entries
    EVT_TREE_SEL_CHANGED( XRCID("ID_BODIES"), CBodiesDlg::OnBodiesSelChanged )
    EVT_TREE_ITEM_ACTIVATED( XRCID("ID_BODIES"), CBodiesDlg::OnBodiesItemActivated )

    EVT_CHOICE( XRCID("ID_CLASSES"), CBodiesDlg::OnClassesSelected )
    EVT_UPDATE_UI( XRCID("ID_CLASSES"), CBodiesDlg::OnClassesUpdate )

    EVT_BUTTON( XRCID("ID_ADDBODY"), CBodiesDlg::OnAddbodyClick )

    EVT_BUTTON( XRCID("ID_REMOVEBODY"), CBodiesDlg::OnRemovebodiesClick )
    EVT_UPDATE_UI( XRCID("ID_REMOVEBODY"), CBodiesDlg::OnRemovebodiesUpdate )

    EVT_BUTTON( XRCID("ID_PROPSBODY"), CBodiesDlg::OnPropsbodyClick )
    EVT_UPDATE_UI( XRCID("ID_PROPSBODY"), CBodiesDlg::OnPropsbodyUpdate )

    EVT_BUTTON( XRCID("ID_REMOVEBODIES"), CBodiesDlg::OnRemovebodiesClick )
    EVT_UPDATE_UI( XRCID("ID_REMOVEBODIES"), CBodiesDlg::OnRemovebodiesUpdate )

    EVT_BUTTON( XRCID("ID_ADDBODIES"), CBodiesDlg::OnAddbodiesClick )
    EVT_UPDATE_UI( XRCID("ID_ADDBODIES"), CBodiesDlg::OnAddbodiesUpdate )

    EVT_BUTTON( XRCID("ID_SPICEFILES"), CBodiesDlg::OnSpicefilesClick )

////@end CBodiesDlg event table entries

END_EVENT_TABLE()

/*!
 * CBodiesDlg constructors
 */

CBodiesDlg::CBodiesDlg( )
{
}

CBodiesDlg::CBodiesDlg( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Create(parent, id, caption, pos, size, style);
}

/*!
 * CBodiesDlg creator
 */

bool CBodiesDlg::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CBodiesDlg member initialisation
    m_bodiesCtrl = NULL;
    m_classes = NULL;
    m_availBodiesCtrl = NULL;
////@end CBodiesDlg member initialisation

////@begin CBodiesDlg creation
    SetExtraStyle(GetExtraStyle()|wxWS_EX_BLOCK_EVENTS);
    SetParent(parent);
    CreateControls();
    GetSizer()->Fit(this);
    GetSizer()->SetSizeHints(this);
    Centre();
////@end CBodiesDlg creation
    return true;
}

/*!
 * Control creation for CBodiesDlg
 */

void CBodiesDlg::CreateControls()
{    
    wxXmlResource::Get()->Load(wxT("xrc/bodies.xrc")); 

////@begin CBodiesDlg content construction
    wxXmlResource::Get()->LoadDialog(this, GetParent(), _T("IDD_BODIES"));
    m_bodiesCtrl = XRCCTRL(*this, "ID_BODIES", wxTreeCtrl);
    m_classes = XRCCTRL(*this, "ID_CLASSES", wxChoice);
    m_availBodiesCtrl = XRCCTRL(*this, "ID_AVAILABLEBODIES", wxTreeCtrl);
////@end CBodiesDlg content construction

    // Create custom windows not generated automatically here.
	UpdateBodies();

	m_mapClasses.insert( std::map<wxString, wxString>::value_type( "Body", "Body" ) );
	m_mapClasses.insert( std::map<wxString, wxString>::value_type( "ExtendedBody", "Extended Body" ) );
	m_mapClasses.insert( std::map<wxString, wxString>::value_type( "Star", "Star" ) );
	m_mapClasses.insert( std::map<wxString, wxString>::value_type( "IrregularBody", "Irregular Body" ) );
	m_mapClasses.insert( std::map<wxString, wxString>::value_type( "Comet", "Comet" ) );

	std::map<wxString, wxString>::const_iterator it;
	for ( it = m_mapClasses.begin( ) ; it != m_mapClasses.end( ) ; it++ )
	{
		m_classes->Append( it->second );
	}

	

////@begin CBodiesDlg content initialisation

////@end CBodiesDlg content initialisation
}

/*!
 * Should we show tooltips?
 */

bool CBodiesDlg::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CBodiesDlg::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CBodiesDlg bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CBodiesDlg bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CBodiesDlg::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CBodiesDlg icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CBodiesDlg icon retrieval
}
/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_SPICEFILES
 */

void CBodiesDlg::OnSpicefilesClick( wxCommandEvent& event )
{
	Simulation * sim = reinterpret_cast<Simulation*>( GetClientData() );
	
	//  create new copies
	std::set<std::string> spkFiles(	sim->GetSpkFiles()->begin(), sim->GetSpkFiles()->end() );
	std::set<std::string> pckFiles(	sim->GetPckFiles()->begin(), sim->GetPckFiles()->end() );
	
	std::set<std::string> * vecFiles[2] = { &spkFiles, &pckFiles };

	CSpiceFilesDlg dlg;

	dlg.SetClientData( vecFiles );

	dlg.Create(this);

	if ( dlg.ShowModal() == wxID_OK )
	{
		//  copy new data over old
		sim->GetSpkFiles()->clear();
		sim->GetSpkFiles()->insert( spkFiles.begin(), spkFiles.end() );

		sim->GetPckFiles()->clear();
		sim->GetPckFiles()->insert( pckFiles.begin(), pckFiles.end() );

		CPhaethonDoc * doc = ::wxGetApp().GetDocument();
		doc->Modify( TRUE );

		UpdateBodies();
	}

}

class CBodyID: public wxTreeItemData
{
public:	
	CBodyID(int spiceID = 0) { m_spiceID = spiceID; }
	int m_spiceID;
};

void CBodiesDlg::UpdateAvailableBodies(std::set<std::string> * set)
{
	wxBusyInfo wait( _T("Please wait, loading bodies from SPK file(s)...") );

	Simulation * sim = reinterpret_cast<Simulation*>( GetClientData() );

	std::map<std::string, int> bodiesNames = 
		JakubSchwarzmeier::Astronomy::CSpice::GetSpkObjects( *(sim->GetSpkFiles()) );

	m_availBodiesCtrl->Hide();
	m_availBodiesCtrl->DeleteAllItems();
	wxTreeItemId idRoot = m_availBodiesCtrl->AddRoot( _T("Bodies") );

	std::map<std::string, int>::const_iterator it;
	for( it = bodiesNames.begin(); it !=  bodiesNames.end(); it++ )
	{
		std::set<std::string>::const_iterator it2 = set->find( (it->first) );
		if ( it2 == set->end() )
		{
			//  bodyname not in active bodies set

            wxString append( (it->first).c_str() );
			//  check if we have PCK mass entry for SPK body ID
			double dummy;
			try
			{
				sim->GetSpice()->GetMass( it->second, dummy );
				append += _T("(M)");
			}
			catch (...)
			{}
			try
			{
				sim->GetSpice()->GetRadii( it->second, dummy );
				append += _T("(R)");
			}
			catch (...)
			{}
			try
			{
				sim->GetSpice()->GetPoleEcliptic( it->second, dummy, dummy, dummy );
				sim->GetSpice()->GetPM( it->second, dummy );
				append += _T("(O)");		//  orientation
			}
			catch (...)
			{}

			wxTreeItemId nId = m_availBodiesCtrl->AppendItem( m_availBodiesCtrl->GetRootItem(), append, -1, -1, new CBodyID( it->second ) );
		}
	}

	m_availBodiesCtrl->Show();
}

class CBodyPtr: public wxTreeItemData
{
public:	
	CBodyPtr(Body * pBody = NULL) { m_pBody = pBody; }
	Body * m_pBody;
};

void CBodiesDlg::LocalTreeStructure( Body * body, wxTreeItemId & parentTreeItem, std::set<std::string> * set )
{
	if ( body->GetUseSPICE() ) set->insert( body->GetName() );

	wxTreeItemId nId = m_bodiesCtrl->AppendItem( parentTreeItem, wxString( body->GetName().c_str() ), -1, -1, new CBodyPtr( body ) );

	//  create tree from descendats
	std::vector<Body*>::const_iterator it;
	for ( it = body->GetChildList()->begin( ) ; it != body->GetChildList()->end( ) ; it++ )
	{
		Body * childBody = *it;
		LocalTreeStructure( childBody, nId, set );
	}
}

void CBodiesDlg::UpdateActiveBodies(std::set<std::string> * set)
{
	Simulation * sim = reinterpret_cast<Simulation*>( GetClientData() );
	std::vector<Body*> * bodies = sim->GetBodies();

	m_bodiesCtrl->Hide();
	m_bodiesCtrl->DeleteAllItems();
	wxTreeItemId idRoot = m_bodiesCtrl->AddRoot( _T("Bodies"), -1, -1, new CBodyPtr(NULL) );

	std::vector<Body*>::const_iterator it;
	for( it = bodies->begin(); it !=  bodies->end(); it++ )
	{
		if ((*it)->GetParent() == NULL)  LocalTreeStructure( (*it), idRoot, set );
	}

	m_bodiesCtrl->Show();
}

void CBodiesDlg::UpdateBodies()
{
	std::set<std::string> activeBodiesSet;
	UpdateActiveBodies( &activeBodiesSet );
	UpdateAvailableBodies( &activeBodiesSet );
}

/*!
 * wxEVT_UPDATE_UI event handler for ID_REMOVEBODIES
 */

void CBodiesDlg::OnRemovebodiesUpdate( wxUpdateUIEvent& event )
{
	wxArrayTreeItemIds dummy;
	event.Enable( m_bodiesCtrl->GetSelections( dummy ) != 0 );
}
/*!
 * wxEVT_UPDATE_UI event handler for ID_ADDBODIES
 */

void CBodiesDlg::OnAddbodiesUpdate( wxUpdateUIEvent& event )
{
	wxArrayTreeItemIds dummy;
	event.Enable( m_availBodiesCtrl->GetSelections( dummy ) != 0 );
}

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_ADDBODIES
 */

void CBodiesDlg::OnAddbodiesClick( wxCommandEvent& event )
{
	Simulation * sim = reinterpret_cast<Simulation*>( GetClientData() );

	wxArrayTreeItemIds sel;
	size_t count = m_availBodiesCtrl->GetSelections( sel );
    for ( size_t i = 0; i < count; i++ )
    {
		CBodyID * bodyID = dynamic_cast<CBodyID*>( m_availBodiesCtrl->GetItemData( sel.Item(i) ) );

		Body * body = new Body();
		sim->Add(body);

		body->SetSpiceBodyID( bodyID->m_spiceID );
		body->SetUseSPICE( true );

		//  active bodies
		wxArrayTreeItemIds abSel;
		size_t iAbCount = m_bodiesCtrl->GetSelections( abSel );
		if (iAbCount == 1)
		{
			//  I can assign body to its parent
			CBodyPtr * parentBodyPtr = dynamic_cast<CBodyPtr*>( m_bodiesCtrl->GetItemData( abSel.Item(0) ) );
			if (parentBodyPtr->m_pBody != NULL)
			{
				body->SetParent( parentBodyPtr->m_pBody );
				parentBodyPtr->m_pBody->GetChildList()->push_back( body );
			}
		}
	}

	UpdateBodies();
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_REMOVEBODIES
 */

void CBodiesDlg::OnRemovebodiesClick( wxCommandEvent& event )
{
	Simulation * sim = reinterpret_cast<Simulation*>( GetClientData() );

	wxArrayTreeItemIds sel;
	size_t count = m_bodiesCtrl->GetSelections( sel );
    for ( size_t i = 0; i < count; i++ )
    {
		CBodyPtr * bodyPtr = dynamic_cast<CBodyPtr*>( m_bodiesCtrl->GetItemData( sel.Item(i) ) );

		sim->Remove( bodyPtr->m_pBody );
	}

	UpdateBodies();
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_PROPSBODY
 */

void CBodiesDlg::OnPropsbodyUpdate( wxUpdateUIEvent& event )
{
	wxArrayTreeItemIds dummy;
	event.Enable( m_bodiesCtrl->GetSelections( dummy ) == 1 );
}

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_PROPSBODY
 */

void CBodiesDlg::OnPropsbodyClick( wxCommandEvent& event )
{
	wxArrayTreeItemIds bodySel;
	size_t count = m_bodiesCtrl->GetSelections( bodySel );
	if (count != 1)  throw;

	CBodyPtr * bodyPtr = dynamic_cast<CBodyPtr*>( m_bodiesCtrl->GetItemData( bodySel.Item(0) ) );
	Body * body = wxDynamicCast( bodyPtr->m_pBody->GetClassInfo()->CreateObject(), Body );
	
	body->Assign( bodyPtr->m_pBody );

	Simulation * sim = reinterpret_cast<Simulation*>( GetClientData() );
	body->SetSimulation( sim );

	CBodyPropertiesDlg dialog;
	dialog.SetClientData( body );

	dialog.Create(this);
    if ( dialog.ShowModal() == wxID_OK )
	{
		bodyPtr->m_pBody->Assign( body );

		CPhaethonDoc * doc = ::wxGetApp().GetDocument();
		doc->Modify( TRUE );

		UpdateBodies();
	}

	delete body;
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_ADDBODY
 */

void CBodiesDlg::OnAddbodyClick( wxCommandEvent& event )
{
	wxArrayTreeItemIds bodySel;
	size_t count = m_bodiesCtrl->GetSelections( bodySel );

	CBodyPtr * parent; 
	if (count == 1)  
		parent = dynamic_cast<CBodyPtr*>( m_bodiesCtrl->GetItemData( bodySel.Item(0) ) );
	else
		parent = NULL;

	Body * body = new Body;
	
	Simulation * sim = reinterpret_cast<Simulation*>( GetClientData() );
	body->SetSimulation( sim );

	CBodyNewDlg dialog;
	dialog.SetClientData( body );

	dialog.Create(this);
    if ( dialog.ShowModal() == wxID_OK )
	{
		sim->Add( body );

		CPhaethonDoc * doc = ::wxGetApp().GetDocument();
		doc->Modify( TRUE );

		UpdateBodies();
	}
	else
	{
		delete body;
	}

}


/*!
 * wxEVT_COMMAND_TREE_SEL_CHANGED event handler for ID_BODIES
 */

void CBodiesDlg::OnBodiesSelChanged( wxTreeEvent& event )
{
	wxArrayTreeItemIds bodySel;
	size_t count = m_bodiesCtrl->GetSelections( bodySel );

	if (count == 1)
	{
		CBodyPtr * b = dynamic_cast<CBodyPtr*>( m_bodiesCtrl->GetItemData( bodySel.Item(0) ) );

		std::map<wxString, wxString>::const_iterator it = m_mapClasses.find( b->m_pBody->GetClassInfo()->GetClassName() );
		if ( it != m_mapClasses.end() )
		{
			m_classes->SetSelection( m_classes->FindString( it->second ) );
		}
	}
}


/*!
 * wxEVT_COMMAND_CHOICE_SELECTED event handler for ID_CLASSES
 */

void CBodiesDlg::OnClassesSelected( wxCommandEvent& event )
{
	wxArrayTreeItemIds bodySel;
	size_t count = m_bodiesCtrl->GetSelections( bodySel );

	if (count == 1)
	{
		CBodyPtr * p = dynamic_cast<CBodyPtr*>( m_bodiesCtrl->GetItemData( bodySel.Item(0) ) );	
		Body * oldBody = p->m_pBody;

		Simulation * sim = reinterpret_cast<Simulation*>( GetClientData() );
        
		Body * b;

		std::map<wxString, wxString>::const_iterator it;
		for ( it = m_mapClasses.begin( ) ; it != m_mapClasses.end( ) ; it++ )
		{
			if ( m_classes->GetStringSelection().Cmp( it->second ) == 0 )
			{
				b = dynamic_cast<Body*>( ::wxCreateDynamicObject( it->first ) );
				break;
			}
		}

		b->SetSimulation( sim );
		b->SetParent( oldBody->GetParent() );

		b->Assign( oldBody );

		//  remove from main list
		std::vector<Body*>::iterator it2;
		for ( it2 = sim->GetBodies()->begin( ) ; it2 != sim->GetBodies()->end( ) ; it2++ )
		{
            if ((*it2) == oldBody)
			{
				sim->GetBodies()->erase( it2 );
				break;
			}
		}

		//  insert into new list
		sim->GetBodies()->push_back( b );

		//  remove old and add new into parent's list
		if (oldBody->GetParent() != NULL)
		{
			for ( it2 = oldBody->GetParent()->GetChildList()->begin( ) ; it2 != oldBody->GetParent()->GetChildList()->end( ) ; it2++ )
			{
				if ((*it2) == oldBody)
				{
					oldBody->GetParent()->GetChildList()->erase( it2 );
					oldBody->GetParent()->GetChildList()->push_back( b );
					break;
				}			
			}
		}

		//  children
		for ( it2 = oldBody->GetChildList()->begin( ) ; it2 != oldBody->GetChildList()->end( ) ; it2++ )
		{
			(*it2)->SetParent( b );
			b->GetChildList()->push_back( *it2 );
		}
		
		delete oldBody;

		UpdateBodies();
	}

}


/*!
 * wxEVT_UPDATE_UI event handler for ID_CLASSES
 */

void CBodiesDlg::OnClassesUpdate( wxUpdateUIEvent& event )
{
	wxArrayTreeItemIds dummy;
	size_t count = m_bodiesCtrl->GetSelections( dummy );

	m_classes->Enable( count == 1 );
}


/*!
 * wxEVT_COMMAND_TREE_ITEM_ACTIVATED event handler for ID_BODIES
 */

void CBodiesDlg::OnBodiesItemActivated( wxTreeEvent& event )
{
	OnPropsbodyClick( wxCommandEvent() );
}


