/************************************************************
*
*  Copyright (c) 2002-2004 Jakub Schwarzmeier
*
************************************************************/

#include "stdafx.h"

#include "body.h"
#include "simulation.h"
#include <schwarzmeier/astronomy.h>
#include <locale>
#include <algorithm>

using namespace JakubSchwarzmeier::Astronomy;

IMPLEMENT_DYNAMIC_CLASS (Body, wxObject)


Body::Body()
{
	m_mass = -1.0;
	m_position = 0;
	m_velocity = 0;
	m_acceleration = 0;

	m_simulation = NULL;
	m_spiceBodyID = 0;
	m_bUseSPICE = false;

	for(int i = 0; i < 3; i++)	m_color[i] = 255;

	m_parent = NULL;

    //  dispatch
	mapDispatch.insert( MAPDISPATCH::value_type( this->GetClassInfo() , Assign2 ) );
}

Body::Body( const Body & original )
{
	operator=( original );
}

void Body::Assign(Body * body) 
{
	MAPDISPATCH::const_iterator it1 = mapDispatch.find( this->GetClassInfo() );
	MAPDISPATCH::const_iterator it2 = mapDispatch.find( body->GetClassInfo() );

	if ( it1->first->IsKindOf( it2->first ) )
	{
		it2->second( this, body );
	}
	else
	{
		it1->second( this, body );
	}
}	

/* static */
void Body::Assign2(Body * body, Body * right_side) 
{
	body->operator=( *right_side );
}

Body & Body::operator=( const Body & right_side )
{
	if ( &right_side == this )  return (*this);

	m_names = right_side.m_names;	
	m_mass = right_side.m_mass;
	m_position = right_side.m_position;
	m_velocity = right_side.m_velocity;
	m_acceleration = right_side.m_acceleration;
	m_persPosition = right_side.m_persPosition;
	m_persVelocity = right_side.m_persVelocity;

	m_spiceBodyID = right_side.m_spiceBodyID;
	m_bUseSPICE = right_side.m_bUseSPICE;
	for(int i = 0; i < 3; i++)
		m_color[i] = right_side.m_color[i];
	
	return *this;
}

xmlNode * Body::Serialize(xmlDoc * pXmlDoc, xmlNode * pXmlNode)
{
	//  class
	xmlNode * pBodyNode =  xmlNewNode( NULL, (const xmlChar *)s_strBody.c_str() );
	xmlAddChild( pXmlNode, pBodyNode );
	xmlNewProp( pBodyNode, (const xmlChar *)s_strClass.c_str(), (const xmlChar *)this->GetClassInfo()->GetClassName() );

	//  names
	MAP_NAMES::const_iterator it;
	for ( it = m_names.begin( ) ; it != m_names.end( ) ; it++ )
	{
		char szCode[16];
		sprintf(szCode, "%d", it->first );

		xmlNode * pNameNode =  xmlNewNode( NULL, (const xmlChar *)s_strName.c_str() );
		xmlAddChild( pBodyNode, pNameNode );
		xmlNewProp( pNameNode, (const xmlChar *)s_strLang.c_str(), (const xmlChar *)szCode );

		//  convert to right characters
		int size = 512;
		int inLen = strlen((const char *)it->second.c_str());
		char out[512];
		isolat1ToUTF8((unsigned char *)out, &size, (const unsigned char *)it->second.c_str(), &inLen );
		out[size] = '\0';

		SetTextNode( pXmlDoc, pNameNode, out );
	}

	//  SPICE node
	xmlNode * pSpiceNode =  xmlNewNode( NULL, (const xmlChar *)s_strSPICE.c_str() );
	xmlAddChild( pBodyNode, pSpiceNode );
	SetAttribute( pSpiceNode, s_strUse, m_bUseSPICE );

	//  SPICE body ID
	xmlNode * pBodyIDNode =  xmlNewNode( NULL, (const xmlChar *)s_strID.c_str() );
	xmlAddChild( pSpiceNode, pBodyIDNode );
	SetTextNode( pXmlDoc, pBodyIDNode, m_spiceBodyID );

	//  mass
	xmlNode * pMassNode =  xmlNewNode( NULL, (const xmlChar *)s_strMass.c_str() );
	xmlAddChild( pBodyNode, pMassNode );
	SetTextNode( pXmlDoc, pMassNode, m_mass );

	//  initial conditions node -- position, velocity
	xmlNode * pICsNode =  xmlNewNode( NULL, (const xmlChar *)s_strICs.c_str() );
	xmlAddChild( pBodyNode, pICsNode );

	xmlNode * pPositionNode =  xmlNewNode( NULL, (const xmlChar *)s_strPosition.c_str() );
	xmlAddChild( pICsNode, pPositionNode );
	
	SetAttribute( pPositionNode, std::string("x"), m_persPosition[0] );
	SetAttribute( pPositionNode, std::string("y"), m_persPosition[1] );
	SetAttribute( pPositionNode, std::string("z"), m_persPosition[2] );
	
	xmlNode * pVelocityNode =  xmlNewNode( NULL, (const xmlChar *)s_strVelocity.c_str() );
	xmlAddChild( pICsNode, pVelocityNode );
	
	SetAttribute( pVelocityNode, std::string("x"), m_persVelocity[0] );
	SetAttribute( pVelocityNode, std::string("y"), m_persVelocity[1] );
	SetAttribute( pVelocityNode, std::string("z"), m_persVelocity[2] );

	//  graphics node
	xmlNode * pGraphicsNode =  xmlNewNode( NULL, (const xmlChar *)s_strGraphics.c_str() );
	xmlAddChild( pBodyNode, pGraphicsNode );


	//  color
	xmlNode * pColorNode =  xmlNewNode( NULL, (const xmlChar *)s_strColor.c_str() );
	xmlAddChild( pGraphicsNode, pColorNode );
	SetAttribute( pColorNode, std::string("R"), m_color[0] );
	SetAttribute( pColorNode, std::string("G"), m_color[1] );
	SetAttribute( pColorNode, std::string("B"), m_color[2] );

	return pBodyNode;
}

Body * Body::Deserialize( xmlNode * pXmlNode )
{
	xmlChar * attr = xmlGetProp(pXmlNode, (const xmlChar *) s_strClass.c_str() );
	std::string strClassName( reinterpret_cast<char*>(attr) );
	xmlFree(attr);

	wxClassInfo * classInfo = wxClassInfo::FindClass( strClassName.c_str() );
	if ( !classInfo->IsKindOf( CLASSINFO(Body) ) )
	{
		throw;
	}

	return dynamic_cast<Body*>( ::wxCreateDynamicObject( strClassName.c_str() ) );
}

void Body::Deserialize(xmlDoc * pXmlDoc, xmlNode * pXmlNode )
{
	//  get name of body
	xmlNodeSet * nameNodes = SelectNodes( pXmlDoc, pXmlNode, s_strName );
	if ( nameNodes != NULL)
	{
		for(int i = 0; i < nameNodes->nodeNr; i++)
		{
			xmlChar * text = xmlNodeListGetString(pXmlDoc, nameNodes->nodeTab[i]->children, 1);
			if (text == NULL)  continue;

			//  convert to right characters
			int size = 512;
			int inLen = strlen((const char *)text);
			char out[512];
			UTF8Toisolat1((unsigned char *)out, &size, text, &inLen );
			out[size] = '\0';

			xmlChar * szCode = xmlGetProp(nameNodes->nodeTab[i], (const xmlChar *) s_strLang.c_str() );
			int code = atoi( (const char*) szCode );
			xmlFree(szCode);

			m_names.insert( MAP_NAMES::value_type(code, out) );

			xmlFree( text );
		}
	}


	//  SPICE
	//
	xmlNode * spiceNode = SelectSingleNode(pXmlDoc, pXmlNode, s_strSPICE );
	GetAttribute( spiceNode, s_strUse, &m_bUseSPICE );
	
	//  SPICE ID
	std::string xpathIDStr(s_strID);
	xpathIDStr += "/text()";
	std::string strID = GetXPathResponse( pXmlDoc, spiceNode, xpathIDStr );
	m_spiceBodyID = atoi( strID.c_str() );

	//  get mass of body
	std::string xpathMassStr(s_strMass);
	xpathMassStr += "/text()";

	std::string strMass = GetXPathResponse( pXmlDoc, pXmlNode, xpathMassStr  );
	m_mass = _tstof( strMass.c_str() );

	//  initial conditions (position and velocity)
	//
	xmlNode * ICsNode = SelectSingleNode(pXmlDoc, pXmlNode, s_strICs );

	if ( ICsNode != NULL )
	{
		//  get position
		xmlNode * positionNode = SelectSingleNode( pXmlDoc, ICsNode, s_strPosition );
		GetAttribute( positionNode, std::string("x"), &m_position[0] );
		GetAttribute( positionNode, std::string("y"), &m_position[1] );
		GetAttribute( positionNode, std::string("z"), &m_position[2] );
		m_persPosition = m_position;

		//  get velocity
		xmlNode * velocityNode = SelectSingleNode( pXmlDoc, ICsNode, s_strVelocity );
		GetAttribute( velocityNode, std::string("x"), &m_velocity[0] );
		GetAttribute( velocityNode, std::string("y"), &m_velocity[1] );
		GetAttribute( velocityNode, std::string("z"), &m_velocity[2] );
		m_persVelocity = m_velocity;
	}
	//
	//  end initial conditions (position and velocity)

	if (m_bUseSPICE)
	{
		InitializeFromSPICE();
	}

	//  graphics node
	xmlNode * graphicsNode = SelectSingleNode( pXmlDoc, pXmlNode, s_strGraphics );

	//  color
	xmlNode * colorNode = SelectSingleNode( pXmlDoc, graphicsNode, s_strColor );
	GetAttribute( colorNode, std::string("R"), &m_color[0] );
	GetAttribute( colorNode, std::string("G"), &m_color[1] );
	GetAttribute( colorNode, std::string("B"), &m_color[2] );

}

void Body::SetSimulation( Simulation * sim )
{
	m_simulation = sim;
}

void Body::SetName( std::string name )
{
	m_names.insert( MAP_NAMES::value_type( wxLANGUAGE_ENGLISH, name ) );
}

void Body::SetMass( double mass )
{
	m_mass = mass;
}

void Body::SetPosition( const JakubSchwarzmeier::Std::Math::Vector<> & position )
{
	m_position = position;
}

void Body::SetVelocity( const JakubSchwarzmeier::Std::Math::Vector<> & velocity )
{
	m_velocity = velocity;
}

void Body::SetSpiceBodyID( int bodyID )
{
	m_spiceBodyID = bodyID;
}


JakubSchwarzmeier::Std::Math::Vector<> & Body::GetPosition()
{
	return m_position;
}

JakubSchwarzmeier::Std::Math::Vector<> & Body::GetVelocity()
{
	return m_velocity;
}

JakubSchwarzmeier::Std::Math::Vector<> & Body::GetAcceleration()
{
	return m_acceleration;
}

void Body::SetColor(int r, int g, int b)
{
	m_color[0] = r;
	m_color[1] = g;
	m_color[2] = b;
}

void Body::GetColor(int * r, int * g, int * b)
{
	*r = m_color[0];
	*g = m_color[1];
	*b = m_color[2];
}

void Body::Render()
{
/*
	glPushAttrib(GL_LIGHTING|GL_ENABLE_BIT );
	glDisable( GL_LIGHTING);
	glDisable (GL_TEXTURE_2D);

	glPushMatrix();

	//  set color for body and path
	glColor3ub(m_color[0], m_color[1], m_color[2]);

	//
	//

	Vector<> observer = 0;
	if (m_simulation->m_openGL != NULL)
	{
		observer = m_simulation->m_openGL->GetObserver()->GetPosition();
	}

	Vector<> vecCenter = 0;
	if (m_parent != NULL)		//  if i have parent, draw parent-centric trajectory
		vecCenter = m_parent->GetPosition();

	glBegin(GL_LINE_STRIP);
	for(int i = m_iPositionsArray; i < m_prevPositions.GetSize(); i+=4)	//  //  draw only every fourth, it's faster
		glVertex3d(vecCenter[0] + m_prevPositions.GetAt(i).values[0] - observer[0], vecCenter[1] + m_prevPositions.GetAt(i).values[1] - observer[1], vecCenter[2] + m_prevPositions.GetAt(i).values[2] - observer[2] );

	for(int i = 0; i < m_iPositionsArray; i+=4)	//  draw only every fourth, it's faster
		glVertex3d(vecCenter[0] + m_prevPositions.GetAt(i).values[0] - observer[0], vecCenter[1] + m_prevPositions.GetAt(i).values[1] - observer[1], vecCenter[2] + m_prevPositions.GetAt(i).values[2] - observer[2] );
	glEnd();
	

	//
	//  end draw path 

	//  draw body  
	//

	glPointSize(5);

	if (m_simulation->m_openGL != NULL)
	{
		Vector<> observer = m_simulation->m_openGL->GetObserver()->GetPosition();

		glTranslated( -observer[0]+m_position[0], -observer[1]+m_position[1], -observer[2]+m_position[2] );
	
	}

	glBegin (GL_POINTS);
		glVertex3f( 0, 0, 0 );
	glEnd();
	//
	//  end draw body  

	glPopMatrix();

	//  draw body name
	//
	glPushMatrix();
	CString strBodyName = GetName( CString(_T("csy")) );
	strBodyName = CString(_T("  ")) + strBodyName;
	
	
	if (m_simulation->m_openGL != NULL)
	{
		Vector<> observer = m_simulation->m_openGL->GetObserver()->GetPosition();

		glRasterPos3d( -observer[0]+m_position[0], -observer[1]+m_position[1], -observer[2]+m_position[2] );
	
	}

	m_simulation->m_openGL->DrawText(strBodyName, _T("Times New Roman"));

	glPopMatrix();
	//
	//  end draw body name

	glPopAttrib();
*/
}

void Body::Update()
{
	//  store previous position (spline?)

}

void Body::SetRelation( bool parent, Body * body )
{
	if (parent)
	{
		//  parent
		m_parent = body;
	}
	else
	{
		//  child
		m_childList.push_back( body );
	}
}

void Body::RemoveRelation( bool parent, Body * body )
{
	if (parent)
	{
		//  parent
		m_parent = NULL;
	}
	else
	{
		//  child
		std::vector<Body*>::iterator it;
		for ( it = m_childList.begin( ) ; it != m_childList.end( ) ; it++ )
		{
			if (*it == body)
			{
				m_childList.erase( it );
				break;
			}
		}
	}
}

double Body::GetMass()
{
	return m_mass;
}

std::string Body::GetName( )
{
	MAP_NAMES::const_iterator it = m_names.find( wxLocale::GetSystemLanguage() );
	if ( it == m_names.end() )
	{
		InitializeFromSPICE();

		//  not found
		it = m_names.find( wxLANGUAGE_ENGLISH );
		if ( it == m_names.end() )
		{
			if ( m_names.begin() != m_names.end() )
			{
				it = m_names.begin();
			}			
			else
				return "";
		}
	}

	return it->second;
}

Body::MAP_NAMES * Body::GetNames()
{
	return &m_names;
}

int Body::GetSpiceBodyID()
{
	return m_spiceBodyID;
}

void Body::SetUseSPICE( bool bUseSPICE )
{
	m_bUseSPICE = bUseSPICE;
	if (m_bUseSPICE)  InitializeFromSPICE();
}

bool Body::GetUseSPICE( )
{
	return m_bUseSPICE;
}

void Body::InitializeFromSPICE()
{
	if (!m_bUseSPICE)  return;

	CSpice * spice = m_simulation->GetSpice();

	std::string name;
	spice->GetName( m_spiceBodyID, name );

	bool prevNonLetter = true;
	for(int i = 0; i < name.length(); i++)
	{
		if (!prevNonLetter)
		{
			name[i] = tolower( name[i] );
		}

		if ( isalpha(name[i]) )
			prevNonLetter = false;
		else
			prevNonLetter = true;
	}

	MAP_NAMES::const_iterator it = m_names.find( wxLANGUAGE_ENGLISH );
	if ( it != m_names.end() )
	{
		//  found
		m_names.erase( it->first );
	}
	m_names.insert( MAP_NAMES::value_type( wxLANGUAGE_ENGLISH, name ) );

	try
	{
		JakubSchwarzmeier::Std::Math::Vector<> position, velocity;
		spice->GetEphemerisEclipticRectangular( 0/*SSB*/, m_spiceBodyID,
			m_simulation->GetEt().GetJulianDayNumber(), position, velocity );
		m_position = position;
		m_velocity = velocity;
	}
	catch( ... )
	{
		//  there are no ephemerides data for given time
	}

	try
	{
		double mass;
		spice->GetMass( m_spiceBodyID, mass );
		m_mass = mass;
	}
	catch (...)
	{
	}
}

std::vector<Body*> * Body::GetChildList()
{
	return &m_childList;
}

Body * Body::GetParent()
{
	return m_parent;
}

void Body::SetParent( Body * parent )
{
	m_parent = parent;
}

std::string Body::s_strBody( _T("body") );
std::string Body::s_strClass( _T("class") );
std::string Body::s_strName( _T("name") );
std::string Body::s_strLang( _T("lang") );
std::string Body::s_strMass( _T("mass") );
std::string Body::s_strICs( _T("ics") );
std::string Body::s_strPosition( _T("position") );
std::string Body::s_strVelocity( _T("velocity") );
std::string Body::s_strSPICE( _T("spice") );
std::string Body::s_strUse( _T("use") );
std::string Body::s_strID( _T("ID") );
std::string Body::s_strColor( _T("color") );
std::string Body::s_strGraphics( _T("graphics") );
