/************************************************************
*
*  Copyright (c) 2002-2004 Jakub Schwarzmeier
*
************************************************************/

#ifndef __BODY__
#define __BODY__

#include <schwarzmeier/std.h>

//  forward declaration
class Simulation;

class Body: public wxObject
{
	friend Simulation;

public:

	Body();
	virtual ~Body() {};
	Body( const Body & original );
	Body & operator=( const Body & right_side );

	void Assign(Body * body);

	static Body * Deserialize( xmlNode * pXmlNode ) ;

	virtual xmlNode * Serialize(xmlDoc * pXmlDoc, xmlNode * pXmlNode );
	virtual void Deserialize( xmlDoc * pXmlDoc, xmlNode * pXmlNode );
	virtual void InitializeFromSPICE();

	void SetSimulation( Simulation * sim );

	virtual void Render();					//  render itself
	virtual void Update();					//  integrate

	void SetName( std::string name );
	void SetMass( double mass );
	void SetPosition( const JakubSchwarzmeier::Std::Math::Vector<> & position );
	void SetVelocity( const JakubSchwarzmeier::Std::Math::Vector<> & velocity );

	typedef std::map<int, std::string> MAP_NAMES;

	std::string GetName();
	MAP_NAMES * GetNames();
	double GetMass();
	JakubSchwarzmeier::Std::Math::Vector<> & GetPosition();
	JakubSchwarzmeier::Std::Math::Vector<> & GetVelocity();
	JakubSchwarzmeier::Std::Math::Vector<> & GetAcceleration();

	int GetSpiceBodyID();
	void SetSpiceBodyID( int bodyID );

	void SetUseSPICE( bool bUseSPICE );
	bool GetUseSPICE( );

	void SetColor(int r, int g, int b);
	void GetColor(int * r, int * g, int * b);

	std::vector<Body*> * GetChildList();
	Body * GetParent();
	void SetParent( Body * parent );

	void SetRelation( bool parent, Body * body );
	void RemoveRelation( bool parent, Body * body );

protected:

	MAP_NAMES m_names;		//  name in various languages
	double m_mass;				//  body mass
	JakubSchwarzmeier::Std::Math::Vector<> m_position;		//  body position
	JakubSchwarzmeier::Std::Math::Vector<> m_velocity;		//  body velocity
	JakubSchwarzmeier::Std::Math::Vector<> m_acceleration;	//  body acceleration
	JakubSchwarzmeier::Std::Math::Vector<> m_persPosition;		//  persistent body position
	JakubSchwarzmeier::Std::Math::Vector<> m_persVelocity;		//  persistent body velocity

	//  SPICE fields
	//
	int m_spiceBodyID;			//  ID of body under SPICE (name)
	bool m_bUseSPICE;
	//
	//  end SPICE fields

	int m_color[3];				//  RGB color of body and orbital path
	
	//  relation data fields
	Simulation * m_simulation;
	Body * m_parent;
	std::vector<Body*> m_childList;

	//  dispatch table
	typedef std::map<const wxClassInfo*, void(*)(Body*,Body*)> MAPDISPATCH;
	MAPDISPATCH mapDispatch;

	static void Assign2(Body * body, Body * right_side);

	static std::string s_strBody;
	static std::string s_strClass;
	static std::string s_strName;
	static std::string s_strLang;
	static std::string s_strMass;
	static std::string s_strICs;
	static std::string s_strPosition;
	static std::string s_strVelocity;
	static std::string s_strSPICE;
	static std::string s_strUse;
	static std::string s_strID;
	static std::string s_strGraphics;
	static std::string s_strColor;

	DECLARE_DYNAMIC_CLASS (Body)

};	//  end class Body

#endif //  __BODY__