/////////////////////////////////////////////////////////////////////////////
// Name:        BodyLocaleNameDlg.cpp
// Purpose:     
// Author:      Jakub Schwarzmeier
// Modified by: 
// Created:     09/17/06 09:01:40
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#if defined(__GNUG__) && !defined(__APPLE__)
#pragma implementation "BodyLocaleNameDlg.h"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
////@end includes

#include "BodyLocaleNameDlg.h"
#include "Body.h"
#include "PhaethonApp.h"

////@begin XPM images
////@end XPM images

/*!
 * CBodyLocaleNameDlg type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CBodyLocaleNameDlg, wxDialog )

/*!
 * CBodyLocaleNameDlg event table definition
 */

BEGIN_EVENT_TABLE( CBodyLocaleNameDlg, wxDialog )

////@begin CBodyLocaleNameDlg event table entries
    EVT_BUTTON( XRCID("ID_ADD"), CBodyLocaleNameDlg::OnAddClick )

    EVT_BUTTON( wxID_OK, CBodyLocaleNameDlg::OnOkClick )

////@end CBodyLocaleNameDlg event table entries

END_EVENT_TABLE()

/*!
 * CBodyLocaleNameDlg constructors
 */

CBodyLocaleNameDlg::CBodyLocaleNameDlg( )
{
}

CBodyLocaleNameDlg::CBodyLocaleNameDlg( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Create(parent, id, caption, pos, size, style);
}

/*!
 * CBodyLocaleNameDlg creator
 */

bool CBodyLocaleNameDlg::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CBodyLocaleNameDlg member initialisation
    m_localeCtrl = NULL;
////@end CBodyLocaleNameDlg member initialisation

////@begin CBodyLocaleNameDlg creation
    SetExtraStyle(GetExtraStyle()|wxWS_EX_BLOCK_EVENTS);
    SetParent(parent);
    CreateControls();
    GetSizer()->Fit(this);
    GetSizer()->SetSizeHints(this);
    Centre();
////@end CBodyLocaleNameDlg creation
    return true;
}

/*!
 * Control creation for CBodyLocaleNameDlg
 */

void CBodyLocaleNameDlg::CreateControls()
{    
	wxXmlResource::Get()->Load("xrc/bodylocalename.xrc");
////@begin CBodyLocaleNameDlg content construction
    wxXmlResource::Get()->LoadDialog(this, GetParent(), _T("IDD_BODYLOCALENAME"));
    m_localeCtrl = XRCCTRL(*this, "ID_LOCALENAME", wxListCtrl);
////@end CBodyLocaleNameDlg content construction

	//  Listbox
	wxListItem itemCol;
    itemCol.m_format = wxLIST_FORMAT_LEFT;
	itemCol.m_mask = wxLIST_MASK_FORMAT;
    m_localeCtrl->InsertColumn(0, itemCol);
    m_localeCtrl->InsertColumn(1, itemCol);
	
    // to speed up inserting we hide the control temporarily
    m_localeCtrl->Hide();

	void * clientData = GetClientData();
	Body::MAP_NAMES * mapNames = reinterpret_cast<Body::MAP_NAMES*>( clientData );

	long l = 0;
	Body::MAP_NAMES::const_iterator it;
	for( it = mapNames->begin(); it != mapNames->end(); it++ )
	{
		char szCode[16];
		sprintf(szCode, "%d", it->first);

		m_localeCtrl->InsertItem( l, szCode );
		m_localeCtrl->SetItem( l, 1, it->second.c_str() );

		l++;
	}

	m_localeCtrl->SetColumnWidth(0, wxLIST_AUTOSIZE);
	m_localeCtrl->SetColumnWidth(1, wxLIST_AUTOSIZE);
    m_localeCtrl->Show();

	// Create custom windows not generated automatically here.

////@begin CBodyLocaleNameDlg content initialisation

////@end CBodyLocaleNameDlg content initialisation
}

/*!
 * Should we show tooltips?
 */

bool CBodyLocaleNameDlg::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CBodyLocaleNameDlg::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CBodyLocaleNameDlg bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CBodyLocaleNameDlg bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CBodyLocaleNameDlg::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CBodyLocaleNameDlg icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CBodyLocaleNameDlg icon retrieval
}
/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_ADD
 */

void CBodyLocaleNameDlg::OnAddClick( wxCommandEvent& event )
{
	wxTextEntryDialog txt( this, _T("Please enter localized name of the body:"),
		_T("Please enter text"), _(""), wxOK);

	txt.ShowModal();

	wxString strCode;
	strCode << wxLocale::GetSystemLanguage();

	wxListItem itemCol;
	itemCol.SetText( strCode );
	itemCol.m_mask = wxLIST_MASK_TEXT;

	long i = m_localeCtrl->InsertItem( itemCol );
	m_localeCtrl->SetItem( i, 1, txt.GetValue() );

	m_localeCtrl->SetColumnWidth(0, wxLIST_AUTOSIZE);
	m_localeCtrl->SetColumnWidth(1, wxLIST_AUTOSIZE);

	m_localeCtrl->EditLabel( i );
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_OK
 */

void CBodyLocaleNameDlg::OnOkClick( wxCommandEvent& event )
{
	if ( Validate() && TransferDataFromWindow() )
	{
		void * clientData = GetClientData();
		Body::MAP_NAMES * mapNames = reinterpret_cast<Body::MAP_NAMES*>( clientData );

		mapNames->clear();

		long item = -1;
		for ( ;; )
		{
			item = m_localeCtrl->GetNextItem(item);
			if ( item == -1 )
				break;

			wxString text = m_localeCtrl->GetItemText( item );

			wxListItem info;
			info.SetId( item );
			info.m_col = 1;
			info.m_mask = wxLIST_MASK_TEXT;
			bool v = m_localeCtrl->GetItem(info);

			long code;
			text.ToLong(&code);
			mapNames->insert( Body::MAP_NAMES::value_type( code, info.m_text.c_str() ) );
		}

		EndDialog( wxID_OK );
	}
}


