/////////////////////////////////////////////////////////////////////////////
// Name:        BodyPropertiesPnl.cpp
// Purpose:     
// Author:      Jakub Schwarzmeier
// Modified by: 
// Created:     08/13/06 15:02:41
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#if defined(__GNUG__) && !defined(__APPLE__)
#pragma implementation "BodyPropertiesPnl.h"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
////@end includes

#include "BodyPropertiesPnl.h"
#include "Body.h"
#include "BodyLocaleNameDlg.h"
#include <schwarzmeier/std.h>

////@begin XPM images
////@end XPM images

/*!
 * CBodyPropertiesPnl type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CBodyPropertiesPnl, wxPanel )

/*!
 * CBodyPropertiesPnl event table definition
 */

BEGIN_EVENT_TABLE( CBodyPropertiesPnl, wxPanel )

////@begin CBodyPropertiesPnl event table entries
    EVT_BUTTON( XRCID("ID_LOCALE"), CBodyPropertiesPnl::OnLocaleClick )

    EVT_UPDATE_UI( XRCID("ID_VZ"), CBodyPropertiesPnl::OnUpdateNonSpice )

    EVT_BUTTON( XRCID("ID_CHOOSECOLOR"), CBodyPropertiesPnl::OnChoosecolorClick )

////@end CBodyPropertiesPnl event table entries

END_EVENT_TABLE()

/*!
 * CBodyPropertiesPnl constructors
 */

CBodyPropertiesPnl::CBodyPropertiesPnl( )
{
}

CBodyPropertiesPnl::CBodyPropertiesPnl( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
    Create(parent, id, pos, size, style);
}

/*!
 * CBodyPropertiesPnl creator
 */

bool CBodyPropertiesPnl::Create( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CBodyPropertiesPnl member initialisation
    m_px = 0;
    m_py = 0;
    m_pz = 0;
    m_vx = 0;
    m_vy = 0;
    m_vz = 0;
    m_mass = 0;
    m_spiceID = 0;
    m_useSpiceCtrl = NULL;
////@end CBodyPropertiesPnl member initialisation

////@begin CBodyPropertiesPnl creation
    SetParent(parent);
    CreateControls();
    GetSizer()->Fit(this);
    GetSizer()->SetSizeHints(this);
    Centre();
////@end CBodyPropertiesPnl creation
    return true;
}

/*!
 * Control creation for CBodyPropertiesPnl
 */

void CBodyPropertiesPnl::CreateControls()
{    
    wxXmlResource::Get()->Load(wxT("xrc/bodyproperties.xrc")); 

////@begin CBodyPropertiesPnl content construction
    wxXmlResource::Get()->LoadPanel(this, GetParent(), _T("IDD_BODYPROPERTIES"));
    m_useSpiceCtrl = XRCCTRL(*this, "ID_USESPICE", wxCheckBox);
    // Set validators
    if (FindWindow(XRCID("ID_NAME")))
        FindWindow(XRCID("ID_NAME"))->SetValidator( wxNotNullValidator(& m_name, _T("Please enter the name of body.") ) );
    if (FindWindow(XRCID("ID_MASS")))
        FindWindow(XRCID("ID_MASS"))->SetValidator( wxDoubleValidator(& m_mass) );
    if (FindWindow(XRCID("ID_SPICEID")))
        FindWindow(XRCID("ID_SPICEID"))->SetValidator( wxIntegerValidator(& m_spiceID) );
    if (FindWindow(XRCID("ID_PX")))
        FindWindow(XRCID("ID_PX"))->SetValidator( wxDoubleValidator(& m_px) );
    if (FindWindow(XRCID("ID_PY")))
        FindWindow(XRCID("ID_PY"))->SetValidator( wxDoubleValidator(& m_py) );
    if (FindWindow(XRCID("ID_PZ")))
        FindWindow(XRCID("ID_PZ"))->SetValidator( wxDoubleValidator(& m_pz) );
    if (FindWindow(XRCID("ID_VX")))
        FindWindow(XRCID("ID_VX"))->SetValidator( wxDoubleValidator(& m_vx) );
    if (FindWindow(XRCID("ID_VY")))
        FindWindow(XRCID("ID_VY"))->SetValidator( wxDoubleValidator(& m_vy) );
    if (FindWindow(XRCID("ID_VZ")))
        FindWindow(XRCID("ID_VZ"))->SetValidator( wxDoubleValidator(& m_vz) );
////@end CBodyPropertiesPnl content construction

    // Create custom windows not generated automatically here.
	Body * body = reinterpret_cast<Body*>( GetClientData() );
	m_name = body->GetName().c_str();
	m_mass = body->GetMass();
	m_spiceID = body->GetSpiceBodyID();

	using namespace JakubSchwarzmeier::Std::Math;
	Vector<> pos = body->GetPosition();
	Vector<> vel = body->GetVelocity();
	m_px = pos[0];
	m_py = pos[1];
	m_pz = pos[2];

	m_vx = vel[0];
	m_vy = vel[1];
	m_vz = vel[2];

	m_useSpiceCtrl->SetValue( body->GetUseSPICE() );

////@begin CBodyPropertiesPnl content initialisation

////@end CBodyPropertiesPnl content initialisation
}

/*!
 * Should we show tooltips?
 */

bool CBodyPropertiesPnl::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CBodyPropertiesPnl::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CBodyPropertiesPnl bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CBodyPropertiesPnl bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CBodyPropertiesPnl::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CBodyPropertiesPnl icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CBodyPropertiesPnl icon retrieval
}

bool CBodyPropertiesPnl::TransferDataFromWindow()
{
	bool r = wxPanel::TransferDataFromWindow();

	if (r)
	{
		Body * body = reinterpret_cast<Body*>( GetClientData() );
		body->SetUseSPICE( m_useSpiceCtrl->GetValue( ) );

		body->SetSpiceBodyID( m_spiceID );

		body->SetName( m_name.c_str() );
		body->SetMass( m_mass );

		using namespace JakubSchwarzmeier::Std::Math;
		Vector<> pos(m_px, m_py, m_pz);
		Vector<> vel(m_vx, m_vy, m_vz);

		body->SetPosition( pos );
		body->SetVelocity( vel );

	}

	return r;
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_CHOOSECOLOR
 */

void CBodyPropertiesPnl::OnChoosecolorClick( wxCommandEvent& event )
{
	Body * body = reinterpret_cast<Body*>( GetClientData() );
	int r, g, b;
	body->GetColor(&r, &g, &b);

	wxColour color(r, g, b);

	wxColourData colData;
	colData.SetColour( color );
	wxColourDialog colorDlg(this, &colData);

	if ( colorDlg.ShowModal() == wxID_OK )
	{
		color = colorDlg.GetColourData().GetColour();
		body->SetColor( color.Red(), color.Green(), color.Blue() );
	}
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_LOCALE
 */

void CBodyPropertiesPnl::OnLocaleClick( wxCommandEvent& event )
{
	CBodyLocaleNameDlg dlg;

	Body * body = reinterpret_cast<Body*>( GetClientData() );

	dlg.SetClientData( body->GetNames() );
	dlg.Create(this);

	if ( dlg.ShowModal() == wxID_OK )
	{
	}
}














/*!
 * wxEVT_UPDATE_UI event handler for ID_VZ
 */

void CBodyPropertiesPnl::OnUpdateNonSpice( wxUpdateUIEvent& event )
{
////@begin wxEVT_UPDATE_UI event handler for ID_VZ in CBodyPropertiesPnl.
    // Before editing this code, remove the block markers.
    event.Skip();
////@end wxEVT_UPDATE_UI event handler for ID_VZ in CBodyPropertiesPnl. 
}


