/////////////////////////////////////////////////////////////////////////////
// Name:        CamerasPanel.cpp
// Purpose:     
// Author:      Jakub Schwarzmeier
// Modified by: 
// Created:     08/20/06 10:55:54
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#if defined(__GNUG__) && !defined(__APPLE__)
#pragma implementation "CCamerasPanel.h"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
////@end includes

#include "PhaethonApp.h"
#include "PhaethonDoc.h"
#include "PhaethonView.h"
#include "CamerasPanel.h"
#include "app_resources.h"
#include "OrbiterCameraDlg.h"

#include "ThreedCanvas.h"

////@begin XPM images
////@end XPM images

/*!
 * CCamerasPanel type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CCamerasPanel, wxPanel )

/*!
 * CCamerasPanel event table definition
 */

BEGIN_EVENT_TABLE( CCamerasPanel, wxPanel )

////@begin CCamerasPanel event table entries
    EVT_BUTTON( XRCID("ID_ADDCAMERA"), CCamerasPanel::OnAddcameraClick )

    EVT_BUTTON( XRCID("ID_REMOVECAMERA"), CCamerasPanel::OnRemovecameraClick )
    EVT_UPDATE_UI( XRCID("ID_REMOVECAMERA"), CCamerasPanel::OnRemovecameraUpdate )

    EVT_TREE_ITEM_ACTIVATED( XRCID("ID_CAMERAS"), CCamerasPanel::OnCamerasItemActivated )

////@end CCamerasPanel event table entries

    EVT_MENU( XRCID("ID_ORBITER"), CCamerasPanel::OnOrbiterClick )
    EVT_MENU( XRCID("ID_RECTANGULAR"), CCamerasPanel::OnRectangularClick )

    EVT_UPDATE_UI( XRCID("ID_ORBITER"), CCamerasPanel::OnPopupUpdate )
    EVT_UPDATE_UI( XRCID("ID_RECTANGULAR"), CCamerasPanel::OnPopupUpdate )

END_EVENT_TABLE()

/*!
 * CCamerasPanel constructors
 */

CCamerasPanel::CCamerasPanel( )
{
}

CCamerasPanel::~CCamerasPanel()
{
	m_rectangularObserver->Destroy();
	m_sphericalObserver->Destroy();
}

CCamerasPanel::CCamerasPanel( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
    Create(parent, id, pos, size, style);
}

/*!
 * CCamerasPanel creator
 */

bool CCamerasPanel::Create( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CCamerasPanel member initialisation
    m_addCtrl = NULL;
    m_camerasCtrl = NULL;
////@end CCamerasPanel member initialisation

	m_rectangularObserver = NULL;
	m_sphericalObserver = NULL;

////@begin CCamerasPanel creation
    SetParent(parent);
    CreateControls();
    GetSizer()->Fit(this);
    GetSizer()->SetSizeHints(this);
    Centre();
////@end CCamerasPanel creation
    return true;
}

/*!
 * Control creation for CCamerasPanel
 */

void CCamerasPanel::CreateControls()
{    
    wxXmlResource::Get()->Load(wxT("xrc/cameraspanel.xrc")); 

////@begin CCamerasPanel content construction
    wxXmlResource::Get()->LoadPanel(this, GetParent(), _T("IDD_CAMERASPANEL"));
    m_addCtrl = XRCCTRL(*this, "ID_ADDCAMERA", wxButton);
    m_camerasCtrl = XRCCTRL(*this, "ID_CAMERAS", wxTreeCtrl);
////@end CCamerasPanel content construction

    // Create custom windows not generated automatically here.
	if (::wxGetApp().GetDocument())  UpdateCameras();

	m_rectangularObserver = new CRectangularObserver;
	m_rectangularObserver->OnCreateCamera();
	m_sphericalObserver = new CSphericalObserver;
	m_sphericalObserver->OnCreateCamera();

////@begin CCamerasPanel content initialisation

////@end CCamerasPanel content initialisation
}

/*!
 * Should we show tooltips?
 */

bool CCamerasPanel::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CCamerasPanel::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CCamerasPanel bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CCamerasPanel bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CCamerasPanel::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CCamerasPanel icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CCamerasPanel icon retrieval
}
/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_ADDCAMERA
 */

void CCamerasPanel::OnAddcameraClick( wxCommandEvent& event )
{
	int x, y;
	m_addCtrl->GetPosition(&x, &y);
	wxMenu * menu = AppResources::CreateCamerasPanelMenu();
	PopupMenu( menu );
	delete menu;
}

void CCamerasPanel::OnOrbiterClick( wxCommandEvent& event )
{
	CMainFrame * frame = ::wxGetApp().GetMainFrame();
	wxDocMDIChildFrame * childFrame = wx_reinterpret_cast( wxDocMDIChildFrame*, frame->GetActiveChild() );
	CPhaethonView * view = wx_reinterpret_cast( CPhaethonView*, childFrame->GetView() );

	reinterpret_cast<CThreedCanvas*>( view->GetCanvas() )->AcquireRendering();
	view->GetCanvas()->SetObserver( m_sphericalObserver );
	reinterpret_cast<CThreedCanvas*>( view->GetCanvas() )->ReleaseRendering();

    COrbiterCameraDlg* window 
		= new COrbiterCameraDlg;

	window->SetClientData( NULL );
	window->Create(this);
    window->Show(true);

	CPhaethonDoc * doc = ::wxGetApp().GetDocument();
	doc->Modify( TRUE );
}

void CCamerasPanel::OnRectangularClick( wxCommandEvent& event )
{
	CMainFrame * frame = ::wxGetApp().GetMainFrame();
	wxDocMDIChildFrame * childFrame = wx_reinterpret_cast( wxDocMDIChildFrame*, frame->GetActiveChild() );
	CPhaethonView * view = wx_reinterpret_cast( CPhaethonView*, childFrame->GetView() );

	reinterpret_cast<CThreedCanvas*>( view->GetCanvas() )->AcquireRendering();
	view->GetCanvas()->SetObserver( m_rectangularObserver );
	reinterpret_cast<CThreedCanvas*>( view->GetCanvas() )->ReleaseRendering();

    COrbiterCameraDlg* window 
		= new COrbiterCameraDlg;

	window->SetClientData( NULL );
	window->Create(this);
    window->Show(true);

	CPhaethonDoc * doc = ::wxGetApp().GetDocument();
	doc->Modify( TRUE );
}

void CCamerasPanel::OnPopupUpdate( wxUpdateUIEvent& event )
{
    wxWindow * orbiterWindow = wxWindow::FindWindowById( IDD_ORBITERCAMERA );
	//  dodat rectangular
//    wxWindow * rectangularWindow = wxWindow::FindWindowById( IDD_RECTANGULARCAMERA );

	event.Enable( !(orbiterWindow /*|| rectangularWindow*/) );
}

class CChunkPtr: public wxTreeItemData
{
public:	
	CChunkPtr(CChunkDesc * pChunkDesc = NULL) { m_pChunkDesc = pChunkDesc; }
	CChunkDesc * m_pChunkDesc;
};

void CCamerasPanel::UpdateCameras()
{
	m_camerasCtrl->Hide();
	m_camerasCtrl->DeleteAllItems();
	wxTreeItemId idRoot = m_camerasCtrl->AddRoot( _T("Cameras"), -1, -1, new CChunkPtr(NULL) );

	std::vector<CChunkDesc*> * chunks = ::wxGetApp().GetDocument()->GetVisualization()->GetChunks();
	std::vector<CChunkDesc*>::const_iterator it;
	int i = 1;
	for( it = chunks->begin(); it != chunks->end(); it++)
	{
        wxString name;
		name.sprintf( _T("Camera%d"), i );
		wxTreeItemId nId = m_camerasCtrl->AppendItem( idRoot, name.c_str(), -1, -1, new CChunkPtr( (*it) ) );
		i++;
	}

	m_camerasCtrl->Show();
}

void CCamerasPanel::UpdateObservers()
{
	CRectangularObserver * new_rectangularObserver = new CRectangularObserver;
	new_rectangularObserver->OnCreateCamera();
	CSphericalObserver * new_sphericalObserver = new CSphericalObserver;
	new_sphericalObserver->OnCreateCamera();

	CMainFrame * frame = ::wxGetApp().GetMainFrame();
	wxDocMDIChildFrame * childFrame = wx_reinterpret_cast( wxDocMDIChildFrame*, frame->GetActiveChild() );
	CPhaethonView * view = wx_reinterpret_cast( CPhaethonView*, childFrame->GetView() );

	reinterpret_cast<CThreedCanvas*>( view->GetCanvas() )->AcquireRendering();
	if ( view->GetCanvas()->GetObserver()->IsKindOf( 
		CLASSINFO(CRectangularObserver) ) )
	{
		new_rectangularObserver->SetCamera( dynamic_cast<CCamera*>( m_rectangularObserver->GetCamera() ) );
		new_sphericalObserver->SetCamera( dynamic_cast<CCamera*>( m_sphericalObserver->GetCamera() ) );
	}

    m_rectangularObserver->Destroy();
	m_sphericalObserver->Destroy();

	m_rectangularObserver = new_rectangularObserver;
	m_sphericalObserver = new_sphericalObserver;

	if ( view->GetCanvas()->GetObserver()->IsKindOf( CLASSINFO(CSphericalObserver) ) )
	{
		view->GetCanvas()->SetObserver( m_sphericalObserver );
	}
	else
	{
		view->GetCanvas()->SetObserver( m_rectangularObserver );
	}

	reinterpret_cast<CThreedCanvas*>( view->GetCanvas() )->ReleaseRendering();
}

/*!
 * wxEVT_UPDATE_UI event handler for ID_REMOVECAMERA
 */

void CCamerasPanel::OnRemovecameraUpdate( wxUpdateUIEvent& event )
{
	wxTreeItemId x = m_camerasCtrl->GetSelection();
	event.Enable( x.IsOk() && IsEnabled() );
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_REMOVECAMERA
 */

void CCamerasPanel::OnRemovecameraClick( wxCommandEvent& event )
{
	CChunkPtr* chunk = reinterpret_cast<CChunkPtr*>( m_camerasCtrl->GetItemData( m_camerasCtrl->GetSelection() ) );

	std::vector<CChunkDesc*> * chunks = ::wxGetApp().GetDocument()->GetVisualization()->GetChunks();
	std::vector<CChunkDesc*>::iterator it;
	for( it = chunks->begin(); it != chunks->end(); it++)
	{
		if ( (*it) == chunk->m_pChunkDesc )
		{
			delete (*it);
			chunks->erase( it );
			break;
		}
	}

	UpdateCameras();

	CPhaethonDoc * doc = ::wxGetApp().GetDocument();
	doc->Modify( TRUE );
}


/*!
 * wxEVT_COMMAND_TREE_ITEM_ACTIVATED event handler for ID_CAMERAS
 */

void CCamerasPanel::OnCamerasItemActivated( wxTreeEvent& event )
{
	CMainFrame * frame = ::wxGetApp().GetMainFrame();
	wxDocMDIChildFrame * childFrame = wx_reinterpret_cast( wxDocMDIChildFrame*, frame->GetActiveChild() );
	CPhaethonView * view = wx_reinterpret_cast( CPhaethonView*, childFrame->GetView() );

	CChunkPtr* chunk = reinterpret_cast<CChunkPtr*>( m_camerasCtrl->GetItemData( m_camerasCtrl->GetSelection() ) );

	reinterpret_cast<CThreedCanvas*>( view->GetCanvas() )->AcquireRendering();
	if ( chunk->m_pChunkDesc->GetStartCamera()->IsKindOf( CLASSINFO( CSphericalCamera ) ) )
	{
		view->GetCanvas()->SetObserver( m_sphericalObserver );
	}
	else
	{
		view->GetCanvas()->SetObserver( m_rectangularObserver );
	}
	reinterpret_cast<CThreedCanvas*>( view->GetCanvas() )->ReleaseRendering();


    COrbiterCameraDlg* window 
		= new COrbiterCameraDlg;

	window->SetClientData( chunk->m_pChunkDesc );
	window->Create(this);
    window->Show(true);

	CPhaethonDoc * doc = ::wxGetApp().GetDocument();
	doc->Modify( TRUE );
}

CRectangularObserver * CCamerasPanel::GetRectangularObserver()
{
	return m_rectangularObserver;
}

CSphericalObserver * CCamerasPanel::GetSphericalObserver()
{
	return m_sphericalObserver;
}