#include "StdAfx.h"
#include "chunkdesc.h"

CChunkDesc::CChunkDesc(void)
{
	m_startCamera = new CCamera;
	m_endCamera = new CCamera;
	m_startTime = 0;
	m_endTime = 0;
	m_durationTime = 0;
}

CChunkDesc::~CChunkDesc(void)
{
	delete m_startCamera;
	delete m_endCamera;
}

CChunkDesc::CChunkDesc( const CChunkDesc & original )
{
	operator=( original );
}

CChunkDesc & CChunkDesc::operator=( const CChunkDesc & right_side )
{
	if ( &right_side == this )  return (*this);

	delete m_startCamera;
	delete m_endCamera;

	m_startCamera 
		= dynamic_cast<CCamera*>( right_side.m_startCamera->GetClassInfo()->CreateObject() );

	m_endCamera 
		= dynamic_cast<CCamera*>( right_side.m_endCamera->GetClassInfo()->CreateObject() );

	m_startCamera->Assign( right_side.m_startCamera );
	m_endCamera->Assign( right_side.m_endCamera );

	m_startTime = right_side.m_startTime;
	m_endTime = right_side.m_endTime;
	m_durationTime = right_side.m_durationTime;	

	return (*this);
}

double CChunkDesc::GetDuration()
{
	return m_durationTime;
}

void CChunkDesc::SetDuration(double durationTime)
{
	m_durationTime = durationTime;
}

double CChunkDesc::GetStartTime()
{
	return m_startTime;
}

void CChunkDesc::SetStartTime(double startTime)
{
	m_startTime = startTime;
}

double CChunkDesc::GetEndTime()
{
	return m_endTime;
}

void CChunkDesc::SetEndTime(double endTime)
{
	m_endTime = endTime;
}

CCamera * CChunkDesc::GetStartCamera()
{
	return m_startCamera;
}

void CChunkDesc::SetStartCamera( CCamera * startMatrix )
{
	delete m_startCamera;
	m_startCamera 
		= dynamic_cast<CCamera*>( startMatrix->GetClassInfo()->CreateObject() );
	m_startCamera->Assign( startMatrix );
}

CCamera * CChunkDesc::GetEndCamera()
{
	return m_endCamera;
}

void CChunkDesc::SetEndCamera( CCamera * endMatrix )
{
	delete m_endCamera;

	m_endCamera 
		= dynamic_cast<CCamera*>( endMatrix->GetClassInfo()->CreateObject() );

	m_endCamera->Assign( endMatrix );
}

xmlNode * CChunkDesc::Serialize(xmlDoc * pXmlDoc, xmlNode * pXmlNode)
{
	xmlNode * pChunkNode =  xmlNewNode( NULL, (const xmlChar *)s_strCameraChunk.c_str() );
	xmlAddChild( pXmlNode, pChunkNode );

	//  starttime
	xmlNode * starttimeNode = xmlNewNode( NULL, (const xmlChar *)s_strStartTime.c_str() );
	xmlAddChild( pChunkNode, starttimeNode );
	SetTextNode( pXmlDoc, starttimeNode, m_startTime );

	//  endtime
	xmlNode * endtimeNode = xmlNewNode( NULL, (const xmlChar *)s_strEndTime.c_str() );
	xmlAddChild( pChunkNode, endtimeNode );
	SetTextNode( pXmlDoc, endtimeNode, m_endTime );

	//  durationtime
	xmlNode * durationtimeNode = xmlNewNode( NULL, (const xmlChar *)s_strDurationTime.c_str() );
	xmlAddChild( pChunkNode, durationtimeNode );
	SetTextNode( pXmlDoc, durationtimeNode, m_durationTime );

	//  startmatrix
	xmlNode * startMatrixNode = xmlNewNode( NULL, (const xmlChar *)s_strStartCamera.c_str() );
	xmlAddChild( pChunkNode, startMatrixNode );
	m_startCamera->Serialize( pXmlDoc, startMatrixNode );

	//  endmatrix
	xmlNode * endMatrixNode = xmlNewNode( NULL, (const xmlChar *)s_strEndCamera.c_str() );
	xmlAddChild( pChunkNode, endMatrixNode );
	m_endCamera->Serialize( pXmlDoc, endMatrixNode );

	return pChunkNode;
}

void CChunkDesc::Deserialize(xmlDoc * pXmlDoc, xmlNode * pXmlNode )
{
	std::string xpathIDStr(s_strStartTime);
	xpathIDStr += "/text()";
	std::string strID = GetXPathResponse( pXmlDoc, pXmlNode, xpathIDStr );
	m_startTime = _tstof( strID.c_str() );

	std::string xpathEndStr(s_strEndTime);
	xpathEndStr += "/text()";
	std::string end = GetXPathResponse( pXmlDoc, pXmlNode, xpathEndStr );
	m_endTime = _tstof( end.c_str() );

	std::string xpathDurationStr(s_strDurationTime);
	xpathDurationStr += "/text()";
	std::string durat = GetXPathResponse( pXmlDoc, pXmlNode, xpathDurationStr );
	m_durationTime = _tstof( durat.c_str() );

	xmlNode * startmatNode = SelectSingleNode( pXmlDoc, pXmlNode, s_strStartCamera );
	delete m_startCamera;
	m_startCamera = CCamera::Deserialize( startmatNode ) ;
	m_startCamera->Deserialize( pXmlDoc, startmatNode );

	xmlNode * endmatNode = SelectSingleNode( pXmlDoc, pXmlNode, s_strEndCamera );
	delete m_endCamera;
	m_endCamera = CCamera::Deserialize( endmatNode ) ;
	m_endCamera->Deserialize( pXmlDoc, endmatNode );

}

std::string CChunkDesc::s_strCameraChunk( _T("camerachunk") );
std::string CChunkDesc::s_strStartTime( _T("starttime") );
std::string CChunkDesc::s_strEndTime( _T("endtime") );
std::string CChunkDesc::s_strDurationTime( _T("durationtime") );
std::string CChunkDesc::s_strStartCamera( _T("startmatrix") );
std::string CChunkDesc::s_strEndCamera( _T("endmatrix") );
