#ifndef __COMET__
#define __COMET__

#include "irregularbody.h"

class Comet: public IrregularBody
{
public:

	Comet();
	Comet( const Comet & original );
	virtual ~Comet() {};

	Comet & operator=( const Comet & right_side );

	virtual xmlNode * Serialize(xmlDoc * pXmlDoc, xmlNode * pXmlNode);
	virtual void Deserialize( xmlDoc * pXmlDoc, xmlNode * pXmlNode );

	virtual void Update();

	enum MODEL { Marsden = 0, noprecession, linearprecession, forcedprecession };

	MODEL GetModel();
	void SetModel(MODEL model);

	double GetA1();
	void SetA1(double A1);
	double GetA2();
	void SetA2(double A2);
	double GetA3();
	void SetA3(double A3);

	double GetEta();
	void SetEta(double eta);

	double GetTau();
	void SetTau(double tau);

protected:

	MODEL m_model;					//  active model for computing

	double m_A[3];		//  Marsden's nongravitational parameters [AU*day-2]

	double m_eta;					//  lag angle [deg]

	double m_tau;					//  perihelion shift in assymetrical model [day]

	static void Assign2(Body * body, Body * right_side);

	static std::string s_strNongrav;
	static std::string s_strModel;
	static std::string s_strParameters;
	static std::string s_strEta;
	static std::string s_strTau;

	DECLARE_DYNAMIC_CLASS (Comet)
};

#endif // __COMET__