#include "stdafx.h"

#include "Body.h"
#include "ExtendedBody.h"
#include "Simulation.h"
#include <spicelib/spicelib.h>

using namespace JakubSchwarzmeier::Std::Math;

IMPLEMENT_DYNAMIC_CLASS (ExtendedBody, Body)

ExtendedBody::ExtendedBody()
{
    //  dispatch
	mapDispatch.insert( MAPDISPATCH::value_type( this->GetClassInfo() , Assign2 ) );
}

ExtendedBody::ExtendedBody( const ExtendedBody & original )
{
	operator=( original );
}

/* static */
void ExtendedBody::Assign2(Body * body, Body * right_side) 
{
	//  retype
	ExtendedBody * comet_left = dynamic_cast<ExtendedBody*>( body );
	ExtendedBody * comet_right = dynamic_cast<ExtendedBody*>( right_side );
	comet_left->operator=( *comet_right );
}

ExtendedBody & ExtendedBody::operator=( const ExtendedBody & right_side )
{
	if ( &right_side == this )  return (*this);

	Body::operator=( right_side );
	m_texture = right_side.m_texture;

	return *this;
}

double ExtendedBody::GetPoleLongitude()
{
	if (m_bUseSPICE)
	{
		SpiceDouble et = m_simulation->GetEt().GetJDN();
		double longitude, latitude;

		m_simulation->GetSpice()->GetPoleEcliptic(m_spiceBodyID, et, longitude,
			latitude);

		return longitude;
	}
	else
        return 0.0;
}

double ExtendedBody::GetPoleLatitude()
{
	if (m_bUseSPICE)
	{
		SpiceDouble et = m_simulation->GetEt().GetJDN();
		double longitude, latitude;

		m_simulation->GetSpice()->GetPoleEcliptic(m_spiceBodyID, et, longitude,
			latitude);

		return latitude;
	}
	else
        return 0.0;
}

double ExtendedBody::GetPM()
{
	if (m_bUseSPICE)
	{
		SpiceDouble et = m_simulation->GetEt().GetJDN();
		return m_simulation->GetSpice()->GetPM(m_spiceBodyID, et);
	}
	else
        return 0.0;
}

double ExtendedBody::GetRadius()
{
	double r;
	m_simulation->GetSpice()->GetRadii(m_spiceBodyID, r);

	return r;
}


std::string ExtendedBody::GetTexture()
{
	return m_texture;
}

void ExtendedBody::SetTexture(std::string texture)
{
	m_texture = texture;
}

xmlNode * ExtendedBody::Serialize(xmlDoc * pXmlDoc, xmlNode * pXmlNode)
{
	//  call ancestor's implementation
	xmlNode * pBodyNode = Body::Serialize(pXmlDoc, pXmlNode);		

	//  texture
	xmlNode * pTextureNode = xmlNewNode( NULL, (const xmlChar *)s_strTexture.c_str() );
	xmlAddChild( pBodyNode, pTextureNode );

	if (!m_texture.empty())
	{
		wxFileName textureFile( m_texture.c_str() );
		textureFile.MakeRelativeTo( wxFileName::GetCwd() );

		SetTextNode( pXmlDoc, pTextureNode, textureFile.GetFullPath().c_str() );
	}
	else
	{
		SetTextNode( pXmlDoc, pTextureNode, m_texture );
	}

	return pBodyNode;
}


void ExtendedBody::Deserialize(xmlDoc * pXmlDoc, xmlNode * pXmlNode )
{
	Body::Deserialize(pXmlDoc, pXmlNode);		//  call ancestor
	
	//  retrieve texture
	std::string strTextureXpath( s_strTexture );
	strTextureXpath += _T("/text()");
	std::string strTexture = GetXPathResponse( pXmlDoc, pXmlNode, strTextureXpath );

	if (!strTexture.empty())
	{
		wxFileName textureFile( strTexture.c_str() );
		textureFile.MakeAbsolute( );
		m_texture = textureFile.GetFullPath().c_str();
	}
}

const double ExtendedBody::s_eps = 23.5;

std::string ExtendedBody::s_strTexture( _T("texture") );
