#include "StdAfx.h"

#include ".\irregularbodytarget.h"

#include "extendedbody.h"
#include "irregularbody.h"

using namespace JakubSchwarzmeier::Std::Math;
using namespace JakubSchwarzmeier::Std::Graphics;

CIrregularBodyTarget::CIrregularBodyTarget(Body * body, CVisualization * visualization):
	CExtendedBodyTarget( body, visualization )
{
	m_model = NULL;
}

CIrregularBodyTarget::~CIrregularBodyTarget(void)
{
	if (m_model != NULL)
		throw;
}
/*
void CreatePyramid(float x, float y, float z, int width)
{
	// Start rendering the 4 triangles for the sides
	glBegin(GL_TRIANGLES);		
		
		// These vertices create the Back Side
		glColor3ub(255, 255, 0);   glVertex3f(x-width/2, y + width/2, z-width/2);					// Top point
		glColor3ub(255, 255, 255); glVertex3f(x - width/2, y - width/2, z - width/2);	// Bottom left point
		glColor3ub(255, 0, 0); glVertex3f(x, y, z + width/2);  // Bottom right point

		// These vertices create the Front Side
		glColor3ub(255, 255, 0);   glVertex3f(x-width/2, y - width/2, z-width/2);					// Top point
		glColor3ub(255, 255, 255); glVertex3f(x + width/2, y - width/2, z - width/2);  // Bottom right point
		glColor3ub(255, 0, 0); glVertex3f(x, y, z + width/2);	// Bottom left point

		// These vertices create the Front Left Side
		glColor3ub(255, 255, 0);   glVertex3f(x + width/2, y - width/2, z-width/2);					// Top point
		glColor3ub(255, 0, 0); glVertex3f(x + width/2, y + width/2, z - width/2);	// Front bottom point
		glColor3ub(255, 255, 255); glVertex3f(x, y, z + width/2);	// Bottom back point

		// These vertices create the Front right Side
		glColor3ub(255, 255, 0);   glVertex3f(x+width/2, y + width/2, z-width/2);					// Top point
		glColor3ub(255, 0, 0); glVertex3f(x - width/2, y + width/2, z - width/2);	// Bottom back point
		glColor3ub(255, 255, 255); glVertex3f(x, y, z + width/2);	// Front bottom point
			
	glEnd();

	// Now render the bottom of our pyramid

	glBegin(GL_QUADS);

		// These vertices create the bottom of the pyramid
		glColor3ub(0, 0, 255); glVertex3f(x + width/2, y + width/2, z - width/2);	// Front left point
		glColor3ub(0, 0, 255); glVertex3f(x - width/2, y + width/2, z - width/2);    // Front right point
		glColor3ub(0, 0, 255); glVertex3f(x - width/2, y - width/2, z - width/2);    // Back right point
		glColor3ub(0, 0, 255); glVertex3f(x + width/2, y - width/2, z - width/2);    // Back left point
	glEnd();
}
*/


void CIrregularBodyTarget::Render()
{
	CThreedCanvas * openGLCanvas = m_pVisualization->GetThreedCanvas();

	RotationMatrix objectInEye = GetObjectInEye();

	const CFrustum * frustum = openGLCanvas->GetFrustum();

	Trajectory();

	if ( frustum->SphereInFrustum( objectInEye[12], objectInEye[13], objectInEye[14], m_radius ) )
	{
		if ( IsLarge() )
		{
			if ( GetSize() < 25 )	DrawName();

			//  prepare drawing
			int r, g, b;
			m_pBody->GetColor(&r, &g, &b);
			glColor3ub(r, g, b);	
			glLoadMatrixd( objectInEye.GetMatrix() );
			//  end prepare drawing

			//  load texture if required
			if (m_model == NULL)
			{
				wxString strFn = dynamic_cast<IrregularBody*>( m_pBody )->GetModel().c_str();
				bool b = wxFileName::FileExists( strFn );

				if (b)  
				{
					m_model = JakubSchwarzmeier::wx::CThreedModel::Create( strFn.c_str(), openGLCanvas );
				}
			}

			if (m_model != NULL) m_model->Render();

			RenderRotationAxis();

			glPopAttrib();

			PrintOSD();
		}
		else
		{
			//  draw body name
			DrawName();

			//  prepare drawing
			int r, g, b;
			m_pBody->GetColor(&r, &g, &b);
			glColor3ub(r, g, b);	
			glLoadMatrixd( objectInEye.GetMatrix() );
			//  end prepare drawing

			//  draw just point
			glPointSize( 3 );

			glPushAttrib( GL_ENABLE_BIT );
				glDisable( GL_TEXTURE_2D );
				glDisable( GL_LIGHTING );

				glBegin (GL_POINTS);
					glVertex2f( 0, 0 );
				glEnd();
			glPopAttrib();
		}
	}
}

void CIrregularBodyTarget::OnContextTerminate()
{
	if (m_model != NULL)
	{
		delete m_model;
		m_model = NULL;
	}
}