/////////////////////////////////////////////////////////////////////////////
// Name:        OrbiterCameraDlg.cpp
// Purpose:     
// Author:      Jakub Schwarzmeier
// Modified by: 
// Created:     08/20/06 18:54:47
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#if defined(__GNUG__) && !defined(__APPLE__)
#pragma implementation "OrbiterCameraDlg.h"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
////@end includes

#include "OrbiterCameraDlg.h"
#include "PhaethonApp.h"
#include "PhaethonDoc.h"
#include "PhaethonView.h"
#include "CamerasPanel.h"
#include "ChunkDesc.h"
#include <schwarzmeier/std.h>
#include "Camera.h"

////@begin XPM images
////@end XPM images

/*!
 * COrbiterCameraDlg type definition
 */

IMPLEMENT_DYNAMIC_CLASS( COrbiterCameraDlg, wxDialog )

/*!
 * COrbiterCameraDlg event table definition
 */

BEGIN_EVENT_TABLE( COrbiterCameraDlg, wxDialog )

////@begin COrbiterCameraDlg event table entries
    EVT_PAINT( COrbiterCameraDlg::OnPaint )

    EVT_BUTTON( XRCID("ID_GETSTARTING"), COrbiterCameraDlg::OnGetstartingClick )
    EVT_UPDATE_UI( XRCID("ID_GETSTARTING"), COrbiterCameraDlg::OnUpdateUI )

    EVT_BUTTON( XRCID("ID_APPLYSTARTING"), COrbiterCameraDlg::OnApplystartingClick )
    EVT_UPDATE_UI( XRCID("ID_APPLYSTARTING"), COrbiterCameraDlg::OnUpdateUI )

    EVT_BUTTON( XRCID("ID_GETENDING"), COrbiterCameraDlg::OnGetendingClick )

    EVT_BUTTON( XRCID("ID_APPLYENDING"), COrbiterCameraDlg::OnApplyendingClick )

    EVT_BUTTON( wxID_OK, COrbiterCameraDlg::OnOkClick )

    EVT_BUTTON( wxID_CANCEL, COrbiterCameraDlg::OnCancelClick )

////@end COrbiterCameraDlg event table entries

END_EVENT_TABLE()

using namespace JakubSchwarzmeier::Std::Graphics;
using namespace JakubSchwarzmeier::Std::Math;

/*!
 * COrbiterCameraDlg constructors
 */

COrbiterCameraDlg::COrbiterCameraDlg( )
{
}

COrbiterCameraDlg::~COrbiterCameraDlg( )
{
	delete m_localChunkDesc;
}

COrbiterCameraDlg::COrbiterCameraDlg( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Create(parent, id, caption, pos, size, style);
}

/*!
 * COrbiterCameraDlg creator
 */

bool COrbiterCameraDlg::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin COrbiterCameraDlg member initialisation
////@end COrbiterCameraDlg member initialisation

	m_bNewChunk = false;

////@begin COrbiterCameraDlg creation
    SetExtraStyle(GetExtraStyle()|wxWS_EX_BLOCK_EVENTS);
    SetParent(parent);
    CreateControls();
    GetSizer()->Fit(this);
    GetSizer()->SetSizeHints(this);
    Centre();
////@end COrbiterCameraDlg creation
    return true;
}

/*!
 * Control creation for COrbiterCameraDlg
 */

void COrbiterCameraDlg::CreateControls()
{    
    wxXmlResource::Get()->Load(wxT("xrc/orbitercamera.xrc")); 

////@begin COrbiterCameraDlg content construction
    wxXmlResource::Get()->LoadDialog(this, GetParent(), _T("IDD_ORBITERCAMERA"));
    // Set validators
    if (FindWindow(XRCID("ID_STARTINGMATRIX")))
        FindWindow(XRCID("ID_STARTINGMATRIX"))->SetValidator( wxGenericValidator(& m_startMatrix) );
    if (FindWindow(XRCID("ID_STARTINGTIME")))
        FindWindow(XRCID("ID_STARTINGTIME"))->SetValidator( wxDoubleValidator( & m_startTime) );
    if (FindWindow(XRCID("ID_ENDINGMATRIX")))
        FindWindow(XRCID("ID_ENDINGMATRIX"))->SetValidator( wxGenericValidator(& m_endMatrix) );
    if (FindWindow(XRCID("ID_ENDINGTIME")))
        FindWindow(XRCID("ID_ENDINGTIME"))->SetValidator( wxDoubleValidator( & m_endTime) );
    if (FindWindow(XRCID("ID_ANIMATIONLENGTH")))
        FindWindow(XRCID("ID_ANIMATIONLENGTH"))->SetValidator( wxDoubleValidator(& m_animationDuration) );
////@end COrbiterCameraDlg content construction

    // Create custom windows not generated automatically here.

	GetParent()->Enable( false );

	m_localChunkDesc = new CChunkDesc;
	if ( GetClientData() == NULL )
	{
		m_bNewChunk = true;
		m_localChunkDesc->SetStartCamera( reinterpret_cast<CCamera*>( GetActiveView()->GetObserver()->GetCamera() ) );
		m_localChunkDesc->SetEndCamera( reinterpret_cast<CCamera*>( GetActiveView()->GetObserver()->GetCamera() ) );
	}
	else
	{
		m_bNewChunk = false;
		*m_localChunkDesc = *( reinterpret_cast<CChunkDesc*>( GetClientData() ) );
	}

	wxUpdateUIEvent e;
	OnUpdateUI( e );
	if ( !e.GetEnabled() )
	{
		//  Not first in list, copy starting time and matrix from previous
		//  ending time and ending matrix.
		std::vector<CChunkDesc*> * chVec = ::wxGetApp().GetDocument()->GetVisualization()->GetChunks();
		m_localChunkDesc->SetStartCamera( chVec->back()->GetEndCamera() );
		m_localChunkDesc->SetStartTime( chVec->back()->GetEndTime() );
	}

	m_animationDuration = m_localChunkDesc->GetDuration(); 
	
	RotationMatrix mat;
	Vector<> vec;

	mat = *( m_localChunkDesc->GetStartCamera() );
	vec = mat.GetPosition();
	m_startMatrix.sprintf(s_strSprintf.c_str(),  vec[0], vec[1], vec[2],
		mat.GetQuaternion().GetAngle(), mat.GetQuaternion().GetLongitude(), 
		mat.GetQuaternion().GetLatitude() );

	mat = *( m_localChunkDesc->GetEndCamera() );
	vec = mat.GetPosition();
	m_endMatrix.sprintf(s_strSprintf.c_str(),  vec[0], vec[1], vec[2],
		mat.GetQuaternion().GetAngle(), mat.GetQuaternion().GetLongitude(), 
		mat.GetQuaternion().GetLatitude() );

	m_startTime = m_localChunkDesc->GetStartTime();
	m_endTime = m_localChunkDesc->GetEndTime();


////@begin COrbiterCameraDlg content initialisation

////@end COrbiterCameraDlg content initialisation
}

/*!
 * Should we show tooltips?
 */

bool COrbiterCameraDlg::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap COrbiterCameraDlg::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin COrbiterCameraDlg bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end COrbiterCameraDlg bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon COrbiterCameraDlg::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin COrbiterCameraDlg icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end COrbiterCameraDlg icon retrieval
}

JakubSchwarzmeier::wx::COpenGLCanvas * COrbiterCameraDlg::GetActiveView()
{
	CMainFrame * frame = ::wxGetApp().GetMainFrame();
	wxDocMDIChildFrame * childFrame = wx_reinterpret_cast( wxDocMDIChildFrame*, frame->GetActiveChild() );
	CPhaethonView * view = wx_reinterpret_cast( CPhaethonView*, childFrame->GetView() );

	return view->GetCanvas();
}

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_GETSTARTING
 */

void COrbiterCameraDlg::OnGetstartingClick( wxCommandEvent& event )
{
	RotationMatrix mat;
	Vector<> vec;

	CPhaethonDoc * doc = ::wxGetApp().GetDocument();
	doc->GetVisualization()->GetThreedCanvas()->AcquireRendering();
	mat = *(GetActiveView()->GetObserver()->GetCamera());
	doc->GetVisualization()->GetThreedCanvas()->ReleaseRendering();

	vec = mat.GetPosition();
	m_startMatrix.sprintf(s_strSprintf.c_str(),  vec[0], vec[1], vec[2],
		mat.GetQuaternion().GetAngle(), mat.GetQuaternion().GetLongitude(), 
		mat.GetQuaternion().GetLatitude() );

	m_localChunkDesc->SetStartCamera( reinterpret_cast<CCamera*>( GetActiveView()->GetObserver()->GetCamera() ) );

	m_startTime = ::wxGetApp().GetDocument()->GetVisualization()->GetEt();
	m_localChunkDesc->SetStartTime( m_startTime );

	TransferDataToWindow();
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_APPLYSTARTING
 */

void COrbiterCameraDlg::OnApplystartingClick( wxCommandEvent& event )
{
	CPhaethonDoc * doc = ::wxGetApp().GetDocument();
	doc->GetVisualization()->GetThreedCanvas()->AcquireRendering();

	if ( m_localChunkDesc->GetStartCamera()->IsKindOf( CLASSINFO(CSphericalCamera) ) )
	{
		CSphericalObserver * obs = dynamic_cast<CCamerasPanel*>( GetParent() )->GetSphericalObserver();
		obs->SetCamera( m_localChunkDesc->GetStartCamera() );
		GetActiveView()->SetObserver( obs );
	}
	else
	{
		CRectangularObserver * obs = dynamic_cast<CCamerasPanel*>( GetParent() )->GetRectangularObserver();
		obs->SetCamera( m_localChunkDesc->GetStartCamera() );
		GetActiveView()->SetObserver( obs );
	}

	double prevEt = ::wxGetApp().GetDocument()->GetVisualization()->GetEt();
	::wxGetApp().GetDocument()->GetVisualization()->SetEt( m_localChunkDesc->GetStartTime() );
	if ( fabs(prevEt - m_localChunkDesc->GetStartTime() ) > 10.0 )	::wxGetApp().GetDocument()->GetVisualization()->ClearTrajectories();


	doc->GetVisualization()->GetThreedCanvas()->ReleaseRendering();

	GetActiveView()->ReRender();
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_GETENDING
 */

void COrbiterCameraDlg::OnGetendingClick( wxCommandEvent& event )
{
	RotationMatrix mat;
	Vector<> vec;

	CPhaethonDoc * doc = ::wxGetApp().GetDocument();
	doc->GetVisualization()->GetThreedCanvas()->AcquireRendering();
	mat = *(GetActiveView()->GetObserver()->GetCamera());
	doc->GetVisualization()->GetThreedCanvas()->ReleaseRendering();

	vec = mat.GetPosition();
	m_endMatrix.sprintf(s_strSprintf.c_str(),  vec[0], vec[1], vec[2],
		mat.GetQuaternion().GetAngle(), mat.GetQuaternion().GetLongitude(), 
		mat.GetQuaternion().GetLatitude() );

	m_localChunkDesc->SetEndCamera( reinterpret_cast<CCamera*>( GetActiveView()->GetObserver()->GetCamera() ) );

	m_endTime = ::wxGetApp().GetDocument()->GetVisualization()->GetEt();
	m_localChunkDesc->SetEndTime( m_endTime );

	TransferDataToWindow();
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_APPLYENDING
 */

void COrbiterCameraDlg::OnApplyendingClick( wxCommandEvent& event )
{
	CPhaethonDoc * doc = ::wxGetApp().GetDocument();
	doc->GetVisualization()->GetThreedCanvas()->AcquireRendering();

	if ( m_localChunkDesc->GetEndCamera()->IsKindOf( CLASSINFO(CSphericalCamera) ) )
	{
		CSphericalObserver * obs = dynamic_cast<CCamerasPanel*>( GetParent() )->GetSphericalObserver();
		obs->SetCamera( m_localChunkDesc->GetEndCamera() );
		GetActiveView()->SetObserver( obs );
	}
	else
	{
		CRectangularObserver * obs = dynamic_cast<CCamerasPanel*>( GetParent() )->GetRectangularObserver();
		obs->SetCamera( m_localChunkDesc->GetEndCamera() );
		GetActiveView()->SetObserver( obs );
	}

	double prevEt = ::wxGetApp().GetDocument()->GetVisualization()->GetEt();
	::wxGetApp().GetDocument()->GetVisualization()->SetEt( m_localChunkDesc->GetEndTime() );
	if ( fabs(prevEt - m_localChunkDesc->GetEndTime() ) > 10.0 )	::wxGetApp().GetDocument()->GetVisualization()->ClearTrajectories();

	doc->GetVisualization()->GetThreedCanvas()->ReleaseRendering();

	GetActiveView()->ReRender();
}

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_OK
 */

void COrbiterCameraDlg::OnOkClick( wxCommandEvent& event )
{
	if ( Validate() && TransferDataFromWindow() )
	{
		m_localChunkDesc->SetDuration( m_animationDuration ); 
		m_localChunkDesc->SetStartTime( m_startTime );
		m_localChunkDesc->SetEndTime( m_endTime );

		if (m_bNewChunk)
		{
			// add to list
			CChunkDesc * newChunk = new CChunkDesc;
			*newChunk = *m_localChunkDesc;
			::wxGetApp().GetDocument()->GetVisualization()->GetChunks()->push_back ( newChunk );

			CCamerasPanel* father = reinterpret_cast<CCamerasPanel*>( GetParent() );
			father->UpdateCameras();
		}
		else
		{
			//  copy to original
			*( reinterpret_cast<CChunkDesc*>( GetClientData() ) ) = *m_localChunkDesc;
		}

		GetParent()->Enable( true );
	
		Destroy();

		EndDialog( wxID_OK );
	} 	
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_CANCEL
 */

void COrbiterCameraDlg::OnCancelClick( wxCommandEvent& event )
{
	GetParent()->Enable( true );

	Destroy();

	wxDialog::OnCancel( event );
}

std::string COrbiterCameraDlg::s_strSprintf( _T("%f %f %f %f %f %f") );

/*!
 * wxEVT_PAINT event handler for IDD_ORBITERCAMERA
 */

void COrbiterCameraDlg::OnPaint( wxPaintEvent& event )
{
    wxPaintDC dc(this);

	if (GetSizer())
	{
		GetSizer()->Fit(this);
		GetSizer()->SetSizeHints(this);
		Layout();
	}
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_GETSTARTING
 */

void COrbiterCameraDlg::OnUpdateUI( wxUpdateUIEvent& event )
{
	std::vector<CChunkDesc*> * chVec = ::wxGetApp().GetDocument()->GetVisualization()->GetChunks();
	CChunkDesc * ch = reinterpret_cast<CChunkDesc*>( GetClientData() );

	if (( m_bNewChunk && chVec->empty() ) || ( (ch != NULL) && ( chVec->front() == ch )) )
	{
		event.Enable( true );
	}
	else
	{
		event.Enable( false );
	}

}


