// PhaethonDoc.cpp : implementation of the CPhaethonDoc class
//

#include "stdafx.h"
#include "PhaethonApp.h"
#include "PhaethonDoc.h"
#include "PhaethonView.h"
#include "MainFrame.h"

// CPhaethonDoc

IMPLEMENT_DYNAMIC_CLASS (CPhaethonDoc, wxDocument)

BEGIN_EVENT_TABLE(CPhaethonDoc, wxDocument)
END_EVENT_TABLE()

CPhaethonDoc::CPhaethonDoc()
{
	m_simulation = NULL;
	m_visualization = NULL;
}

CPhaethonDoc::~CPhaethonDoc()
{
	if (m_simulation == NULL)  throw;
	delete m_simulation;

	if (m_visualization == NULL)  throw;
	delete m_visualization;
}

bool CPhaethonDoc::OnCreate(const wxString& path, long flags)
{
	if (m_simulation != NULL)  throw;
	if (m_visualization != NULL)  throw;
	
	m_simulation = new Simulation;
	m_visualization = new CVisualization;

	CMainFrame * frame = ::wxGetApp().GetMainFrame();

	//  Parent implementation creates view, so we do

	//  Create first frame
	frame->CreateChildFrame(this, 0);

	wxDocMDIChildFrame * childFrame = wx_reinterpret_cast( wxDocMDIChildFrame*, frame->GetActiveChild() );
	CPhaethonView * view = wx_reinterpret_cast( CPhaethonView*, childFrame->GetView() );

	return true;
}

bool CPhaethonDoc::Close()
{
	return wxDocument::Close();
}


bool CPhaethonDoc::OnOpenDocument(const wxString& filename)
{
	xmlThrDefIndentTreeOutput( 1 );
	xmlKeepBlanksDefault(0);

	char * szLoc = setlocale( LC_NUMERIC, NULL ); \
	setlocale( LC_NUMERIC, "English" );

	//  Read parameters from XML file
	xmlDoc * pXmlDoc = xmlParseFile( (const char *)filename.c_str() );
	if (pXmlDoc == NULL)	return false;
	xmlNode * pRootNode = xmlDocGetRootElement(pXmlDoc);
	if (pRootNode == NULL)	return false;
	
	bool res;
	if ( strcmp( reinterpret_cast<const char*>( pRootNode->name ), s_strPhaethon.c_str() ) == 0 )
	{
		const wxString originalWD = wxFileName::GetCwd();

		wxFileName fn( filename );
		wxFileName::SetCwd( fn.GetPath() );

		m_simulation->Deserialize( pXmlDoc, pRootNode );
		m_visualization->Deserialize( pXmlDoc, pRootNode );

		wxFileName::SetCwd( originalWD );

		m_visualization->CreateRenderingTargets();

		res = true;
	}
	else
	{
		res = false;
	}

	xmlFreeDoc(pXmlDoc);

	if (res)
	{
		res = wxDocument::OnOpenDocument( filename );
	}

	setlocale( LC_NUMERIC, szLoc );

	return res;
}

bool CPhaethonDoc::OnSaveDocument(const wxString& filename)
{
/*	JakubSchwarzmeier::wx::CSerializeWriter writer( filename );
	CPhaethonDoc * doc = this;
	writer.Append( reinterpret_cast<wxObject**>( &doc ), wxString("document"));
*/
	char * szLoc = setlocale( LC_NUMERIC, NULL ); \
	setlocale( LC_NUMERIC, "English" );

	//  Write XML file
	xmlThrDefIndentTreeOutput( 1 );
	xmlKeepBlanksDefault(0);
	xmlDoc * pXmlDoc = xmlNewDoc( (const xmlChar *)"1.0" );
	
	//  create new node
	xmlNode * pRootNode = xmlNewNode( NULL, (const xmlChar *)s_strPhaethon.c_str() );

	//  connect root node to document
	xmlDocSetRootElement( pXmlDoc, pRootNode );

	//
	const wxString originalWD = wxFileName::GetCwd();

	wxFileName fn( filename );
	wxFileName::SetCwd( fn.GetPath() );

	//
	m_simulation->Serialize( pXmlDoc, pRootNode );
	m_visualization->Serialize( pXmlDoc, pRootNode );

	//
	wxFileName::SetCwd( originalWD );

	//
	xmlSaveFormatFileEnc( filename.c_str(), pXmlDoc, "ISO-8859-2", 1 );
//	xmlSaveFormatFile( filename.c_str(), pXmlDoc, 1 );
	xmlFreeDoc(pXmlDoc);

	Modify( false );

	setlocale( LC_NUMERIC, szLoc );

	return true;
}

bool CPhaethonDoc::OnSerialize( JakubSchwarzmeier::wx::CSerializeReaderWriter * dir )
{
//	dir->Append(  reinterpret_cast<wxObject**>( &m_visual ), wxString("m_visual") );

	return true;
}

Simulation * CPhaethonDoc::GetSimulation()
{
	return m_simulation;
}

void CPhaethonDoc::SetSimulation( Simulation * sim )
{
	*m_simulation = *sim;
}


CVisualization * CPhaethonDoc::GetVisualization()
{
	return m_visualization;
}


std::string CPhaethonDoc::s_strPhaethon( _("phaethon") );
