#pragma once

#include "camera.h"
#include "body.h"

class CRectangularCamera: public CCamera
{
public:
	CRectangularCamera();
	virtual ~CRectangularCamera(void);

	CRectangularCamera( const CRectangularCamera & original );
	CRectangularCamera & operator=( const CRectangularCamera & right_side );

	int GetCentralBody();
	void SetCentralBody(int iNaifId);

	Body * GetCentralBodyPtr();

	const RotationMatrix & GetOrbiter();
	void SetOrbiter(RotationMatrix & orbiter);

	virtual void Update();

	//
	virtual void ChangeHeading( double angle );	//  rotate around local "y", knodwn as "yaw" too
	virtual void ChangePitch( double angle );	//  around "x"
	virtual void ChangeRoll( double angle );	//  around "z"
	virtual void ChangeRotation( JakubSchwarzmeier::Std::Math::Matrix<> & rotation );
    virtual void ChangePosition( JakubSchwarzmeier::Std::Math::Vector<> & position );
	//  sets
	virtual void ChangeLambda( double angle );	//  rotate around y
	virtual void ChangePhi( double angle );	//  rotate around x
	virtual void ChangeTheta( double angle );	//  rotate around z

	virtual void SetPosition( JakubSchwarzmeier::Std::Math::Vector<> & position );
	virtual JakubSchwarzmeier::Std::Math::Vector<> GetPosition() const;

	virtual xmlNode * Serialize(xmlDoc * pXmlDoc, xmlNode * pXmlNode);
	virtual void Deserialize( xmlDoc * pXmlDoc, xmlNode * pXmlNode );

protected:

    int m_iNaifId;		//  position of body in bodies vector
	Body * m_pBody;

	RotationMatrix m_orbiter;

	static void Assign2(CCamera * camera, CCamera * right_side);

	static std::string s_strCentralBodyNaifId;
	static std::string s_strOrbiterMatrix;

	DECLARE_DYNAMIC_CLASS (CRectangularCamera)
};
