#include "StdAfx.h"
#include ".\rectangularobserver.h"
#include "PhaethonApp.h"
#include "rectangularcamera.h"


IMPLEMENT_DYNAMIC_CLASS( CRectangularObserver, JakubSchwarzmeier::wx::CObserver )

/*!
 * CCamerasPanel event table definition
 */

BEGIN_EVENT_TABLE( CRectangularObserver, JakubSchwarzmeier::wx::CObserver )

    EVT_BUTTON( XRCID("ID_SET"), CRectangularObserver::OnSetClick )

END_EVENT_TABLE()

using namespace JakubSchwarzmeier::wx;
using namespace JakubSchwarzmeier::Std::Math;

bool CRectangularObserver::Create( wxWindow* parent, JakubSchwarzmeier::wx::COpenGLCanvas * pglCanvas, wxWindowID id)
{
	return CObserver::Create( parent, pglCanvas, id );
}

void CRectangularObserver::OnCreateCamera()
{
	if (m_pCamera == NULL)
		m_pCamera = new CRectangularCamera;
}

void CRectangularObserver::CreateControls()
{
	CObserver::CreateControls();

	m_initial = dynamic_cast<CRectangularCamera*>( m_pCamera )->GetOrbiter();

    wxXmlResource::Get()->Load(wxT("xrc/rectangularobserver.xrc")); 

	wxNotebook * notebook = XRCCTRL(*this, "ID_NOTEBOOK", wxNotebook);

	wxNotebookPage * page =
		wxXmlResource::Get()->LoadPanel(notebook, _T("IDD_RECTANGULAROBSERVER"));
	//  Do not delete the page, it will be deleted by the notebook

	notebook->AddPage( page, _T("Rectangular") );

    m_centralBodyCtrl = XRCCTRL(*this, "ID_CENTRALBODY", wxChoice);

	std::vector<Body*> * bodies = ::wxGetApp().GetDocument()->GetSimulation()->GetBodies();
	std::vector<Body*>::const_iterator it;
	for( it = bodies->begin(); it != bodies->end(); it++)
	{
		int pos = m_centralBodyCtrl->Append( (*it)->GetName().c_str() );
		if ( (*it)->GetSpiceBodyID() == dynamic_cast<CRectangularCamera*>( m_pCamera )->GetCentralBody() )
		{
			m_centralBodyCtrl->SetSelection( pos );
		}
	}

}

void CRectangularObserver::OnSetClick( wxCommandEvent& event )
{
	int i = m_centralBodyCtrl->GetSelection();
	std::vector<Body*> * bodies = ::wxGetApp().GetDocument()->GetSimulation()->GetBodies();
	
	int naifCode = 0;
	if ( bodies->size() > i )  
		naifCode = bodies->at( i )->GetSpiceBodyID();

	dynamic_cast<CRectangularCamera*>( m_pCamera )->SetCentralBody( naifCode );

	OnUpdateInputs();

	m_pglCanvas->ReRender();
}

void CRectangularObserver::SetCamera( CCamera * camera )
{
	dynamic_cast<CCamera*>( m_pCamera )->Assign( camera );
}

void CRectangularObserver::OnInitialClick( wxCommandEvent& event )
{
	dynamic_cast<CRectangularCamera*>( m_pCamera )->SetOrbiter( m_initial );
    OnUpdateInputs();
	m_pglCanvas->ReRender();
}
