/////////////////////////////////////////////////////////////////////////////
// Name:        RunSimulationDlg.cpp
// Purpose:     
// Author:      Jakub Schwarzmeier
// Modified by: 
// Created:     08/14/06 16:57:48
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#if defined(__GNUG__) && !defined(__APPLE__)
#pragma implementation "RunSimulationDlg.h"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
////@end includes

#include "RunSimulationDlg.h"
#include "Simulation.h"
#include "PhaethonApp.h"

using namespace JakubSchwarzmeier::Astronomy;

////@begin XPM images
////@end XPM images

/*!
 * CRunSimulationDlg type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CRunSimulationDlg, wxDialog )

/*!
 * CRunSimulationDlg event table definition
 */

BEGIN_EVENT_TABLE( CRunSimulationDlg, wxDialog )

////@begin CRunSimulationDlg event table entries
    EVT_BUTTON( wxID_OK, CRunSimulationDlg::OnOkClick )

////@end CRunSimulationDlg event table entries

END_EVENT_TABLE()

/*!
 * CRunSimulationDlg constructors
 */

CRunSimulationDlg::CRunSimulationDlg( )
{
}

CRunSimulationDlg::CRunSimulationDlg( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Create(parent, id, caption, pos, size, style);
}

/*!
 * CRunSimulationDlg creator
 */

bool CRunSimulationDlg::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CRunSimulationDlg member initialisation
    m_epsilon = 0;
    m_stepSize = 0;
    m_forceMethodCtrl = NULL;
    m_integrationMethodCtrl = NULL;
    m_startingTimeCtrl = NULL;
    m_stoppingTimeCtrl = NULL;
#if defined(__WXMSW__)
    m_genPlotsCtrl = NULL;
#endif
////@end CRunSimulationDlg member initialisation

////@begin CRunSimulationDlg creation
    SetExtraStyle(GetExtraStyle()|wxWS_EX_BLOCK_EVENTS);
    SetParent(parent);
    CreateControls();
    GetSizer()->Fit(this);
    GetSizer()->SetSizeHints(this);
    Centre();
////@end CRunSimulationDlg creation
    return true;
}

/*!
 * Control creation for CRunSimulationDlg
 */

void CRunSimulationDlg::CreateControls()
{    
    wxXmlResource::Get()->Load(wxT("xrc/runsimulation.xrc")); 

////@begin CRunSimulationDlg content construction
    wxXmlResource::Get()->LoadDialog(this, GetParent(), _T("IDD_RUNSIMULATION"));
    m_forceMethodCtrl = XRCCTRL(*this, "ID_FORCEMETHOD", wxChoice);
    m_integrationMethodCtrl = XRCCTRL(*this, "ID_INTEGRATIONMETHOD", wxChoice);
    m_startingTimeCtrl = XRCCTRL(*this, "ID_CALENDARSTART", wxCalendarCtrl);
    m_stoppingTimeCtrl = XRCCTRL(*this, "ID_CALENDARSTOP", wxCalendarCtrl);
#if defined(__WXMSW__)
    m_genPlotsCtrl = XRCCTRL(*this, "ID_GENPLOTS", wxCheckBox);
#endif
    // Set validators
    if (FindWindow(XRCID("ID_EPSILON")))
        FindWindow(XRCID("ID_EPSILON"))->SetValidator( wxDoubleValidator(& m_epsilon) );
    if (FindWindow(XRCID("ID_STEPSIZE")))
        FindWindow(XRCID("ID_STEPSIZE"))->SetValidator( wxDoubleValidator(& m_stepSize) );
    if (FindWindow(XRCID("ID_STARTINGTIME")))
        FindWindow(XRCID("ID_STARTINGTIME"))->SetValidator( wxTimeValidator(& m_startingTime) );
    if (FindWindow(XRCID("ID_STOPPINGTIME")))
        FindWindow(XRCID("ID_STOPPINGTIME"))->SetValidator( wxTimeValidator(& m_stoppingTime) );
    if (FindWindow(XRCID("ID_OUTPUTFILE")))
        FindWindow(XRCID("ID_OUTPUTFILE"))->SetValidator( wxNotNullValidator(& m_outputSPKFile, _T("Please enter the name of output file.") ) );
////@end CRunSimulationDlg content construction

    // Create custom windows not generated automatically here.
	Simulation * sim = reinterpret_cast<Simulation*>( GetClientData() );
	
	m_stepSize = sim->GetDt();
	m_epsilon = sim->GetEpsilon();
	m_startingTimeCtrl->SetDate( sim->GetEt().ToUTC(true) );
	m_stoppingTimeCtrl->SetDate( sim->GetEt().ToUTC(true) + wxDateSpan(1) );

	m_forceMethodCtrl->SetSelection( sim->GetAccelerationMethod() );
	m_integrationMethodCtrl->SetSelection( sim->GetIntegrationMethod() );

	m_startingTime = sim->GetEt();
	m_stoppingTime = sim->GetEt();

	m_outputSPKFile = sim->GetOutputFile().c_str();
	if ( m_outputSPKFile.IsEmpty() )
	{
		wxFileName newFN( wxFileName::GetHomeDir(), ::wxGetApp().GetDocument()->GetTitle() );
		newFN.SetExt( _T("spk") );
		newFN.MakeAbsolute();
		m_outputSPKFile = newFN.GetFullPath();
	}

////@begin CRunSimulationDlg content initialisation

////@end CRunSimulationDlg content initialisation
}

/*!
 * Should we show tooltips?
 */

bool CRunSimulationDlg::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CRunSimulationDlg::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CRunSimulationDlg bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CRunSimulationDlg bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CRunSimulationDlg::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CRunSimulationDlg icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CRunSimulationDlg icon retrieval
}
/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_OK
 */

void CRunSimulationDlg::OnOkClick( wxCommandEvent& event )
{
	if ( Validate() && TransferDataFromWindow() )
	{
		CDateTime datetime( m_startingTimeCtrl->GetDate().ToUTC(true) );
		datetime.SetHour( m_startingTime.GetHour() );
		datetime.SetMinute( m_startingTime.GetMinute() );
		datetime.SetSecond( m_startingTime.GetSecond() );
		datetime.SetMillisecond( m_startingTime.GetMillisecond() );

		CDateTime stopdatetime( m_stoppingTimeCtrl->GetDate().ToUTC(true) );
		stopdatetime.SetHour( m_stoppingTime.GetHour() );
		stopdatetime.SetMinute( m_stoppingTime.GetMinute() );
		stopdatetime.SetSecond( m_stoppingTime.GetSecond() );
		stopdatetime.SetMillisecond( m_stoppingTime.GetMillisecond() );

		if (datetime == stopdatetime)
		{
			wxMessageDialog msg( this, _T("Starting and ending date time cannot be the same.") );
			msg.ShowModal();
			return;
		}

		if (m_stepSize == 0)
		{
			wxMessageDialog msg( this, _T("Timestep cannot be set to zero.") );
			msg.ShowModal();
			return;
		}

		Simulation * sim = reinterpret_cast<Simulation*>( GetClientData() );

		sim->SetDt( m_stepSize );
		sim->SetEpsilon( m_epsilon );

		sim->SetEt( datetime );

		sim->SetAccelerationMethod( static_cast<Simulation::ACCELERATION_METHOD>( m_forceMethodCtrl->GetSelection() ) );
		sim->SetIntegrationMethod( static_cast<Simulation::INTEGRATION_METHOD>( m_integrationMethodCtrl->GetSelection() ) );

		wxFileName fn( m_outputSPKFile );
		fn.MakeAbsolute();

		if ( wxFileName::FileExists( fn.GetFullPath() ) )
		{
			remove( fn.GetFullPath().c_str() );
		}

		sim->SetOutputFile( fn.GetFullPath().c_str() );

		#if defined(__WXMSW__)
			sim->SetGeneratePlots( m_genPlotsCtrl->GetValue() );
		#endif

		// perform computations
		bool cont = true;

		//  how many time steps?
		wxTimeSpan ts = stopdatetime - datetime;
		//  ts can be negative, if we step into the past
		double totalSecs = labs(  ts.GetSeconds().ToDouble() );
		size_t steps = labs( totalSecs / m_stepSize );

		if ( ts.GetSeconds().ToDouble() < 0 )	sim->SetDt( -fabs(m_stepSize) );

		//  progress dialog
		wxProgressDialog dialog(_T("Simulation Run"), 
			_T("Computing forces and integrating..."), steps, this, 
			wxPD_AUTO_HIDE | wxPD_APP_MODAL | wxPD_CAN_ABORT |  wxPD_ELAPSED_TIME |
            wxPD_ESTIMATED_TIME | wxPD_REMAINING_TIME | wxPD_SMOOTH );

		sim->StartSimulation();

		for(size_t i = 0; i < steps; i++)
		{
			//  actual computation
            sim->Update();

			//  progress dialog stuff
			if (steps >= 100)
				if ( (i % (steps/100)) == 0 ) // only 100 updates, this makes it much faster
					cont = dialog.Update(i);

			if ( !cont )
			{
				if ( wxMessageBox(_T("Do you really want to cancel?"),
								_T("Progress dialog question"),  // caption
								wxYES_NO | wxICON_QUESTION) == wxYES )
				{
					break;
				}
				else
				{
					cont = true;
					dialog.Resume();
				}
			}
		}

		if (cont)
		{
			if ( m_genPlotsCtrl->GetValue() )
				wxBusyInfo wait("Please be patient, generating graphs...");
			sim->StopSimulation();
			RequestUserAttention( wxUSER_ATTENTION_INFO );
		}

		if (cont) 
		{
			wxMessageBox(_T("Simulation succeeded!") );
			dialog.Destroy();
		}

		EndDialog( wxID_OK );
	}
}


