/////////////////////////////////////////////////////////////////////////////
// Name:        RunSimulationDlg.h
// Purpose:     
// Author:      Jakub Schwarzmeier
// Modified by: 
// Created:     08/14/06 16:57:48
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

#ifndef _RUNSIMULATIONDLG_H_
#define _RUNSIMULATIONDLG_H_

#if defined(__GNUG__) && !defined(__APPLE__)
#pragma interface "RunSimulationDlg.cpp"
#endif

/*!
 * Includes
 */

////@begin includes
#include "wx/xrc/xmlres.h"
#include "wx/calctrl.h"
////@end includes

/*!
 * Forward declarations
 */

#include <schwarzmeier/astronomy.h>

////@begin forward declarations
class wxCalendarCtrl;
////@end forward declarations

/*!
 * Control identifiers
 */

////@begin control identifiers
#define IDD_RUNSIMULATION 10025
#define SYMBOL_CRUNSIMULATIONDLG_STYLE wxCAPTION|wxRESIZE_BORDER|wxSYSTEM_MENU|wxCLOSE_BOX
#define SYMBOL_CRUNSIMULATIONDLG_TITLE _("Run Simulation")
#define SYMBOL_CRUNSIMULATIONDLG_IDNAME IDD_RUNSIMULATION
#define SYMBOL_CRUNSIMULATIONDLG_SIZE wxSize(400, 300)
#define SYMBOL_CRUNSIMULATIONDLG_POSITION wxDefaultPosition
////@end control identifiers

/*!
 * Compatibility
 */

#ifndef wxCLOSE_BOX
#define wxCLOSE_BOX 0x1000
#endif
#ifndef wxFIXED_MINSIZE
#define wxFIXED_MINSIZE 0
#endif

/*!
 * CRunSimulationDlg class declaration
 */

class CRunSimulationDlg: public wxDialog
{    
    DECLARE_DYNAMIC_CLASS( CRunSimulationDlg )
    DECLARE_EVENT_TABLE()

public:
    /// Constructors
    CRunSimulationDlg( );
    CRunSimulationDlg( wxWindow* parent, wxWindowID id = SYMBOL_CRUNSIMULATIONDLG_IDNAME, const wxString& caption = SYMBOL_CRUNSIMULATIONDLG_TITLE, const wxPoint& pos = SYMBOL_CRUNSIMULATIONDLG_POSITION, const wxSize& size = SYMBOL_CRUNSIMULATIONDLG_SIZE, long style = SYMBOL_CRUNSIMULATIONDLG_STYLE );

    /// Creation
    bool Create( wxWindow* parent, wxWindowID id = SYMBOL_CRUNSIMULATIONDLG_IDNAME, const wxString& caption = SYMBOL_CRUNSIMULATIONDLG_TITLE, const wxPoint& pos = SYMBOL_CRUNSIMULATIONDLG_POSITION, const wxSize& size = SYMBOL_CRUNSIMULATIONDLG_SIZE, long style = SYMBOL_CRUNSIMULATIONDLG_STYLE );

    /// Creates the controls and sizers
    void CreateControls();

////@begin CRunSimulationDlg event handler declarations

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_OK
    void OnOkClick( wxCommandEvent& event );

////@end CRunSimulationDlg event handler declarations

////@begin CRunSimulationDlg member function declarations

    double GetEpsilon() const { return m_epsilon ; }
    void SetEpsilon(double value) { m_epsilon = value ; }

    double GetStepSize() const { return m_stepSize ; }
    void SetStepSize(double value) { m_stepSize = value ; }

    wxString GetOutputSPKFile() const { return m_outputSPKFile ; }
    void SetOutputSPKFile(wxString value) { m_outputSPKFile = value ; }

    JakubSchwarzmeier::Astronomy::CDateTime GetStartingTime() const { return m_startingTime ; }
    void SetStartingTime(JakubSchwarzmeier::Astronomy::CDateTime value) { m_startingTime = value ; }

    JakubSchwarzmeier::Astronomy::CDateTime GetStoppingTime() const { return m_stoppingTime ; }
    void SetStoppingTime(JakubSchwarzmeier::Astronomy::CDateTime value) { m_stoppingTime = value ; }

    /// Retrieves bitmap resources
    wxBitmap GetBitmapResource( const wxString& name );

    /// Retrieves icon resources
    wxIcon GetIconResource( const wxString& name );
////@end CRunSimulationDlg member function declarations

    /// Should we show tooltips?
    static bool ShowToolTips();

////@begin CRunSimulationDlg member variables
    wxChoice* m_forceMethodCtrl;
    wxChoice* m_integrationMethodCtrl;
    wxCalendarCtrl* m_startingTimeCtrl;
    wxCalendarCtrl* m_stoppingTimeCtrl;
#if defined(__WXMSW__)
    wxCheckBox* m_genPlotsCtrl;
#endif
    double m_epsilon;
    double m_stepSize;
    wxString m_outputSPKFile;
    JakubSchwarzmeier::Astronomy::CDateTime m_startingTime;
    JakubSchwarzmeier::Astronomy::CDateTime m_stoppingTime;
////@end CRunSimulationDlg member variables
};

#endif
    // _RUNSIMULATIONDLG_H_
