/////////////////////////////////////////////////////////////////////////////
// Name:        SettingsDlg.cpp
// Purpose:     
// Author:      Jakub Schwarzmeier
// Modified by: 
// Created:     09/17/06 15:33:07
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#if defined(__GNUG__) && !defined(__APPLE__)
#pragma implementation "SettingsDlg.h"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
////@end includes

#include "SettingsDlg.h"
#include "Visualization.h"

////@begin XPM images
////@end XPM images

/*!
 * CSettingsDlg type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CSettingsDlg, wxDialog )

/*!
 * CSettingsDlg event table definition
 */

BEGIN_EVENT_TABLE( CSettingsDlg, wxDialog )

////@begin CSettingsDlg event table entries
    EVT_BUTTON( wxID_OK, CSettingsDlg::OnOkClick )

////@end CSettingsDlg event table entries

END_EVENT_TABLE()

/*!
 * CSettingsDlg constructors
 */

CSettingsDlg::CSettingsDlg( )
{
}

CSettingsDlg::CSettingsDlg( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Create(parent, id, caption, pos, size, style);
}

/*!
 * CSettingsDlg creator
 */

bool CSettingsDlg::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CSettingsDlg member initialisation
    m_showOSDCtrl = NULL;
    m_autoUnitsCtrl = NULL;
    m_rotationAxisCtrl = NULL;
    m_bodyCentricFabricCtrl = NULL;
    m_drawTrajectoriesCtrl = NULL;
////@end CSettingsDlg member initialisation

////@begin CSettingsDlg creation
    SetExtraStyle(GetExtraStyle()|wxWS_EX_BLOCK_EVENTS);
    SetParent(parent);
    CreateControls();
    GetSizer()->Fit(this);
    GetSizer()->SetSizeHints(this);
    Centre();
////@end CSettingsDlg creation
    return true;
}

/*!
 * Control creation for CSettingsDlg
 */

void CSettingsDlg::CreateControls()
{    
	wxXmlResource::Get()->Load("xrc/settings.xrc");
////@begin CSettingsDlg content construction
    wxXmlResource::Get()->LoadDialog(this, GetParent(), _T("IDD_SETTINGS"));
    m_showOSDCtrl = XRCCTRL(*this, "ID_SHOW_OSD", wxCheckBox);
    m_autoUnitsCtrl = XRCCTRL(*this, "ID_AUTO_UNITS", wxCheckBox);
    m_rotationAxisCtrl = XRCCTRL(*this, "ID_ROTATION_AXIS", wxCheckBox);
    m_bodyCentricFabricCtrl = XRCCTRL(*this, "ID_BODYCENTRIC_FABRIC", wxCheckBox);
    m_drawTrajectoriesCtrl = XRCCTRL(*this, "ID_DRAW_TRAJECTORIES", wxCheckBox);
////@end CSettingsDlg content construction

	m_showOSDCtrl->SetValue( GetShowOsd() );
	m_autoUnitsCtrl->SetValue( GetAutoUnits() );
	m_rotationAxisCtrl->SetValue( GetRotationAxis() );
	m_bodyCentricFabricCtrl->SetValue( GetBodycentricFabric() );
	m_drawTrajectoriesCtrl->SetValue( GetDrawTrajectories() );

    // Create custom windows not generated automatically here.

////@begin CSettingsDlg content initialisation

////@end CSettingsDlg content initialisation
}

/*!
 * Should we show tooltips?
 */

bool CSettingsDlg::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CSettingsDlg::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CSettingsDlg bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CSettingsDlg bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CSettingsDlg::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CSettingsDlg icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CSettingsDlg icon retrieval
}
/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_OK
 */

bool CSettingsDlg::GetShowOsd()
{
	wxCriticalSectionLocker locker(m_csMTSafe);

	wxConfig config;
	bool b = false;
	config.Read( CSettingsDlg::m_sShowOsd, &b);

	return b;
}

bool CSettingsDlg::GetAutoUnits()
{
	wxCriticalSectionLocker locker(m_csMTSafe);

	wxConfig config;
	bool b = false;
	config.Read( CSettingsDlg::m_sAutoUnits, &b);

	return b;
}

bool CSettingsDlg::GetRotationAxis()
{
	wxCriticalSectionLocker locker(m_csMTSafe);

	wxConfig config;
	bool b = false;
	config.Read( CSettingsDlg::m_sRotationAxis, &b);

	return b;
}

bool CSettingsDlg::GetBodycentricFabric()
{
	wxCriticalSectionLocker locker(m_csMTSafe);

	wxConfig config;
	bool b = false;
	config.Read( CSettingsDlg::m_sBodycentricFabric, &b);

	return b;
}

bool CSettingsDlg::GetDrawTrajectories()
{
	wxCriticalSectionLocker locker(m_csMTSafe);

	wxConfig config;
	bool b = false;
	config.Read( CSettingsDlg::m_sDrawTrajectories, &b);

	return b;
}

void CSettingsDlg::SetShowOsd(bool b)
{
	wxCriticalSectionLocker locker(m_csMTSafe);

	wxConfig config;
	config.Write(CSettingsDlg::m_sShowOsd, b);
}

void CSettingsDlg::SetAutoUnits(bool b)
{
	wxCriticalSectionLocker locker(m_csMTSafe);

	wxConfig config;
	config.Write(CSettingsDlg::m_sAutoUnits, b);
}

void CSettingsDlg::SetRotationAxis(bool b)
{
	wxCriticalSectionLocker locker(m_csMTSafe);

	wxConfig config;
	config.Write(CSettingsDlg::m_sRotationAxis, b);
}

void CSettingsDlg::SetBodycentricFabric(bool b)
{
	wxCriticalSectionLocker locker(m_csMTSafe);

	wxConfig config;
	config.Write(CSettingsDlg::m_sBodycentricFabric, b);
}

void CSettingsDlg::SetDrawTrajectories(bool b)
{
	wxCriticalSectionLocker locker(m_csMTSafe);

	wxConfig config;
	config.Write(CSettingsDlg::m_sDrawTrajectories, b);
}

wxString CSettingsDlg::m_sShowOsd( wxT("/Settings/ShowOsd") );
wxString CSettingsDlg::m_sAutoUnits( wxT("/Settings/AutoUnits") );
wxString CSettingsDlg::m_sRotationAxis( wxT("/Settings/RotationAxis") );
wxString CSettingsDlg::m_sBodycentricFabric( wxT("/Settings/BodycentricFabric") );
wxString CSettingsDlg::m_sDrawTrajectories( wxT("/Settings/DrawTrajectories") );
wxCriticalSection CSettingsDlg::m_csMTSafe;

void CSettingsDlg::OnOkClick( wxCommandEvent& event )
{
	if ( Validate() && TransferDataFromWindow() )
	{
		CVisualization * vis = reinterpret_cast<CVisualization*>( GetClientData() );

		SetShowOsd( m_showOSDCtrl->GetValue() );
		SetAutoUnits( m_autoUnitsCtrl->GetValue() );
		SetRotationAxis( m_rotationAxisCtrl->GetValue() );
		SetBodycentricFabric( m_bodyCentricFabricCtrl->GetValue() );
		SetDrawTrajectories( m_drawTrajectoriesCtrl->GetValue() );

		EndDialog( wxID_OK );
	}
}
