#include "StdAfx.h"
#include ".\sphericalobserver.h"
#include "PhaethonApp.h"
#include "sphericalcamera.h"
#include "ExtendedBody.h"

#include <schwarzmeier/std.h>

IMPLEMENT_DYNAMIC_CLASS( CSphericalObserver, CRectangularObserver )

/*!
 * CCamerasPanel event table definition
 */

BEGIN_EVENT_TABLE( CSphericalObserver, CRectangularObserver )

    EVT_BUTTON( XRCID("ID_SETSPHERICAL"), CSphericalObserver::OnSetClick )

    EVT_BUTTON( XRCID("IDC_ARROW_RIGHT_UP"), CSphericalObserver::OnCArrowRightUpClick )
    EVT_BUTTON( XRCID("IDC_ARROW_UP"), CSphericalObserver::OnCArrowUpClick )
    EVT_BUTTON( XRCID("IDC_ARROW_LEFT"), CSphericalObserver::OnCArrowLeftClick )
    EVT_BUTTON( XRCID("IDC_ARROW_RIGHT"), CSphericalObserver::OnCArrowRightClick )
    EVT_BUTTON( XRCID("IDC_ARROW_DOWN"), CSphericalObserver::OnCArrowDownClick )
    EVT_BUTTON( XRCID("IDC_ARROW_LEFT_DOWN"), CSphericalObserver::OnCArrowLeftDownClick )

END_EVENT_TABLE()


using namespace JakubSchwarzmeier::Std::Math;

CSphericalObserver::CSphericalObserver()
{
	m_bMouseRightDrag = false;
}

bool CSphericalObserver::Create( wxWindow* parent, JakubSchwarzmeier::wx::COpenGLCanvas * pglCanvas, wxWindowID id)
{
	return CRectangularObserver::Create( parent, pglCanvas, id );
}

void CSphericalObserver::OnCreateCamera()
{
	if (m_pCamera == NULL)
		m_pCamera = new CSphericalCamera;
}

void CSphericalObserver::CreateControls()
{
	CRectangularObserver::CreateControls();

    wxPanel * matrixPanel = XRCCTRL(*this, "ID_MATRIX", wxPanel);
	matrixPanel->Enable( false );

	wxButton * buttons[6];
	buttons[0] = XRCCTRL(*this, "ID_HEADING_LEFT", wxButton);
	buttons[1] = XRCCTRL(*this, "ID_HEADING_RIGHT", wxButton);
	buttons[2] = XRCCTRL(*this, "ID_PITCH_UP", wxButton);
	buttons[3] = XRCCTRL(*this, "ID_PITCH_DOWN", wxButton);
	buttons[4] = XRCCTRL(*this, "ID_ROLL_CCW", wxButton);
	buttons[5] = XRCCTRL(*this, "ID_ROLL_CW", wxButton);
	for(int i = 0; i < 6; i++)  buttons[i]->Enable(false);

    wxXmlResource::Get()->Load(wxT("xrc/sphericalobserver.xrc")); 

	wxNotebook * notebook = XRCCTRL(*this, "ID_NOTEBOOK", wxNotebook);

	wxNotebookPage * page =
		wxXmlResource::Get()->LoadPanel(notebook, _T("IDD_SPHERICALOBSERVER"));
	//  Do not delete the page, it will be deleted by the notebook

	notebook->AddPage( page, _T("Spherical") );

	m_distance = m_latitude = m_longitude = 0;
    if (FindWindow(XRCID("ID_DISTANCE")))
        FindWindow(XRCID("ID_DISTANCE"))->SetValidator( wxDoubleValidator(& m_distance) );

    if (FindWindow(XRCID("ID_LATITUDE")))
        FindWindow(XRCID("ID_LATITUDE"))->SetValidator( wxDoubleValidator(& m_latitude) );

    if (FindWindow(XRCID("ID_LONGITUDE")))
        FindWindow(XRCID("ID_LONGITUDE"))->SetValidator( wxDoubleValidator(& m_longitude) );

	m_distance = dynamic_cast<CSphericalCamera*>( m_pCamera )->GetDistance();
	m_latitude = dynamic_cast<CSphericalCamera*>( m_pCamera )->GetLatitude();
	m_longitude = dynamic_cast<CSphericalCamera*>( m_pCamera )->GetLongitude();
}


void CSphericalObserver::OnSetClick( wxCommandEvent& event )
{
	if ( Validate() && TransferDataFromWindow() )
	{
		CSphericalCamera* cam = dynamic_cast<CSphericalCamera*>( m_pCamera );

		if ( m_distance != cam->GetDistance() )
			cam->SetDistance( m_distance );

		if ( m_latitude != cam->GetLatitude() )
			cam->SetLatitude( m_latitude );

		if ( m_longitude != cam->GetLongitude() )
			cam->SetLongitude( m_longitude );

		OnUpdateInputs();
		m_pglCanvas->ReRender();
	}	
}

bool CSphericalObserver::UpdateInputs()
{
	bool shift;
	if (m_keys[WXK_SHIFT])
		shift = true;
	else
		shift = false;

	CalculateFrameRate();

	// Once we have the frame interval, we just need to find the current speed.
	// This is done by multiplying our speed by the elapsed time between frames.
	// We can then pass that speed into our camera movement functions.
	float positionSpeed = m_stepSize.position; // / m_framesPerSecond;
	float angleSpeed = m_stepSize.angle; // / m_framesPerSecond;

	bool used = false;

	CSphericalCamera* m_pSphericalCamera = dynamic_cast< CSphericalCamera* >( m_pCamera );

	if ( m_keys[WXK_LEFT] && !shift )
	{
		m_pSphericalCamera->ChangeLongitude( -angleSpeed );
		OnUpdateInputs();
		used = true;
	}

	if ( m_keys[WXK_RIGHT] && !shift )	
	{
		m_pSphericalCamera->ChangeLongitude( angleSpeed );
		OnUpdateInputs();
		used = true;
	}

	if ( m_keys[WXK_UP] && !shift )		
	{
		m_pSphericalCamera->ChangeDistance(-positionSpeed);
		OnUpdateInputs();
		used = true;
	}

	if ( m_keys[WXK_DOWN] && !shift )
	{
		m_pSphericalCamera->ChangeDistance(positionSpeed);
		OnUpdateInputs();
		used = true;
	}

	if ( m_keys[WXK_UP] && shift )		
	{
		m_pSphericalCamera->ChangeLatitude( angleSpeed );
		OnUpdateInputs();
		used = true;
	}

	if ( m_keys[WXK_DOWN]&& shift )
	{
		m_pSphericalCamera->ChangeLatitude( -angleSpeed );
		OnUpdateInputs();
		used = true;
	}

	if ( m_keys[WXK_LEFT] && shift )		
	{
		m_pSphericalCamera->ChangeLongitude( angleSpeed );
		OnUpdateInputs();
		used = true;
	}

	if ( m_keys[WXK_RIGHT] && shift )	
	{
		m_pSphericalCamera->ChangeLongitude( -angleSpeed );
		OnUpdateInputs();
		used = true;
	}


	return used;
}

void CSphericalObserver::OnUpdateInputs()
{
	CSphericalCamera* m_pSphericalCamera = dynamic_cast< CSphericalCamera* >( m_pCamera );
	m_distance = m_pSphericalCamera->GetDistance();
	m_longitude = m_pSphericalCamera->GetLongitude();
	m_latitude = m_pSphericalCamera->GetLatitude();

	CRectangularObserver::OnUpdateInputs();
}

void CSphericalObserver::OnCArrowRightUpClick( wxCommandEvent& event )
{
	CSphericalCamera* m_pSphericalCamera = dynamic_cast< CSphericalCamera* >( m_pCamera );
	m_pSphericalCamera->ChangeDistance(-m_stepSize.position);
	OnUpdateInputs();
	m_pglCanvas->ReRender();
}

void CSphericalObserver::OnCArrowLeftDownClick( wxCommandEvent& event )
{
	CSphericalCamera* m_pSphericalCamera = dynamic_cast< CSphericalCamera* >( m_pCamera );
	m_pSphericalCamera->ChangeDistance(m_stepSize.position);
	OnUpdateInputs();
	m_pglCanvas->ReRender();
}

void CSphericalObserver::OnCArrowUpClick( wxCommandEvent& event )
{
	CSphericalCamera* m_pSphericalCamera = dynamic_cast< CSphericalCamera* >( m_pCamera );
	m_pSphericalCamera->ChangeLatitude( m_stepSize.angle );
	OnUpdateInputs();
	m_pglCanvas->ReRender();
}

void CSphericalObserver::OnCArrowDownClick( wxCommandEvent& event )
{
	CSphericalCamera* m_pSphericalCamera = dynamic_cast< CSphericalCamera* >( m_pCamera );
	m_pSphericalCamera->ChangeLatitude( -m_stepSize.angle );
	OnUpdateInputs();
	m_pglCanvas->ReRender();
}

void CSphericalObserver::OnCArrowLeftClick( wxCommandEvent& event )
{
	CSphericalCamera* m_pSphericalCamera = dynamic_cast< CSphericalCamera* >( m_pCamera );
	m_pSphericalCamera->ChangeLongitude( -m_stepSize.angle );
	OnUpdateInputs();
	m_pglCanvas->ReRender();
}

void CSphericalObserver::OnCArrowRightClick( wxCommandEvent& event )
{
	CSphericalCamera* m_pSphericalCamera = dynamic_cast< CSphericalCamera* >( m_pCamera );
	m_pSphericalCamera->ChangeLongitude( m_stepSize.angle );
	OnUpdateInputs();
	m_pglCanvas->ReRender();

}

void CSphericalObserver::OnMouse(wxMouseEvent& event)
{
	if ( event.RightDown() )
	{
		m_bMouseRightDrag = true;
		m_posRMouse.x = event.GetX();
		m_posRMouse.y = event.GetY();
	}
	else if ( event.RightUp() )
	{
		m_bMouseRightDrag = false;
	}
	else if (m_bMouseRightDrag)
	{
 		CSphericalCamera* m_pSphericalCamera = dynamic_cast< CSphericalCamera* >( m_pCamera );

		POINT m_RMouseRotate;
		m_RMouseRotate.x = (event.GetX() - m_posRMouse.x);
 		m_RMouseRotate.y = (event.GetY() - m_posRMouse.y);
		
		m_pSphericalCamera->ChangeLongitude( m_RMouseRotate.x * m_stepSize.angle );
		m_pSphericalCamera->ChangeLatitude( m_RMouseRotate.y * m_stepSize.angle );

		m_posRMouse.x = event.GetX();
		m_posRMouse.y = event.GetY();
	}

	int rot = event.GetWheelRotation();
	if ( rot != 0)
	{
 		CSphericalCamera* m_pSphericalCamera = dynamic_cast< CSphericalCamera* >( m_pCamera );

		ExtendedBody * b = dynamic_cast<ExtendedBody*>( m_pSphericalCamera->GetCentralBodyPtr() );
		if (b != NULL)
			m_pSphericalCamera->ChangeDistance( -rot * (1.0/ event.GetWheelDelta()) * (1.0/3.0) * b->GetRadius() );
		else
			m_pSphericalCamera->ChangeDistance( -rot * (1.0/ event.GetWheelDelta()) * m_stepSize.position );
	}

	UpdateInputs();
	m_pglCanvas->ReRender();
}