/////////////////////////////////////////////////////////////////////////////
// Name:        SpiceFilesDlg.cpp
// Purpose:     
// Author:      Jakub Schwarzmeier
// Modified by: 
// Created:     08/11/06 16:37:58
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#if defined(__GNUG__) && !defined(__APPLE__)
#pragma implementation "CSpiceFilesDlg.h"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
////@end includes

#include "SpiceFilesDlg.h"
#include <set>
#include <schwarzmeier/astronomy.h>

using namespace JakubSchwarzmeier::Astronomy;

////@begin XPM images
////@end XPM images

/*!
 * CSpiceFilesDlg type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CSpiceFilesDlg, wxDialog )

/*!
 * CSpiceFilesDlg event table definition
 */

BEGIN_EVENT_TABLE( CSpiceFilesDlg, wxDialog )

////@begin CSpiceFilesDlg event table entries
    EVT_BUTTON( XRCID("ID_ADD"), CSpiceFilesDlg::OnAddClick )

    EVT_BUTTON( XRCID("ID_REMOVE"), CSpiceFilesDlg::OnRemoveClick )
    EVT_UPDATE_UI( XRCID("ID_REMOVE"), CSpiceFilesDlg::OnRemoveUpdate )

////@end CSpiceFilesDlg event table entries

END_EVENT_TABLE()

/*!
 * CSpiceFilesDlg constructors
 */

CSpiceFilesDlg::CSpiceFilesDlg( )
{
}

CSpiceFilesDlg::CSpiceFilesDlg( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Create(parent, id, caption, pos, size, style);
}

/*!
 * CSpiceFilesDlg creator
 */

bool CSpiceFilesDlg::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CSpiceFilesDlg member initialisation
    m_list = NULL;
////@end CSpiceFilesDlg member initialisation

////@begin CSpiceFilesDlg creation
    SetExtraStyle(GetExtraStyle()|wxWS_EX_BLOCK_EVENTS);
    SetParent(parent);
    CreateControls();
    GetSizer()->Fit(this);
    GetSizer()->SetSizeHints(this);
    Centre();
////@end CSpiceFilesDlg creation
    return true;
}

/*!
 * Control creation for CSpiceFilesDlg
 */

void CSpiceFilesDlg::CreateControls()
{    
    wxXmlResource::Get()->Load(wxT("xrc/spicefiles.xrc")); 

////@begin CSpiceFilesDlg content construction
    wxXmlResource::Get()->LoadDialog(this, GetParent(), _T("IDD_SPICEFILES"));
    m_list = XRCCTRL(*this, "ID_LISTCTRL", wxListCtrl);
////@end CSpiceFilesDlg content construction

    // Create custom windows not generated automatically here.

////@begin CSpiceFilesDlg content initialisation

////@end CSpiceFilesDlg content initialisation

	//  Listbox
	wxListItem itemCol;
    itemCol.m_format = wxLIST_FORMAT_LEFT;
	itemCol.m_mask = wxLIST_MASK_FORMAT;
    m_list->InsertColumn(0, itemCol);
	
    // to speed up inserting we hide the control temporarily
    m_list->Hide();

	void * clientData = GetClientData();
	std::set<std::string> ** vecFiles = reinterpret_cast<std::set<std::string>**>( clientData );

	//  SPK
	long l = 0;
	std::set<std::string>::const_iterator it;
	for( it = (vecFiles[0])->begin(); it != (vecFiles[0])->end(); it++ )
	{
		m_list->InsertItem( l++, it->c_str() );
	}

	//  PCK
	for( it = (vecFiles[1])->begin(); it != (vecFiles[1])->end(); it++ )
	{
		m_list->InsertItem( l++, it->c_str() );
	}

	m_list->SetColumnWidth(0, wxLIST_AUTOSIZE);
    m_list->Show();
}

/*!
 * Should we show tooltips?
 */

bool CSpiceFilesDlg::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CSpiceFilesDlg::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CSpiceFilesDlg bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CSpiceFilesDlg bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CSpiceFilesDlg::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CSpiceFilesDlg icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CSpiceFilesDlg icon retrieval
}
/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_ADD
 */

void CSpiceFilesDlg::OnAddClick( wxCommandEvent& event )
{
	wxFileDialog fileDialog(this, wxFileSelectorPromptStr, wxEmptyString, wxEmptyString,
                wxFileSelectorDefaultWildcardStr, wxFD_DEFAULT_STYLE | wxMULTIPLE | wxFILE_MUST_EXIST );

    fileDialog.CentreOnParent();
	if ( fileDialog.ShowModal() == wxID_OK )
	{
		void * clientData = GetClientData();
		std::set<std::string> ** vecFiles = reinterpret_cast<std::set<std::string>**>( clientData );

		m_list->Hide();
		wxArrayString paths;
		fileDialog.GetPaths( paths );
		for(size_t i = 0; i < paths.GetCount(); i++)
		{
			std::string filePath( paths.Item( i ).c_str() );

			std::string type = CSpice::GetFileType( filePath );
			
			wxListItem itemCol;
			itemCol.SetText( filePath.c_str() );
			itemCol.m_mask = wxLIST_MASK_TEXT;

			if ( type.compare( "SPK" ) == 0 )
			{
				std::pair < std::set<std::string>::iterator, bool > pr = vecFiles[0]->insert( filePath );
				if (pr.second == true)	m_list->InsertItem( itemCol );
			}
			else if ( type.compare( "?" ) != 0 )
			{
				std::pair < std::set<std::string>::iterator, bool > pr = vecFiles[1]->insert( filePath );
				if (pr.second == true)	m_list->InsertItem( itemCol );
			}
			else
			{
				wxString str;
				str.sprintf( _("File %s is not known to SPICE."), filePath.c_str());
				wxMessageDialog msg(this, str );
				msg.ShowModal();
			}
		}

		m_list->SetColumnWidth(0, wxLIST_AUTOSIZE);
		m_list->Show();
	}
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_REMOVE
 */

void CSpiceFilesDlg::OnRemoveUpdate( wxUpdateUIEvent& event )
{
	event.Enable( (m_list->GetSelectedItemCount() != 0) );
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_REMOVE
 */

void CSpiceFilesDlg::OnRemoveClick( wxCommandEvent& event )
{
	//  starting selected item
    long item = m_list->GetNextItem(-1, wxLIST_NEXT_ALL,
                                        wxLIST_STATE_SELECTED);
	void * clientData = GetClientData();
	std::set<std::string> ** vecFiles = reinterpret_cast<std::set<std::string>**>( clientData );

	while ( item != -1 )
    {
		//  we do not know from which list
		vecFiles[0]->erase( m_list->GetItemText(item).c_str() );
		vecFiles[1]->erase( m_list->GetItemText(item).c_str() );

		long prevItem = item;

		//  next item
        item = m_list->GetNextItem(item, wxLIST_NEXT_ALL,
                                       wxLIST_STATE_SELECTED);

		m_list->DeleteItem(prevItem);
    }
}


