#include "StdAfx.h"
#include "startarget.h"

using namespace JakubSchwarzmeier::Std::Graphics;

CStarTarget::CStarTarget(Body * body, CVisualization * visualization):
	CExtendedBodyTarget( body, visualization )
{
	m_glLight = 0;
}

CStarTarget::~CStarTarget(void)
{
	if (m_glLight != 0)	throw;
}

void CStarTarget::Render()
{
	glPushAttrib( GL_ENABLE_BIT );
	glDisable(GL_LIGHTING);
		CExtendedBodyTarget::Render( );
	glPopAttrib();

	//  always draw glare, even when not in frustum
	if( m_glLight == 0 )
	{
		m_glLight = m_pVisualization->GetThreedCanvas()->ReserveLight();
		glEnable( m_glLight );

		float ambience[4] = {0.0f, 0.0f, 0.0f, 1.0};		// The color of the light in the world
		float diffuse[4] = {1.f, 1.f, 1.f, 1.0};			// The color of the positioned light

		glLightfv( m_glLight, GL_AMBIENT,  ambience );		// Set our ambience values (Default color without direct light)
		glLightfv( m_glLight, GL_DIFFUSE,  diffuse );		// Set our diffuse color (The light color)
	}	

	//  Tell GL new light source position.
	RotationMatrix objectInEye = GetObjectInEye();

	//  set light position
	float position[4] = {objectInEye[12], objectInEye[13], objectInEye[14], 1};
    glLightfv(m_glLight, GL_POSITION, position ); 

}

void CStarTarget::OnContextTerminate()
{
	CExtendedBodyTarget::OnContextTerminate();

	if (m_glLight != 0)
	{
		glDisable( m_glLight );
		m_pVisualization->GetThreedCanvas()->DisposeLight(m_glLight);
		m_glLight = 0;
	}
}