	// RecognitaView.cpp : implementation of the CThreedCanvas class
//

#include "stdafx.h"
#include "PhaethonApp.h"

#include "PhaethonDoc.h"
#include "ThreedCanvas.h"

#include "PhaethonView.h"

#include <schwarzmeier/std.h>

using namespace JakubSchwarzmeier::Std::Math;

IMPLEMENT_DYNAMIC_CLASS(CThreedCanvas, JakubSchwarzmeier::wx::COpenGLCanvas)

BEGIN_EVENT_TABLE(CThreedCanvas, JakubSchwarzmeier::wx::COpenGLCanvas)

    EVT_KEY_DOWN(CThreedCanvas::OnKeyDown)

END_EVENT_TABLE()


// CThreedCanvas

// CThreedCanvas construction/destruction

CThreedCanvas::CThreedCanvas(): m_condVideo(m_mutex)
{
}

CThreedCanvas::CThreedCanvas(wxWindow *parent): m_condVideo(m_mutex)
{
	Create( parent );
}

void CThreedCanvas::Create(wxWindow *parent, bool bDeserialize)
{
	m_pDocument = ::wxGetApp().GetDocument();
	m_pVisualization = m_pDocument->GetVisualization();

	m_bAnimationActive = false;

	COpenGLCanvas::Create(parent, bDeserialize);

	m_pVisualization->Create( m_pDocument->GetSimulation(), this );

	GetMyContext()->CreatePBuffer();

	SetSignaled( false );
	m_bLighting = true;
	m_bColorMaterial = true;
}

//virtual
void CThreedCanvas::OnRender()
{
	//  clear the color and depth buffers 
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); 

	//  modeling transformation in ModelView matrix

	glLoadIdentity();

	//  simulate viewing transformation in ModelView matrix
//	glLoadMatrixd( m_observer->GetCamera()->GetMatrix() );

	//  objects drawing

	// We don't need to calculate this every frame, only when the camera view changes.
	// I just did it every frame anyway.  In this case it isn't a big deal.
    m_frustum.CreateSimpleFrustum();		//  calculate frustum

	AcquireRendering();
	m_pVisualization->Render();
	ReleaseRendering();

	//  flush rendering pipe
	glFinish();	
}

//virtual
void CThreedCanvas::PreRender()
{
	COpenGLCanvas::PreRender();

	//  If enabled, draw points with proper filtering. 
	//  Otherwise, draw aliased points.
	glEnable( GL_POINT_SMOOTH );

	m_pVisualization->PreRender();
}

void CThreedCanvas::OnContextTerminate()
{
	m_pVisualization->OnContextTerminate();
}

bool CThreedCanvas::OnSerialize( JakubSchwarzmeier::wx::CSerializeReaderWriter * dir )
{
	COpenGLCanvas::OnSerialize( dir );

	return true;
}

void CThreedCanvas::RenderFinished()
{
	COpenGLCanvas::RenderFinished();

	if (m_bAnimationActive)	m_condVideo.Signal();
}


void CThreedCanvas::Draw3DGrid()
{
	// Turn the lines GREEN
	glColor3ub(0, 255, 0);

	// Draw a 1x1 grid along the X and Z axis'
	for(float i = -50; i <= 50; i += 1)
	{
		// Start drawing some lines
		glBegin(GL_LINES);

			// Do the horizontal lines (along the X)
			glVertex3f(-50, 0, i);
			glVertex3f(50, 0, i);

			// Do the vertical lines (along the Z)
			glVertex3f(i, 0, -50);
			glVertex3f(i, 0, 50);

		// Stop drawing lines
		glEnd();
	}
} 

void CThreedCanvas::OnKeyDown(wxKeyEvent& event)
{
	m_csRenderingContext.Enter();

	switch (event.GetKeyCode())
	{
	case WXK_ADD:
	case WXK_NUMPAD_ADD:
	
		m_pVisualization->SetEt( m_pVisualization->GetEt() + m_pVisualization->GetDt() );
		break;

	case WXK_SUBTRACT:
	case WXK_NUMPAD_SUBTRACT:
	
		m_pVisualization->SetEt( m_pVisualization->GetEt() - m_pVisualization->GetDt() );
		break;
	}

	m_csRenderingContext.Leave();

	event.Skip();
}

const CFrustum * CThreedCanvas::GetFrustum()
{
	return &m_frustum;
}

void CThreedCanvas::AcquireRendering()
{
//	m_csRendering.Enter();
	m_csRenderingContext.Enter();
}

void CThreedCanvas::ReleaseRendering()
{
//	m_csRendering.Leave();
	m_csRenderingContext.Leave();
}

void CThreedCanvas::ForceReRender()
{
	if (m_bAnimationActive)	m_mutex.Lock();
	COpenGLCanvas::ForceReRender();
	if (m_bAnimationActive) m_condVideo.Wait();
}

void CThreedCanvas::SetAnimation(bool active)
{
	m_bAnimationActive = active;
}
