#ifndef __VISUALIZATION__
#define __VISUALIZATION__

#include "Simulation.h"
#include "RenderingTarget.h"
#include "ChunkDesc.h"
#include <vector>
#include "ThreedCanvas.h"
#include <schwarzmeier/wx.h>

//  forward
class CThreedCanvas;
class CRenderingTarget;

class CVisualization
{
public:
	CVisualization();
	void Create(Simulation * simulation, CThreedCanvas * pOpenGlCanvas);
	~CVisualization(void);

	void SetEt(double et);
	double GetEt();

	void SetDt(double dt);
	double GetDt();

	void SetOutputVideo(std::string outputVideo);
	std::string GetOutputVideo();

	void SetFps(double fps);
	double GetFps();

	bool GetCoordSolar();
	void SetCoordSolar(bool coordSolar);
	std::string GetCoordCustom();
	void SetCoordCustom(std::string coordCustom);

	std::vector<CChunkDesc*> * GetChunks();

	virtual void Serialize(xmlDoc * pXmlDoc, xmlNode * branch);
	virtual void Deserialize(xmlDoc * pXmlDoc, xmlNode * pXmlNode);

	void CreateRenderingTargets();
	void Render();
	void PreRender();
	void OnContextTerminate();

	CThreedCanvas * GetThreedCanvas();

	void SetNearestAndFurthermost( double nearest, double furthermost );

	void AddHeading(std::string heading);
	void AddInformation(wxString info);

	JakubSchwarzmeier::wx::CFreetype * GetBodyFont();
	Simulation * GetSimulation();

	bool GetShowOsd();
	bool GetAutoUnits();
	bool GetRotationAxis();
	bool GetBodycentricFabric();
	bool GetDrawTrajectories();

	void ClearTrajectories();

	void DistanceAutoUnit( double & val, wxString & strUnit );

protected:

	std::vector<CChunkDesc*> m_chunks;
	double m_et;
	double m_dt;
	std::string m_outputVideo;
	double m_fps;
	bool m_bCoordSolar;
	std::string m_coordCustom;

	Simulation * m_pSimulation;
	CThreedCanvas * m_pOpenGlCanvas;

	std::vector<CRenderingTarget*> m_renderingTargets;

	JakubSchwarzmeier::wx::CFreetype * m_pBodyFont;
	JakubSchwarzmeier::wx::CFreetype * m_pHeadingFont;
	JakubSchwarzmeier::wx::CFreetype * m_pInformationFont;

	std::vector<std::string> m_headings;
	std::vector<std::string> m_informations;

	unsigned int m_sfDisplayList;
	GLuint m_starTexture;

	void CreateStarField();
	void CreateStar();

	GLEWContext * glewGetContext();
	WGLEWContext * wglewGetContext();

	static std::string s_strVisualization;
	static std::string s_strTimestep;
	static std::string s_strTime;
	static std::string s_strFps;
	static std::string s_strCoordSolar;
	static std::string s_strCoordCustom;
	static std::string s_strOutputVideo;
	static std::string s_strCameraChunks;
	static std::string s_strCameraChunk;
	static std::string s_strUse;

	static std::string s_strSolar;
};


#endif //  __VISUALIZATION__