/////////////////////////////////////////////////////////////////////////////
// Name:        VisualizationSettingsDlg.cpp
// Purpose:     
// Author:      Jakub Schwarzmeier
// Modified by: 
// Created:     08/21/06 19:41:01
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#if defined(__GNUG__) && !defined(__APPLE__)
#pragma implementation "VisualizationSettingsDlg.h"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
////@end includes

#include "VisualizationSettingsDlg.h"
#include "PhaethonApp.h"
#include "Visualization.h"

////@begin XPM images
////@end XPM images

using namespace JakubSchwarzmeier::Astronomy;

/*!
 * CVisualizationSettingsDlg type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CVisualizationSettingsDlg, wxDialog )

/*!
 * CVisualizationSettingsDlg event table definition
 */

BEGIN_EVENT_TABLE( CVisualizationSettingsDlg, wxDialog )

////@begin CVisualizationSettingsDlg event table entries
    EVT_UPDATE_UI( XRCID("ID_CUSTOM_COORD"), CVisualizationSettingsDlg::OnCustomCoordUpdate )

    EVT_BUTTON( wxID_OK, CVisualizationSettingsDlg::OnOkClick )

////@end CVisualizationSettingsDlg event table entries

END_EVENT_TABLE()

/*!
 * CVisualizationSettingsDlg constructors
 */

CVisualizationSettingsDlg::CVisualizationSettingsDlg( )
{
}

CVisualizationSettingsDlg::CVisualizationSettingsDlg( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Create(parent, id, caption, pos, size, style);
}

/*!
 * CVisualizationSettingsDlg creator
 */

bool CVisualizationSettingsDlg::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CVisualizationSettingsDlg member initialisation
    m_dateEtCtrl = NULL;
    m_coordSolarCtrl = NULL;
////@end CVisualizationSettingsDlg member initialisation

////@begin CVisualizationSettingsDlg creation
    SetExtraStyle(GetExtraStyle()|wxWS_EX_BLOCK_EVENTS);
    SetParent(parent);
    CreateControls();
    GetSizer()->Fit(this);
    GetSizer()->SetSizeHints(this);
    Centre();
////@end CVisualizationSettingsDlg creation
    return true;
}

/*!
 * Control creation for CVisualizationSettingsDlg
 */

void CVisualizationSettingsDlg::CreateControls()
{    
    wxXmlResource::Get()->Load(wxT("xrc/visualizationsettings.xrc")); 

////@begin CVisualizationSettingsDlg content construction
    wxXmlResource::Get()->LoadDialog(this, GetParent(), _T("ID_DIALOG"));
    m_dateEtCtrl = XRCCTRL(*this, "ID_ET", wxCalendarCtrl);
    m_coordSolarCtrl = XRCCTRL(*this, "ID_COORD_SOLAR", wxRadioButton);
    // Set validators
    if (FindWindow(XRCID("ID_TIME")))
        FindWindow(XRCID("ID_TIME"))->SetValidator( wxTimeValidator(& m_timeEt) );
    if (FindWindow(XRCID("ID_TEXTCTRL5")))
        FindWindow(XRCID("ID_TEXTCTRL5"))->SetValidator( wxDoubleValidator(& m_dt) );
    if (FindWindow(XRCID("ID_CUSTOM_COORD")))
        FindWindow(XRCID("ID_CUSTOM_COORD"))->SetValidator( wxGenericValidator( & m_customCoord ) );
////@end CVisualizationSettingsDlg content construction

    // Create custom windows not generated automatically here.
	CVisualization * vis = ::wxGetApp().GetDocument()->GetVisualization();
	
	m_dt = vis->GetDt();
	m_timeEt = wxDateTime( vis->GetEt() );
	m_dateEtCtrl->SetDate( vis->GetEt() );

	if ( vis->GetCoordSolar() )	m_coordSolarCtrl->SetValue( true );
	m_customCoord = vis->GetCoordCustom().c_str();

////@begin CVisualizationSettingsDlg content initialisation

////@end CVisualizationSettingsDlg content initialisation
}

/*!
 * Should we show tooltips?
 */

bool CVisualizationSettingsDlg::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CVisualizationSettingsDlg::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CVisualizationSettingsDlg bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CVisualizationSettingsDlg bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CVisualizationSettingsDlg::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CVisualizationSettingsDlg icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CVisualizationSettingsDlg icon retrieval
}
/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_OK
 */

void CVisualizationSettingsDlg::OnOkClick( wxCommandEvent& event )
{
	if ( Validate() && TransferDataFromWindow() )
	{
		CDateTime datetime( m_dateEtCtrl->GetDate().ToUTC(true) );
		datetime.SetHour( m_timeEt.GetHour() );
		datetime.SetMinute( m_timeEt.GetMinute() );
		datetime.SetSecond( m_timeEt.GetSecond() );
		datetime.SetMillisecond( m_timeEt.GetMillisecond() );

		CVisualization * vis = ::wxGetApp().GetDocument()->GetVisualization();
		vis->SetDt( m_dt );

		double prevEt = vis->GetEt();
		vis->SetEt( datetime.GetJDN() );
		if ( fabs(prevEt - datetime.GetJDN() ) > 10.0 )	vis->ClearTrajectories();

		vis->SetCoordSolar( m_coordSolarCtrl->GetValue() );
		vis->SetCoordCustom( m_customCoord.c_str() );

		EndDialog( wxID_OK );
	}
}

/*!
 * wxEVT_UPDATE_UI event handler for ID_CUSTOM_COORD
 */

void CVisualizationSettingsDlg::OnCustomCoordUpdate( wxUpdateUIEvent& event )
{
	event.Enable( !m_coordSolarCtrl->GetValue() );
}


