/////////////////////////////////////////////////////////////////////////////
// Name:        mainframe.cpp
// Purpose:     
// Author:      Jakub Schwarzmeier
// Modified by: 
// Created:     08/11/06 10:11:05
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#if defined(__GNUG__) && !defined(__APPLE__)
#pragma implementation "mainframe.h"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
////@end includes

#include "mainframe.h"
#include "phaethonapp.h"
#include "phaethondoc.h"
#include "simulation.h"
#include "runsimulationdlg.h"
#include "app_resources.h"
#include "bodiesdlg.h"
#include "cameraspanel.h"
#include "visualizationsettingsdlg.h"
#include "phaethonview.h"
#include "threedcanvas.h"
#include "videomakerdlg.h"
#include "settingsdlg.h"

////@begin XPM images
////@end XPM images

/*!
 * CMainFrame type definition
 */

IMPLEMENT_CLASS( CMainFrame, wxDocMDIParentFrame )

/*!
 * CMainFrame event table definition
 */

BEGIN_EVENT_TABLE( CMainFrame, wxDocMDIParentFrame )

////@begin CMainFrame event table entries
    EVT_CLOSE( CMainFrame::OnFileExit )

////@end CMainFrame event table entries

    EVT_MENU( XRCID("ID_SETTINGS"), CMainFrame::OnFileSettings)

    EVT_MENU( XRCID("ID_SIMULATION_RUN"), CMainFrame::OnSimulationRunClick )
    EVT_MENU( XRCID("ID_SIMULATION_BODIES"), CMainFrame::OnSimulationBodiesClick )
    EVT_MENU( XRCID("ID_VISUALISATION_ANIMATION"), CMainFrame::OnVisualizationAnimationClick )
    EVT_MENU( XRCID("ID_VISUALISATION_SETTINGS"), CMainFrame::OnVisualizationSettingsClick )
    EVT_MENU( XRCID("ID_CLEAR_TRAJECTORIES"), CMainFrame::OnClearTrajectories )

    EVT_MENU( XRCID("ID_NEW"), CMainFrame::OnFileNew)
    EVT_MENU( XRCID("ID_OPEN"), CMainFrame::OnFileOpen)
    EVT_MENU( XRCID("ID_CLOSE"), CMainFrame::OnFileClose)
    EVT_MENU( XRCID("ID_EXIT"), CMainFrame::OnFileExit)

    EVT_MENU_RANGE(wxID_FILE1, wxID_FILE9, CMainFrame::OnMRUFile)

    EVT_SIZE(CMainFrame::OnSize)

END_EVENT_TABLE()

/*!
 * CMainFrame constructors
 */

CMainFrame::CMainFrame( )
{
}

CMainFrame::CMainFrame( wxDocManager *manager, wxFrame *parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Create( manager, parent, id, caption, pos, size, style );
}

/*!
 * CMainFrame creator
 */

bool CMainFrame::Create( wxDocManager *manager, wxFrame *parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CMainFrame member initialisation
////@end CMainFrame member initialisation

////@begin CMainFrame creation
    SetParent(parent);
    CreateControls();
    Centre();
////@end CMainFrame creation
    return true;
}

/*!
 * Control creation for CMainFrame
 */

void CMainFrame::CreateControls()
{    
    wxXmlResource::Get()->Load(wxT("xrc/mainframe.xrc")); 
    wxXmlResource::Get()->Load(wxT("xrc/menus.xrc")); 

////@begin CMainFrame content construction
    wxXmlResource::Get()->LoadObject(this, GetParent(), wxT("ID_MAINFRAME"), wxT("wxDocMDIParentFrame"));
////@end CMainFrame content construction

    //  Set to this wxDocMDIParentFrame member m_docManager
    m_docManager = ::wxGetApp().GetDocManager(); 

    // Create custom windows not generated automatically here.

    // associate the menu bar with the frame
	SetMenuBar( AppResources::CreatePartialMenubar() );

	m_camerasPanel = NULL;

	// A history of files visited. Use this menu.
    m_docManager->FileHistoryUseMenu( GetMenuBar()->GetMenu(0) );
	m_docManager->FileHistoryAddFilesToMenu();

	int x = wxSystemSettings::GetMetric( wxSYS_SCREEN_X );
	int y = wxSystemSettings::GetMetric( wxSYS_SCREEN_Y );

	SetSize( -1, -1, x * 0.95, y * 0.95 );

////@begin CMainFrame content initialisation

////@end CMainFrame content initialisation
}

/*!
 * Should we show tooltips?
 */

bool CMainFrame::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CMainFrame::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CMainFrame bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CMainFrame bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CMainFrame::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CMainFrame icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CMainFrame icon retrieval
}

CChildFrame * CMainFrame::CreateChildFrame(wxDocument * pDocument, int pViewClass)
{
	//  Get template class, which is a factory for creating new views
	wxDocTemplate * pTemplate;
	if (pViewClass < 0)
	{
		//  Newly created view will be of the same type as currently 
		//  active window.
		//  tady musi bejt zjistovacka jaky vokno je aktiv
		//  wxMDIChildFrame * pMDIActive = GetActiveChild();	// Get the currently active MDI child window. 
		pTemplate = pDocument->GetDocumentTemplate();
	}
	else
	{
		//  Newly created view will be other type than is the curretly opened window.
		wxList list = ::wxGetApp().GetDocManager()->GetTemplates();
		pTemplate = reinterpret_cast<wxDocTemplate*>( list.Item(pViewClass)->GetData() );
	}
    
	wxView * pView = pTemplate->CreateView( pDocument );
	CChildFrame * subframe = new CChildFrame(pDocument, pView, this );
	//  specialni ikona
	//    subframe->SetIcon(wxIcon(_T("chrt_icn")));

    subframe->Show(true);
//	subframe->Update();		//  first redraw of frame and its child

	return subframe;
}

void CMainFrame::OnSize(wxSizeEvent& 
                                  event
                                  )
{
	if ( GetSizer() != NULL )
	{
		int wPanel, hPanel;
		GetSizer()->GetItem((size_t)0)->GetWindow()->GetSize( &wPanel, &hPanel );

		int w, h;
		GetClientSize(&w, &h);
		GetClientWindow()->SetSize(wPanel, 0, w - wPanel, h);
		Layout();

		#ifdef __WXUNIVERSAL__   
			event.Skip();
		#endif	
	}
	else
	{
		event.Skip();
	}

}

void CMainFrame::OnFileSettings( wxCommandEvent& event )
{
	CSettingsDlg dlg(this);
	dlg.ShowModal();
}

/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_SIMULATION_BODIES
 */

void CMainFrame::OnSimulationBodiesClick( wxCommandEvent& event )
{
	CPhaethonDoc * doc = ::wxGetApp().GetDocument();
	Simulation sim = *( doc->GetSimulation() );

	CBodiesDlg dlg;
	dlg.SetClientData( &sim );
	dlg.Create( this  );
	if ( dlg.ShowModal() == wxID_OK )
	{
		doc->GetVisualization()->GetThreedCanvas()->AcquireRendering();
		*( doc->GetSimulation() ) = sim;
		m_camerasPanel->UpdateObservers();
		doc->GetVisualization()->CreateRenderingTargets();
		doc->GetVisualization()->GetThreedCanvas()->ReleaseRendering();

		doc->Modify( TRUE );
	}
}/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_SIMULATION_RUN
 */

void CMainFrame::OnSimulationRunClick( wxCommandEvent& event )
{
	CPhaethonDoc * doc = ::wxGetApp().GetDocument();
	if (doc->GetSimulation()->GetBodies()->size() < 2)
	{
		wxMessageDialog dlg( this, _T("There are too few bodies in the sumulation. At least two bodies needed. Please, add more.") );
		dlg.ShowModal();
		return;
	}

	Simulation sim = *( doc->GetSimulation() );

	CRunSimulationDlg dlg;
	dlg.SetClientData( &sim );
	dlg.Create( this );
	if ( dlg.ShowModal() == wxID_OK )
	{
		doc->GetVisualization()->GetThreedCanvas()->AcquireRendering();
		*( doc->GetSimulation() ) = sim;
		doc->GetVisualization()->CreateRenderingTargets();
		doc->GetVisualization()->GetThreedCanvas()->ReleaseRendering();

		doc->Modify( TRUE );
	}
}

void CMainFrame::OnFileNew(wxCommandEvent& event)
{
	if ( ::wxGetApp().GetDocManager()->GetDocuments().GetCount() == 1 )
	{
		::wxGetApp().GetDocManager()->OnFileClose( event );
	}

	OnNewDocument();

	::wxGetApp().GetDocManager()->OnFileNew( event );

	m_camerasPanel->UpdateObservers();
}

void CMainFrame::OnFileOpen(wxCommandEvent& event)
{
	if ( ::wxGetApp().GetDocManager()->GetDocuments().GetCount() == 1 )
	{
		OnFileClose( event );
	}

	::wxGetApp().GetDocManager()->OnFileOpen( event );

	if ( ::wxGetApp().GetDocManager()->GetDocuments().GetCount() == 1 )
	{
		OnNewDocument();
		m_camerasPanel->UpdateObservers();
	}
}

void CMainFrame::OnFileClose(wxCommandEvent& event)
{
	::wxGetApp().GetDocManager()->OnFileClose( event );

	if ( ::wxGetApp().GetDocManager()->GetDocuments().GetCount() == 0 )
	{
		wxMenuBar * oldMenu = GetMenuBar();
		SetMenuBar( AppResources::CreatePartialMenubar() );

		m_docManager->FileHistoryRemoveMenu( oldMenu->GetMenu(0) );
		m_docManager->FileHistoryUseMenu( GetMenuBar()->GetMenu(0) );
		m_docManager->FileHistoryAddFilesToMenu();
		oldMenu->Destroy();

		GetSizer()->GetItem((size_t)0)->GetWindow()->Destroy();
		SetSizer( NULL );
		int w, h;
		GetClientSize(&w, &h);
		GetClientWindow()->SetSize(0, 0, w, h);

		m_camerasPanel = NULL;
	}
}

void CMainFrame::OnFileExit( wxCloseEvent& event )
{
	if ( ::wxGetApp().GetDocManager()->GetDocuments().GetCount() == 1 )
	{
		OnFileClose( wxCommandEvent() );
	}

	event.Skip();
}

void CMainFrame::OnFileExit(wxCommandEvent& event)
{
	OnFileExit( wxCloseEvent() );

	wxDocMDIParentFrame::OnExit(event);
}

void CMainFrame::OnNewDocument()
{
	wxBoxSizer * mainSizer = new wxBoxSizer( wxHORIZONTAL );
	SetSizer( mainSizer );

	m_camerasPanel = new CCamerasPanel( this );
	mainSizer->Insert(0, m_camerasPanel, 0, wxGROW );
	Layout();

	//  initial size of mainframe
	int wPanel, hPanel;
	mainSizer->GetItem((size_t)0)->GetWindow()->GetSize( &wPanel, &hPanel );

	int w, h;
	GetClientSize(&w, &h);
	GetClientWindow()->SetSize(wPanel, 0, w - wPanel, h);

	//  create full menu
	wxMenuBar * oldMenu = GetMenuBar();
	SetMenuBar( AppResources::CreateFullMenubar() );

	m_docManager->FileHistoryRemoveMenu( oldMenu->GetMenu(0) );
    m_docManager->FileHistoryUseMenu( GetMenuBar()->GetMenu(0) );
	m_docManager->FileHistoryAddFilesToMenu();
	oldMenu->Destroy();
}

void CMainFrame::OnVisualizationAnimationClick( wxCommandEvent& event )
{
	CPhaethonDoc * doc = ::wxGetApp().GetDocument();
	CVisualization * vis = doc->GetVisualization();

	vis->GetThreedCanvas()->SetAutoRender( false );

	CVideoMakerDlg dlg;
	dlg.SetClientData( vis );
	dlg.Create( this );
	if ( dlg.ShowModal() == wxID_OK )
	{
		doc->Modify( TRUE );
	}

	doc->GetVisualization()->GetThreedCanvas()->AcquireRendering();
	vis->GetThreedCanvas()->SetObserver( m_camerasPanel->GetRectangularObserver() );
	doc->GetVisualization()->GetThreedCanvas()->ReleaseRendering();

	vis->GetThreedCanvas()->SetAutoRender( true );

}

void CMainFrame::OnVisualizationSettingsClick( wxCommandEvent& event )
{
	CPhaethonDoc * doc = ::wxGetApp().GetDocument();
	CVisualization * vis = doc->GetVisualization();

	vis->GetThreedCanvas()->AcquireRendering();

	CVisualizationSettingsDlg dlg(this);
	dlg.ShowModal();

	vis->GetThreedCanvas()->ReleaseRendering();
}

void CMainFrame::OnMRUFile(wxCommandEvent& event)
{
	if ( ::wxGetApp().GetDocManager()->GetDocuments().GetCount() == 1 )
	{
		OnFileClose( event );
	}

	wxDocMDIParentFrame::OnMRUFile( event );

	if ( ::wxGetApp().GetDocManager()->GetDocuments().GetCount() == 1 )
	{
		OnNewDocument();
	}
}

void CMainFrame::OnClearTrajectories(wxCommandEvent& event)
{
	CPhaethonDoc * doc = ::wxGetApp().GetDocument();
	CVisualization * vis = doc->GetVisualization();
	vis->ClearTrajectories();
}
